Package: report
Type: Package
Title: Automated Reporting of Results and Statistical Models
Version: 0.6.4
Authors@R: 
    c(person(given = "Dominique",
             family = "Makowski",
             role = "aut",
             email = "dom.makowski@gmail.com",
             comment = c(ORCID = "0000-0001-5375-9967")),
      person(given = "Daniel",
             family = "Lüdecke",
             role = "aut",
             email = "d.luedecke@uke.de",
             comment = c(ORCID = "0000-0002-8895-3206")),
      person(given = "Indrajeet",
             family = "Patil",
             role = "aut",
             email = "patilindrajeet.science@gmail.com",
             comment = c(ORCID = "0000-0003-1995-6531")),
      person(given = "Rémi",
             family = "Thériault",
             role = c("aut", "cre"),
             email = "remi.theriault@mail.mcgill.ca",
             comment = c(ORCID = "0000-0003-4315-6788")),
      person(given = "Mattan S.",
             family = "Ben-Shachar",
             role = "aut",
             email = "matanshm@post.bgu.ac.il",
             comment = c(ORCID = "0000-0002-4287-4801")),
      person(given = "Brenton M.",
             family = "Wiernik",
             role = "aut",
             email = "brenton@wiernik.org",
             comment = c(ORCID = "0000-0001-9560-6336")),
      person(given = "Rudolf",
             family = "Siegel",
             role = "ctb",
             email = "mutlusun@users.noreply.github.com",
             comment = c(ORCID = "0000-0002-6021-804X")),
     person(given = "Camden",
            family = "Bock",
            role = "ctb",
            email = "camden.bock@maine.edu",
            comment = c(ORCID = "0000-0002-3907-7748")))
Maintainer: Rémi Thériault <remi.theriault@mail.mcgill.ca>
Description: The aim of the 'report' package is to bridge the gap between
    R’s output and the formatted results contained in your manuscript.
    This package converts statistical models and data frames into textual
    reports suited for publication, ensuring standardization and quality
    in results reporting.
License: MIT + file LICENSE
URL: https://easystats.github.io/report/
BugReports: https://github.com/easystats/report/issues
Depends: R (>= 3.6)
Imports: bayestestR (>= 0.17.0), effectsize (>= 1.0.2), insight (>=
        1.4.6), parameters (>= 0.28.3), performance (>= 0.16.0),
        datawizard (>= 1.3.0), stats, tools, utils
Suggests: BayesFactor, brms, collapse, GLMMadaptive, glmmTMB, ivreg,
        knitr, lavaan, lme4, loo, nlme, dplyr, Formula, rmarkdown,
        rstanarm, survival, modelbased (>= 0.9.0), emmeans,
        marginaleffects (>= 0.25.0), RcppEigen, BH, testthat (>= 3.2.1)
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
Config/testthat/edition: 3
Config/Needs/website: rstudio/bslib, r-lib/pkgdown,
        easystats/easystatstemplate
Collate: 'cite_easystats.R' 'format_algorithm.R' 'format_citation.R'
        'format_formula.R' 'format_model.R' 'reexports.R'
        'report-package.R' 'report.BFBayesFactor.R'
        'utils_combine_tables.R' 'report.lm.R' 'report.MixMod.R'
        'report_text.R' 'report.R' 'report.htest.R' 'report.aov.R'
        'report.bayesfactor_models.R' 'report.lme4.R'
        'report.stanreg.R' 'report.brmsfit.R' 'report.character.R'
        'report.compare.loo.R' 'report.compare_performance.R'
        'report.coxph.R' 'report.data.frame.R' 'report.default.R'
        'report.estimate_contrasts.R' 'report.factor.R' 'report.glm.R'
        'report.glmmTMB.R' 'report.ivreg.R' 'report.lavaan.R'
        'report.lme.R' 'report.numeric.R' 'report.sessionInfo.R'
        'report.survreg.R' 'report.test_performance.R'
        'report.zeroinfl.R' 'report_effectsize.R' 'report_htest_chi2.R'
        'report_htest_cor.R' 'report_htest_fisher.R'
        'report_htest_friedman.R' 'report_htest_kruskal.R'
        'report_htest_ttest.R' 'report_htest_wilcox.R' 'report_info.R'
        'report_intercept.R' 'report_misc.R' 'report_model.R'
        'report_parameters.R' 'report_participants.R'
        'report_performance.R' 'report_priors.R' 'report_random.R'
        'report_s.R' 'report_sample.R' 'report_statistics.R'
        'report_table.R' 'utils_error_message.R' 'utils_grouped_df.R'
        'utils_misspelled_variables.R' 'zzz.R'
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-04 17:38:21 UTC; rt3105
Author: Dominique Makowski [aut] (ORCID:
    <https://orcid.org/0000-0001-5375-9967>),
  Daniel Lüdecke [aut] (ORCID: <https://orcid.org/0000-0002-8895-3206>),
  Indrajeet Patil [aut] (ORCID: <https://orcid.org/0000-0003-1995-6531>),
  Rémi Thériault [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4315-6788>),
  Mattan S. Ben-Shachar [aut] (ORCID:
    <https://orcid.org/0000-0002-4287-4801>),
  Brenton M. Wiernik [aut] (ORCID:
    <https://orcid.org/0000-0001-9560-6336>),
  Rudolf Siegel [ctb] (ORCID: <https://orcid.org/0000-0002-6021-804X>),
  Camden Bock [ctb] (ORCID: <https://orcid.org/0000-0002-3907-7748>)
Repository: CRAN
Date/Publication: 2026-05-04 21:10:12 UTC
Built: R 4.6.0; ; 2026-05-05 00:11:18 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as.character,report_parameters)
S3method(as.data.frame,report)
S3method(as.report_table,default)
S3method(as.report_table,report)
S3method(as.report_text,default)
S3method(as.report_text,report)
S3method(c,report_table)
S3method(display,report_table)
S3method(format,report_table)
S3method(format_model,character)
S3method(format_model,default)
S3method(print,cite_easystats)
S3method(print,report)
S3method(print,report_effectsize)
S3method(print,report_info)
S3method(print,report_intercept)
S3method(print,report_model)
S3method(print,report_parameters)
S3method(print,report_performance)
S3method(print,report_priors)
S3method(print,report_random)
S3method(print,report_sample)
S3method(print,report_statistics)
S3method(print,report_table)
S3method(print,report_text)
S3method(print_html,report_sample)
S3method(print_md,report_sample)
S3method(report,BFBayesFactor)
S3method(report,Date)
S3method(report,MixMod)
S3method(report,anova)
S3method(report,aov)
S3method(report,aovlist)
S3method(report,bayesfactor_inclusion)
S3method(report,bayesfactor_models)
S3method(report,brmsfit)
S3method(report,character)
S3method(report,compare.loo)
S3method(report,compare_performance)
S3method(report,coxph)
S3method(report,data.frame)
S3method(report,default)
S3method(report,estimate_contrasts)
S3method(report,factor)
S3method(report,glm)
S3method(report,glmmTMB)
S3method(report,grouped_df)
S3method(report,htest)
S3method(report,ivreg)
S3method(report,lavaan)
S3method(report,lm)
S3method(report,lme)
S3method(report,logical)
S3method(report,merMod)
S3method(report,numeric)
S3method(report,sessionInfo)
S3method(report,stanreg)
S3method(report,survreg)
S3method(report,test_performance)
S3method(report,zeroinfl)
S3method(report_effectsize,MixMod)
S3method(report_effectsize,anova)
S3method(report_effectsize,aov)
S3method(report_effectsize,aovlist)
S3method(report_effectsize,brmsfit)
S3method(report_effectsize,coxph)
S3method(report_effectsize,default)
S3method(report_effectsize,glm)
S3method(report_effectsize,glmmTMB)
S3method(report_effectsize,htest)
S3method(report_effectsize,ivreg)
S3method(report_effectsize,lm)
S3method(report_effectsize,lme)
S3method(report_effectsize,merMod)
S3method(report_effectsize,stanreg)
S3method(report_effectsize,survreg)
S3method(report_effectsize,zeroinfl)
S3method(report_info,MixMod)
S3method(report_info,anova)
S3method(report_info,aov)
S3method(report_info,aovlist)
S3method(report_info,brmsfit)
S3method(report_info,coxph)
S3method(report_info,default)
S3method(report_info,glm)
S3method(report_info,glmmTMB)
S3method(report_info,htest)
S3method(report_info,ivreg)
S3method(report_info,lm)
S3method(report_info,lme)
S3method(report_info,merMod)
S3method(report_info,stanreg)
S3method(report_info,survreg)
S3method(report_info,zeroinfl)
S3method(report_intercept,MixMod)
S3method(report_intercept,brmsfit)
S3method(report_intercept,coxph)
S3method(report_intercept,default)
S3method(report_intercept,glm)
S3method(report_intercept,glmmTMB)
S3method(report_intercept,ivreg)
S3method(report_intercept,lm)
S3method(report_intercept,lme)
S3method(report_intercept,merMod)
S3method(report_intercept,stanreg)
S3method(report_intercept,survreg)
S3method(report_intercept,zeroinfl)
S3method(report_model,MixMod)
S3method(report_model,anova)
S3method(report_model,aov)
S3method(report_model,aovlist)
S3method(report_model,brmsfit)
S3method(report_model,coxph)
S3method(report_model,default)
S3method(report_model,glm)
S3method(report_model,glmmTMB)
S3method(report_model,htest)
S3method(report_model,ivreg)
S3method(report_model,lm)
S3method(report_model,lme)
S3method(report_model,merMod)
S3method(report_model,stanreg)
S3method(report_model,survreg)
S3method(report_model,zeroinfl)
S3method(report_parameters,Date)
S3method(report_parameters,MixMod)
S3method(report_parameters,anova)
S3method(report_parameters,aov)
S3method(report_parameters,aovlist)
S3method(report_parameters,brmsfit)
S3method(report_parameters,character)
S3method(report_parameters,compare_performance)
S3method(report_parameters,coxph)
S3method(report_parameters,data.frame)
S3method(report_parameters,default)
S3method(report_parameters,factor)
S3method(report_parameters,glm)
S3method(report_parameters,glmmTMB)
S3method(report_parameters,grouped_df)
S3method(report_parameters,htest)
S3method(report_parameters,ivreg)
S3method(report_parameters,lm)
S3method(report_parameters,lme)
S3method(report_parameters,logical)
S3method(report_parameters,merMod)
S3method(report_parameters,numeric)
S3method(report_parameters,sessionInfo)
S3method(report_parameters,stanreg)
S3method(report_parameters,survreg)
S3method(report_parameters,test_performance)
S3method(report_parameters,zeroinfl)
S3method(report_performance,MixMod)
S3method(report_performance,brmsfit)
S3method(report_performance,coxph)
S3method(report_performance,default)
S3method(report_performance,glm)
S3method(report_performance,glmmTMB)
S3method(report_performance,ivreg)
S3method(report_performance,lavaan)
S3method(report_performance,lm)
S3method(report_performance,lme)
S3method(report_performance,merMod)
S3method(report_performance,stanreg)
S3method(report_performance,survreg)
S3method(report_performance,zeroinfl)
S3method(report_priors,brmsfit)
S3method(report_priors,default)
S3method(report_priors,stanreg)
S3method(report_random,MixMod)
S3method(report_random,brmsfit)
S3method(report_random,default)
S3method(report_random,glmmTMB)
S3method(report_random,lme)
S3method(report_random,merMod)
S3method(report_random,stanreg)
S3method(report_statistics,BFBayesFactor)
S3method(report_statistics,Date)
S3method(report_statistics,MixMod)
S3method(report_statistics,anova)
S3method(report_statistics,aov)
S3method(report_statistics,aovlist)
S3method(report_statistics,brmsfit)
S3method(report_statistics,character)
S3method(report_statistics,compare_performance)
S3method(report_statistics,coxph)
S3method(report_statistics,data.frame)
S3method(report_statistics,default)
S3method(report_statistics,factor)
S3method(report_statistics,glm)
S3method(report_statistics,glmmTMB)
S3method(report_statistics,grouped_df)
S3method(report_statistics,htest)
S3method(report_statistics,ivreg)
S3method(report_statistics,lm)
S3method(report_statistics,lme)
S3method(report_statistics,merMod)
S3method(report_statistics,numeric)
S3method(report_statistics,stanreg)
S3method(report_statistics,survreg)
S3method(report_statistics,test_performance)
S3method(report_statistics,zeroinfl)
S3method(report_table,Date)
S3method(report_table,MixMod)
S3method(report_table,anova)
S3method(report_table,aov)
S3method(report_table,aovlist)
S3method(report_table,bayesfactor_inclusion)
S3method(report_table,bayesfactor_models)
S3method(report_table,brmsfit)
S3method(report_table,character)
S3method(report_table,compare_performance)
S3method(report_table,coxph)
S3method(report_table,data.frame)
S3method(report_table,default)
S3method(report_table,estimate_contrasts)
S3method(report_table,factor)
S3method(report_table,glm)
S3method(report_table,glmmTMB)
S3method(report_table,grouped_df)
S3method(report_table,htest)
S3method(report_table,ivreg)
S3method(report_table,lavaan)
S3method(report_table,lm)
S3method(report_table,lme)
S3method(report_table,logical)
S3method(report_table,merMod)
S3method(report_table,numeric)
S3method(report_table,sessionInfo)
S3method(report_table,stanreg)
S3method(report_table,survreg)
S3method(report_table,test_performance)
S3method(report_table,zeroinfl)
S3method(report_text,Date)
S3method(report_text,MixMod)
S3method(report_text,anova)
S3method(report_text,aov)
S3method(report_text,aovlist)
S3method(report_text,bayesfactor_inclusion)
S3method(report_text,bayesfactor_models)
S3method(report_text,brmsfit)
S3method(report_text,character)
S3method(report_text,compare_performance)
S3method(report_text,coxph)
S3method(report_text,data.frame)
S3method(report_text,default)
S3method(report_text,estimate_contrasts)
S3method(report_text,factor)
S3method(report_text,glm)
S3method(report_text,glmmTMB)
S3method(report_text,grouped_df)
S3method(report_text,htest)
S3method(report_text,ivreg)
S3method(report_text,lm)
S3method(report_text,lme)
S3method(report_text,logical)
S3method(report_text,merMod)
S3method(report_text,numeric)
S3method(report_text,sessionInfo)
S3method(report_text,stanreg)
S3method(report_text,survreg)
S3method(report_text,test_performance)
S3method(report_text,zeroinfl)
S3method(summary,cite_easystats)
S3method(summary,report_effectsize)
S3method(summary,report_info)
S3method(summary,report_intercept)
S3method(summary,report_model)
S3method(summary,report_parameters)
S3method(summary,report_performance)
S3method(summary,report_priors)
S3method(summary,report_random)
S3method(summary,report_statistics)
S3method(summary,report_table)
S3method(summary,report_text)
export(as.report)
export(as.report_effectsize)
export(as.report_info)
export(as.report_intercept)
export(as.report_model)
export(as.report_parameters)
export(as.report_performance)
export(as.report_priors)
export(as.report_random)
export(as.report_statistics)
export(as.report_table)
export(as.report_text)
export(cite_citation)
export(cite_easystats)
export(cite_packages)
export(clean_citation)
export(display)
export(format_algorithm)
export(format_citation)
export(format_formula)
export(format_model)
export(is.report)
export(print_html)
export(print_md)
export(report)
export(report_date)
export(report_effectsize)
export(report_info)
export(report_intercept)
export(report_model)
export(report_packages)
export(report_parameters)
export(report_participants)
export(report_performance)
export(report_priors)
export(report_random)
export(report_s)
export(report_sample)
export(report_statistics)
export(report_story)
export(report_system)
export(report_table)
export(report_text)
importFrom(insight,display)
importFrom(insight,print_html)
importFrom(insight,print_md)

# report <img src='man/figures/logo.png' align="right" height="139" />

[![R-CMD-check](https://github.com/easystats/report/actions/workflows/R-CMD-check.yml/badge.svg)](https://github.com/easystats/report/actions)
[![CRAN](http://www.r-pkg.org/badges/version/report)](https://cran.r-project.org/package=report)
[![r-universe](https://easystats.r-universe.dev/badges/report)](https://easystats.r-universe.dev/report)
[![downloads](http://cranlogs.r-pkg.org/badges/report)](https://cran.r-project.org/package=report)
[![total](https://cranlogs.r-pkg.org/badges/grand-total/report)](https://cranlogs.r-pkg.org/)
[![stars](https://img.shields.io/github/stars/easystats/report?style=social)](https://github.com/easystats/report/stargazers)

***“From R to your manuscript”***

**report**’s primary goal is to bridge the gap between R’s output and
the formatted results contained in your manuscript. It automatically
produces reports of models and data frames according to **best
practices** guidelines (e.g., [APA](https://apastyle.apa.org/)’s style),
ensuring **standardization** and **quality** in results reporting.

``` r
library(report)

model <- lm(Sepal.Length ~ Species, data = iris)
report(model)
# We fitted a linear model (estimated using OLS) to predict Sepal.Length with
# Species (formula: Sepal.Length ~ Species). The model explains a statistically
# significant and substantial proportion of variance (R2 = 0.62, F(2, 147) =
# 119.26, p < .001, adj. R2 = 0.61). The model's intercept, corresponding to
# Species = setosa, is at 5.01 (95% CI [4.86, 5.15], t(147) = 68.76, p < .001).
# Within this model:
# 
#   - The effect of Species [versicolor] is statistically significant and positive
# (beta = 0.93, 95% CI [0.73, 1.13], t(147) = 9.03, p < .001; Std. beta = 1.12,
# 95% CI [0.88, 1.37])
#   - The effect of Species [virginica] is statistically significant and positive
# (beta = 1.58, 95% CI [1.38, 1.79], t(147) = 15.37, p < .001; Std. beta = 1.91,
# 95% CI [1.66, 2.16])
# 
# Standardized parameters were obtained by fitting the model on a standardized
# version of the dataset. 95% Confidence Intervals (CIs) and p-values were
# computed using a Wald t-distribution approximation.
```

## Installation

The package is available on `CRAN` and can be downloaded by running:

``` r
install.packages("report")
```

If you would instead like to experiment with the development version,
you can download it from `GitHub`:

``` r
install.packages("remotes")
remotes::install_github("easystats/report") # You only need to do that once
```

Load the package every time you start R

``` r
library("report")
```

> **Tip**
>
> **Instead of `library(report)`, use `library(easystats)`.** **This
> will make all features of the easystats-ecosystem available.**
>
> **To stay updated, use `easystats::install_latest()`.**

## Documentation

The package documentation can be found
[**here**](https://easystats.github.io/report/).

## Report all the things

<a href=https://easystats.github.io/report/><img src="man/figures/allthethings.jpg" height="60" alt="All the things meme by Allie Brosh" ></a>

### General Workflow

The `report` package works in a two step fashion. First, you create a
`report` object with the `report()` function. Then, this report object
can be displayed either textually (the default output) or as a table,
using `as.data.frame()`. Moreover, you can also access a more digest and
compact version of the report using `summary()` on the report object.

[![workflow](man/figures/workflow.png)](https://easystats.github.io/report/)

The `report()` function works on a variety of models, as well as other
objects such as dataframes:

``` r
report(iris)
```

    # The data contains 150 observations of the following 5 variables:
    # 
    #   - Sepal.Length: n = 150, Mean = 5.84, SD = 0.83, Median = 5.80, MAD = 1.04,
    # range: [4.30, 7.90], Skewness = 0.31, Kurtosis = -0.55, 0% missing
    #   - Sepal.Width: n = 150, Mean = 3.06, SD = 0.44, Median = 3.00, MAD = 0.44,
    # range: [2, 4.40], Skewness = 0.32, Kurtosis = 0.23, 0% missing
    #   - Petal.Length: n = 150, Mean = 3.76, SD = 1.77, Median = 4.35, MAD = 1.85,
    # range: [1, 6.90], Skewness = -0.27, Kurtosis = -1.40, 0% missing
    #   - Petal.Width: n = 150, Mean = 1.20, SD = 0.76, Median = 1.30, MAD = 1.04,
    # range: [0.10, 2.50], Skewness = -0.10, Kurtosis = -1.34, 0% missing
    #   - Species: 3 levels, namely setosa (n = 50, 33.33%), versicolor (n = 50,
    # 33.33%) and virginica (n = 50, 33.33%)

These reports nicely work within the
[*tidyverse*](https://github.com/tidyverse) workflow:

``` r
iris %>%
  select(-starts_with("Sepal")) %>%
  group_by(Species) %>%
  report() %>%
  summary()
```

    # The data contains 150 observations, grouped by Species, of the following 3
    # variables:
    # 
    # - setosa (n = 50):
    #   - Petal.Length: Mean = 1.46, SD = 0.17, range: [1, 1.90]
    #   - Petal.Width: Mean = 0.25, SD = 0.11, range: [0.10, 0.60]
    # 
    # - versicolor (n = 50):
    #   - Petal.Length: Mean = 4.26, SD = 0.47, range: [3, 5.10]
    #   - Petal.Width: Mean = 1.33, SD = 0.20, range: [1, 1.80]
    # 
    # - virginica (n = 50):
    #   - Petal.Length: Mean = 5.55, SD = 0.55, range: [4.50, 6.90]
    #   - Petal.Width: Mean = 2.03, SD = 0.27, range: [1.40, 2.50]

### *t*-tests and correlations

Reports can be used to automatically format tests like *t*-tests or
correlations.

``` r
report(t.test(mtcars$mpg ~ mtcars$am))
```

    # Effect sizes were labelled following Cohen's (1988) recommendations.
    # 
    # The Welch Two Sample t-test testing the difference of mtcars$mpg by mtcars$am
    # (mean in group 0 = 17.15, mean in group 1 = 24.39) suggests that the effect is
    # negative, statistically significant, and large (difference = -7.24, 95% CI
    # [-11.28, -3.21], t(18.33) = -3.77, p = 0.001; Cohen's d = -1.41, 95% CI [-2.26,
    # -0.53])

As mentioned, you can also create tables with the `as.data.frame()`
functions, like for example with this correlation test:

``` r
cor.test(iris$Sepal.Length, iris$Sepal.Width) %>%
  report() %>%
  as.data.frame()
# Pearson's product-moment correlation
# 
# Parameter1        |       Parameter2 |     r |        95% CI | t(148) |     p
# -----------------------------------------------------------------------------
# iris$Sepal.Length | iris$Sepal.Width | -0.12 | [-0.27, 0.04] |  -1.44 | 0.152
# 
# Alternative hypothesis: two.sided
```

### ANOVAs

This works great with ANOVAs, as it includes **effect sizes** and their
interpretation.

``` r
aov(Sepal.Length ~ Species, data = iris) %>%
  report()
```

    # The ANOVA (formula: Sepal.Length ~ Species) suggests that:
    # 
    #   - The main effect of Species is statistically significant and large (F(2, 147)
    # = 119.26, p < .001; Eta2 = 0.62, 95% CI [0.54, 1.00])
    # 
    # Effect sizes were labelled following Field's (2013) recommendations.

### Generalized Linear Models (GLMs)

Reports are also compatible with GLMs, such as this **logistic
regression**:

``` r
model <- glm(vs ~ mpg * drat, data = mtcars, family = "binomial")

report(model)
```

    # We fitted a logistic model (estimated using ML) to predict vs with mpg and drat
    # (formula: vs ~ mpg * drat). The model's explanatory power is substantial
    # (Tjur's R2 = 0.51). The model's intercept, corresponding to mpg = 0 and drat =
    # 0, is at -33.43 (95% CI [-77.90, 3.25], p = 0.083). Within this model:
    # 
    #   - The effect of mpg is statistically non-significant and positive (beta = 1.79,
    # 95% CI [-0.10, 4.05], p = 0.066; Std. beta = 3.63, 95% CI [1.36, 7.50])
    #   - The effect of drat is statistically non-significant and positive (beta =
    # 5.96, 95% CI [-3.75, 16.26], p = 0.205; Std. beta = -0.36, 95% CI [-1.96,
    # 0.98])
    #   - The effect of mpg × drat is statistically non-significant and negative (beta
    # = -0.33, 95% CI [-0.83, 0.15], p = 0.141; Std. beta = -1.07, 95% CI [-2.66,
    # 0.48])
    # 
    # Standardized parameters were obtained by fitting the model on a standardized
    # version of the dataset. 95% Confidence Intervals (CIs) and p-values were
    # computed using a Wald z-distribution approximation.

### Mixed Models

Mixed models, whose popularity and usage is exploding, can also be
reported:

``` r
library(lme4)

model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)

report(model)
```

    # We fitted a linear mixed model (estimated using REML and nloptwrap optimizer)
    # to predict Sepal.Length with Petal.Length (formula: Sepal.Length ~
    # Petal.Length). The model included Species as random effect (formula: ~1 |
    # Species). The model's total explanatory power is substantial (conditional R2 =
    # 0.97) and the part related to the fixed effects alone (marginal R2) is of 0.66.
    # The model's intercept, corresponding to Petal.Length = 0, is at 2.50 (95% CI
    # [1.19, 3.82], t(146) = 3.75, p < .001). Within this model:
    # 
    #   - The effect of Petal Length is statistically significant and positive (beta =
    # 0.89, 95% CI [0.76, 1.01], t(146) = 13.93, p < .001; Std. beta = 1.89, 95% CI
    # [1.63, 2.16])
    # 
    # Standardized parameters were obtained by fitting the model on a standardized
    # version of the dataset. 95% Confidence Intervals (CIs) and p-values were
    # computed using a Wald t-distribution approximation.

### Bayesian Models

Bayesian models can also be reported using the new
[**SEXIT**](https://easystats.github.io/bayestestR/reference/sexit.html)
framework, which combines clarity, precision and usefulness.

``` r
library(rstanarm)

model <- stan_glm(mpg ~ qsec + wt, data = mtcars)

report(model)
```

    # We fitted a Bayesian linear model (estimated using MCMC sampling with 4 chains
    # of 1000 iterations and a warmup of 500) to predict mpg with qsec and wt
    # (formula: mpg ~ qsec + wt). Priors over parameters were all set as normal (mean
    # = 0.00, SD = 8.43; mean = 0.00, SD = 15.40) distributions. The model's
    # explanatory power is substantial (R2 = 0.81, 95% CI [0.71, 0.90], adj. R2 =
    # 0.79). The model's intercept, corresponding to qsec = 0 and wt = 0, is at 19.67
    # (95% CI [8.34, 30.67]). Within this model:
    # 
    #   - The effect of qsec (Median = 0.93, 95% CI [0.39, 1.51]) has a 99.90%
    # probability of being positive (> 0), 98.65% of being significant (> 0.30), and
    # 0.30% of being large (> 1.81). The estimation successfully converged (Rhat =
    # 1.000) and the indices are reliable (ESS = 1762)
    #   - The effect of wt (Median = -5.05, 95% CI [-6.01, -4.05]) has a 100.00%
    # probability of being negative (< 0), 100.00% of being significant (< -0.30),
    # and 100.00% of being large (< -1.81). The estimation successfully converged
    # (Rhat = 1.000) and the indices are reliable (ESS = 2213)
    # 
    # Following the Sequential Effect eXistence and sIgnificance Testing (SEXIT)
    # framework, we report the median of the posterior distribution and its 95% CI
    # (Highest Density Interval), along the probability of direction (pd), the
    # probability of significance and the probability of being large. The thresholds
    # beyond which the effect is considered as significant (i.e., non-negligible) and
    # large are |0.30| and |1.81| (corresponding respectively to 0.05 and 0.30 of the
    # outcome's SD). Convergence and stability of the Bayesian sampling has been
    # assessed using R-hat, which should be below 1.01 (Vehtari et al., 2019), and
    # Effective Sample Size (ESS), which should be greater than 1000 (Burkner, 2017).

## Other types of reports

### Specific parts

One can, for complex reports, directly access the pieces of the reports:

``` r
model <- lm(Sepal.Length ~ Species, data = iris)

report_model(model)
# linear model (estimated using OLS) to predict Sepal.Length with Species (formula: Sepal.Length ~ Species)

report_performance(model)
# The model explains a statistically significant and substantial proportion of
# variance (R2 = 0.62, F(2, 147) = 119.26, p < .001, adj. R2 = 0.61)

report_statistics(model)
# beta = 5.01, 95% CI [4.86, 5.15], t(147) = 68.76, p < .001; Std. beta = -1.01, 95% CI [-1.18, -0.84]
# beta = 0.93, 95% CI [0.73, 1.13], t(147) = 9.03, p < .001; Std. beta = 1.12, 95% CI [0.88, 1.37]
# beta = 1.58, 95% CI [1.38, 1.79], t(147) = 15.37, p < .001; Std. beta = 1.91, 95% CI [1.66, 2.16]
```

### Report participants’ details

This can be useful to complete the **Participants** paragraph of your
manuscript.

``` r
data <- data.frame(
  "Age" = c(22, 23, 54, 21),
  "Sex" = c("F", "F", "M", "M")
)

paste(
  report_participants(data, spell_n = TRUE),
  "were recruited in the study by means of torture and coercion."
)
```

    # Four participants (Mean age = 30.0, SD = 16.0, range: [21, 54]; Sex:
    #   50.0% females, 50.0% males, 0.0% other) were recruited in the study by
    #   means of torture and coercion.

### Report sample

Report can also help you create a sample description table (also
referred to as **Table 1**).

``` r
report_sample(iris, by = "Species")
```

| Variable               | setosa (n=50) | versicolor (n=50) | virginica (n=50) | Total (n=150) |
|:-----------------------|:--------------|:------------------|:-----------------|:--------------|
| Mean Sepal.Length (SD) | 5.01 (0.35)   | 5.94 (0.52)       | 6.59 (0.64)      | 5.84 (0.83)   |
| Mean Sepal.Width (SD)  | 3.43 (0.38)   | 2.77 (0.31)       | 2.97 (0.32)      | 3.06 (0.44)   |
| Mean Petal.Length (SD) | 1.46 (0.17)   | 4.26 (0.47)       | 5.55 (0.55)      | 3.76 (1.77)   |
| Mean Petal.Width (SD)  | 0.25 (0.11)   | 1.33 (0.20)       | 2.03 (0.27)      | 1.20 (0.76)   |

### Report system and packages

Finally, **report** includes some functions to help you write the data
analysis paragraph about the tools used.

``` r
report(sessionInfo())
```

    # Analyses were conducted using the R Statistical language (version 4.4.1; R Core
    # Team, 2024) on Windows 11 x64 (build 22631), using the packages lme4 (version
    # 1.1.35.5; Bates D et al., 2015), Matrix (version 1.7.0; Bates D et al., 2024),
    # Rcpp (version 1.0.13; Eddelbuettel D et al., 2024), rstanarm (version 2.32.1;
    # Goodrich B et al., 2024), report (version 0.5.9; Makowski D et al., 2023) and
    # dplyr (version 1.1.4; Wickham H et al., 2023).
    # 
    # References
    # ----------
    #   - Bates D, Mächler M, Bolker B, Walker S (2015). "Fitting Linear Mixed-Effects
    # Models Using lme4." _Journal of Statistical Software_, *67*(1), 1-48.
    # doi:10.18637/jss.v067.i01 <https://doi.org/10.18637/jss.v067.i01>.
    #   - Bates D, Maechler M, Jagan M (2024). _Matrix: Sparse and Dense Matrix Classes
    # and Methods_. R package version 1.7-0,
    # <https://CRAN.R-project.org/package=Matrix>.
    #   - Eddelbuettel D, Francois R, Allaire J, Ushey K, Kou Q, Russell N, Ucar I,
    # Bates D, Chambers J (2024). _Rcpp: Seamless R and C++ Integration_. R package
    # version 1.0.13, <https://CRAN.R-project.org/package=Rcpp>. Eddelbuettel D,
    # François R (2011). "Rcpp: Seamless R and C++ Integration." _Journal of
    # Statistical Software_, *40*(8), 1-18. doi:10.18637/jss.v040.i08
    # <https://doi.org/10.18637/jss.v040.i08>. Eddelbuettel D (2013). _Seamless R and
    # C++ Integration with Rcpp_. Springer, New York. doi:10.1007/978-1-4614-6868-4
    # <https://doi.org/10.1007/978-1-4614-6868-4>, ISBN 978-1-4614-6867-7.
    # Eddelbuettel D, Balamuta J (2018). "Extending R with C++: A Brief Introduction
    # to Rcpp." _The American Statistician_, *72*(1), 28-36.
    # doi:10.1080/00031305.2017.1375990
    # <https://doi.org/10.1080/00031305.2017.1375990>.
    #   - Goodrich B, Gabry J, Ali I, Brilleman S (2024). "rstanarm: Bayesian applied
    # regression modeling via Stan." R package version 2.32.1,
    # <https://mc-stan.org/rstanarm/>. Brilleman S, Crowther M, Moreno-Betancur M,
    # Buros Novik J, Wolfe R (2018). "Joint longitudinal and time-to-event models via
    # Stan." StanCon 2018. 10-12 Jan 2018. Pacific Grove, CA, USA.,
    # <https://github.com/stan-dev/stancon_talks/>.
    #   - Makowski D, Lüdecke D, Patil I, Thériault R, Ben-Shachar M, Wiernik B (2023).
    # "Automated Results Reporting as a Practical Tool to Improve Reproducibility and
    # Methodological Best Practices Adoption." _CRAN_.
    # <https://easystats.github.io/report/>.
    #   - R Core Team (2024). _R: A Language and Environment for Statistical
    # Computing_. R Foundation for Statistical Computing, Vienna, Austria.
    # <https://www.R-project.org/>.
    #   - Wickham H, François R, Henry L, Müller K, Vaughan D (2023). _dplyr: A Grammar
    # of Data Manipulation_. R package version 1.1.4,
    # <https://CRAN.R-project.org/package=dplyr>.

## Credits

If you like it, you can put a *star* on this repo, and cite the package
as follows:

``` r
citation("report")
To cite in publications use:

  Makowski, D., Lüdecke, D., Patil, I., Thériault, R., Ben-Shachar,
  M.S., & Wiernik, B.M. (2023). Automated Results Reporting as a
  Practical Tool to Improve Reproducibility and Methodological Best
  Practices Adoption. CRAN. Available from
  https://easystats.github.io/report/ doi: .

A BibTeX entry for LaTeX users is

  @Article{,
    title = {Automated Results Reporting as a Practical Tool to Improve Reproducibility and Methodological Best Practices Adoption},
    author = {Dominique Makowski and Daniel Lüdecke and Indrajeet Patil and Rémi Thériault and Mattan S. Ben-Shachar and Brenton M. Wiernik},
    year = {2023},
    journal = {CRAN},
    url = {https://easystats.github.io/report/},
  }
```

## Contribute

***report* is a young package in need of affection**. You can easily be
a part of the
[developing](https://github.com/easystats/report/blob/master/.github/CONTRIBUTING.md)
community of this open-source software and improve science! Don’t be
shy, try to code and submit a pull request (See the [contributing
guide](https://github.com/easystats/report/blob/master/.github/CONTRIBUTING.md)).
Even if it’s not perfect, we will help you make it great!

## Code of Conduct

Please note that the report project is released with a [Contributor Code
of Conduct](https://easystats.github.io/report/CODE_OF_CONDUCT.html). By
contributing to this project, you agree to abide by its terms.

## ----echo = FALSE-----------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  comment = "#"
)

options(
  knitr.kable.NA = "",
  width = 60
)

if (!requireNamespace("dplyr", quietly = TRUE)) {
  knitr::opts_chunk$set(eval = FALSE)
}

## ----results='asis'---------------------------------------
library(report)

cite_packages()

## ----results='asis'---------------------------------------
cite_easystats()

#' @rdname report.data.frame
#' @export
report.character <- function(
  x,
  n_entries = 3,
  levels_percentage = "auto",
  missing_percentage = "auto",
  ...
) {
  report_table <- report_table(
    x,
    n_entries = n_entries,
    levels_percentage = levels_percentage,
    missing_percentage = missing_percentage,
    ...
  )

  report_text <- report_text(
    x,
    n_entries = n_entries,
    levels_percentage = levels_percentage,
    missing_percentage = missing_percentage,
    ...
  )

  as.report(report_text, table = report_table, ...)
}


# report_table ------------------------------------------------------------

#' @export
report_table.character <- function(
  x,
  n_entries = 3,
  levels_percentage = "auto",
  missing_percentage = "auto",
  ...
) {
  levels_percentage <- .report_dataframe_percentage(x, levels_percentage)
  missing_percentage <- .report_dataframe_percentage(x, missing_percentage)

  n_char <- as.data.frame(table(x))
  n_char <- n_char[order(n_char$Freq, decreasing = TRUE), ]
  names(n_char) <- c("Entry", "n_Entry")
  n_char$percentage_Entry <- n_char$n_Entry / length(x)

  if (n_entries == "all" || n_entries > nrow(n_char)) {
    n_entries <- nrow(n_char)
  }

  table_full <- data.frame(
    n_Entries = nrow(n_char),
    n_Obs = length(x),
    n_Missing = sum(is.na(x))
  )
  table_full$percentage_Missing <- table_full$n_Missing / table_full$n_Obs * 100
  summary_table <- table_full

  if (isTRUE(missing_percentage)) {
    summary_table <- datawizard::data_remove(summary_table, "n_Missing")
  } else {
    summary_table <- datawizard::data_remove(
      summary_table,
      "percentage_Missing"
    )
  }

  as.report_table(table_full, summary = summary_table, entries = n_char)
}


# report_parameters -------------------------------------------------------

#' @export
report_parameters.character <- function(
  x,
  table = NULL,
  n_entries = 3,
  levels_percentage = "auto",
  missing_percentage = "auto",
  ...
) {
  levels_percentage <- .report_dataframe_percentage(x, levels_percentage)

  # Get table
  if (is.null(table)) {
    table <- report_table(
      x,
      n_entries = n_entries,
      levels_percentage = levels_percentage,
      missing_percentage = missing_percentage,
      ...
    )
  }
  entries <- attributes(table)$entries

  if (levels_percentage) {
    param_text <- paste0(
      entries$Entry,
      " (",
      insight::format_value(entries$percentage_Entry, as_percent = TRUE),
      ")"
    )
  } else {
    param_text <- paste0(entries$Entry, " (n = ", entries$n_Entry, ")")
  }

  n_entries_actual <- min(n_entries, length(param_text))
  as.report_parameters(
    param_text,
    summary = param_text[1:n_entries_actual],
    ...
  )
}


# report_text -------------------------------------------------------------

#' @export
report_text.character <- function(
  x,
  table = NULL,
  n_entries = 3,
  levels_percentage = "auto",
  missing_percentage = "auto",
  ...
) {
  if (!is.null(list(...)$varname)) {
    name <- list(...)$varname
  } else if (is.null(names(x))) {
    name <- deparse(substitute(x))
  } else {
    name <- "Character variable"
  }

  if (is.null(table)) {
    table <- report_table(
      x,
      n_entries = n_entries,
      levels_percentage = levels_percentage,
      missing_percentage = missing_percentage,
      ...
    )
  }
  entries <- attributes(table)$entries
  params <- report_parameters(
    x,
    table = table,
    n_entries = n_entries,
    levels_percentage = levels_percentage,
    missing_percentage = missing_percentage,
    ...
  )

  report_text <- paste(summary(params), collapse = "; ")
  if (nrow(entries) > 1) {
    report_text <- paste0(
      name,
      ": ",
      nrow(entries),
      " entries, such as ",
      report_text
    )
  } else {
    report_text <- paste0(
      name,
      ": ",
      nrow(entries),
      " entry, such as ",
      report_text
    )
  }

  if (nrow(entries) - n_entries == 1) {
    report_text <- paste0(
      report_text,
      " and ",
      nrow(entries) - n_entries,
      " other"
    )
  }
  if (nrow(entries) - n_entries > 1) {
    report_text <- paste0(
      report_text,
      " and ",
      nrow(entries) - n_entries,
      " others"
    )
  }

  text_full <- report_text

  # Missing
  text_n_missing <- paste0(table$n_Missing[1], " missing")
  text_percentage_missing <- paste0(
    insight::format_value(table$percentage_Missing[1]),
    "% missing"
  )
  if (isTRUE(missing_percentage)) {
    text_full <- paste0(text_full, "(", text_percentage_missing, ")")
    if (table$n_Missing[1] > 0) {
      report_text <- paste0(report_text, " (", text_percentage_missing, ")")
    }
  } else {
    text_full <- paste0(text_full, " (", text_n_missing, ")")
    table <- datawizard::data_remove(table, "percentage_Missing")
    if (table$n_Missing[1] > 0) {
      report_text <- paste0(report_text, " (", text_n_missing, ")")
    }
  }

  as.report_text(text_full, summary = report_text)
}


# report_statistics -------------------------------------------------------

#' @export
report_statistics.character <- function(
  x,
  table = NULL,
  n_entries = 3,
  levels_percentage = "auto",
  missing_percentage = "auto",
  ...
) {
  levels_percentage <- .report_dataframe_percentage(x, levels_percentage)

  if (is.null(table)) {
    table <- report_table(
      x,
      n_entries = n_entries,
      levels_percentage = levels_percentage,
      missing_percentage = missing_percentage,
      ...
    )
  }
  entries <- attributes(table)$entries

  if (levels_percentage) {
    stat_text <- paste0(
      entries$Entry,
      ", ",
      insight::format_value(entries$percentage_Entry, as_percent = TRUE),
      "%"
    )
  } else {
    stat_text <- paste0(entries$Entry, ", n = ", entries$n_Entry)
  }

  n_entries_actual <- min(n_entries, length(stat_text))
  as.report_statistics(
    paste(stat_text, collapse = "; "),
    summary = paste(stat_text[1:n_entries_actual], collapse = "; ")
  )
}

test_that("report.character() works correctly", {
  # Test basic character vector
  char_data <- c("red", "blue", "red", "green", "blue", "red")

  result <- report(char_data)

  expect_s3_class(result, "report")
  expect_type(result, "character") # report objects are character type

  # Check that it contains the expected components
  expect_match(as.character(result), "entries")
  expect_match(as.character(result), "red")

  # Check table
  table_result <- as.data.frame(result)
  expect_s3_class(table_result, "data.frame")
  expect_true("n_Entries" %in% names(table_result))
  expect_true("n_Obs" %in% names(table_result))
})

test_that("report.character() handles missing values", {
  char_data <- c("red", "blue", NA, "green", "blue", "red", NA)

  result <- report(char_data)

  expect_match(as.character(result), "missing")

  # Check table contains missing info
  table_result <- as.data.frame(result)
  expect_gt(table_result$n_Missing[1], 0)
})

test_that("report.character() respects parameters", {
  char_data <- c(
    "red",
    "blue",
    "red",
    "green",
    "blue",
    "red",
    "yellow",
    "orange"
  )

  # Test with different n_entries
  result1 <- report(char_data, n_entries = 2)
  expect_match(as.character(result1), "others")

  # Test with levels_percentage
  result2 <- report(char_data, levels_percentage = TRUE)
  expect_match(as.character(result2), "%")

  result3 <- report(char_data, levels_percentage = FALSE)
  expect_match(as.character(result3), "n =")
})

test_that("report.character() handles single unique value", {
  # Test with a single unique value
  single_char <- "big"
  result <- report(single_char)

  expect_s3_class(result, "report")
  expect_match(as.character(result), "1 entry")
  expect_match(as.character(result), "big")
  expect_no_match(as.character(result), "NA")

  # Test with single unique value repeated multiple times
  single_repeated <- c("big", "big", "big", "big")
  result2 <- report(single_repeated)

  expect_s3_class(result2, "report")
  expect_match(as.character(result2), "1 entry")
  expect_match(as.character(result2), "big")
  expect_match(as.character(result2), "n = 4")

  # Test in data frame context (original issue)
  df <- data.frame(id = 1:6, size = single_char)
  result3 <- report(df)

  expect_s3_class(result3, "report")
  expect_match(as.character(result3), "size")
  expect_match(as.character(result3), "big")
  expect_match(as.character(result3), "1 entry")
  expect_match(as.character(result3), "n = 6")
  expect_no_match(as.character(result3), "NA")
})

test_that("report.factor() works correctly", {
  # Test basic factor
  factor_data <- factor(c("low", "medium", "high", "low", "medium", "high"))

  result <- report(factor_data)

  expect_s3_class(result, "report")
  expect_match(as.character(result), "levels")
  expect_match(as.character(result), "low")
  expect_match(as.character(result), "medium")
  expect_match(as.character(result), "high")

  # Check table
  table_result <- as.data.frame(result)
  expect_s3_class(table_result, "data.frame")
  expect_true("Level" %in% names(table_result))
  expect_true("n_Obs" %in% names(table_result))
})

test_that("report.factor() handles missing values", {
  factor_data <- factor(c("low", NA, "medium", "high", "low", NA, "medium"))

  result <- report(factor_data)

  expect_match(as.character(result), "missing")

  # Check table includes missing level
  table_result <- as.data.frame(result)
  expect_true("missing" %in% table_result$Level)
})

test_that("report.factor() works with logical vectors", {
  logical_data <- c(TRUE, FALSE, TRUE, TRUE, FALSE)

  result <- report(logical_data)

  expect_s3_class(result, "report")
  expect_match(as.character(result), "levels")
  expect_match(as.character(result), "TRUE|FALSE")
})

test_that("report.factor() respects levels_percentage parameter", {
  factor_data <- factor(c("A", "B", "A", "B", "A"))

  result_perc <- report(factor_data, levels_percentage = TRUE)
  expect_match(as.character(result_perc), "%")

  result_count <- report(factor_data, levels_percentage = FALSE)
  expect_match(as.character(result_count), "n =")
})

test_that("report.default() methods throw appropriate errors", {
  # Test with unsupported object
  unsupported_obj <- structure(list(x = 1), class = "unsupported_class")

  expect_error(
    report(unsupported_obj),
    "objects of class.*unsupported_class.*not supported"
  )
  expect_error(
    report_table(unsupported_obj),
    "objects of class.*unsupported_class.*not supported"
  )
  expect_error(
    report_parameters(unsupported_obj),
    "objects of class.*unsupported_class.*not supported"
  )
  expect_error(
    report_statistics(unsupported_obj),
    "objects of class.*unsupported_class.*not supported"
  )
  expect_error(
    report_effectsize(unsupported_obj),
    "objects of class.*unsupported_class.*not supported"
  )
  expect_error(
    report_model(unsupported_obj),
    "objects of class.*unsupported_class.*not supported"
  )
  expect_error(
    report_random(unsupported_obj),
    "objects of class.*unsupported_class.*not supported"
  )
  expect_error(
    report_priors(unsupported_obj),
    "objects of class.*unsupported_class.*not supported"
  )
  expect_error(
    report_performance(unsupported_obj),
    "objects of class.*unsupported_class.*not supported"
  )
  expect_error(
    report_info(unsupported_obj),
    "objects of class.*unsupported_class.*not supported"
  )
  expect_error(
    report_text(unsupported_obj),
    "objects of class.*unsupported_class.*not supported"
  )
  expect_error(
    report_intercept(unsupported_obj),
    "objects of class.*unsupported_class.*not supported"
  )
})

test_that("report component functions work for character vectors", {
  char_data <- c("red", "blue", "red", "green", "blue", "red")

  # Test report_table
  table_result <- report_table(char_data)
  expect_s3_class(table_result, "report_table")
  expect_true("n_Entries" %in% names(table_result))

  # Test report_parameters
  params_result <- report_parameters(char_data)
  expect_s3_class(params_result, "report_parameters")

  # Test report_text
  text_result <- report_text(char_data)
  expect_s3_class(text_result, "report_text")
  expect_match(as.character(text_result), "char_data") # Variable name, not "Character variable"

  # Test report_statistics
  stats_result <- report_statistics(char_data)
  expect_s3_class(stats_result, "report_statistics")
})

test_that("report component functions work for factor vectors", {
  factor_data <- factor(c("low", "medium", "high", "low", "medium"))

  # Test report_table
  table_result <- report_table(factor_data)
  expect_s3_class(table_result, "report_table")
  expect_true("Level" %in% names(table_result))

  # Test report_parameters
  params_result <- report_parameters(factor_data)
  expect_s3_class(params_result, "report_parameters")

  # Test report_text
  text_result <- report_text(factor_data)
  expect_s3_class(text_result, "report_text")
  expect_match(as.character(text_result), "factor_data") # Variable name, not "Factor"

  # Test report_statistics
  stats_result <- report_statistics(factor_data)
  expect_s3_class(stats_result, "report_statistics")
})

#' Report a descriptive table
#'
#' Creates tables to describe different objects (see list of supported objects
#' in [report()]).
#'
#' @inheritParams report
#'
#' @return An object of class [report_table()].
#'
#' @examples
#' \donttest{
#' # Miscellaneous
#' r <- report_table(sessionInfo())
#' r
#' summary(r)
#'
#' # Data
#' report_table(iris$Sepal.Length)
#' report_table(as.character(round(iris$Sepal.Length, 1)))
#' report_table(iris$Species)
#' report_table(iris)
#'
#' # h-tests
#' report_table(t.test(mtcars$mpg ~ mtcars$am))
#'
#' # ANOVAs
#' report_table(aov(Sepal.Length ~ Species, data = iris))
#'
#' # GLMs
#' report_table(lm(Sepal.Length ~ Petal.Length * Species, data = iris))
#' report_table(glm(vs ~ disp, data = mtcars, family = "binomial"))
#' }
#'
#' @examplesIf requireNamespace("lme4", quietly = TRUE)
#' \donttest{
#' # Mixed models
#' library(lme4)
#' model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
#' report_table(model)
#' }
#'
#' @examplesIf requireNamespace("rstanarm", quietly = TRUE)
#' \donttest{
#' # Bayesian models
#' library(rstanarm)
#' model <- suppressWarnings(stan_glm(Sepal.Length ~ Species, data = iris, refresh = 0, iter = 600))
#' report_table(model, effectsize_method = "basic")
#' }
#'
#' @examplesIf requireNamespace("lavaan", quietly = TRUE)
#' \donttest{
#' # Structural Equation Models (SEM)
#' library(lavaan)
#' structure <- "ind60 =~ x1 + x2 + x3
#'               dem60 =~ y1 + y2 + y3
#'               dem60 ~ ind60"
#' model <- lavaan::sem(structure, data = PoliticalDemocracy)
#' suppressWarnings(report_table(model))
#' }
#' @export
report_table <- function(x, ...) {
  UseMethod("report_table")
}


# METHODS -----------------------------------------------------------------

#' @rdname as.report
#' @export
as.report_table <- function(x, ...) {
  UseMethod("as.report_table")
}

#' @export
as.report_table.default <- function(x, summary = NULL, as_is = FALSE, ...) {
  if (as_is) {
    class(x) <- unique(c(
      class(x)[1],
      "report_table",
      utils::tail(class(x), -1)
    ))
  } else {
    class(x) <- unique(c("report_table", class(x)))
  }

  attributes(x) <- c(attributes(x), list(...))

  if (!is.null(summary)) {
    if (as_is) {
      class(summary) <- unique(c(
        class(summary)[1],
        "report_table",
        utils::tail(class(summary), -1)
      ))
    } else {
      class(summary) <- unique(c("report_table", class(summary)))
    }
    attr(x, "summary") <- summary
  }

  x
}

#' @export
as.report_table.report <- function(x, summary = NULL, ...) {
  if (is.null(summary) || isFALSE(summary)) {
    attributes(x)$table
  } else if (isTRUE(summary)) {
    summary(attributes(x)$table)
  }
}


#' @export
summary.report_table <- function(object, ...) {
  if (is.null(attributes(object)$summary)) {
    object
  } else {
    attributes(object)$summary
  }
}


#' @export
format.report_table <- function(x, ...) {
  # remove unwanted columns
  x$Method <- NULL
  x$Alternative <- NULL

  insight::format_table(x, ...)
}


#' @export
print.report_table <- function(x, ...) {
  # try to guess appropriate caption and footer
  caption <- .report_table_caption(x)
  footer <- .report_table_footer(x)

  cat(insight::export_table(
    format(x, ...),
    caption = caption,
    footer = footer,
    ...
  ))
}


#' @export
c.report_table <- function(...) {
  x <- list(...)

  out <- x[[1]]
  for (i in 2:length(x)) {
    out <- datawizard::data_join(out, x[[i]], join = "bind")
  }
  out
}


#' @export
display.report_table <- function(object, ...) {
  # fix caption
  if (is.null(list(...)$caption)) {
    attr(object, "no_caption") <- TRUE
  }
  class(object) <- c("report_table", "parameters_model", "data.frame")
  NextMethod()
}


# helper to create table captions and footer -----------------------

.report_table_caption <- function(x) {
  caption <- NULL

  # footer for htest objects
  if (!is.null(x$Method)) {
    caption <- x$Method
  }

  caption
}


.report_table_footer <- function(x) {
  footer <- NULL

  # footer for htest objects
  if (is.null(x$Alternative)) {
    footer <- NULL
  } else {
    footer <- "Alternative hypothesis: "
    if (is.null(x$null.value)) {
      # handle missing null.value
      alt.char <- switch(
        x$Alternative,
        two.sided = "two.sided",
        less = "less",
        greater = "greater"
      )
      footer <- paste0(footer, alt.char)
    } else if (length(x$null.value) == 1L) {
      alt.char <- switch(
        x$Alternative,
        two.sided = "not equal to",
        less = "less than",
        greater = "greater than"
      )
      footer <- paste0(
        footer,
        "true ",
        names(x$null.value),
        " is ",
        alt.char,
        " ",
        x$null.value
      )
    } else {
      footer <- paste0(footer, x$Alternative)
    }

    footer <- c(paste0("\n", footer), "blue")
  }

  footer
}

test_that("report.htest-correlation", {
  set.seed(123)
  r <- report(cor.test(iris$Sepal.Width, iris$Sepal.Length))
  expect_equal(as.report_table(r)$r, -0.117, tolerance = 0.01)

  set.seed(123)
  r <- cor.test(
    iris$Sepal.Width,
    iris$Sepal.Length,
    method = "spearman",
    exact = FALSE
  )
  r <- report(r)
  expect_equal(as.report_table(r)$rho, -0.166, tolerance = 0.01)

  set.seed(123)
  r <- report(cor.test(iris$Sepal.Width, iris$Sepal.Length, method = "kendall"))
  expect_equal(as.report_table(r)$tau, -0.077, tolerance = 0.01)

  # snapshot tests with a different dataset
  set.seed(123)
  expect_snapshot(variant = "windows", report(cor.test(mtcars$wt, mtcars$mpg)))

  set.seed(123)
  expect_snapshot(
    variant = "windows",
    report(suppressWarnings(cor.test(
      mtcars$wt,
      mtcars$mpg,
      method = "spearman"
    )))
  )

  set.seed(123)
  expect_snapshot(
    variant = "windows",
    report(suppressWarnings(cor.test(
      mtcars$wt,
      mtcars$mpg,
      method = "kendall"
    )))
  )
})

test_that("report.htest-t-test", {
  # two samples unpaired t-tests summaries ---------------------

  set.seed(123)
  r <- report(t.test(iris$Sepal.Width, iris$Sepal.Length, var.equal = TRUE))
  expect_equal(
    as.report_table(r, summary = TRUE)$Difference,
    -2.786,
    tolerance = 0.01
  )

  set.seed(123)
  r <- report(t.test(iris$Sepal.Width, iris$Sepal.Length))
  expect_equal(
    as.report_table(r, summary = TRUE)$Difference,
    -2.786,
    tolerance = 0.01
  )

  set.seed(123)
  r <- report(t.test(mtcars$mpg ~ mtcars$vs))
  expect_equal(
    as.report_table(r, summary = TRUE)$Difference,
    -7.9404,
    tolerance = 0.01
  )

  set.seed(123)
  r <- report(t.test(iris$Sepal.Width, mu = 1))
  expect_equal(
    as.report_table(r, summary = TRUE)$Difference,
    2.057,
    tolerance = 0.01
  )

  # one-sample t-test ---------------------

  set.seed(123)
  expect_snapshot(variant = "windows", report(t.test(iris$Sepal.Width, mu = 1)))

  set.seed(123)
  expect_snapshot(
    variant = "windows",
    report(t.test(
      iris$Sepal.Width,
      mu = -1,
      alternative = "l"
    ))
  )

  set.seed(123)
  expect_snapshot(
    variant = "windows",
    report(t.test(
      iris$Sepal.Width,
      mu = 5,
      alternative = "g"
    ))
  )

  # two-sample unpaired t-test ---------------------

  set.seed(123)
  expect_snapshot(variant = "windows", report(t.test(mtcars$wt ~ mtcars$am)))

  set.seed(123)
  expect_snapshot(
    variant = "windows",
    report(t.test(mtcars$wt ~ mtcars$am, alternative = "l"))
  )

  set.seed(123)
  expect_snapshot(
    variant = "windows",
    report(t.test(mtcars$wt ~ mtcars$am, alternative = "g"))
  )

  # two-sample paired t-test ---------------------

  x <<- c(1.83, 0.50, 1.62, 2.48, 1.68, 1.88, 1.55, 3.06, 1.30)
  y <<- c(0.878, 0.647, 0.598, 2.05, 1.06, 1.29, 1.06, 3.14, 1.29)

  set.seed(123)
  expect_snapshot(variant = "windows", report(t.test(x, y, paired = TRUE)))

  set.seed(123)
  expect_snapshot(
    variant = "windows",
    report(t.test(x, y, paired = TRUE, alternative = "l"))
  )

  set.seed(123)
  expect_snapshot(
    variant = "windows",
    report(t.test(x, y, paired = TRUE, alternative = "g"))
  )

  if (getRversion() > "4.0") {
    sleep2 <<- reshape(
      sleep,
      direction = "wide",
      idvar = "ID",
      timevar = "group"
    )
    set.seed(123)
    expect_snapshot(
      variant = "windows",
      report(t.test(sleep2$extra.1, sleep2$extra.2, paired = TRUE))
    )
  }

  # type, rules ---------------------

  x <- t.test(mtcars$mpg ~ mtcars$vs)
  expect_snapshot(
    variant = "windows",
    report_effectsize(x)
  )
  expect_snapshot(
    variant = "windows",
    report_effectsize(x, type = "d")
  )
  expect_snapshot(
    variant = "windows",
    report_effectsize(x, type = "g")
  )
  expect_snapshot(
    variant = "windows",
    report_effectsize(x, rules = "cohen1988")
  )
  expect_snapshot(
    variant = "windows",
    report_effectsize(x, rules = "cohen1988", type = "d")
  )
  expect_snapshot(
    variant = "windows",
    report_effectsize(x, rules = "cohen1988", type = "g")
  )
  expect_snapshot(
    variant = "windows",
    report_effectsize(x, rules = "sawilowsky2009")
  )
  expect_snapshot(
    variant = "windows",
    report_effectsize(x, rules = "sawilowsky2009", type = "d")
  )
  expect_snapshot(
    variant = "windows",
    report_effectsize(x, rules = "sawilowsky2009", type = "g")
  )
  expect_snapshot(
    variant = "windows",
    report_effectsize(x, rules = "gignac2016")
  )
  expect_snapshot(
    variant = "windows",
    report_effectsize(x, rules = "gignac2016", type = "d")
  )
  expect_snapshot(
    variant = "windows",
    report_effectsize(x, rules = "gignac2016", type = "g")
  )
})

# Coverage tests for report.MixMod functions

skip_if_not_installed("GLMMadaptive")
skip_on_cran() # GLMMadaptive mixed models are computationally intensive

test_that("report.MixMod coverage test", {
  set.seed(999)
  # Create longitudinal data suitable for GLMMadaptive
  n_subjects <- 8
  n_timepoints <- 4

  data_mixmod <- data.frame(
    id = rep(1:n_subjects, each = n_timepoints),
    time = rep(c(0, 1, 2, 3), n_subjects),
    x = rnorm(n_subjects * n_timepoints)
  )

  # Create binary outcome appropriate for GLMMadaptive
  data_mixmod$y <- rbinom(
    nrow(data_mixmod),
    1,
    plogis(-1 + 0.5 * data_mixmod$time + 0.3 * data_mixmod$x)
  )

  # Create GLMMadaptive mixed model
  suppressWarnings({
    model <- GLMMadaptive::mixed_model(
      fixed = y ~ time + x,
      random = ~ 1 | id,
      data = data_mixmod,
      family = binomial()
    )
  })

  # Test report.MixMod (which is aliased to report.lm)
  r <- suppressWarnings(report(model, data = data_mixmod))
  expect_s3_class(r, "report")
  expect_type(summary(r), "character")
  expect_s3_class(as.data.frame(r), c("report_table", "data.frame"))

  # Test report_random.MixMod specifically
  rr <- report_random(model)
  expect_s3_class(rr, c("report_random", "character"))
  expect_true(grepl("random effect", rr, fixed = TRUE))

  # Test other MixMod-specific functions for coverage
  rt <- report_table(model)
  expect_s3_class(rt, c("report_table", "data.frame"))

  # Test report_performance for MixMod
  rp <- suppressWarnings(report_performance(model))
  expect_s3_class(rp, c("report_performance", "character"))

  # Test report_effectsize for MixMod
  re <- suppressWarnings(report_effectsize(model))
  expect_s3_class(re, "report_effectsize")
})

# Coverage tests for report.glmmTMB functions

skip_if_not_installed("glmmTMB")
skip_on_cran() # glmmTMB models are computationally intensive

test_that("report.glmmTMB coverage test", {
  set.seed(789)
  # Create simple test data
  test_data <- data.frame(
    y = rpois(20, lambda = 1.5),
    x = rnorm(20),
    group = factor(rep(1:4, 5))
  )

  # Create minimal glmmTMB model
  suppressWarnings({
    model <- glmmTMB::glmmTMB(
      y ~ x + (1 | group),
      data = test_data,
      family = poisson()
    )
  })

  # Test report.glmmTMB (which is aliased to report.lm)
  r <- report(model)
  expect_s3_class(r, "report")
  expect_type(summary(r), "character")
  expect_s3_class(as.data.frame(r), c("report_table", "data.frame"))

  # Test report_random.glmmTMB specifically
  rr <- report_random(model)
  expect_s3_class(rr, c("report_random", "character"))
  expect_true(grepl("random effect", rr, fixed = TRUE))
  expect_true(grepl("group", rr, fixed = TRUE))

  # Test other glmmTMB-specific functions for coverage
  rt <- report_table(model)
  expect_s3_class(rt, c("report_table", "data.frame"))
  expect_true("Parameter" %in% names(rt))

  # Test report_performance for glmmTMB
  rp <- report_performance(model)
  expect_s3_class(rp, c("report_performance", "character"))

  # Test report_effectsize for glmmTMB
  re <- suppressWarnings(report_effectsize(model))
  expect_s3_class(re, "report_effectsize")
})

skip_if_not_installed("glmmTMB")

test_that("report.glmmTMB", {
  skip_on_cran() # glmmTMB models can be computationally intensive

  # Create a simple dataset for testing
  set.seed(123)
  data_test <- data.frame(
    y = rpois(100, lambda = 2),
    x = rnorm(100),
    group = factor(rep(1:10, 10))
  )

  # Create a simple glmmTMB model
  suppressWarnings({
    model <- glmmTMB::glmmTMB(
      y ~ x + (1 | group),
      data = data_test,
      family = poisson()
    )
  })

  # Test main report function
  r <- report(model)
  expect_s3_class(r, "report")
  expect_s3_class(summary(r), "character")
  expect_s3_class(as.data.frame(r), c("report_table", "data.frame"))

  # Test that basic structure is correct
  expect_gt(nchar(summary(r)), 0)
  expect_gt(nrow(as.data.frame(r)), 0)

  # Test report components
  rt <- report_table(model)
  expect_s3_class(rt, c("report_table", "data.frame"))
  expect_true("Parameter" %in% names(rt))

  # Test that random effects are reported
  rr <- report_random(model)
  expect_s3_class(rr, c("report_random", "character"))
  expect_true(grepl("random effect", rr, fixed = TRUE))
  expect_true(grepl("group", rr, fixed = TRUE))
})

skip_if_not_installed("lavaan")
skip_on_cran() # lavaan SEM models can be computationally intensive

structure <- " ind60 =~ x1 + x2 + x3
  dem60 =~ y1 + y2 + y3
  dem60 ~ ind60 "

set.seed(123)
model <- lavaan::sem(structure, data = lavaan::PoliticalDemocracy)

# Specific reports
test_that("model-lavaan detailed report", {
  expect_snapshot(variant = "windows", report(model))
})

test_that("model-lavaan detailed table", {
  expect_snapshot(variant = "windows", report_table(model))
})

test_that("model-lavaan detailed performance", {
  expect_snapshot(variant = "windows", report_performance(model))
})

test_that("report.aov", {
  model <- anova(lm(Sepal.Width ~ Species, data = iris))
  r1 <- report(model)
  expect_equal(
    c(
      ncol(as.report_table(r1, summary = TRUE)),
      nrow(as.report_table(r1, summary = TRUE))
    ),
    c(7, 2),
    ignore_attr = FALSE
  )
  expect_equal(
    as.report_table(r1, summary = TRUE)$Mean_Square[1],
    5.6724,
    tolerance = 0.01
  )

  model <- aov(Sepal.Width ~ Species, data = iris)
  r2 <- report(model)
  expect_equal(
    c(
      ncol(as.report_table(r2, summary = TRUE)),
      nrow(as.report_table(r2, summary = TRUE))
    ),
    c(7, 2),
    ignore_attr = FALSE
  )
  expect_equal(
    as.report_table(r2, summary = TRUE)$Mean_Square[1],
    5.6724,
    tolerance = 0.01
  )

  r3 <- report(model)
  expect_equal(
    c(
      ncol(as.report_table(r3, summary = TRUE)),
      nrow(as.report_table(r3, summary = TRUE))
    ),
    c(7, 2),
    ignore_attr = FALSE
  )
  expect_equal(
    sum(as.report_table(r3, summary = TRUE)$Mean_Square),
    5.787854,
    tolerance = 0.01
  )

  data <- iris
  data$Cat1 <- rep_len(c("X", "X", "Y"), nrow(data))
  data$Cat2 <- rep_len(c("A", "B"), nrow(data))

  model <- aov(Sepal.Length ~ Species * Cat1 * Cat2, data = data)
  r4 <- report(model)
  expect_equal(
    c(
      ncol(as.report_table(r4, summary = TRUE)),
      nrow(as.report_table(r4, summary = TRUE))
    ),
    c(7, 8),
    ignore_attr = FALSE
  )
  expect_equal(
    as.report_table(r4, summary = TRUE)$Mean_Square[1],
    31.6060,
    tolerance = 0.01
  )

  model <- aov(Sepal.Length ~ Species * Cat1 + Error(Cat2), data = data)
  r5 <- report(model, verbose = FALSE)
  expect_warning(report(model), "non-finite")
  expect_equal(
    c(
      ncol(as.report_table(r5, summary = TRUE)),
      nrow(as.report_table(r5, summary = TRUE))
    ),
    c(8, 5),
    ignore_attr = FALSE
  )
  expect_equal(
    as.report_table(r5, summary = TRUE)$Mean_Square[1],
    31.60607,
    tolerance = 0.01
  )

  # snapshot tests -----

  set.seed(123)
  expect_snapshot(
    variant = "windows",
    suppressWarnings(report(anova(lm(Sepal.Width ~ Species, data = iris))))
  )

  set.seed(123)
  expect_snapshot(
    variant = "windows",
    suppressWarnings(report(anova(lm(
      wt ~ as.factor(am) * as.factor(cyl),
      data = mtcars
    ))))
  )

  set.seed(123)
  expect_snapshot(
    variant = "windows",
    suppressWarnings(report(aov(wt ~ cyl + Error(gear), data = mtcars)))
  )
})

skip_if_not_installed("modelbased")

test_that("report.estimate_contrasts", {
  # Create a simple model for testing contrasts
  model <- lm(Sepal.Width ~ Species, data = iris)

  # Skip if marginaleffects or other dependencies are not available
  skip_if_not_installed("marginaleffects")
  skip_if_not_installed("collapse")
  skip_if_not_installed("Formula")

  contr <- modelbased::estimate_contrasts(model)

  # Test main report function
  r <- report(contr)
  expect_s3_class(r, "report")
  expect_s3_class(summary(r), "character")
  expect_s3_class(as.data.frame(r), c("report_table", "data.frame"))

  # Test report_table
  rt <- report_table(contr)
  expect_s3_class(rt, c("report_table", "data.frame"))
  expect_gt(nrow(rt), 0)

  # Test report_text
  rtx <- report_text(contr, table = rt)
  expect_s3_class(rtx, c("report_text", "character"))
  expect_gt(nchar(rtx), 0)
  expect_true(grepl("marginal contrasts", rtx, fixed = TRUE))
})

skip_if_not_installed("GLMMadaptive")
skip_if_not_installed("glmmTMB") # Required by the report.MixMod method

test_that("report.MixMod", {
  # Create a test GLMMadaptive model
  skip_on_cran() # GLMMadaptive models can be computationally intensive

  # Use example from GLMMadaptive documentation
  # Create binary outcome data (GLMMadaptive works best with binomial)
  set.seed(123)
  n <- 50
  K <- 8
  t.max <- 15

  times <- as.vector(replicate(n, c(0, sort(runif(K - 1, 0, t.max)))))
  group <- sample(rep(0:1, each = n / 2))

  DF <- data.frame(
    id = rep(seq_len(n), each = K),
    time = times,
    group = factor(rep(group, each = K))
  )

  # Add binary outcome
  DF$y <- rbinom(nrow(DF), 1, plogis(-2.13 + 0.24 * DF$time))

  suppressWarnings({
    model <- GLMMadaptive::mixed_model(
      fixed = y ~ time + group,
      random = ~ 1 | id,
      data = DF,
      family = binomial()
    )
  })

  # Test main report function
  r <- suppressWarnings(report(model, data = DF))
  expect_s3_class(r, "report")
  expect_s3_class(summary(r), "character")
  expect_s3_class(as.data.frame(r), c("report_table", "data.frame"))

  # Test that it uses lm methods (since report.MixMod <- report.lm)
  expect_gt(nchar(summary(r)), 0)
  expect_gt(nrow(as.data.frame(r)), 0)

  # Test specific MixMod methods
  rt <- report_table(model)
  expect_s3_class(rt, c("report_table", "data.frame"))

  rr <- report_random(model)
  expect_s3_class(rr, c("report_random", "character"))
  expect_true(grepl("random effect", rr, fixed = TRUE))
})

test_that("report_table() works with sessionInfo", {
  si <- sessionInfo()

  result <- report_table(si)

  expect_s3_class(result, "report_table")
  expect_s3_class(result, "data.frame")

  # Should have some basic information about the session
  expect_gt(nrow(result), 0)
})

test_that("report_table() works with basic data types", {
  # Numeric vector
  numeric_data <- iris$Sepal.Length
  result1 <- report_table(numeric_data)
  expect_s3_class(result1, "report_table")

  # Character vector
  char_data <- as.character(round(iris$Sepal.Length, 1))
  result2 <- report_table(char_data)
  expect_s3_class(result2, "report_table")
  expect_true("n_Entries" %in% names(result2))

  # Factor
  factor_data <- iris$Species
  result3 <- report_table(factor_data)
  expect_s3_class(result3, "report_table")
  expect_true("Level" %in% names(result3))
})

test_that("report_table() works with data frames", {
  result <- report_table(iris)

  expect_s3_class(result, "report_table")
  expect_s3_class(result, "data.frame")
  expect_gt(nrow(result), 0)
})

test_that("report_table() works with statistical tests", {
  # t-test
  test_result <- t.test(mtcars$mpg ~ mtcars$am)
  result1 <- report_table(test_result)
  expect_s3_class(result1, "report_table")

  # ANOVA
  aov_result <- aov(Sepal.Length ~ Species, data = iris)
  result2 <- report_table(aov_result)
  expect_s3_class(result2, "report_table")
})

test_that("report_table() works with linear models", {
  # Simple linear model
  model1 <- lm(Sepal.Length ~ Petal.Length, data = iris)
  result1 <- report_table(model1)
  expect_s3_class(result1, "report_table")
  expect_true(
    "Parameter" %in% names(result1) || "Coefficient" %in% names(result1)
  )

  # Interaction model
  model2 <- lm(Sepal.Length ~ Petal.Length * Species, data = iris)
  result2 <- report_table(model2)
  expect_s3_class(result2, "report_table")

  # GLM
  model3 <- glm(vs ~ disp, data = mtcars, family = "binomial")
  result3 <- report_table(model3)
  expect_s3_class(result3, "report_table")
})

test_that("as.report_table() works correctly", {
  # Create a data frame to convert
  df <- data.frame(
    Parameter = c("A", "B"),
    Value = c(1, 2),
    stringsAsFactors = FALSE
  )

  result <- as.report_table(df)
  expect_s3_class(result, "report_table")
  expect_s3_class(result, "data.frame")
  expect_identical(nrow(result), 2L)
  expect_identical(result$Parameter, c("A", "B"))
})

test_that("report_table methods work correctly", {
  # Create a simple report table
  df <- data.frame(x = 1:3, y = 4:6)
  result <- as.report_table(df)

  # Test summary
  summ <- summary(result)
  expect_s3_class(summ, "report_table")

  # Test print (should not error)
  expect_output(print(result))
})

test_that("report.numeric", {
  r <- report(seq(0, 1, length.out = 100))
  expect_equal(as.data.frame(r)$Mean, 0.5, tolerance = 0)
  expect_null(as.report_table(r, summary = TRUE)$Median)

  r <- report(
    seq(0, 1, length.out = 100),
    centrality = "median",
    range = FALSE,
    dispersion = FALSE,
    missing_percentage = TRUE
  )
  expect_equal(as.data.frame(r)$Median, 0.5, tolerance = 0)
  expect_equal(as.data.frame(r)$percentage_Missing, 0, tolerance = 0)
  expect_null(summary(as.data.frame(r))$Mean)
  expect_null(summary(as.data.frame(r))$Min)
  expect_null(summary(as.data.frame(r))$MAD)
  expect_warning(report(c(0, 0, 0, 1, 1)), "factor")
})


test_that("report.character", {
  x <- c("A", "B", "C", "A", "B", "B", "D", "E", "B", "D", "A")
  r <- report(x)
  expect_equal(nrow(as.data.frame(r)), 1, tolerance = 0)
  expect_null(as.data.frame(r)$Median)

  r <- report(x, levels_percentage = FALSE, missing_percentage = TRUE)
  expect_equal(nrow(as.data.frame(r)), 1, tolerance = 0)
  expect_equal(as.data.frame(r)$percentage_Missing[1], 0, tolerance = 0)
})

test_that("report.factor", {
  r <- report(factor(rep(c("A", "B", "C"), 10)))
  expect_equal(nrow(as.data.frame(r)), 3, tolerance = 0)
  expect_null(as.data.frame(r)$Median)
  expect_snapshot(variant = "windows", r)

  r <- report(factor(rep(c("A", "B", "C", NA), 10)), levels_percentage = FALSE)
  expect_equal(nrow(as.data.frame(r)), 4, tolerance = 0)
})

test_that("report.Date", {
  set.seed(123)
  x <- sample(
    seq(as.Date("1999/01/01"), as.Date("1999/01/05"), by = "day"),
    30,
    replace = TRUE
  )
  r <- report(x)
  expect_equal(
    as.character(r),
    paste0(
      "x: 5 levels, namely 1999-01-01 (n = 6, 20.00%), 1999-01-02 (n = 6, 20.00%), ",
      "1999-01-03 (n = 9, 30.00%), 1999-01-04 (n = 4, 13.33%) and 1999-01-05 (n = 5, 16.67%)"
    ),
    ignore_attr = TRUE
  )
})

test_that("report.data.frame", {
  skip_if_not_installed("dplyr")

  r <- report(iris)
  expect_equal(nrow(as.data.frame(r)), 7, tolerance = 0)
  expect_equal(mean(as.data.frame(r)$Median, na.rm = TRUE), 3.6125)

  r <- report(
    iris,
    levels_percentage = FALSE,
    missing_percentage = TRUE,
    median = TRUE,
    range = FALSE,
    dispersion = FALSE
  )
  expect_equal(nrow(as.data.frame(r)), 7, tolerance = 0)
  expect_equal(mean(as.data.frame(r)$n_Obs), 107, tolerance = 0.01)

  r <- report(dplyr::group_by(iris, Species))
  expect_equal(nrow(as.data.frame(r)), 12, tolerance = 0)
  expect_equal(mean(as.data.frame(r)$n_Obs), 50, tolerance = 0)

  expect_snapshot(variant = "windows", r)
})

test_that("report.data.frame - with NAs", {
  skip_if_not_installed("dplyr")

  df <- mtcars
  df[1, 2] <- NA
  df[1, 6] <- NA

  report_grouped_df <- report(dplyr::group_by(df, cyl))
  expect_snapshot(variant = "windows", report_grouped_df)
})

test_that("report.data.frame - with list columns", {
  skip_if_not_installed("dplyr")

  set.seed(123)
  expect_snapshot(variant = "windows", report(dplyr::starwars))
})

skip_if_not_installed("performance")
library(performance)

test_that("report.test_performance", {
  # Create test models
  m1 <- lm(Sepal.Length ~ Petal.Length * Species, data = iris)
  m2 <- lm(Sepal.Length ~ Petal.Length + Species, data = iris)
  m3 <- lm(Sepal.Length ~ Petal.Length, data = iris)

  x <- performance::test_performance(m1, m2, m3)

  # Test main report function
  r <- report(x)
  expect_s3_class(r, "report")
  expect_s3_class(summary(r), "character")
  expect_s3_class(as.data.frame(r), c("report_table", "data.frame"))

  # Test report_table
  rt <- report_table(x)
  expect_s3_class(rt, c("report_table", "data.frame"))
  expect_gte(nrow(rt), 1) # At least one comparison

  # Test report_statistics
  rs <- report_statistics(x)
  expect_s3_class(rs, "character")
  expect_true(nzchar(rs[[1]], keepNA = TRUE)) # May be empty if no significant comparisons

  # Test report_parameters
  rp <- report_parameters(x)
  expect_s3_class(rp, "character")
  expect_gt(nchar(rp[[1]]), 0)

  # Test report_text
  rtx <- report_text(x)
  expect_s3_class(rtx, c("report_text", "character"))
  expect_gt(nchar(rtx), 0)
  expect_true(grepl("compared", rtx, fixed = TRUE))
})

#' @rdname report.data.frame
#' @export
report.factor <- function(x, levels_percentage = "auto", ...) {
  if (!is.factor(x)) {
    x <- as.factor(x)
  }
  report_table_obj <- report_table(
    x,
    levels_percentage = levels_percentage,
    ...
  )
  report_text_obj <- report_text(x, levels_percentage = levels_percentage, ...)

  as.report(report_text_obj, table = report_table_obj, ...)
}

#' @export
report.logical <- report.factor

#' @export
report.Date <- report.factor


# report_table ------------------------------------------------------------

#' @export
report_table.factor <- function(x, levels_percentage = "auto", ...) {
  if (!is.factor(x)) {
    x <- as.factor(x)
  }
  levels_percentage <- .report_dataframe_percentage(x, levels_percentage)

  if (length(x[is.na(x)]) != 0) {
    x <- factor(
      ifelse(is.na(x), "missing", as.character(x)),
      levels = c(levels(x), "missing")
    )
  }

  # Table
  table_full <- as.data.frame(table(x))
  names(table_full) <- c("Level", "n_Obs")
  table_full$percentage_Obs <- table_full$n_Obs / length(x) * 100

  # Shorten
  report_table_obj <- table_full
  if (!levels_percentage) {
    report_table_obj <- datawizard::data_remove(
      report_table_obj,
      "percentage_Obs"
    )
  }

  as.report_table(table_full, summary = report_table_obj)
}

#' @export
report_table.logical <- report_table.factor

#' @export
report_table.Date <- report_table.factor


# report_parameters -------------------------------------------------------

#' @export
report_parameters.factor <- function(
  x,
  table = NULL,
  levels_percentage = "auto",
  ...
) {
  if (!is.factor(x)) {
    x <- as.factor(x)
  }
  # Get table
  if (is.null(table)) {
    table <- report_table(x, levels_percentage = levels_percentage, ...)
  }

  text_levels <- paste0(table$Level)
  text_n_Obs <- paste0("n = ", table$n_Obs)
  text_percentage_Obs <- paste0(
    insight::format_value(table$percentage_Obs),
    "%"
  )

  text_full <- paste0(
    text_levels,
    " (",
    text_n_Obs,
    ", ",
    text_percentage_Obs,
    ")"
  )

  if (isTRUE(levels_percentage)) {
    report_text_obj <- paste0(
      text_levels,
      " (",
      text_percentage_Obs,
      ")"
    )
  } else {
    report_text_obj <- paste0(
      text_levels,
      " (",
      text_n_Obs,
      ")"
    )
  }

  as.report_parameters(text_full, summary = report_text_obj, ...)
}

#' @export
report_parameters.logical <- report_parameters.factor

#' @export
report_parameters.Date <- report_parameters.factor


# report_text -------------------------------------------------------------

#' @export
report_text.factor <- function(
  x,
  table = NULL,
  levels_percentage = "auto",
  ...
) {
  if (!is.factor(x)) {
    x <- as.factor(x)
  }
  if (!is.null(list(...)$varname)) {
    name <- list(...)$varname
  } else if (is.null(names(x))) {
    name <- deparse(substitute(x))
  } else {
    name <- "Factor"
  }

  if (is.null(table)) {
    table <- report_table(x, levels_percentage = levels_percentage, ...)
  }

  table_no_missing <- table[table$Level != "missing", ]
  params <- report_parameters(
    x,
    table = table,
    levels_percentage = levels_percentage,
    ...
  )

  if (nrow(table) > 1) {
    text_total_levels <- paste0(
      name,
      ": ",
      nrow(table_no_missing),
      " levels, namely "
    )
  } else {
    text_total_levels <- paste0(
      name,
      ": ",
      nrow(table_no_missing),
      " level, namely "
    )
  }

  text_full <- paste0(
    text_total_levels,
    datawizard::text_concatenate(params, sep = ", ")
  )
  report_text_obj <- paste0(
    text_total_levels,
    datawizard::text_concatenate(summary(params), sep = ", ")
  )

  as.report_text(text_full, summary = report_text_obj)
}

#' @export
report_text.logical <- report_text.factor

#' @export
report_text.Date <- report_text.factor


# report_statistics -------------------------------------------------------

#' @export
report_statistics.factor <- function(
  x,
  table = NULL,
  levels_percentage = "auto",
  ...
) {
  if (!is.factor(x)) {
    x <- as.factor(x)
  }
  if (is.null(table)) {
    table <- report_table(x, levels_percentage = levels_percentage, ...)
  }

  text_levels <- paste0(table$Level)
  text_n_Obs <- paste0("n = ", table$n_Obs)
  text_percentage_Obs <- paste0(
    insight::format_value(table$percentage_Obs),
    "%"
  )

  text_full <- paste0(
    text_levels,
    ", ",
    text_n_Obs,
    ", ",
    text_percentage_Obs
  )

  if (isTRUE(levels_percentage)) {
    report_text_obj <- paste0(
      text_levels,
      ", ",
      text_percentage_Obs
    )
  } else {
    report_text_obj <- paste0(
      text_levels,
      ", ",
      text_n_Obs
    )
  }

  as.report_statistics(
    paste(text_full, collapse = "; "),
    summary = paste(report_text_obj, collapse = "; ")
  )
}

#' @export
report_statistics.Date <- report_statistics.factor

#' Reporting ANOVAs
#'
#' Create reports for ANOVA models.
#'
#' @param x Object of class `aov`, `anova` or `aovlist`.
#' @param include_intercept	Set to `TRUE` to include the intercept (relevant for type-3 ANOVA tables).
#' @inheritParams report
#' @inheritParams report.htest
#' @inherit report return seealso
#'
#' @examples
#' data <- iris
#' data$Cat1 <- rep(c("A", "B"), length.out = nrow(data))
#'
#' model <- aov(Sepal.Length ~ Species * Cat1, data = data)
#' r <- report(model)
#' r
#' summary(r)
#' as.data.frame(r)
#' summary(as.data.frame(r))
#' @return An object of class [report()].
#' @export
report.aov <- function(x, ...) {
  results_table <- report_table(x, ...)
  result_text <- report_text(x, table = results_table, ...)

  as.report(result_text, table = results_table, ...)
}

#' @export
report.anova <- report.aov

#' @export
report.aovlist <- report.aov


# report_effectsize -------------------------------------------------------

#' @rdname report.aov
#' @inheritParams report.lm
#' @export
report_effectsize.aov <- function(x, include_intercept = FALSE, ...) {
  results_table <- suppressMessages(effectsize::effectsize(
    x,
    include_intercept = include_intercept,
    ...
  ))
  estimate <- names(results_table)[effectsize::is_effectsize_name(names(
    results_table
  ))]

  interpret <- switch(
    estimate,
    Eta2_partial = effectsize::interpret_eta_squared(
      results_table[[estimate]],
      ...
    ),
    Eta2 = effectsize::interpret_eta_squared(results_table[[estimate]], ...),
    Omega2_partial = effectsize::interpret_omega_squared(
      results_table[[estimate]],
      ...
    ),
    Omega2 = effectsize::interpret_omega_squared(
      results_table[[estimate]],
      ...
    ),
    Epsilon2_partial = effectsize::interpret_epsilon_squared(
      results_table[[estimate]],
      ...
    ),
    Epsilon2 = effectsize::interpret_epsilon_squared(
      results_table[[estimate]],
      ...
    )
  )

  interpretation <- interpret

  main <- switch(
    estimate,
    Eta2_partial = paste0(
      "Eta2 (partial) = ",
      insight::format_value(results_table[[estimate]])
    ),
    Eta2 = paste0("Eta2 = ", insight::format_value(results_table[[estimate]])),
    Omega2_partial = paste0(
      "Omega2 (partial) = ",
      insight::format_value(results_table[[estimate]])
    ),
    Omega2 = paste0(
      "Epsilon2 = ",
      insight::format_value(results_table[[estimate]])
    ),
    Epsilon2_partial = paste0(
      "Epsilon2 (partial) = ",
      insight::format_value(results_table[[estimate]])
    ),
    Epsilon2 = paste0(
      "Epsilon2 = ",
      insight::format_value(results_table[[estimate]])
    )
  )

  ci <- results_table$CI
  statistics <- paste0(
    main,
    ", ",
    insight::format_ci(results_table$CI_low, results_table$CI_high, ci)
  )

  effsize_table <- as.data.frame(results_table)[c(
    "Parameter",
    estimate,
    "CI_low",
    "CI_high"
  )]
  names(effsize_table)[3:ncol(effsize_table)] <- c(
    paste0(estimate, "_CI_low"),
    paste0(estimate, "_CI_high")
  )

  rules <- .text_effectsize(attr(attr(interpret, "rules"), "rule_name"))
  parameters <- paste0(interpretation, " (", statistics, ")")

  as.report_effectsize(
    parameters,
    summary = parameters,
    table = effsize_table,
    interpretation = interpretation,
    statistics = statistics,
    rules = rules,
    ci = ci,
    main = main
  )
}

#' @export
report_effectsize.anova <- report_effectsize.aov

#' @export
report_effectsize.aovlist <- report_effectsize.aov

# report_table ------------------------------------------------------------

#' @rdname report.aov
#' @export
report_table.aov <- function(x, include_intercept = FALSE, ...) {
  effsize <- report_effectsize(x, include_intercept = include_intercept, ...)
  effsize_table <- attributes(effsize)$table
  params <- parameters::model_parameters(x, ...)

  if (!include_intercept) {
    params <- params[params$Parameter != "(Intercept)", ]
  }

  if ("Group" %in% names(params)) {
    effsize_table$Group <- "Within"
    params <- params[params$Group == "Within", ]
    table_full <- merge(params, effsize_table, all = TRUE)
    table_full <- table_full[
      order(
        match(
          paste(table_full$Group, table_full$Parameter),
          paste(params$Group, params$Parameter)
        )
      ),
    ]
  } else {
    table_full <- merge(params, effsize_table, all = TRUE)
    table_full <- table_full[
      order(
        match(table_full$Parameter, params$Parameter)
      ),
    ]
  }

  row.names(table_full) <- NULL

  results_table <- datawizard::data_remove(
    table_full,
    select = "(_CI_low|_CI_high)$",
    regex = TRUE
  )

  as.report_table(
    table_full,
    summary = results_table,
    ci = attributes(effsize)$ci,
    effsize = effsize
  )
}

#' @export
report_table.anova <- report_table.aov

#' @export
report_table.aovlist <- report_table.aov


# report_statistics ------------------------------------------------------------

#' @rdname report.aov
#' @export
report_statistics.aov <- function(x, table = NULL, ...) {
  if (is.null(table) || is.null(attributes(table)$effsize)) {
    table <- report_table(x, ...)
  }
  effsize <- attributes(table)$effsize

  parameters <- table[table$Parameter != "Residuals", ]
  if ("Group" %in% names(parameters)) {
    parameters <- parameters[parameters$Group == "Within", ]
  }

  # Get residuals' DoFs
  if ("Residuals" %in% table$Parameter) {
    DoF_residual <- table[table$Parameter == "Residuals", "df"]
  } else {
    DoF_residual <- NULL
  }

  # DoFs
  result_text <- paste0(
    "F(",
    insight::format_value(parameters$df, protect_integers = TRUE)
  )

  if (!is.null(DoF_residual)) {
    result_text <- paste0(
      result_text,
      ", ",
      insight::format_value(DoF_residual, protect_integers = TRUE)
    )
  } else if ("DoF_Residuals" %in% names(parameters)) {
    result_text <- paste0(
      result_text,
      ", ",
      insight::format_value(parameters$DoF_Residuals, protect_integers = TRUE)
    )
  }

  # Indices
  result_text <- paste0(
    result_text,
    ") = ",
    insight::format_value(parameters[["F"]]),
    ", ",
    insight::format_p(parameters$p)
  )

  # Effect size
  text_full <- paste0(result_text, "; ", attributes(effsize)$statistics)
  result_text <- paste0(result_text, ", ", attributes(effsize)$main)

  as.report_statistics(
    text_full,
    summary = result_text,
    table = table,
    effsize = effsize
  )
}

#' @export
report_statistics.anova <- report_statistics.aov

#' @export
report_statistics.aovlist <- report_statistics.aov


# report_parameters ------------------------------------------------------------

#' @rdname report.aov
#' @export
report_parameters.aov <- function(x, ...) {
  stats <- report_statistics(x, ...)
  stats_table <- attributes(stats)$table
  effsize <- attributes(stats)$effsize

  params <- stats_table[stats_table$Parameter != "Residuals", ]

  # Text parameters
  parameter_text <- vapply(
    params$Parameter,
    .format_parameters_aov,
    USE.NAMES = FALSE,
    "string"
  )

  # Significance
  parameter_text <- paste0(
    parameter_text,
    " is statistically ",
    effectsize::interpret_p(params$p),
    " and ",
    attributes(effsize)$interpretation,
    " ("
  )

  text_full <- paste0(parameter_text, stats, ")")
  parameter_text <- paste0(parameter_text, summary(stats), ")")

  as.report_parameters(
    text_full,
    summary = parameter_text,
    table = stats_table,
    effectsize = effsize,
    ...
  )
}

#' @export
report_parameters.anova <- report_parameters.aov

#' @export
report_parameters.aovlist <- report_parameters.aov


# report_model ------------------------------------------------------------

#' @rdname report.aov
#' @export
report_model.aov <- function(x, table = NULL, ...) {
  if (is.null(table)) {
    table <- report_table(x, ...)
  }

  if ("Group" %in% names(table)) {
    model_text <- "repeated-measures ANOVA"
  } else {
    model_text <- "ANOVA"
  }

  if (inherits(x, "anova")) {
    text_full <- model_text # Because anova() does not save the formula.
  } else {
    text_full <- paste0(
      model_text,
      " (",
      format_formula(x),
      ")"
    )
  }

  as.report_model(text_full, summary = model_text)
}

#' @export
report_model.anova <- report_model.aov

#' @export
report_model.aovlist <- report_model.aov


# report_info ------------------------------------------------------------

#' @rdname report.aov
#' @include report.htest.R
#' @export
report_info.aov <- function(x, effectsize = NULL, ...) {
  if (is.null(effectsize)) {
    effectsize <- report_effectsize(x, ...)
  }
  as.report_info(attributes(effectsize)$rules)
}

#' @export
report_info.anova <- report_info.aov

#' @export
report_info.aovlist <- report_info.aov


# report_text ------------------------------------------------------------

#' @rdname report.aov
#' @export
report_text.aov <- function(x, table = NULL, ...) {
  params <- report_parameters(x, table = table, ...)
  table <- attributes(params)$table
  model <- report_model(x, table = table, ...)
  info <- report_info(x, effectsize = attributes(params)$effectsize, ...)

  text_full <- paste0(
    "The ",
    model,
    " suggests that:\n\n",
    as.character(params, ...),
    "\n\n",
    info
  )

  result_text <- paste0(
    "The ",
    summary(model),
    " suggests that:\n\n",
    as.character(summary(params), ...)
  )

  as.report_text(text_full, summary = result_text)
}

#' @export
report_text.anova <- report_text.aov

#' @export
report_text.aovlist <- report_text.aov

# Neutralize any C/C++ flags leaking into test compiles
Sys.setenv(
  R_MAKEVARS_USER = "",
  R_MAKEVARS_SITE = "",
  R_MAKEVARS = ""
)

# Unset env vars that force Stan/RcppEigen includes into ALL compilations
vars <- c(
  "PKG_CPPFLAGS",
  "CPPFLAGS",
  "CFLAGS",
  "CXXFLAGS",
  "CXX11FLAGS",
  "CXX14FLAGS",
  "CXX17FLAGS",
  "CPLUS_INCLUDE_PATH",
  "INCLUDE"
)
Sys.unsetenv(vars)

# (Optional) double-check during tests
# message("FLAGS after cleanup:\n",
#         paste(sprintf("%s='%s'", vars, Sys.getenv(vars)), collapse="\n"))

#' @include report.lm.R
#' @export
report.coxph <- report.lm

#' @export
report_effectsize.coxph <- report_effectsize.lm

#' @export
report_table.coxph <- report_table.lm

#' @export
report_statistics.coxph <- report_statistics.lm

#' @export
report_parameters.coxph <- report_parameters.lm

#' @export
report_intercept.coxph <- report_intercept.lm

#' @export
report_model.coxph <- report_model.lm

#' @export
report_performance.coxph <- report_performance.lm

#' @export
report_info.coxph <- report_info.lm

#' @export
report_text.coxph <- report_text.lm

# report_table -----------------

.report_table_chi2 <- function(table_full, effsize) {
  table_full <- cbind(table_full, attributes(effsize)$table)
  list(table = NULL, table_full = table_full)
}


# report_effectsize ---------------------

.report_effectsize_chi2 <- function(x, dot_args, rules = "funder2019") {
  if (!chi2_type(x) %in% c("pearson", "probabilities")) {
    insight::format_error(
      "This test is not yet supported. Please open an issue at {.url https://github.com/easystats/report/issues}."
    )
  }
  es_args <- c(list(x), dot_args)
  es_table <- do.call(effectsize::effectsize, es_args)
  table_footer <- attributes(es_table)$table_footer
  ci <- attributes(es_table)$ci
  estimate <- names(es_table)[1]
  dot_args$rules <- ifelse(is.null(dot_args$rules), rules, dot_args$rules)

  es_args <- c(list(es_table), dot_args)
  interpretation <- do.call(effectsize::interpret, es_args)$Interpretation
  rules <- .text_effectsize(attr(attr(interpretation, "rules"), "rule_name"))

  main <- switch(
    estimate,
    Cramers_v_adjusted = paste0(
      "Adjusted Cramer's v = ",
      insight::format_value(es_table[[estimate]])
    ),
    Fei = paste0("Fei = ", insight::format_value(es_table[[estimate]])),
    Tschuprows_t = paste0(
      "Tschuprow's t = ",
      insight::format_value(es_table[[estimate]])
    ),
    Tschuprows_t_adjusted = paste0(
      "Adjusted Tschuprow's t = ",
      insight::format_value(es_table[[estimate]])
    ),
    Pearsons_c = paste0(
      "Pearson's c = ",
      insight::format_value(es_table[[estimate]])
    ),
    phi_adjusted = paste0(
      "Adjusted Phi = ",
      insight::format_value(es_table[[estimate]])
    ),
    Cohens_h = paste0(
      "Cohen's h = ",
      insight::format_value(es_table[[estimate]])
    ),
    Odds_ratio = paste0(
      "Odds ratio = ",
      insight::format_value(es_table[[estimate]])
    ),
    Ris_kratio = paste0(
      "Risk ratio = ",
      insight::format_value(es_table[[estimate]])
    ),
    cohens_h = paste0(
      "Cohen's w = ",
      insight::format_value(es_table[[estimate]])
    ),
    paste0(estimate, " = ", insight::format_value(es_table[[estimate]]))
  )

  statistics <- paste0(
    main,
    ", ",
    insight::format_ci(es_table$CI_low, es_table$CI_high, ci)
  )

  result_table <- datawizard::data_rename(
    as.data.frame(es_table),
    select = c("CI_low", "CI_high"),
    replacement = paste0(estimate, c("_CI_low", "_CI_high"))
  )

  result_table <- result_table[c(
    estimate,
    paste0(estimate, c("_CI_low", "_CI_high"))
  )]
  attributes(result_table)$table_footer <- table_footer

  list(
    table = result_table,
    statistics = statistics,
    interpretation = interpretation,
    rules = rules,
    ci = ci,
    main = main
  )
}

# report_model ----------------------------

.report_model_chi2 <- function(x, table) {
  if (chi2_type(x) == "pearson") {
    type <- " of independence between"
    vars_full <- paste(
      names(attributes(x$observed)$dimnames),
      collapse = " and "
    )
  } else if (chi2_type(x) == "probabilities") {
    type <- " / goodness of fit of "
    distr <- ifelse(
      grepl("non", attr(table, "table_footer"), fixed = TRUE),
      "a uniform distribution",
      paste0(
        "a distribution of [",
        paste0(
          names(x$expected),
          ": n=",
          x$expected,
          collapse = ", "
        ),
        "]"
      )
    )

    vars_full <- paste(x$data.name, "to", distr)
  }

  paste0(
    trimws(x$method),
    type,
    paste0(" ", vars_full)
  )
}

chi2_type <- function(x) {
  if (grepl("probabilities", x$method, fixed = TRUE)) {
    out <- "probabilities"
  } else if (grepl("Pearson", x$method, fixed = TRUE)) {
    out <- "pearson"
  }
  out
}

# report_parameters ----------------------------

.report_parameters_chi2 <- function(table, stats, effsize, ...) {
  if (is.null(attributes(effsize)$interpretation)) {
    and <- ""
  } else {
    and <- paste0(", and ", attributes(effsize)$interpretation)
  }

  text_full <- paste0(
    "statistically ",
    effectsize::interpret_p(table$p, rules = "default"),
    and,
    " (",
    stats,
    ")"
  )

  text_short <- paste0(
    "statistically ",
    effectsize::interpret_p(table$p, rules = "default"),
    and,
    " (",
    summary(stats),
    ")"
  )

  list(text_short = text_short, text_full = text_full)
}

#' @include report.lm.R
#' @export
report.MixMod <- report.lm

#' @export
report_effectsize.MixMod <- report_effectsize.lm

#' @export
report_table.MixMod <- report_table.lm

#' @export
report_performance.MixMod <- report_performance.lm

#' @export
report_statistics.MixMod <- report_statistics.lm

#' @export
report_parameters.MixMod <- report_parameters.lm

#' @export
report_intercept.MixMod <- report_intercept.lm

#' @export
report_random.MixMod <- function(x, ...) {
  random_terms <- insight::find_terms(x)$random
  if (!is.null(random_terms)) {
    random_text <- random_terms
    random_text <- paste0(
      "The model included ",
      random_text,
      " as random effect"
    )
    random_text <- ifelse(
      length(random_terms) > 1,
      paste0(random_text, "s"),
      random_text
    )
    text_full <- paste0(random_text, " (", format_formula(x, "random"), ")")
  }

  as.report_random(text_full, summary = random_text, ...)
}


#' @export
report_model.MixMod <- report_model.lm

#' @export
report_info.MixMod <- report_info.lm

#' @export
report_text.MixMod <- report_text.lm

#' Report the model's quality and fit indices
#'
#' Investigating the fit of statistical models to data often involves selecting
#' the best fitting model amongst many competing models. This function helps
#' report indices of model fit for various models.  Reports the type of
#' different R objects . For a list of supported objects, see
#' [report()]).
#'
#' @inheritParams report
#' @inheritParams report_table
#' @inheritParams report_text
#' @inheritParams as.report
#'
#' @return An object of class [report_performance()].
#'
#' @examples
#' \donttest{
#' # GLMs
#' report_performance(lm(Sepal.Length ~ Petal.Length * Species, data = iris))
#' report_performance(glm(vs ~ disp, data = mtcars, family = "binomial"))
#' }
#'
#' @examplesIf requireNamespace("lme4", quietly = TRUE)
#' \donttest{
#' # Mixed models
#' library(lme4)
#' model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
#' report_performance(model)
#' }
#'
#' @examplesIf requireNamespace("rstanarm", quietly = TRUE)
#' \donttest{
#' # Bayesian models
#' library(rstanarm)
#' model <- suppressWarnings(stan_glm(Sepal.Length ~ Species, data = iris, refresh = 0, iter = 600))
#' report_performance(model)
#' }
#'
#' @examplesIf requireNamespace("lavaan", quietly = TRUE) && packageVersion("effectsize") >= "0.6.0.1"
#' \donttest{
#' # Structural Equation Models (SEM)
#' library(lavaan)
#' structure <- "ind60 =~ x1 + x2 + x3
#'               dem60 =~ y1 + y2 + y3
#'               dem60 ~ ind60 "
#' model <- lavaan::sem(structure, data = PoliticalDemocracy)
#' suppressWarnings(report_performance(model))
#' }
#' @export
report_performance <- function(x, table = NULL, ...) {
  UseMethod("report_performance")
}


# METHODS -----------------------------------------------------------------

#' @rdname as.report
#' @export
as.report_performance <- function(x, summary = NULL, ...) {
  class(x) <- unique(c("report_performance", class(x)))
  attributes(x) <- c(attributes(x), list(...))

  if (!is.null(summary)) {
    attr(x, "summary") <- summary
  }

  x
}


#' @export
summary.report_performance <- function(object, ...) {
  if (is.null(attributes(object)$summary)) {
    object
  } else {
    attributes(object)$summary
  }
}

#' @export
print.report_performance <- print.report_text

# Utils -------------------------------------------------------------------

#' @keywords internal
.text_r2 <- function(x, info, performance, ...) {
  r2_text <- ""
  text_full <- ""

  # R2
  if ("R2" %in% names(performance)) {
    r2 <- attributes(performance)$r2

    r2_text <- paste0(
      "The model's explanatory power is ",
      effectsize::interpret_r2(performance$R2, ...),
      " (R2 = ",
      insight::format_value(performance$R2)
    )

    # Frequentist
    if (all(c("p", "df", "df_residual") %in% names(r2))) {
      text_full <- paste0(
        "The model explains a statistically ",
        effectsize::interpret_p(r2$p),
        " and ",
        effectsize::interpret_r2(performance$R2, ...),
        " proportion of variance (R2 = ",
        insight::format_value(performance$R2),
        ", F(",
        insight::format_value(r2$df, protect_integers = TRUE),
        ", ",
        insight::format_value(r2$df_residual, protect_integers = TRUE),
        ") = ",
        insight::format_value(r2[["F"]]),
        ", ",
        insight::format_p(r2$p)
      )
    } else {
      text_full <- r2_text
    }

    if ("CI" %in% names(r2)) {
      text_full <- paste0(
        text_full,
        ", ",
        insight::format_ci(
          r2$CI$R2_Bayes$CI_low,
          r2$CI$R2_Bayes$CI_high,
          r2$CI$R2_Bayes$CI
        )
      )
    }

    if ("R2_adjusted" %in% names(performance)) {
      r2_text <- paste0(
        r2_text,
        ", adj. R2 = ",
        insight::format_value(performance$R2_adjusted),
        ")"
      )
      text_full <- paste0(
        text_full,
        ", adj. R2 = ",
        insight::format_value(performance$R2_adjusted),
        ")"
      )
    } else {
      if (datawizard::text_lastchar(text_full) != ")") {
        text_full <- paste0(text_full, ")")
      }
      if (datawizard::text_lastchar(r2_text) != ")") {
        r2_text <- paste0(r2_text, ")")
      }
    }
  }

  # Tjur's R2
  if ("R2_Tjur" %in% names(performance)) {
    r2_text <- text_full <- paste0(
      "The model's explanatory power is ",
      effectsize::interpret_r2(performance$R2_Tjur, ...),
      " (Tjur's R2 = ",
      insight::format_value(performance$R2_Tjur),
      ")"
    )
  }

  # Nagelkerke's R2
  if ("R2_Nagelkerke" %in% names(performance)) {
    r2_text <- text_full <- paste0(
      "The model's explanatory power is ",
      effectsize::interpret_r2(performance$R2_Nagelkerke, ...),
      " (Nagelkerke's R2 = ",
      insight::format_value(performance$R2_Nagelkerke),
      ")"
    )
  }

  # CoxSnell's R2
  if ("R2_CoxSnell" %in% names(performance)) {
    r2_text <- text_full <- paste0(
      "The model's explanatory power is ",
      effectsize::interpret_r2(performance$R2_CoxSnell, ...),
      " (R2_CoxSnell's R2 = ",
      insight::format_value(performance$R2_CoxSnell),
      ")"
    )
  }

  # McFadden's R2
  if ("R2_McFadden" %in% names(performance)) {
    r2_text <- text_full <- paste0(
      "The model's explanatory power is ",
      effectsize::interpret_r2(performance$R2_McFadden, ...),
      " (McFadden's R2 = ",
      insight::format_value(performance$R2_McFadden),
      ")"
    )
  }

  # R2 Conditional
  if (
    "R2_conditional" %in%
      names(performance) &&
      !is.na(performance$R2_conditional)
  ) {
    r2_text <- text_full <- paste0(
      "The model's total explanatory power is ",
      effectsize::interpret_r2(performance$R2_conditional, ...),
      " (conditional R2 = ",
      insight::format_value(performance$R2_conditional),
      ")"
    )
  }

  # R2 marginal
  if ("R2_marginal" %in% names(performance)) {
    if (r2_text == "") {
      r2_text <- text_full <- "The model's explanatory power"
      of <- ""
    } else {
      r2_text <- paste0(r2_text, " and the part")
      text_full <- paste0(text_full, " and the part")
      of <- "of "
    }
    text_r2marginal <- paste0(
      " related to the fixed effects alone (marginal R2) is ",
      of,
      insight::format_value(performance$R2_marginal)
    )
    r2_text <- paste0(r2_text, text_r2marginal)

    if (is.null(attributes(performance)$r2_bayes$CI$R2_Bayes_marginal)) {
      text_full <- paste0(text_full, text_r2marginal)
    } else {
      r2 <- attributes(performance)$r2_bayes$CI$R2_Bayes_marginal
      text_full <- paste0(
        text_full,
        text_r2marginal,
        " (",
        insight::format_ci(
          r2$CI_low,
          r2$CI_high,
          r2$CI
        ),
        ")"
      )
    }
  }

  list(text_full = text_full, text = r2_text)
}


#' @keywords internal
.text_performance_lavaan <- function(perf_table, ...) {
  perf_table$Text <- paste0(
    perf_table$Name,
    " (",
    substring(insight::format_value(perf_table$Value), 2),
    ifelse(perf_table$Value > perf_table$Threshold, " > ", " < "),
    substring(insight::format_value(perf_table$Threshold), 2),
    ")"
  )

  # Satisfactory
  if (
    length(perf_table[perf_table$Interpretation == "satisfactory", "Text"]) >= 1
  ) {
    text_satisfactory <- paste0(
      "The ",
      perf_table[perf_table$Interpretation == "satisfactory", "Text"],
      ifelse(
        length(perf_table[
          perf_table$Interpretation == "satisfactory",
          "Text"
        ]) >
          1,
        " suggest",
        " suggests"
      ),
      " a satisfactory fit."
    )
  } else {
    text_satisfactory <- ""
  }

  # Poor
  if (length(perf_table[perf_table$Interpretation == "poor", "Text"]) >= 1) {
    text_poor <- paste0(
      "The ",
      perf_table[perf_table$Interpretation == "poor", "Text"],
      ifelse(
        length(perf_table[perf_table$Interpretation == "poor", "Text"]) > 1,
        " suggest",
        " suggests"
      ),
      " a poor fit."
    )
  } else {
    text_poor <- ""
  }

  fit_text <- datawizard::text_paste(text_satisfactory, text_poor, sep = " ")
  fit_text
}

test_that("report_performance Linear)", {
  set.seed(123)
  # Linear
  x1 <- lm(Sepal.Length ~ Petal.Length * Species, data = iris)
  expect_identical(
    as.character(report_performance(x1)),
    paste(
      "The model explains a statistically significant and substantial proportion of",
      "variance (R2 = 0.84, F(5, 144) = 151.71, p < .001, adj. R2 = 0.83)"
    )
  )
  expect_identical(
    as.character(summary(report_performance(x1))),
    "The model's explanatory power is substantial (R2 = 0.84, adj. R2 = 0.83)"
  )
})

test_that("report_performance GLM)", {
  set.seed(123)
  # GLM
  x2 <- glm(vs ~ disp, data = mtcars, family = "binomial")
  expect_identical(
    as.character(report_performance(x2)),
    "The model's explanatory power is substantial (Tjur's R2 = 0.53)"
  )
  expect_identical(
    as.character(summary(report_performance(x2))),
    "The model's explanatory power is substantial (Tjur's R2 = 0.53)"
  )
})

test_that("report_performance Mixed models)", {
  set.seed(123)
  # Mixed models
  skip_if_not_installed("lme4")

  x3 <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
  expect_identical(
    as.character(report_performance(x3)),
    paste(
      "The model's total explanatory power is substantial (conditional R2 = 0.97) and the",
      "part related to the fixed effects alone (marginal R2) is of 0.66"
    )
  )
  expect_identical(
    as.character(summary(report_performance(x3))),
    paste(
      "The model's total explanatory power is substantial (conditional R2 = 0.97) and the",
      "part related to the fixed effects alone (marginal R2) is of 0.66"
    )
  )

  x4 <- lme4::glmer(vs ~ mpg + (1 | cyl), data = mtcars, family = "binomial")
  expect_identical(
    as.character(report_performance(x4)),
    paste(
      "The model's total explanatory power is substantial (conditional R2 = 0.59) and the",
      "part related to the fixed effects alone (marginal R2) is of 0.13"
    )
  )
  expect_identical(
    as.character(summary(report_performance(x4))),
    paste(
      "The model's total explanatory power is substantial (conditional R2 = 0.59) and the",
      "part related to the fixed effects alone (marginal R2) is of 0.13"
    )
  )
})

test_that("report_performance Bayesian)", {
  set.seed(123)
  # Bayesian
  skip_if_not_installed("rstanarm")

  x5 <- rstanarm::stan_glm(
    Sepal.Length ~ Species,
    data = iris,
    refresh = 0,
    iter = 1000,
    seed = 333
  )
  expect_snapshot(
    variant = "windows",
    report_performance(x5)
  )
  expect_snapshot(
    variant = "windows",
    summary(report_performance(x5))
  )

  x6 <- rstanarm::stan_glm(
    vs ~ disp,
    data = mtcars,
    family = "binomial",
    refresh = 0,
    iter = 1000,
    seed = 333
  )
  expect_snapshot(
    variant = "windows",
    report_performance(x6)
  )
  expect_snapshot(
    variant = "windows",
    summary(report_performance(x6))
  )
})

test_that("report_performance Bayesian 2)", {
  set.seed(123)
  # Bayesian
  skip_if_not_installed("rstanarm")
  # Using namespace instead of loading the package throws an error:
  # could not find function "stan_glmer"
  # But we don't call "stan_glmer" directly, I suppose it must be called internally
  # So we must define it manually:

  is_stan_glmer_avail <- !inherits(try(stan_glmer, silent = TRUE), "try-error")
  if (!is_stan_glmer_avail) {
    stan_glmer <<- rstanarm::stan_glmer
    on.exit(remove(stan_glmer, envir = .GlobalEnv))
  }

  x7 <- rstanarm::stan_lmer(
    Sepal.Length ~ Petal.Length + (1 | Species),
    data = iris,
    refresh = 0,
    iter = 1000,
    seed = 333
  )
  expect_snapshot(
    variant = "windows",
    summary(report_performance(x7))
  )
  skip("Skipping because of a .01 decimal difference in snapshots")
  expect_snapshot(
    variant = "windows",
    report_performance(x7)
  )
})

# report_table -----------------

.report_table_wilcox <- function(table_full, effsize) {
  table_full <- cbind(table_full, attributes(effsize)$table)
  list(table = NULL, table_full = table_full)
}


# report_effectsize ---------------------

.report_effectsize_wilcox <- function(x, dot_args) {
  my_args <- c(list(x, es_type = "rank_biserial"), dot_args)
  wilcox_table <- do.call(parameters::model_parameters, my_args)
  ci <- attributes(wilcox_table)$ci
  estimate <- "r_rank_biserial"

  # same as Pearson's r
  my_args <- c(list(wilcox_table$r_rank_biserial), dot_args)
  interpretation <- do.call(effectsize::interpret_r, my_args)
  rules <- .text_effectsize(attr(attr(interpretation, "rules"), "rule_name"))

  main <- paste0(
    "r (rank biserial) = ",
    insight::format_value(wilcox_table$r_rank_biserial)
  )
  statistics <- paste0(
    main,
    ", ",
    insight::format_ci(
      wilcox_table$rank_biserial_CI_low,
      wilcox_table$rank_biserial_CI_high,
      ci
    )
  )

  result_table <- wilcox_table[c(
    "r_rank_biserial",
    "rank_biserial_CI_low",
    "rank_biserial_CI_high"
  )]

  list(
    table = result_table,
    statistics = statistics,
    interpretation = interpretation,
    rules = rules,
    ci = ci,
    main = main
  )
}


# report_model ----------------------------

.report_model_wilcox <- function(x, table) {
  # two-sample
  if ("Parameter1" %in% names(table)) {
    vars_full <- paste0(table$Parameter1[[1]], " and ", table$Parameter2[[1]])

    text_string <- paste0(
      trimws(x$method),
      " testing the difference in ranks between ",
      vars_full
    )
  } else {
    # one-sample
    vars_full <- paste0(table$Parameter[[1]])

    text_string <- paste0(
      trimws(x$method),
      " testing the difference in rank for ",
      vars_full,
      " and true location of 0"
    )
  }

  text_string
}

#' Reporting the participant data
#'
#' A helper function to help you format the participants data (age, sex, ...) in
#' the participants section.
#'
#' @param data A data frame.
#' @param age The name of the column containing the age of the participant.
#' @param sex The name of the column containing the sex of the participant. The
#'   classes should be one of `c("Male", "M", "Female", "F")`. Note that
#'   you can specify other characters here as well (e.g., `"Intersex"`), but
#'   the function will group all individuals in those groups as `"Other"`.
#' @param gender The name of the column containing the gender of the
#'   classes should be one of `c("Man", "M", "Male", Woman", "W", "F",
#'   "Female", Non-Binary", "N")`. Note that you can specify other characters
#'   here as well (e.g., `"Gender Fluid"`), but the function will group all
#'   individuals in those groups as `"Non-Binary"`.
#' @param education The name of the column containing education information.
#' @param country The name of the column containing country information.
#' @param race The name of the column containing race/ethnicity information.
#' @param threshold Percentage after which to combine, e.g., countries (default is 10%,
#' so countries that represent less than 10% will be combined in the "other" category).
#' @param participants The name of the participants' identifier column (for
#'   instance in the case of repeated measures).
#' @param by A character vector indicating the name(s) of the column(s) used
#'   for stratified description.
#' @param spell_n Logical, fully spell the sample size (`"Three participants"`
#'   instead of `"3 participants"`).
#' @inheritParams report.numeric
#' @param group Deprecated. Use `by` instead.
#'
#' @return A character vector with description of the "participants", based on
#'   the information provided in `data`.
#'
#' @examples
#' library(report)
#' data <- data.frame(
#'   "Age" = c(22, 23, 54, 21, 8, 42),
#'   "Sex" = c("Intersex", "F", "M", "M", "NA", NA),
#'   "Gender" = c("N", "W", "W", "M", "NA", NA)
#' )
#' report_participants(data, age = "Age", sex = "Sex")
#'
#' # Years of education (relative to high school graduation)
#' data$Education <- c(0, 8, -3, -5, 3, 5)
#' report_participants(data,
#'   age = "Age", sex = "Sex", gender = "Gender",
#'   education = "Education"
#' )
#'
#' # Education as factor
#' data$Education2 <- c(
#'   "Bachelor", "PhD", "Highschool",
#'   "Highschool", "Bachelor", "Bachelor"
#' )
#' report_participants(data, age = "Age", sex = "Sex", gender = "Gender", education = "Education2")
#'
#' # Country
#' data <- data.frame(
#'   "Age" = c(22, 23, 54, 21, 8, 42, 18, 32, 24, 27, 45),
#'   "Sex" = c("Intersex", "F", "F", "M", "M", "M", "F", "F", "F", "F", "F"),
#'   "Gender" = c("N", "W", "W", "M", "M", "M", "W", "W", "W", "W", "W"),
#'   "Country" = c(
#'     "USA", NA, "Canada", "Canada", "India", "Germany",
#'     "USA", "USA", "USA", "USA", "Canada"
#'   )
#' )
#' report_participants(data)
#'
#' # Country, control presentation treshold
#' report_participants(data, threshold = 5)
#'
#' # Race/ethnicity
#' data <- data.frame(
#'   "Age" = c(22, 23, 54, 21, 8, 42, 18, 32, 24, 27, 45),
#'   "Sex" = c("Intersex", "F", "F", "M", "M", "M", "F", "F", "F", "F", "F"),
#'   "Gender" = c("N", "W", "W", "M", "M", "M", "W", "W", "W", "W", "W"),
#'   "Race" = c(
#'     "Black", NA, "White", "Asian", "Black", "Arab", "Black",
#'     "White", "Asian", "Southeast Asian", "Mixed"
#'   )
#' )
#' report_participants(data)
#'
#' # Race/ethnicity, control presentation treshold
#' report_participants(data, threshold = 5)
#'
#' # Repeated measures data
#' data <- data.frame(
#'   "Age" = c(22, 22, 54, 54, 8, 8),
#'   "Sex" = c("I", "F", "M", "M", "F", "F"),
#'   "Gender" = c("N", "W", "W", "M", "M", "M"),
#'   "Participant" = c("S1", "S1", "s2", "s2", "s3", "s3")
#' )
#' report_participants(data, age = "Age", sex = "Sex", gender = "Gender", participants = "Participant")
#'
#' # Grouped data
#' data <- data.frame(
#'   "Age" = c(22, 22, 54, 54, 8, 8, 42, 42),
#'   "Sex" = c("I", "I", "M", "M", "F", "F", "F", "F"),
#'   "Gender" = c("N", "N", "W", "M", "M", "M", "Non-Binary", "Non-Binary"),
#'   "Participant" = c("S1", "S1", "s2", "s2", "s3", "s3", "s4", "s4"),
#'   "Condition" = c("A", "A", "A", "A", "B", "B", "B", "B")
#' )
#'
#' report_participants(data,
#'   age = "Age",
#'   sex = "Sex",
#'   gender = "Gender",
#'   participants = "Participant",
#'   by = "Condition"
#' )
#'
#' # Spell sample size
#' paste(
#'   report_participants(data, participants = "Participant", spell_n = TRUE),
#'   "were recruited in the study by means of torture and coercion."
#' )
#' @export
report_participants <- function(
  data,
  age = NULL,
  sex = NULL,
  gender = NULL,
  education = NULL,
  country = NULL,
  race = NULL,
  participants = NULL,
  by = NULL,
  spell_n = FALSE,
  digits = 1,
  threshold = 10,
  group = NULL,
  ...
) {
  ## TODO: deprecate later
  if (!is.null(group)) {
    insight::format_warning(
      "Argument `group` is deprecated and will be removed in a future release. Please use `by` instead."
    ) # nolint
    by <- group
  }

  # Convert empty strings to NA
  data_list <- lapply(data, function(x) {
    x[which(x == "")] <- NA # nolint
    x
  })
  data <- as.data.frame(data_list, stringsAsFactors = FALSE)

  # find age variable automatically
  if (is.null(age)) {
    age <- .find_age_in_data(data)
  }

  # find sex variable automatically
  if (is.null(sex)) {
    sex <- .find_sex_in_data(data)
  }

  # find gender variable automatically
  if (is.null(gender)) {
    gender <- .find_gender_in_data(data)
  }

  # find education variable automatically
  if (is.null(education)) {
    education <- .find_education_in_data(data)
  }

  # find country variable automatically
  if (is.null(country)) {
    country <- .find_country_in_data(data)
  }

  # find race variable automatically
  if (is.null(race)) {
    race <- .find_race_in_data(data)
  }

  if (is.null(by)) {
    final_text <- .report_participants(
      data,
      age = age,
      sex = sex,
      gender = gender,
      education = education,
      country = country,
      race = race,
      participants = participants,
      spell_n = spell_n,
      digits = digits,
      threshold = threshold,
      ...
    )
  } else {
    final_text <- NULL
    data[[by]] <- as.character(data[[by]])
    for (i in split(data, data[by])) {
      current_text <- .report_participants(
        i,
        age = age,
        sex = sex,
        gender = gender,
        education = education,
        country = country,
        race = race,
        participants = participants,
        spell_n = spell_n,
        digits = digits,
        threshold = threshold
      )

      pre_text <- paste0(
        "the '",
        paste0(
          names(i[by]),
          " - ",
          vapply(i[by], unique, "character"),
          collapse = " and "
        ),
        "' group: "
      )

      final_text <- c(final_text, paste0(pre_text, current_text))
    }
    final_text <- paste(
      "For",
      datawizard::text_concatenate(
        final_text,
        sep = ", for ",
        last = " and for "
      )
    )
  }
  final_text
}

#' @keywords internal
.check_df_names <- function(data, names) {
  data[names] <- lapply(names, function(x) {
    if (is.null(x) || !all(x %in% names(data))) {
      NA
    } else {
      data[[x]]
    }
  })
  data
}

#' @keywords internal
.replace_names <- function(data, x) {
  if (is.null(x) || !all(x %in% names(data))) {
    tools::toTitleCase(deparse(substitute(x)))
  } else {
    x
  }
}

#' @keywords internal
.report_participants <- function(
  data,
  age = "Age",
  sex = "Sex",
  gender = "Gender",
  education = "Education",
  country = "Country",
  race = "Race",
  participants = NULL,
  spell_n = FALSE,
  digits = 1,
  threshold = 10,
  ...
) {
  # Sanity checks
  demo_names <- c("Age", "Sex", "Gender", "Education", "Country", "Race")
  data <- .check_df_names(data, names = demo_names)

  age <- .replace_names(data, age)
  sex <- .replace_names(data, sex)
  gender <- .replace_names(data, gender)
  education <- .replace_names(data, education)
  country <- .replace_names(data, country)
  race <- .replace_names(data, race)

  # Set age as numeric
  data[[age]] <- as.numeric(data[[age]])

  # Grouped data
  if (!is.null(participants)) {
    data <- data.frame(
      Age = stats::aggregate(
        data[[age]],
        by = list(data[[participants]]),
        FUN = mean
      )[[2]],
      Sex = stats::aggregate(
        data[[sex]],
        by = list(data[[participants]]),
        FUN = utils::head,
        n = 1
      )[[2]],
      Gender = stats::aggregate(
        data[[gender]],
        by = list(data[[participants]]),
        FUN = utils::head,
        n = 1
      )[[2]],
      Education = stats::aggregate(
        data[[education]],
        by = list(data[[participants]]),
        FUN = utils::head,
        n = 1
      )[[2]],
      Country = stats::aggregate(
        data[[country]],
        by = list(data[[participants]]),
        FUN = utils::head,
        n = 1
      )[[2]],
      Race = stats::aggregate(
        data[[race]],
        by = list(data[[participants]]),
        FUN = utils::head,
        n = 1
      )[[2]],
      stringsAsFactors = FALSE
    )
    age <- "Age"
    sex <- "Sex"
    gender <- "Gender"
    education <- "Education"
    country <- "Country"
    race <- "Race"
  }

  if (spell_n) {
    size <- tools::toTitleCase(insight::format_number(nrow(data)))
  } else {
    size <- nrow(data)
  }

  # Create text
  if (all(is.na(data[[age]]))) {
    text_age <- ""
  } else {
    text_age <- summary(
      report_statistics(
        data[[age]],
        n = FALSE,
        centrality = "mean",
        missing_percentage = TRUE,
        digits = digits,
        ...
      )
    )
    text_age <- sub("Mean =", "Mean age =", text_age, fixed = TRUE)
  }

  text_sex <- if (all(is.na(data[[sex]]))) {
    ""
  } else {
    paste0(
      "Sex: ",
      insight::format_value(
        length(data[[sex]][
          tolower(
            data[[sex]]
          ) %in%
            c("female", "f")
        ]) /
          nrow(data) *
          100,
        digits = digits
      ),
      "% females, ",
      insight::format_value(
        length(data[[sex]][
          tolower(
            data[[sex]]
          ) %in%
            c("male", "m")
        ]) /
          nrow(data) *
          100,
        digits = digits
      ),
      "% males, ",
      insight::format_value(
        100 -
          length(data[[sex]][
            tolower(
              data[[sex]]
            ) %in%
              c("male", "m", "female", "f", NA, "na")
          ]) /
            nrow(data) *
            100,
        digits = digits
      ),
      "% other",
      if (
        insight::format_value(
          length(data[[sex]][tolower(data[[sex]]) %in% c(NA, "na")]) /
            nrow(data) *
            100
        ) !=
          "0.00"
      ) {
        # nolint
        paste0(
          ", ",
          insight::format_value(
            length(data[[sex]][
              tolower(
                data[[sex]]
              ) %in%
                c(NA, "na")
            ]) /
              nrow(data) *
              100
          ),
          "% missing"
        )
      }
    )
  }

  genders_woman <- c(
    "woman",
    "w",
    "female",
    "f",
    "women",
    "girl",
    "lady",
    "miss",
    "madam",
    "dame",
    "lass"
  )
  genders_man <- c(
    "man",
    "m",
    "male",
    "men",
    "boy",
    "guy",
    "dude",
    "lad",
    "sir"
  )
  both_genders <- c(genders_woman, genders_man, NA, "na")

  text_gender <- if (all(is.na(data[[gender]]))) {
    ""
  } else {
    paste0(
      "Gender: ",
      insight::format_value(
        length(data[[gender]][
          tolower(
            data[[gender]]
          ) %in%
            genders_woman
        ]) /
          nrow(data) *
          100,
        digits = digits
      ),
      "% women, ",
      insight::format_value(
        length(data[[gender]][
          tolower(
            data[[gender]]
          ) %in%
            genders_man
        ]) /
          nrow(data) *
          100,
        digits = digits
      ),
      "% men, ",
      insight::format_value(
        100 -
          length(data[[gender]][
            tolower(
              data[[gender]]
            ) %in%
              both_genders
          ]) /
            nrow(data) *
            100
      ),
      "% non-binary",
      if (
        insight::format_value(
          length(data[[gender]][
            tolower(
              data[[gender]]
            ) %in%
              c(NA, "na")
          ]) /
            nrow(data) *
            100
        ) !=
          "0.00"
      ) {
        paste0(
          ", ",
          insight::format_value(
            length(data[[gender]][
              tolower(
                data[[gender]]
              ) %in%
                c(NA, "na")
            ]) /
              nrow(data) *
              100
          ),
          "% missing"
        )
      }
    )
  }

  if (all(is.na(data[[education]]))) {
    text_education <- ""
  } else if (is.numeric(data[[education]])) {
    text_education <- summary(
      report_statistics(
        data[[education]],
        n = FALSE,
        centrality = "mean",
        missing_percentage = NULL,
        digits = digits,
        ...
      )
    )

    text_education <- sub(
      "Mean =",
      "Mean education =",
      text_education,
      fixed = TRUE
    )
  } else {
    data[which(data[[education]] %in% c(NA, "NA")), education] <- "missing"
    txt <- summary(report_statistics(
      as.factor(data[[education]]),
      levels_percentage = TRUE,
      digits = digits,
      ...
    ))

    text_education <- paste0("Education: ", txt)
  }

  text_country <- if (all(is.na(data[[country]]))) {
    ""
  } else {
    data[[country]] <- as.character(data[[country]])
    data[which(data[[country]] %in% c(NA, "NA")), country] <- "missing"
    freq_tbl <- table(data[[country]])
    frequency_table <- data.frame(
      Value = names(freq_tbl),
      Percent = as.numeric(freq_tbl) / nrow(data) * 100,
      stringsAsFactors = FALSE
    )
    frequency_table <- frequency_table[order(-frequency_table$Percent), ]
    upper <- frequency_table[which(frequency_table$Percent >= threshold), ]
    lower <- frequency_table[which(frequency_table$Percent < threshold), ]
    if (nrow(lower) > 0) {
      lower_sum <- data.frame(
        Value = "other",
        Percent = sum(lower$Percent),
        stringsAsFactors = FALSE
      )
      combined <- rbind(upper, lower_sum)
    } else {
      combined <- upper
    }
    combined$Percent <- insight::format_value(combined$Percent)
    value_string <- paste0(
      combined$Percent,
      "% ",
      combined$Value,
      collapse = ", "
    )
    text_country <- paste("Country:", value_string)
  }

  text_race <- if (all(is.na(data[[race]]))) {
    ""
  } else {
    data[[race]] <- as.character(data[[race]])
    data[which(data[[race]] %in% c(NA, "NA")), race] <- "missing"
    freq_tbl <- table(data[[race]])
    frequency_table <- data.frame(
      Value = names(freq_tbl),
      Percent = as.numeric(freq_tbl) / nrow(data) * 100,
      stringsAsFactors = FALSE
    )
    frequency_table <- frequency_table[order(-frequency_table$Percent), ]
    upper <- frequency_table[which(frequency_table$Percent >= threshold), ]
    lower <- frequency_table[which(frequency_table$Percent < threshold), ]
    if (nrow(lower) > 0) {
      lower_sum <- data.frame(
        Value = "other",
        Percent = sum(lower$Percent),
        stringsAsFactors = FALSE
      )
      combined <- rbind(upper, lower_sum)
    } else {
      combined <- upper
    }
    combined$Percent <- insight::format_value(combined$Percent)
    value_string <- paste0(
      combined$Percent,
      "% ",
      combined$Value,
      collapse = ", "
    )
    text_race <- paste("Race:", value_string)
  }

  # nolint start
  paste0(
    size,
    " participants (",
    ifelse(text_age == "", "", text_age),
    ifelse(
      text_sex == "",
      "",
      paste0(
        ifelse(
          text_age == "",
          "",
          "; "
        ),
        text_sex
      )
    ),
    ifelse(
      text_gender == "",
      "",
      paste0(
        ifelse(
          text_age == "" & text_sex == "",
          "",
          "; "
        ),
        text_gender
      )
    ),
    ifelse(
      text_education == "",
      "",
      paste0(
        ifelse(
          text_age == "" & text_sex == "" & text_gender == "",
          "",
          "; "
        ),
        text_education
      )
    ),
    ifelse(
      text_country == "",
      "",
      paste0(
        ifelse(
          text_education == "" &
            text_age == "" &
            text_sex == "" &
            text_gender == "",
          "",
          "; "
        ),
        text_country
      )
    ),
    ifelse(
      text_race == "",
      "",
      paste0(
        ifelse(
          text_country == "" &
            text_education == "" &
            text_age == "" &
            text_sex == "" &
            text_gender == "",
          "",
          "; "
        ),
        text_race
      )
    ),
    ")"
  )
  # nolint end
}

#' @keywords internal
.find_age_in_data <- function(data) {
  if ("Age" %in% colnames(data)) {
    "Age"
  } else if ("age" %in% colnames(data)) {
    "age"
  } else if (any(startsWith("Age", colnames(data)))) {
    grep("^Age", colnames(data), value = TRUE)[1]
  } else if (any(startsWith("age", colnames(data)))) {
    grep("^age", colnames(data), value = TRUE)[1]
  } else {
    ""
  }
}

#' @keywords internal
.find_sex_in_data <- function(data) {
  if ("Sex" %in% colnames(data)) {
    "Sex"
  } else if ("sex" %in% colnames(data)) {
    "sex"
  } else if (any(startsWith("Sex", colnames(data)))) {
    grep("^Sex", colnames(data), value = TRUE)[1]
  } else if (any(startsWith("sex", colnames(data)))) {
    grep("^sex", colnames(data), value = TRUE)[1]
  } else {
    ""
  }
}

#' @keywords internal
.find_gender_in_data <- function(data) {
  if ("Gender" %in% colnames(data)) {
    "Gender"
  } else if ("gender" %in% colnames(data)) {
    "gender"
  } else if (any(startsWith("Gender", colnames(data)))) {
    grep("^Gender", colnames(data), value = TRUE)[1]
  } else if (any(startsWith("gender", colnames(data)))) {
    grep("^gender", colnames(data), value = TRUE)[1]
  } else {
    ""
  }
}

#' @keywords internal
.find_education_in_data <- function(data) {
  if ("Education" %in% colnames(data)) {
    "Education"
  } else if ("education" %in% colnames(data)) {
    "education"
  } else if (any(startsWith("Education", colnames(data)))) {
    grep("^Education", colnames(data), value = TRUE)[1]
  } else if (any(startsWith("education", colnames(data)))) {
    grep("^education", colnames(data), value = TRUE)[1]
  } else if ("isced" %in% colnames(data)) {
    "isced"
  } else if (any(startsWith("isced", colnames(data)))) {
    grep("^isced", colnames(data), value = TRUE)[1]
  } else {
    ""
  }
}

#' @keywords internal
.find_country_in_data <- function(data) {
  if ("Country" %in% colnames(data)) {
    "Country"
  } else if ("country" %in% colnames(data)) {
    "country"
  } else if (any(startsWith("Country", colnames(data)))) {
    grep("^Country", colnames(data), value = TRUE)[1]
  } else if (any(startsWith("country", colnames(data)))) {
    grep("^country", colnames(data), value = TRUE)[1]
  } else {
    ""
  }
}

#' @keywords internal
.find_race_in_data <- function(data) {
  if ("Race" %in% colnames(data)) {
    "Race"
  } else if ("race" %in% colnames(data)) {
    "race"
  } else if (any(startsWith("Race", colnames(data)))) {
    grep("^Race", colnames(data), value = TRUE)[1]
  } else if (any(startsWith("race", colnames(data)))) {
    grep("^race", colnames(data), value = TRUE)[1]
  } else {
    ""
  }
}

test_that("report_participants, argument gender works", {
  # Works when capitalized
  data <- data.frame(
    Age = c(22, 22, 54, 54, 8, 8, 42, 42),
    Gender = c("N", "N", "W", "M", "M", "M", "Non-Binary", "Non-Binary"),
    Condition = c("A", "A", "A", "A", "B", "B", "B", "B"),
    stringsAsFactors = FALSE
  )
  out <- report_participants(data)
  expect_identical(
    out,
    paste(
      "8 participants (Mean age = 31.5, SD = 19.0, range: [8, 54];",
      "Gender: 12.5% women, 37.5% men, 50.00% non-binary)"
    )
  )
  out <- report_participants(data, by = "Condition")
  expect_identical(
    out,
    paste(
      "For the 'Condition - A' group: 4 participants (Mean age = 38.0,",
      "SD = 18.5, range: [22, 54]; Gender: 25.0% women, 25.0% men,",
      "50.00% non-binary) and for the 'Condition - B' group: 4 participants",
      "(Mean age = 25.0, SD = 19.6, range: [8, 42]; Gender: 0.0% women,",
      "50.0% men, 50.00% non-binary)"
    )
  )
  # works when lowercase
  out <- report_participants(data, by = "Condition")
  expect_identical(
    out,
    paste(
      "For the 'Condition - A' group: 4 participants (Mean age = 38.0,",
      "SD = 18.5, range: [22, 54]; Gender: 25.0% women, 25.0% men,",
      "50.00% non-binary) and for the 'Condition - B' group: 4 participants",
      "(Mean age = 25.0, SD = 19.6, range: [8, 42]; Gender: 0.0% women,",
      "50.0% men, 50.00% non-binary)"
    )
  )
})

test_that("report_participants", {
  data <- data.frame(
    Age = c(22, 22, 54, 54, 8, 8),
    Sex = c("F", "F", "M", "M", "I", "I"),
    Gender = c("W", "W", "M", "M", "N", "N"),
    Participant = c("S1", "S1", "s2", "s2", "s3", "s3"),
    stringsAsFactors = FALSE
  )

  expect_snapshot(
    variant = "windows",
    report_participants(
      data,
      age = "Age",
      sex = "Sex",
      participant = "Participant"
    )
  )
  expect_snapshot(
    variant = "windows",
    report_participants(data, participant = "Participant", spell_n = TRUE)
  )

  expect_equal(
    nchar(report_participants(
      data,
      participant = "Participant",
      spell_n = TRUE
    )),
    160,
    ignore_attr = TRUE
  )

  data2 <- data.frame(
    Age = c(22, 22, 54, 54, 8, 8),
    Sex = c("F", "F", "M", "O", "F", "O"),
    Gender = c("W", "W", "M", "O", "W", "O"),
    stringsAsFactors = FALSE
  )

  expect_snapshot(
    variant = "windows",
    report_participants(data2)
  )

  data3 <- data.frame(
    Age = c(22, 82, NA, NA, NA, NA),
    Sex = c("F", "F", "", "", "NA", NA),
    Gender = c("W", "W", "", "", "NA", NA),
    stringsAsFactors = FALSE
  )

  expect_snapshot(
    variant = "windows",
    report_participants(data3)
  )

  # Add tests for education, country, race
  data4 <- data.frame(
    Education = c(0, 8, -3, -5, 3, 5, NA),
    Education2 = c(
      "Bachelor",
      "PhD",
      "Highschool",
      "Highschool",
      "Bachelor",
      "Bachelor",
      NA
    ),
    Country = c("USA", "Canada", "Canada", "India", "Germany", "USA", NA),
    Race = c("Black", NA, "White", "Asian", "Black", "Black", "White"),
    stringsAsFactors = FALSE
  )

  expect_snapshot(
    variant = "windows",
    report_participants(data4)
  )
  expect_snapshot(
    variant = "windows",
    report_participants(data4, education = "Education2")
  )
  expect_snapshot(
    variant = "windows",
    report_participants(data4, threshold = 15)
  )
})

test_that("report_participants test NAs no warning", {
  data <- data.frame(
    Age = c(22, 23, 54, 21, 8, 42),
    Sex = (c("Intersex", "F", "M", "M", "NA", NA)),
    Gender = (c("N", "W", "W", "M", "NA", NA)),
    Country = (c("USA", NA, "Canada", "Canada", "India", "Germany")),
    Education = factor(c(0, 8, -3, -5, 3, 5)),
    Race = c(LETTERS[1:5], NA)
  )
  expect_snapshot(
    variant = "windows",
    report_participants(data)
  )

  data <- data.frame(
    Age = factor(c(22, 23, 54, 21, 8, 42)),
    Sex = factor(c("Intersex", "F", "M", "M", "NA", NA)),
    Gender = factor(c("N", "W", "W", "M", "NA", NA)),
    Country = factor(c("USA", NA, "Canada", "Canada", "India", "Germany")),
    Education = factor(c(0, 8, -3, -5, 3, 5)),
    Race = factor(c(LETTERS[1:5], NA))
  )
  expect_snapshot(
    variant = "windows",
    report_participants(data, age = "Age", sex = "Sex")
  )
})

test_that("report_participants age as character", {
  data <- data.frame(
    Age = as.character(c(22, 22, 28, 11, 42, 52), stringsAsFactors = FALSE),
    stringsAsFactors = FALSE
  )
  expect_snapshot(
    variant = "windows",
    report_participants(data)
  )
})

test_that("report_participants different gender spellings", {
  data <- data.frame(
    Gender = c(
      "Man",
      "M",
      "Male",
      "Men",
      "Boy",
      "Guy",
      "Dude",
      "Lad",
      "Sir",
      "Woman",
      "W",
      "Female",
      "Women",
      "Girl",
      "Lady",
      "Miss",
      "Madam",
      "Dame",
      "Lass",
      NA
    ),
    stringsAsFactors = FALSE
  )
  expect_snapshot(
    variant = "windows",
    report_participants(data)
  )
})

#' Miscellaneous reports
#'
#' Other convenient or totally useless reports.
#'
#' @inheritParams report
#'
#' @inherit report return seealso
#'
#' @examples
#' library(report)
#'
#' report_date()
#' summary(report_date())
#' report_story()
#' @return Objects of class [report_text()].
#' @export
report_date <- function(...) {
  current_date <- Sys.time()
  date_text <- format(
    current_date,
    "It's %A, %B %d of the year %Y, at %l%P %M and %S seconds"
  )
  text_short <- format(current_date, "%d/%m/%y - %H:%M:%S")
  as.report_text(date_text, summary = text_short)
}


#' @rdname report_date
#' @export
report_story <- function(...) {
  story_text <-
    paste(
      "Did you ever hear the tragedy of Darth Plagueis The Wise? I thought not.",
      "It's not a story the Jedi would tell you. It's a Sith legend.",
      "Darth Plagueis was a Dark Lord of the Sith, so powerful and so wise he could use the Force",
      "to influence the midichlorians to create life...",
      "He had such a knowledge of the dark side that he could even keep the ones he cared about from dying.",
      "The dark side of the Force is a pathway to many abilities some consider to be unnatural.",
      "He became so powerful...",
      "the only thing he was afraid of was losing his power, which eventually, of course, he did.",
      "Unfortunately, he taught his apprentice everything he knew, then his apprentice killed him in his sleep.",
      "Ironic. He could save others from death, but not himself."
    )

  text_short <- "So this is how liberty dies. With thunderous applause."

  as.report_text(story_text, summary = text_short)
}

#' Reporting Bayesian Models from brms
#'
#' Create reports for Bayesian models. The description of the parameters
#' follows the Sequential Effect eXistence and sIgnificance Testing framework
#' (see [SEXIT documentation][bayestestR::sexit]).
#'
#' @details
#' Message from the `rstan` package: "To avoid recompilation of unchanged
#' Stan programs, we recommend calling `rstan_options(auto_write = TRUE)`"
#'
#' @param effectsize_method Method for computing effect sizes. For `brmsfit` objects,
#'   defaults to `"basic"` (faster, no refitting) instead of `"refit"` to improve
#'   performance with large Bayesian models. See documentation for
#'   [effectsize::effectsize()].
#' @inheritParams report.lm
#' @inherit report return seealso
#'
#' @examplesIf require("brms", quietly = TRUE)
#' \donttest{
#' # Bayesian models
#' library(brms)
#' model <- suppressWarnings(brm(mpg ~ qsec + wt,
#'   data = mtcars,
#'   refresh = 0, iter = 300
#' ))
#' r <- report(model, verbose = FALSE)
#' r
#' summary(r)
#' as.data.frame(r)
#' summary(as.data.frame(r))
#' }
#' @return An object of class [report()].
#' @include report.lm.R report.stanreg.R report.lme4.R
#' @export
report.brmsfit <- function(x, ...) {
  tbl <- report_table(x, include_effectsize = FALSE, ...)
  txt <- report_text(x, table = tbl, ...)

  as.report(txt, table = tbl, ...)
}

#' @rdname report.brmsfit
#' @export
report_effectsize.brmsfit <- function(x, effectsize_method = "basic", ...) {
  # Use faster method for Bayesian models to avoid expensive refitting
  # "basic" method is much faster than "refit" while providing similar results
  effect_table <- suppressWarnings(effectsize::effectsize(
    x,
    method = effectsize_method,
    ...
  ))
  method <- .text_standardize(effect_table)
  estimate <- names(effect_table)[effectsize::is_effectsize_name(names(
    effect_table
  ))]

  # Interpret effect sizes based on model type
  if (insight::model_info(x)$is_logit) {
    interpret <- effectsize::interpret_oddsratio(
      effect_table[[estimate]],
      log = TRUE,
      ...
    )
  } else {
    interpret <- effectsize::interpret_cohens_d(effect_table[[estimate]], ...)
  }

  interpretation <- interpret
  main <- paste0(
    "Std. beta = ",
    insight::format_value(effect_table[[estimate]])
  )

  ci <- effect_table$CI
  names(ci) <- paste0("ci_", estimate)

  statistics <- paste0(
    main,
    ", ",
    insight::format_ci(effect_table$CI_low, effect_table$CI_high, ci)
  )

  if ("Component" %in% colnames(effect_table)) {
    merge_by <- c("Parameter", "Component")
    start_col <- 4L
  } else {
    merge_by <- "Parameter"
    start_col <- 3L
  }

  effect_table <- as.data.frame(effect_table)[c(
    merge_by,
    estimate,
    "CI_low",
    "CI_high"
  )]
  names(effect_table)[start_col:ncol(effect_table)] <- c(
    paste0(estimate, "_CI_low"),
    paste0(estimate, "_CI_high")
  )

  rules <- .text_effectsize(attr(attr(interpret, "rules"), "rule_name"))
  parameters <- paste0(interpretation, " (", statistics, ")")

  as.report_effectsize(
    parameters,
    summary = parameters,
    table = effect_table,
    interpretation = interpretation,
    statistics = statistics,
    rules = rules,
    ci = ci,
    method = method,
    main = main
  )
}

#' @export
report_table.brmsfit <- report_table.lm

#' @export
report_performance.brmsfit <- report_performance.lm

#' @export
report_statistics.brmsfit <- report_statistics.lm

#' @export
report_random.brmsfit <- report_random.merMod

#' @export
report_model.brmsfit <- report_model.lm

#' @export
report_text.brmsfit <- report_text.lm


# ==================== Specific to Bayes ===================================

# report_priors -----------------------------------------------------------

#' @export
report_priors.brmsfit <- function(x, ...) {
  params <- bayestestR::describe_prior(x)

  # Return empty if no priors info
  has_no_prior_information <- (!"Prior_Distribution" %in% names(params)) ||
    nrow(params) == 0L ||
    all(is.na(params$Prior_Scale))

  if (has_no_prior_information) {
    return("")
  }

  # Filter out priors with missing/empty information (both location and
  # scale are NA). This removes uninformative default priors that shouldn't
  # be reported
  valid_priors <- !is.na(params$Prior_Location) |
    !is.na(params$Prior_Scale)
  params <- params[valid_priors, ]

  # Return empty if no valid priors remain after filtering
  if (nrow(params) == 0L) {
    return("")
  }

  # Create enhanced prior descriptions with parameter information
  prior_descriptions <- vector("character", length = 0L)

  # Group parameters by type for cleaner reporting
  intercept_params <- params[params$Parameter == "(Intercept)", ]
  slope_params <- params[
    params$Parameter != "(Intercept)" &
      !grepl("^(sigma|sd_|cor_)", params$Parameter),
  ]
  scale_params <- params[grepl("^(sigma|sd_)", params$Parameter), ]

  # Helper function to format individual priors with mathematical notation
  format_prior <- function(prior_row) {
    prior_dist <- prior_row$Prior_Distribution
    prior_loc <- insight::format_value(prior_row$Prior_Location)
    prior_scale <- insight::format_value(prior_row$Prior_Scale)
    prior_df <- if (
      !is.null(prior_row$Prior_df) && !is.na(prior_row$Prior_df)
    ) {
      paste0("df = ", insight::format_value(prior_row$Prior_df), ", ")
    } else {
      ""
    }

    if (prior_dist == "normal") {
      paste0(
        "Normal(",
        prior_df,
        "\u03bc = ",
        prior_loc,
        ", \u03c3 = ",
        prior_scale,
        ")"
      )
    } else if (prior_dist == "student_t") {
      paste0(
        "Student-t(",
        prior_df,
        "\u03bc = ",
        prior_loc,
        ", \u03c3 = ",
        prior_scale,
        ")"
      )
    } else {
      # Fallback for other distributions
      paste0(
        tools::toTitleCase(prior_dist),
        "(",
        prior_df,
        "location = ",
        prior_loc,
        ", scale = ",
        prior_scale,
        ")"
      )
    }
  }

  # Process intercept parameters
  if (nrow(intercept_params) > 0) {
    intercept_desc <- sapply(seq_len(nrow(intercept_params)), function(i) {
      format_prior(intercept_params[i, ])
    })
    if (length(unique(intercept_desc)) == 1L) {
      prior_descriptions <- c(
        prior_descriptions,
        paste0("Intercept ~ ", intercept_desc[1])
      )
    } else {
      prior_descriptions <- c(
        prior_descriptions,
        paste0("Intercepts ~ ", datawizard::text_concatenate(intercept_desc))
      )
    }
  }

  # Process slope parameters
  if (nrow(slope_params) > 0) {
    slope_names <- slope_params$Parameter
    slope_desc <- sapply(seq_len(nrow(slope_params)), function(i) {
      format_prior(slope_params[i, ])
    })

    if (length(unique(slope_desc)) == 1L) {
      # All slopes have the same prior
      param_list <- if (length(slope_names) > 1) {
        paste0("(", datawizard::text_concatenate(slope_names), ")")
      } else {
        paste0("(", slope_names, ")")
      }
      prior_descriptions <- c(
        prior_descriptions,
        paste0("Slopes ", param_list, " ~ ", slope_desc[1])
      )
    } else {
      # Different priors for different slopes
      individual_slopes <- paste0(slope_names, " ~ ", slope_desc)
      prior_descriptions <- c(
        prior_descriptions,
        datawizard::text_concatenate(individual_slopes)
      )
    }
  }

  # Process scale/sigma parameters
  if (nrow(scale_params) > 0) {
    scale_desc <- sapply(seq_len(nrow(scale_params)), function(i) {
      prior_row <- scale_params[i, ]
      desc <- format_prior(prior_row)
      # Add + notation for positive-only distributions when appropriate
      if (
        grepl("sigma|sd", prior_row$Parameter) && prior_row$Prior_Location >= 0
      ) {
        desc <- gsub("Student-t(", "Student-t\u207a(", desc, fixed = TRUE)
        desc <- gsub("Normal(", "Normal\u207a(", desc, fixed = TRUE)
      }
      desc
    })

    if (length(unique(scale_desc)) == 1L && nrow(scale_params) > 1L) {
      prior_descriptions <- c(
        prior_descriptions,
        paste0("Residual SD (\u03c3) ~ ", scale_desc[1])
      )
    } else {
      scale_names <- gsub(
        "sigma",
        "\u03c3",
        scale_params$Parameter,
        fixed = TRUE
      )
      individual_scales <- paste0(scale_names, " ~ ", scale_desc)
      prior_descriptions <- c(
        prior_descriptions,
        datawizard::text_concatenate(individual_scales)
      )
    }
  }

  # Combine all descriptions
  if (length(prior_descriptions) > 0) {
    report_text <- paste0(
      "Priors were: ",
      datawizard::text_concatenate(
        prior_descriptions,
        sep = "; ",
        last = "; "
      )
    )
  } else {
    report_text <- ""
  }

  as.report_priors(report_text)
}


# report_parameters -------------------------------------------------------

#' @export
report_parameters.brmsfit <- report_parameters.stanreg

# report_intercept --------------------------------------------------------

#' @export
report_intercept.brmsfit <- report_intercept.stanreg

# report_info -------------------------------------------------------------

#' @export
report_info.brmsfit <- report_info.stanreg

#' Reporting (General) Linear Models
#'
#' Create reports for (general) linear models.
#'
#' @param x Object of class `lm` or `glm`.
#' @param include_effectsize If `FALSE`, won't include effect-size related
#'   indices (standardized coefficients, etc.).
#' @param include_diagnostic If `FALSE`, won't include diagnostic related
#'   indices for Bayesian models (ESS, Rhat).
#' @param include_intercept If `FALSE`, won't include the intercept.
#' @param effectsize_method See documentation for
#'   [effectsize::effectsize()].
#' @param parameters Provide the output of `report_parameters()` to avoid
#'   its re-computation.
#' @inheritParams report
#' @inheritParams report.htest
#'
#' @inherit report return seealso
#'
#' @examples
#' \donttest{
#' library(report)
#'
#' # Linear models
#' model <- lm(Sepal.Length ~ Petal.Length * Species, data = iris)
#' r <- report(model)
#' r
#' summary(r)
#' as.data.frame(r)
#' summary(as.data.frame(r))
#'
#' # Logistic models
#' model <- glm(vs ~ disp, data = mtcars, family = "binomial")
#' r <- report(model)
#' r
#' summary(r)
#' as.data.frame(r)
#' summary(as.data.frame(r))
#' }
#'
#' @examplesIf requireNamespace("lme4", quietly = TRUE)
#' \donttest{
#' # Mixed models
#' library(lme4)
#' model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
#' r <- report(model)
#' r
#' summary(r)
#' as.data.frame(r)
#' summary(as.data.frame(r))
#' }
#' @return An object of class [report()].
#' @export
report.lm <- function(
  x,
  include_effectsize = TRUE,
  effectsize_method = "refit",
  ...
) {
  result_table <- report_table(
    x,
    include_effectsize = include_effectsize,
    effectsize_method = effectsize_method,
    ...
  )
  result_text <- report_text(x, table = result_table, ...)

  as.report(result_text, table = result_table, ...)
}


# report_effectsize -------------------------------------------------------

#' @rdname report.lm
#' @export

report_effectsize.lm <- function(x, effectsize_method = "refit", ...) {
  effect_table <- suppressWarnings(effectsize::effectsize(
    x,
    method = effectsize_method,
    ...
  ))
  method <- .text_standardize(effect_table)
  estimate <- names(effect_table)[effectsize::is_effectsize_name(names(
    effect_table
  ))]

  # TODO: finally solve this.
  # interpret <- effectsize::interpret_parameters(x, ...)
  if (insight::model_info(x)$is_logit) {
    interpret <- effectsize::interpret_oddsratio(
      effect_table[[estimate]],
      log = TRUE,
      ...
    )
  } else {
    interpret <- effectsize::interpret_cohens_d(effect_table[[estimate]], ...)
  }

  interpretation <- interpret
  main <- paste0(
    "Std. beta = ",
    insight::format_value(effect_table[[estimate]])
  )

  ci <- effect_table$CI
  names(ci) <- paste0("ci_", estimate)

  statistics <- paste0(
    main,
    ", ",
    insight::format_ci(effect_table$CI_low, effect_table$CI_high, ci)
  )

  if ("Component" %in% colnames(effect_table)) {
    merge_by <- c("Parameter", "Component")
    start_col <- 4L
  } else {
    merge_by <- "Parameter"
    start_col <- 3L
  }

  effect_table <- as.data.frame(effect_table)[c(
    merge_by,
    estimate,
    "CI_low",
    "CI_high"
  )]
  names(effect_table)[start_col:ncol(effect_table)] <- c(
    paste0(estimate, "_CI_low"),
    paste0(estimate, "_CI_high")
  )

  rules <- .text_effectsize(attr(attr(interpret, "rules"), "rule_name"))
  parameters <- paste0(interpretation, " (", statistics, ")")

  as.report_effectsize(
    parameters,
    summary = parameters,
    table = effect_table,
    interpretation = interpretation,
    statistics = statistics,
    rules = rules,
    ci = ci,
    method = method,
    main = main
  )
}


# report_table ------------------------------------------------------------

#' @rdname report.lm
#' @include utils_combine_tables.R
#' @export
report_table.lm <- function(x, include_effectsize = TRUE, ...) {
  params <- parameters::model_parameters(x, ci_random = FALSE, ...)

  # Combine -----
  # Add effectsize
  if (include_effectsize) {
    effsize <- report_effectsize(x, ...)
    params <- .combine_tables_effectsize(params, effsize)
  } else {
    effsize <- NULL
  }

  # Add performance
  performance <- performance::model_performance(x, ...)
  params <- .combine_tables_performance(params, performance)
  params <- params[
    !tolower(params$Parameter) %in%
      c(
        "rmse", # lm
        "logloss",
        "score_log",
        "score_spherical",
        "pcp", # glm
        "icc", # lmer
        "elpd_se",
        "looic_se" # stanreg
      ),
  ]

  # Clean -----
  # Rename some columns
  # names(table_full) <- gsub("Coefficient", "beta", names(table_full))

  # Shorten ----
  if (insight::model_info(x)$is_logit) {
    params <- datawizard::data_remove(params, "df_error")
  }
  table_full <- datawizard::data_remove(params, "SE")
  params_table <- datawizard::data_remove(
    table_full,
    select = "(_CI_low|_CI_high)$",
    regex = TRUE,
    verbose = FALSE
  )
  params_table <- params_table[
    !params_table$Parameter %in% c("AIC", "BIC", "ELPD", "LOOIC", "WAIC"),
  ]

  # Prepare -----
  out <- as.report_table(
    table_full,
    summary = params_table,
    effsize = effsize,
    performance = performance,
    ...
  )
  if (!is.null(effsize)) {
    new_attrs <- attributes(effsize)$ci
    for (idx in seq_along(new_attrs)) {
      if (!is.na(names(new_attrs)[idx])) {
        attr(out, names(new_attrs)[idx]) <- new_attrs[[idx]]
      }
    }
  }
  # Add attributes from params table
  for (att in c(
    "ci",
    "coefficient_name",
    "pretty_names",
    "bootstrap",
    "iterations",
    "ci_method"
  )) {
    attr(out, att) <- attributes(params)[[att]]
  }

  out
}


# report_statistics ------------------------------------------------------------

#' @rdname report.lm
#' @export
report_statistics.lm <- function(
  x,
  table = NULL,
  include_effectsize = TRUE,
  include_diagnostic = TRUE,
  ...
) {
  if (is.null(table)) {
    params_table <- report_table(x, ...)
  } else {
    params_table <- table
  }
  params_table <- .remove_performance(params_table)
  effsize <- attributes(params_table)$effsize

  # For glmmTMB models with multiple components, don't filter the table itself
  # but prepare effectsize alignment for text generation

  # Estimate
  estimate <- .find_regression_estimate(params_table)
  if (
    is.null(estimate) || is.na(estimate) || !estimate %in% names(params_table)
  ) {
    estimate_text <- ""
  } else if (estimate == "Coefficient") {
    estimate_text <- paste0(
      "beta = ",
      insight::format_value(params_table[[estimate]])
    )
  } else {
    estimate_text <- paste0(
      estimate,
      " = ",
      insight::format_value(params_table[[estimate]])
    )
  }

  # CI
  if (!is.null(params_table$CI_low)) {
    estimate_text <- datawizard::text_paste(
      estimate_text,
      insight::format_ci(
        params_table$CI_low,
        params_table$CI_high,
        ci = attributes(params_table)$ci
      )
    )
  }

  # Statistic
  if ("t" %in% names(params_table)) {
    estimate_text <- datawizard::text_paste(
      estimate_text,
      paste0(
        "t(",
        insight::format_value(
          params_table$df,
          protect_integers = TRUE
        ),
        ") = ",
        insight::format_value(params_table$t)
      )
    )
  }

  # p-value
  if ("p" %in% names(params_table)) {
    estimate_text <- datawizard::text_paste(
      estimate_text,
      insight::format_p(params_table$p, stars = FALSE, digits = "apa")
    )
  }

  # pd
  if ("pd" %in% names(params_table)) {
    estimate_text <- datawizard::text_paste(
      estimate_text,
      insight::format_pd(params_table$pd, stars = FALSE)
    )
  }

  # BF
  if ("ROPE_Percentage" %in% names(params_table)) {
    estimate_text <- datawizard::text_paste(
      estimate_text,
      insight::format_rope(params_table$ROPE_Percentage)
    )
  }

  # BF
  if ("BF" %in% names(params_table)) {
    estimate_text <- datawizard::text_paste(
      estimate_text,
      insight::format_bf(params_table$BF, stars = FALSE, exact = TRUE)
    )
  }

  # Effect size
  if (include_effectsize && !is.null(effsize)) {
    # For glmmTMB models with components, align effectsize with the table parameters
    if (inherits(x, "glmmTMB") && "Component" %in% colnames(params_table)) {
      # Find matching indices between table and effectsize
      effsize_table <- attributes(effsize)$table

      # Create matching vectors for both table and effectsize
      table_keys <- paste(params_table$Parameter, params_table$Component)
      effsize_keys <- paste(effsize_table$Parameter, effsize_table$Component)

      # Find which effectsize entries match table entries
      match_idx <- match(table_keys, effsize_keys)

      # Use only the matched effectsize elements, fill missing with empty strings
      n_params <- nrow(params_table)
      effsize_stats <- character(n_params)
      effsize_main <- character(n_params)

      for (i in seq_len(n_params)) {
        if (is.na(match_idx[i])) {
          effsize_stats[i] <- ""
          effsize_main[i] <- ""
        } else {
          effsize_stats[i] <- attributes(effsize)$statistics[match_idx[i]]
          effsize_main[i] <- attributes(effsize)$main[match_idx[i]]
        }
      }
    } else {
      effsize_stats <- attributes(effsize)$statistics
      effsize_main <- attributes(effsize)$main
    }

    # Only include non-empty effectsize information
    has_effectsize <- !is.na(effsize_stats) & effsize_stats != ""
    text_full <- ifelse(
      has_effectsize,
      datawizard::text_paste(estimate_text, effsize_stats, sep = "; "),
      estimate_text
    )
    estimate_text <- ifelse(
      has_effectsize,
      datawizard::text_paste(estimate_text, effsize_main),
      estimate_text
    )
  } else {
    text_full <- estimate_text
  }

  # Quality / Diagnostic
  if (include_diagnostic) {
    text_diagnostic <- ""

    if ("Rhat" %in% names(params_table)) {
      text_diagnostic <- paste0(
        "Rhat = ",
        insight::format_value(params_table$Rhat)
      )
    }

    estimate_text <- datawizard::text_paste(estimate_text, text_diagnostic)

    if ("ESS" %in% names(params_table)) {
      text_diagnostic <- datawizard::text_paste(
        text_diagnostic,
        paste0("ESS = ", insight::format_value(params_table$ESS))
      )
    }

    text_full <- datawizard::text_paste(text_full, text_diagnostic, sep = "; ")
  }

  as.report_statistics(
    text_full,
    summary = estimate_text,
    table = params_table,
    effsize = effsize
  )
}


# report_statistics ------------------------------------------------------------

#' @rdname report.lm
#' @inheritParams report_statistics
#' @export
report_parameters.lm <- function(
  x,
  table = NULL,
  include_effectsize = TRUE,
  include_intercept = TRUE,
  ...
) {
  # Get data
  stats <- report_statistics(
    x,
    table = table,
    include_effectsize = include_effectsize,
    ...
  )
  params <- attributes(stats)$table
  effsize <- attributes(stats)$effsize

  # For glmmTMB models, deduplicate parameters table to prevent repeated output
  # This fixes the issue where same parameter appears multiple times in the report
  if (inherits(x, "glmmTMB")) {
    # Check if we have duplicated parameters (could be due to Component structure or other reasons)
    if ("Component" %in% colnames(params)) {
      # Deduplicate based on Parameter and Component combination
      unique_idx <- !duplicated(paste(params$Parameter, params$Component))
    } else {
      # Deduplicate based on Parameter name only (for tables without Component column)
      unique_idx <- !duplicated(params$Parameter)
    }

    # Only apply deduplication if we actually found duplicates
    if (!all(unique_idx)) {
      params <- params[unique_idx, , drop = FALSE]

      # Also need to adjust the stats object to match the deduplicated table
      stats_vector <- as.character(stats)

      # Keep only the corresponding stats entries
      stats <- structure(
        stats_vector[unique_idx],
        class = class(stats),
        table = params,
        effectsize = effsize
      )
    }
  }

  # Parameters' names
  params_text <- as.character(.parameters_starting_text(x, params))

  # Significance and effect size
  params_text <- paste0(
    params_text,
    " is statistically ",
    effectsize::interpret_p(
      params$p,
      rules = effectsize::rules(0.05, c("significant", "non-significant"))
    ),
    " and ",
    effectsize::interpret_direction(params$Coefficient)
  )

  # Effect size
  # if(include_effectsize){
  #   params_text <- paste0(params_text,  " and ", attributes(effsize)$interpretation)
  # }

  # Include intercept
  if (isFALSE(include_intercept)) {
    idx <- params$Parameter != "(Intercept)"
  } else {
    idx <- rep(TRUE, nrow(params))
  }

  text_full <- paste0(params_text[idx], " (", stats[idx], ")")
  result_text <- paste0(params_text[idx], " (", summary(stats)[idx], ")")

  as.report_parameters(
    text_full,
    summary = result_text,
    table = params,
    effectsize = effsize,
    ...
  )
}


# report_intercept ------------------------------------------------------------
#' @rdname report.lm
#' @export
report_intercept.lm <- function(x, table = NULL, ...) {
  if (is.null(table)) {
    intercept_table <- report_table(x, ...)
  } else {
    intercept_table <- table
  }

  if (
    (insight::model_info(x)$is_zero_inflated ||
      (inherits(x, "glmmTMB") && "dispersion" %in% table$Component)) &&
      "Component" %in% colnames(table)
  ) {
    idx <- !is.na(table$Parameter) &
      table$Parameter == "(Intercept)" &
      table$Component == "conditional"
  } else {
    idx <- !is.na(intercept_table$Parameter) &
      intercept_table$Parameter == "(Intercept)"
  }

  # sanity check - if model has no intercept, return NULL
  if (!any(idx)) {
    return(NULL)
  }

  intercept <- intercept_table[idx, ]

  estimate <- .find_regression_estimate(intercept_table)
  is_at <- insight::format_value(intercept[[estimate]])

  intercept[[estimate]] <- NULL

  intercept_text <- paste0(
    "The model's intercept is at ",
    insight::format_value(is_at),
    " (",
    insight::format_ci(
      intercept$CI_low,
      intercept$CI_high,
      attributes(intercept)$ci
    ),
    ")."
  )
  text_full <- paste0(
    "The model's intercept",
    .find_intercept(x),
    " is at ",
    insight::format_value(is_at),
    " (",
    report_statistics(x, intercept, include_effectsize = FALSE),
    ")."
  )

  text_full <- gsub("std. beta", "std. intercept", text_full, fixed = TRUE)

  as.report_intercept(text_full, summary = intercept_text, ...)
}


# report_model ------------------------------------------------------------

#' @rdname report.lm
#' @export
report_model.lm <- function(x, table = NULL, ...) {
  if (is.null(table)) {
    model_table <- report_table(x, ...)
  } else {
    model_table <- table
  }

  # Model info
  info <- insight::model_info(x)
  is_nullmodel <- suppressWarnings(insight::is_nullmodel(x))

  # Bootstrap
  if (attributes(model_table)$bootstrap) {
    bootstrapped <- paste0(
      "bootstrapped (",
      attributes(model_table)$iterations,
      " iterations) "
    )
  } else {
    bootstrapped <- ""
  }

  # Initial
  model_text <- paste0(
    bootstrapped,
    format_model(x)
  )

  # Algorithm
  algorithm <- format_algorithm(x)
  if (algorithm != "") {
    text_full <- paste0(
      model_text,
      " (estimated using ",
      algorithm,
      ")"
    )
  } else {
    text_full <- model_text
  }

  # To predict
  to_predict_text <- paste0(" to predict ", insight::find_response(x))
  if (!is_nullmodel) {
    to_predict_text <- paste0(
      to_predict_text,
      " with ",
      datawizard::text_concatenate(insight::find_predictors(
        x,
        effects = "fixed",
        flatten = TRUE
      ))
    )
  }

  # Formula
  text_full <- paste0(text_full, to_predict_text, " (", format_formula(x), ")")
  model_text <- paste0(model_text, to_predict_text)

  # Random
  if (info$is_mixed) {
    random_text <- report_random(x)
    text_full <- paste0(text_full, ". ", as.character(random_text))
    model_text <- paste0(model_text, ". ", summary(random_text))
  }

  # Bayesian
  if (info$is_bayesian) {
    priors_text <- report_priors(x)
    text_full <- paste0(text_full, ". ", as.character(priors_text))
    model_text <- paste0(model_text, ". ", summary(priors_text))
  }

  as.report_model(text_full, summary = model_text, ...)
}


# report_info ------------------------------------------------------------

#' @rdname report.lm
#' @export
report_performance.lm <- function(x, table = NULL, ...) {
  if (!is.null(table) || is.null(attributes(table)$performance)) {
    perf_table <- report_table(x, ...)
  } else {
    perf_table <- table
  }
  performance <- attributes(perf_table)$performance

  # Intercept-only
  if (suppressWarnings(insight::is_nullmodel(x))) {
    return(as.report_performance("", summary = ""))
  }

  out <- .text_r2(x, info = insight::model_info(x), performance)
  as.report_performance(out$text_full, summary = out$text)
}


# report_info ------------------------------------------------------------

#' @rdname report.lm
#' @export
report_info.lm <- function(
  x,
  effectsize = NULL,
  include_effectsize = FALSE,
  parameters = NULL,
  ...
) {
  if (is.null(effectsize)) {
    effectsize <- report_effectsize(x, ...)
  }

  info_text <- .info_effectsize(
    x,
    effectsize = effectsize,
    include_effectsize = include_effectsize
  )

  if (is.null(parameters)) {
    parameters <- report_parameters(x, ...)
  }
  if (inherits(parameters, "report_parameters")) {
    att <- attributes(attributes(parameters)$table)
  } else {
    att <- attributes(parameters)
  }

  # Only add CI information if effectsize info is empty or doesn't already contain CI details
  no_ci_info <- !nzchar(info_text) ||
    !grepl("Confidence Intervals.*computed using.*approximation", info_text)
  if ("ci_method" %in% names(att) && no_ci_info) {
    info_text <- paste0(
      info_text,
      ifelse(nzchar(info_text), " ", ""),
      .info_df(
        ci = att$ci,
        ci_method = att$ci_method,
        test_statistic = att$test_statistic,
        bootstrap = att$bootstrap
      )
    )
  }

  # if (!is.null(att$ci_method)) {
  #   .text_ci(ci, ci_method = ci_method, df_method = df_method)
  # }

  as.report_info(info_text)
}


# report_text ------------------------------------------------------------

#' @rdname report.lm
#' @export
report_text.lm <- function(x, table = NULL, ...) {
  params <- report_parameters(x, table = table, include_intercept = FALSE, ...)
  report_table_data <- attributes(params)$table

  info <- report_info(
    x,
    effectsize = attributes(params)$effectsize,
    parameters = params,
    ...
  )
  model <- report_model(x, table = report_table_data, ...)
  perf <- report_performance(x, table = report_table_data, ...)
  intercept <- report_intercept(x, table = report_table_data, ...)

  if (suppressWarnings(insight::is_nullmodel(x))) {
    params_text_full <- params_text <- ""
  } else {
    params_text_full <- paste0("Within this model:\n\n", as.character(params))
    params_text <- paste0(
      "Within this model:\n\n",
      as.character(summary(params), ...)
    )
  }

  # Helpers
  sep_after <- function(x) {
    x <- trimws(x)
    if (!nzchar(x)) {
      return("")
    } # nothing to add if empty
    if (grepl("[.!?]\\s*$", x)) " " else ". " # space if already ends with .!?; else ". "
  }

  # Sanitize fragments (remove accidental leading spaces)
  params_text_full <- sub("^\\s+", "", params_text_full)
  params_text <- sub("^\\s+", "", params_text)
  perf <- trimws(perf)
  intercept <- trimws(intercept)

  # 1) text_full
  text_full <- paste0(
    "We fitted a ",
    model,
    ". ",
    perf,
    sep_after(perf),
    intercept,
    sep_after(intercept),
    params_text_full,
    "\n\n",
    info
  )

  # 2) result_summary
  result_summary <- paste0(
    "We fitted a ",
    summary(model),
    ". ",
    summary(perf),
    sep_after(summary(perf)),
    summary(intercept),
    sep_after(summary(intercept)),
    params_text
  )

  as.report_text(text_full, summary = result_summary)
}


# Utils -------------------------------------------------------------------

#' @keywords internal
.find_regression_estimate <- function(table, ...) {
  candidates <- "(^Coefficient|beta|Median|Mean|MAP)"
  coefname <- attributes(table)$coefficient_name
  if (!is.null(coefname) && coefname %in% names(table)) {
    estimate <- attributes(table)$coefficient_name
  } else {
    estimate <- grep(candidates, names(table), value = TRUE)[1]
  }
  estimate
}

# report_table -----------------

.report_table_fisher <- function(table_full, effsize) {
  table_full <- cbind(table_full, attributes(effsize)$table)
  list(table = NULL, table_full = table_full)
}


# report_effectsize ---------------------

.report_effectsize_fisher <- function(x, dot_args, rules = "funder2019") {
  es_args <- c(list(x), dot_args)
  es_table <- do.call(effectsize::effectsize, es_args)
  ci <- attributes(es_table)$ci
  estimate <- names(es_table)[1]
  dot_args$rules <- ifelse(is.null(dot_args$rules), rules, dot_args$rules)

  es_args <- c(list(es_table), dot_args)
  interpretation <- do.call(effectsize::interpret, es_args)$Interpretation
  rules <- .text_effectsize(attr(attr(interpretation, "rules"), "rule_name"))

  main <- switch(
    estimate,
    Cramers_v_adjusted = paste0(
      "Adjusted Cramer's v = ",
      insight::format_value(es_table[[estimate]])
    ),
    Tschuprows_t = paste0(
      "Tschuprow's t = ",
      insight::format_value(es_table[[estimate]])
    ),
    Tschuprows_t_adjusted = paste0(
      "Adjusted Tschuprow's t = ",
      insight::format_value(es_table[[estimate]])
    ),
    Pearsons_c = paste0(
      "Pearson's c = ",
      insight::format_value(es_table[[estimate]])
    ),
    phi_adjusted = paste0(
      "Adjusted Phi = ",
      insight::format_value(es_table[[estimate]])
    ),
    Cohens_h = paste0(
      "Cohen's h = ",
      insight::format_value(es_table[[estimate]])
    ),
    Odds_ratio = paste0(
      "Odds ratio = ",
      insight::format_value(es_table[[estimate]])
    ),
    Ris_kratio = paste0(
      "Risk ratio = ",
      insight::format_value(es_table[[estimate]])
    ),
    cohens_h = paste0(
      "Cohen's w = ",
      insight::format_value(es_table[[estimate]])
    ),
    paste0(estimate, " = ", insight::format_value(es_table[[estimate]]))
  )

  statistics <- paste0(
    main,
    ", ",
    insight::format_ci(es_table$CI_low, es_table$CI_high, ci)
  )

  result_table <- datawizard::data_rename(
    as.data.frame(es_table),
    select = c("CI_low", "CI_high"),
    replacement = paste0(estimate, c("_CI_low", "_CI_high"))
  )

  result_table <- result_table[c(
    estimate,
    paste0(estimate, c("_CI_low", "_CI_high"))
  )]

  list(
    table = result_table,
    statistics = statistics,
    interpretation = interpretation,
    rules = rules,
    ci = ci,
    main = main
  )
}

# report_model ----------------------------

.report_model_fisher <- function(x, table) {
  vars_full <- paste(names(attributes(x$observed)$dimnames), collapse = " and ")

  final_text <- paste0(
    trimws(x$method),
    " testing the association between the variables of the ",
    x$data.name,
    " dataset "
  )

  final_text
}

chi2_type <- function(x) {
  if (grepl("probabilities", x$method, fixed = TRUE)) {
    out <- "probabilities"
  } else if (grepl("Pearson", x$method, fixed = TRUE)) {
    out <- "pearson"
  } else if (grepl("Fisher", x$method, fixed = TRUE)) {
    out <- "fisher"
  }
  out
}

.report_parameters_fisher <- function(table, stats, effsize, ...) {
  text_full <- paste0(
    "statistically ",
    effectsize::interpret_p(table$p, rules = "default"),
    ", and ",
    attributes(effsize)$interpretation,
    " (",
    stats,
    ")"
  )

  text_short <- paste0(
    "statistically ",
    effectsize::interpret_p(table$p, rules = "default"),
    ", and ",
    attributes(effsize)$interpretation,
    " (",
    summary(stats),
    ")"
  )

  list(text_short = text_short, text_full = text_full)
}

#' Reporting Models' Bayes Factor
#'
#' Create reports of Bayes factors for model comparison.
#'
#' @param x Object of class `bayesfactor_inclusion`.
#' @param interpretation Effect size interpretation set of rules (see
#'   [interpret_bf][effectsize::interpret_bf]).
#' @inheritParams report
#' @inheritParams effectsize::interpret_bf
#' @inherit report return seealso
#'
#' @examplesIf requireNamespace("bayestestR", quietly = TRUE)
#' library(bayestestR)
#' # Bayes factor - models
#' mo0 <- lm(Sepal.Length ~ 1, data = iris)
#' mo1 <- lm(Sepal.Length ~ Species, data = iris)
#' mo2 <- lm(Sepal.Length ~ Species + Petal.Length, data = iris)
#' mo3 <- lm(Sepal.Length ~ Species * Petal.Length, data = iris)
#' BFmodels <- bayesfactor_models(mo1, mo2, mo3, denominator = mo0)
#'
#' r <- report(BFmodels)
#' r
#'
#' # Bayes factor - inclusion
#' inc_bf <- bayesfactor_inclusion(BFmodels, prior_odds = c(1, 2, 3), match_models = TRUE)
#'
#' r <- report(inc_bf)
#' r
#' as.data.frame(r)
#' @return An object of class [report()].
#' @export
report.bayesfactor_models <- function(
  x,
  interpretation = "jeffreys1961",
  exact = TRUE,
  protect_ratio = TRUE,
  ...
) {
  out <- .report.bayesfactor_models(
    x,
    interpretation = interpretation,
    exact = exact,
    protect_ratio = protect_ratio,
    ...
  )

  as.report(
    text = as.report_text(out$text_full, summary = out$text_short),
    table = as.report_table(out$table_full, summary = out$table_short),
    rules = out$rules,
    denominator = out$denominator,
    BF_method = out$BF_method
  )
}


#' @export
report_table.bayesfactor_models <- function(
  x,
  interpretation = "jeffreys1961",
  exact = TRUE,
  protect_ratio = TRUE,
  ...
) {
  out <-
    .report.bayesfactor_models(
      x,
      interpretation = interpretation,
      exact = exact,
      protect_ratio = protect_ratio,
      ...
    )

  as.report_table(
    out$table_full,
    summary = out$table_short,
    rules = out$rules,
    denominator = out$denominator,
    BF_method = out$BF_method
  )
}

#' @export
report_text.bayesfactor_models <- function(
  x,
  table = NULL,
  interpretation = "jeffreys1961",
  exact = TRUE,
  protect_ratio = TRUE,
  ...
) {
  out <- .report.bayesfactor_models(
    x,
    interpretation = interpretation,
    exact = exact,
    protect_ratio = protect_ratio,
    ...
  )

  as.report_text(
    out$text_full,
    summary = out$text_short,
    rules = out$rules,
    denominator = out$denominator,
    BF_method = out$BF_method
  )
}


#' @keywords internal
.report.bayesfactor_models <- function(
  model,
  interpretation = "jeffreys1961",
  exact = TRUE,
  protect_ratio = TRUE,
  ...
) {
  model$Model[model$Model == "1"] <- "(Intercept only)"
  denominator <- attr(model, "denominator")
  BF_method <- attr(model, "BF_method")
  max_den <- which.max(exp(model$log_BF))
  min_den <- which.min(exp(model$log_BF))

  #### text ####
  model_ind <- rep("", nrow(model))
  model_ind[max_den] <- " (the most supported model)"
  model_ind[min_den] <- " (the least supported model)"

  summ_inds <- c(max_den, min_den)
  summ_inds <- summ_inds[summ_inds != denominator]
  bf_text <- paste0(
    "Compared to the ",
    model$Model[denominator],
    " model",
    model_ind[denominator],
    ", ",
    "we found ",
    paste0(
      effectsize::interpret_bf(
        exp(model$log_BF)[summ_inds],
        rules = interpretation,
        include_value = TRUE,
        exact = exact,
        protect_ratio = protect_ratio
      ),
      " the ",
      model$Model[summ_inds],
      " model",
      model_ind[summ_inds],
      collapse = "; "
    ),
    "."
  )

  #### text full ####
  if (grepl("BIC", BF_method, fixed = TRUE)) {
    bf_explain <- paste0(
      "Bayes factors were computed using the BIC approximation, ",
      "by which BF10 = exp((BIC0 - BIC1)/2). "
    )
  } else if (grepl("JZS", BF_method, fixed = TRUE)) {
    bf_explain <- paste0(
      "Bayes factors were computed with the `BayesFactor` package, ",
      "using JZS priors. "
    )
  } else if (grepl("bridgesampling", BF_method, fixed = TRUE)) {
    bf_explain <- paste0(
      "Bayes factors were computed by comparing marginal likelihoods, ",
      "using the `bridgesampling` package. "
    )
  }

  bf_text_full <- paste0(
    bf_explain,
    paste0(
      "Compared to the ",
      model$Model[denominator],
      " model",
      model_ind[denominator],
      ", ",
      "we found ",
      paste0(
        effectsize::interpret_bf(
          exp(model$log_BF)[-denominator],
          rules = interpretation,
          include_value = TRUE,
          exact = exact,
          protect_ratio = protect_ratio
        ),
        " the ",
        model$Model[-denominator],
        " model",
        model_ind[-denominator],
        collapse = "; "
      ),
      "."
    )
  )

  #### table ####
  model$Model <- paste0(" [", seq_len(nrow(model)), "] ", model$Model)
  bf_table <- as.data.frame(model)
  bf_table$BF <- insight::format_bf(
    exp(model$log_BF),
    name = NULL,
    exact = exact,
    protect_ratio = protect_ratio
  )
  colnames(bf_table) <- c("Model", "Bayes factor")

  footer <- list(
    paste0("\nBayes Factor Type: ", BF_method),
    paste0("\nAgainst denominator: Model ", denominator)
  )
  attr(bf_table, "table_footer") <- footer

  #### table full ####
  bf_table_full <- bf_table
  bf_table_full$BF2 <- insight::format_bf(
    exp(model$log_BF) / exp(model$log_BF)[max_den],
    name = NULL,
    exact = exact,
    protect_ratio = protect_ratio
  )

  colnames(bf_table_full) <- c(
    "Model",
    paste0("BF against model ", denominator, ""),
    paste0("BF against model ", max_den, " (best model)")
  )

  # Output
  out <- list(
    table_full = bf_table_full,
    table_short = bf_table,
    text_full = bf_text_full,
    text_short = bf_text,
    rules = interpretation,
    denominator = denominator,
    BF_method = BF_method
  )
}


# bayesfactor_inclusion ---------------------------------------------------

#' @rdname report.bayesfactor_models
#' @export
report.bayesfactor_inclusion <- function(
  x,
  interpretation = "jeffreys1961",
  exact = TRUE,
  protect_ratio = TRUE,
  ...
) {
  out <- .report.bayesfactor_inclusion(
    x,
    interpretation = interpretation,
    exact = exact,
    protect_ratio = protect_ratio,
    ...
  )

  as.report(
    text = as.report_text(out$text_full, summary = out$text_short),
    table = as.report_table(out$table_full, summary = out$table_short),
    interpretation = out$interpretation,
    priorOdds = out$priorOdds,
    matched = out$matched
  )
}


#' @export
report_table.bayesfactor_inclusion <- function(
  x,
  interpretation = "jeffreys1961",
  exact = TRUE,
  protect_ratio = TRUE,
  ...
) {
  out <- .report.bayesfactor_inclusion(
    x,
    interpretation = interpretation,
    exact = exact,
    protect_ratio = protect_ratio,
    ...
  )

  as.report_table(
    out$table_full,
    summary = out$table_short,
    interpretation = out$interpretation,
    priorOdds = out$priorOdds,
    matched = out$matched
  )
}

#' @export
report_text.bayesfactor_inclusion <- function(
  x,
  table = NULL,
  interpretation = "jeffreys1961",
  exact = TRUE,
  protect_ratio = TRUE,
  ...
) {
  out <- .report.bayesfactor_inclusion(
    x,
    interpretation = interpretation,
    exact = exact,
    protect_ratio = protect_ratio,
    ...
  )

  as.report_text(
    out$text_full,
    summary = out$text_short,
    interpretation = out$interpretation,
    priorOdds = out$priorOdds,
    matched = out$matched
  )
}


#' @keywords internal
.report.bayesfactor_inclusion <- function(
  model,
  interpretation = "jeffreys1961",
  exact = TRUE,
  protect_ratio = TRUE,
  ...
) {
  matched <- attr(model, "matched")
  priorOdds <- attr(model, "priorOdds")

  #### text ####
  bf_results <- data.frame(Term = rownames(model), stringsAsFactors = FALSE)
  bf_results$evidence <- effectsize::interpret_bf(
    exp(model$log_BF),
    rules = interpretation,
    include_value = TRUE,
    exact = exact,
    protect_ratio = protect_ratio
  )
  bf_results$postprob <- paste0(round(model$p_posterior * 100, ...), "%")

  bf_text <- paste0(
    "Bayesian model averaging (BMA) was used to obtain the average evidence ",
    "for each predictor. We found ",
    paste(
      paste0(bf_results$evidence, " including ", bf_results$Term),
      collapse = "; "
    ),
    "."
  )

  #### text full ####
  bf_explain <- paste0(
    "Bayesian model averaging (BMA) was used to obtain the average evidence ",
    "for each predictor. Since each model has a prior probability",
    # custom priors?
    if (is.null(priorOdds)) {
      NULL
    } else {
      paste0(
        " (here we used subjective prior odds of ",
        toString(priorOdds),
        ")"
      )
    },
    ", it is possible to sum the prior probability of all models that include ",
    "a predictor of interest (the prior inclusion probability), and of all ",
    "models that do not include that predictor (the prior exclusion probability). ",
    "After the data are observed, we can similarly consider the sums of the ",
    "posterior models' probabilities to obtain the posterior inclusion ",
    "probability and the posterior exclusion probability. The change from ",
    "prior to posterior inclusion odds is the Inclusion Bayes factor. ",
    # matched models?
    if (matched) {
      paste0(
        "For each predictor, averaging was done only across models that ",
        "did not include any interactions with that predictor; ",
        "additionally, for each interaction predictor, averaging was done ",
        "only across models that contained the main effect from which the ",
        "interaction predictor was comprised. This was done to prevent ",
        "Inclusion Bayes factors from being contaminated with non-relevant ",
        "evidence (see Mathot, 2017). "
      )
    } else {
      NULL
    }
  )

  bf_text_full <- paste0(
    bf_explain,
    paste0(
      "We found ",
      paste(
        paste0(
          bf_results$evidence,
          " including ",
          bf_results$Term,
          ", with models including ",
          bf_results$Term,
          " having an overall posterior probability of ",
          bf_results$postprob
        ),
        collapse = "; "
      ),
      "."
    )
  )

  #### table ####
  bf_table <- as.data.frame(model)
  colnames(bf_table) <- c("Pr(prior)", "Pr(posterior)", "Inclusion BF")
  bf_table <- cbind(Terms = rownames(bf_table), bf_table)
  rownames(bf_table) <- NULL
  bf_table$`Inclusion BF` <- insight::format_bf(
    bf_table$`Inclusion BF`,
    name = NULL,
    exact = exact,
    protect_ratio = protect_ratio
  )
  bf_table[, 2:3] <- insight::format_value(bf_table[, 2:3], ...)

  # make table footer
  footer <- list(
    sprintf(
      "\nAcross %s",
      ifelse(matched, "matched models only.", "all models.")
    ),
    ifelse(
      is.null(priorOdds),
      "\nAssuming unifor prior odds.",
      paste0("\nWith custom prior odds of [", toString(priorOdds), "].")
    )
  )
  attr(bf_table, "table_footer") <- footer

  #### table full ####
  bf_table_full <- bf_table

  #### out ####
  out <- list(
    table_full = bf_table_full,
    table_short = bf_table,
    text_full = bf_text_full,
    text_short = bf_text,
    interpretation = interpretation,
    priorOdds = priorOdds,
    matched = matched,
    ...
  )
}

#' Reports of Structural Equation Models (SEM)
#'
#' Create a report for `lavaan` objects.
#'
#' @param x Object of class `lavaan`.
#' @inheritParams report
#' @inheritParams report.htest
#' @inheritParams report.lm
#'
#' @inherit report return seealso
#'
#' @examplesIf requireNamespace("lavaan", quietly = TRUE)
#' \donttest{
#' # Structural Equation Models (SEM)
#' library(lavaan)
#' structure <- "ind60 =~ x1 + x2 + x3
#'               dem60 =~ y1 + y2 + y3
#'               dem60 ~ ind60"
#' model <- lavaan::sem(structure, data = PoliticalDemocracy)
#' r <- report(model)
#' r
#' summary(r)
#' as.data.frame(r)
#' summary(as.data.frame(r))
#'
#' # Specific reports
#' suppressWarnings(report_table(model))
#' suppressWarnings(report_performance(model))
#' }
#' @return An object of class [report()].
#' @export
report.lavaan <- function(x, ...) {
  cat("Support for lavaan not fully implemented yet :(\n")
}


#' @export
report_table.lavaan <- function(x, ...) {
  parameters <- parameters::model_parameters(x, ci_random = FALSE, ...)
  lavaan_table <- as.data.frame(parameters)
  lavaan_table$Parameter <- paste(
    lavaan_table$To,
    lavaan_table$Operator,
    lavaan_table$From
  )
  lavaan_table <- datawizard::data_remove(
    lavaan_table,
    c("To", "Operator", "From")
  )
  lavaan_table <- datawizard::data_reorder(lavaan_table, "Parameter")

  # Combine -----
  # Add performance
  performance <- performance::model_performance(x, ...)
  lavaan_table <- .combine_tables_performance(lavaan_table, performance)
  lavaan_table <- lavaan_table[
    !tolower(lavaan_table$Parameter) %in%
      c("baseline", "baseline_df", "baseline_p"),
  ]

  # Clean -----
  # Rename some columns

  # Shorten ----
  table_full <- datawizard::data_remove(lavaan_table, "SE")
  lavaan_table <- datawizard::data_remove(
    table_full,
    select = "(_CI_low|_CI_high)$",
    regex = TRUE
  )
  lavaan_table <- lavaan_table[
    !lavaan_table$Parameter %in%
      c(
        "AIC",
        "BIC",
        "RMSEA"
      ),
  ]

  # Prepare -----
  out <- as.report_table(
    table_full,
    summary = lavaan_table,
    performance = performance,
    parameters = parameters,
    ...
  )
  # Add attributes from params table
  for (att in "ci") {
    attr(out, att) <- attributes(parameters)[[att]]
  }

  out
}


#' @rdname report.lavaan
#' @export
report_performance.lavaan <- function(x, table = NULL, ...) {
  if (!is.null(table) || is.null(attributes(table)$performance)) {
    table <- report_table(x, ...)
  }
  performance <- attributes(table)$performance

  # Chi2
  text_chi2 <- ""
  if (all(c("p_Chi2", "Chi2", "Chi2_df") %in% names(performance))) {
    sig <- "significantly"
    if (performance$p_Chi2 > 0.05) {
      sig <- "not significantly"
    }
    text_chi2 <- paste0(
      text_chi2,
      "The model is ",
      sig,
      " different from a baseline model (Chi2(",
      insight::format_value(performance$Chi2_df, protect_integers = TRUE),
      ") = ",
      insight::format_value(performance$Chi2),
      ", ",
      insight::format_p(performance$p_Chi2),
      ")."
    )
  }

  perf_table <- effectsize::interpret(performance)
  text_full <- datawizard::text_paste(
    text_chi2,
    .text_performance_lavaan(perf_table),
    sep = " "
  )
  summary_text <- datawizard::text_paste(
    text_chi2,
    .text_performance_lavaan(perf_table[
      perf_table$Name %in% c("RMSEA", "CFI", "SRMR"),
    ]),
    sep = " "
  )

  as.report_performance(text_full, summary = summary_text)
}

#' Report a textual description of an object
#'
#' Creates text containing a description of the parameters of R objects (see
#' list of supported objects in [report()]).
#'
#' @inheritParams report
#' @inheritParams report_table
#' @param table A table obtained via `report_table()`. If not provided,
#'   will run it.
#'
#' @return An object of class [report_text()].
#'
#' @examples
#' library(report)
#'
#' # Miscellaneous
#' r <- report_text(sessionInfo())
#' r
#' summary(r)
#'
#' # Data
#' report_text(iris$Sepal.Length)
#' report_text(as.character(round(iris$Sepal.Length, 1)))
#' report_text(iris$Species)
#' report_text(iris)
#'
#' # h-tests
#' report_text(t.test(iris$Sepal.Width, iris$Sepal.Length))
#'
#' # ANOVA
#' r <- report_text(aov(Sepal.Length ~ Species, data = iris))
#' r
#' summary(r)
#'
#' # GLMs
#' r <- report_text(lm(Sepal.Length ~ Petal.Length * Species, data = iris))
#' r
#' summary(r)
#'
#' @examplesIf requireNamespace("lme4", quietly = TRUE)
#' \donttest{
#' library(lme4)
#' model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
#' r <- report_text(model)
#' r
#' summary(r)
#' }
#'
#' @examplesIf requireNamespace("rstanarm", quietly = TRUE)
#' \donttest{
#' # Bayesian models
#' library(rstanarm)
#' model <- suppressWarnings(stan_glm(mpg ~ cyl + wt, data = mtcars, refresh = 0, iter = 600))
#' r <- report_text(model)
#' r
#' summary(r)
#' }
#' @export
report_text <- function(x, table = NULL, ...) {
  UseMethod("report_text")
}


# METHODS -----------------------------------------------------------------

#' @rdname as.report
#' @export
as.report_text <- function(x, ...) {
  UseMethod("as.report_text")
}

#' @export
as.report_text.default <- function(x, summary = NULL, ...) {
  class(x) <- unique(c("report_text", class(x)))
  attributes(x) <- c(attributes(x), list(...))
  if (!is.null(summary)) {
    class(summary) <- unique(c("report_text", class(summary)))
    attr(x, "summary") <- summary
  }
  x
}

#' @export
as.report_text.report <- function(x, summary = NULL, ...) {
  class(x) <- class(x)[class(x) != "report"]

  if (is.null(summary) || isFALSE(summary)) {
    x
  } else if (isTRUE(summary)) {
    summary(x)
  }
}


#' @export
summary.report_text <- function(object, ...) {
  if (is.null(attributes(object)$summary)) {
    object
  } else {
    attributes(object)$summary
  }
}

#' @export
print.report_text <- function(x, width = NULL, ...) {
  x <- datawizard::text_format(as.character(x), width = width, ...)
  cat(x)
}

test_that("report_info() works for linear models", {
  model <- lm(Sepal.Length ~ Species, data = iris)

  result <- report_info(model)

  expect_s3_class(result, "report_info")
  expect_type(result, "character")

  # Test print method
  expect_output(print(result))
})

test_that("report_info() works for t-test", {
  test_result <- t.test(iris$Sepal.Width, iris$Sepal.Length)

  result <- report_info(test_result)

  expect_s3_class(result, "report_info")
  expect_type(result, "character")
})

test_that("report_info() works for ANOVA", {
  aov_result <- aov(Sepal.Length ~ Species, data = iris)

  result <- report_info(aov_result)

  expect_s3_class(result, "report_info")
  expect_type(result, "character")
})

test_that("as.report_info() creates proper objects", {
  text_info <- "Some information about the model"
  summary_text <- "Short info"

  result <- as.report_info(text_info, summary = summary_text)

  expect_s3_class(result, "report_info")
  expect_identical(as.character(result), text_info)
  expect_identical(as.character(summary(result)), summary_text)
})

test_that("as.report_info() handles NULL summary", {
  text_info <- "Some information"

  result <- as.report_info(text_info)

  expect_s3_class(result, "report_info")
  expect_identical(summary(result), result) # Should return itself if no summary
})

test_that(".info_df() helper function works", {
  # Test basic CI info
  result1 <- report:::.info_df(ci = 0.95, ci_method = "normal")
  expect_type(result1, "character")
  expect_match(result1, "95%")
  expect_match(result1, "Confidence Intervals")
  expect_match(result1, "Wald normal")

  # Test with NULL ci_method
  result2 <- report:::.info_df(ci = 0.95, ci_method = NULL)
  expect_identical(result2, "")

  # Test with bootstrap
  result3 <- report:::.info_df(ci = 0.90, ci_method = "boot", bootstrap = TRUE)
  expect_match(result3, "90%")
  expect_match(result3, "parametric bootstrap")
  expect_match(result3, "intervals")

  # Test with different methods
  result4 <- report:::.info_df(ci = 0.95, ci_method = "bci")
  expect_match(result4, "bias-corrected accelerated bootstrap")
})

test_that(".info_effectsize() helper function works", {
  # Mock effectsize object
  mock_effectsize <- list()
  attr(mock_effectsize, "method") <- "Cohen's d"
  attr(
    mock_effectsize,
    "rules"
  ) <- "Effect sizes interpreted following Cohen (1988)."

  # Test basic usage
  result1 <- report:::.info_effectsize(
    NULL,
    effectsize = mock_effectsize,
    include_effectsize = FALSE
  )
  expect_identical(result1, "Cohen's d")

  # Test with include_effectsize = TRUE
  result2 <- report:::.info_effectsize(
    NULL,
    effectsize = mock_effectsize,
    include_effectsize = TRUE
  )
  expect_match(result2, "Cohen's d")
  expect_match(result2, "interpreted following")

  # Test with NULL effectsize
  result3 <- report:::.info_effectsize(
    NULL,
    effectsize = NULL,
    include_effectsize = FALSE
  )
  expect_identical(result3, "")
})

test_that("report_date() works correctly", {
  result <- report_date()

  expect_s3_class(result, "report_text")
  expect_type(result, "character")
  expect_match(as.character(result), "It's")
  expect_match(as.character(result), "year")

  # Test summary
  summary_result <- summary(result)
  expect_match(as.character(summary_result), "\\d+/\\d+/\\d+") # Date format
  expect_match(as.character(summary_result), "\\d+:\\d+:\\d+") # Time format
})

test_that("report_story() works correctly", {
  result <- report_story()

  expect_s3_class(result, "report_text")
  expect_type(result, "character")
  expect_match(as.character(result), "Darth Plagueis")
  expect_match(as.character(result), "Sith legend")
  expect_match(as.character(result), "Ironic")

  # Test summary
  summary_result <- summary(result)
  expect_match(as.character(summary_result), "thunderous applause")
})

test_that("report_text() works with sessionInfo", {
  si <- sessionInfo()

  result <- report_text(si)

  expect_s3_class(result, "report_text")
  expect_type(result, "character")
})

test_that("as.report_text() works correctly", {
  # Test basic usage
  text_content <- "This is some report text"
  summary_text <- "Short summary"

  result <- as.report_text(text_content, summary = summary_text)

  expect_s3_class(result, "report_text")
  expect_identical(as.character(result), text_content)
  expect_identical(as.character(summary(result)), summary_text)

  # Test with NULL summary
  result2 <- as.report_text(text_content)
  expect_identical(summary(result2), result2) # Should return itself
})

test_that("as.report_text.report() works correctly", {
  # Create a mock report object
  mock_report <- structure(
    list(text = "Full report text", summary = "Short summary"),
    class = c("report", "list")
  )

  # Test with summary = FALSE
  result1 <- as.report_text(mock_report, summary = FALSE)
  expect_false(inherits(result1, "report"))

  # Test with summary = TRUE would require a proper report object with summary method
})

test_that("print.report_text() works correctly", {
  text_content <- "This is a test report text that should be printed nicely."
  report_obj <- as.report_text(text_content)

  # Test that it prints without error
  expect_output(print(report_obj), "test report text")
})

# Generate snapshots only on Windows to avoid having to generate snapshot variant
# corresponding to each OS (#312).
#
# This is especially important for Bayesian models where the results can be different
# across OS, and there is no way to specify a threshold when it comes to snapshots
# since the values included are of character type.
if (tolower(Sys.info()[["sysname"]]) == "windows") {
  library(testthat)

  test_check("report")
}

#' Reporting `htest` objects (Correlation, t-test...)
#'
#' Create reports for `htest` objects (`t.test()`, `cor.test()`,
#' etc.).
#'
#' @param x Object of class `htest`.
#' @param table Provide the output of  `report_table()` to avoid its
#'   re-computation.
#' @param effectsize Provide the output of `report_effectsize()` to avoid
#'   its re-computation.
#' @inheritParams report
#'
#' @inherit report return seealso
#'
#' @examples
#' # t-tests
#' report(t.test(iris$Sepal.Width, iris$Sepal.Length))
#' report(t.test(iris$Sepal.Width, iris$Sepal.Length, var.equal = TRUE))
#' report(t.test(mtcars$mpg ~ mtcars$vs))
#' report(t.test(mtcars$mpg, mtcars$vs, paired = TRUE), verbose = FALSE)
#' report(t.test(iris$Sepal.Width, mu = 1))
#'
#' # Correlations
#' report(cor.test(iris$Sepal.Width, iris$Sepal.Length))
#' @return An object of class [report()].
#' @export
report.htest <- function(x, ...) {
  model_info <- insight::model_info(x, verbose = FALSE)
  tbl <- report_table(x, model_info = model_info, ...)
  txt <- report_text(x, table = tbl, model_info = model_info, ...)

  as.report(txt, table = tbl, ...)
}


# report_effectsize -------------------------------------------------------

#' @rdname report.htest
#' @param table Provide the output of `report_table()` to avoid its re-computation.
#' @export
report_effectsize.htest <- function(x, table = NULL, ...) {
  dot_args <- list(...)
  model_info <- dot_args$model_info
  if (is.null(model_info)) {
    model_info <- suppressWarnings(insight::model_info(x, verbose = FALSE))
  }

  model_data <- insight::get_data(x)
  if (is.null(model_data) && is.null(dot_args$data)) {
    insight::format_alert(
      paste0(
        "In the report() function, for htest objects, you can try providing ",
        "the data argument manually, e.g., report(x, data = data)."
      )
    )
  }

  # remove arg, so dots can be passed to effectsize
  dot_args[["model_info"]] <- NULL

  # Get table if not provided
  if (is.null(table)) {
    call_args <- c(list(x), dot_args)
    table <- do.call(parameters::model_parameters, call_args)
  }

  # For t-tests ----------------

  if (model_info$is_ttest) {
    out <- .report_effectsize_ttest(x, dot_args)
  }

  if (model_info$is_ranktest && !model_info$is_correlation) {
    # For friedman test ---------------

    if (grepl("Friedman", attributes(x$statistic)$names, fixed = TRUE)) {
      out <- .report_effectsize_friedman(x, dot_args)
    } else if (
      !is.null(x$statistic) &&
        grepl(
          "Kruskal",
          attributes(x$statistic)$names,
          fixed = TRUE
        )
    ) {
      # For Kruskal-Wallis test ---------------

      out <- .report_effectsize_kruskal(x, dot_args)
    } else {
      # For wilcox test ---------------

      out <- .report_effectsize_wilcox(x, dot_args)
    }
  }

  ## For correlations ---------------

  if (model_info$is_correlation) {
    out <- .report_effectsize_correlation(x, dot_args)
  }

  ## For Chi2 ---------------

  if (model_info$is_chi2test) {
    if (chi2_type(x) == "fisher") {
      out <- .report_effectsize_fisher(x, dot_args)
    } else {
      out <- .report_effectsize_chi2(x, dot_args)
    }
  }

  # TODO: Chi-squared test -------------

  if (
    model_info$is_proptest ||
      (model_info$is_xtab && !model_info$is_chi2test) ||
      model_info$is_onewaytest
  ) {
    stop(
      insight::format_message(
        "This test is not yet supported. Please open an issue at {.url https://github.com/easystats/report/issues}."
      ),
      call. = FALSE
    )
  }

  parameters <- paste0(out$interpretation, " (", out$statistics, ")")

  as.report_effectsize(
    parameters,
    summary = parameters,
    table = out$table,
    interpretation = out$interpretation,
    statistics = out$statistics,
    rules = out$rules,
    ci = out$ci,
    main = out$main
  )
}


# report_table ------------------------------------------------------------

#' @rdname report.htest
#' @export
report_table.htest <- function(x, ...) {
  dot_args <- list(...)
  model_info <- dot_args$model_info
  if (is.null(model_info)) {
    model_info <- suppressWarnings(insight::model_info(x, verbose = FALSE))
  }

  # remove arg, so dots can be passed to effectsize
  dot_args[["model_info"]] <- NULL

  call_args <- c(list(x), dot_args)
  table_full <- do.call(parameters::model_parameters, call_args)
  call_args <- c(list(x, table = table_full, model_info = model_info), dot_args)
  effsize <- do.call(report_effectsize, call_args)

  if (model_info$is_ttest) {
    # If t-test, effect size
    out <- .report_table_ttest(table_full, effsize)
  } else if (model_info$is_ranktest && !model_info$is_correlation) {
    # wilcox test
    # but same function for Friedman
    out <- .report_table_wilcox(table_full, effsize)
  } else if (model_info$is_chi2test) {
    # chi2 test
    out <- .report_table_chi2(table_full, effsize)
    attr(out$table_full, "table_footer") <- attr(
      attr(effsize, "table"),
      "table_footer"
    )
  } else if (model_info$is_correlation) {
    # correlation test
    out <- .report_table_correlation(table_full)
  } else {
    out <- list(table_full = table_full, table = NULL)
  }

  as.report_table(out$table_full, summary = out$table, effsize = effsize)
}


# report_statistics ------------------------------------------------------------

#' @rdname report.htest
#' @export
report_statistics.htest <- function(x, table = NULL, ...) {
  model_info <- list(...)$model_info
  if (is.null(model_info)) {
    model_info <- suppressWarnings(insight::model_info(x, verbose = FALSE))
  }
  if (is.null(table) || is.null(attributes(table)$effsize)) {
    table <- report_table(x, model_info = model_info)
  }

  effsize <- attributes(table)$effsize
  txt <- NULL

  # Estimate
  candidates <- c(
    "rho",
    "r",
    "tau",
    "Difference",
    "r_rank_biserial",
    "Chi2",
    "Odds Ratio"
  )
  estimate <- candidates[candidates %in% names(table)][1]
  if (!is.null(estimate) && !is.na(estimate)) {
    txt <- paste0(
      tolower(estimate),
      " = ",
      insight::format_value(table[[estimate]])
    )
  }

  # CI
  if (!is.null(attributes(x$conf.int)$conf.level)) {
    txt <- paste0(
      txt,
      ", ",
      insight::format_ci(
        table$CI_low,
        table$CI_high,
        ci = attributes(x$conf.int)$conf.level
      )
    )
  }

  # Statistic
  if ("t" %in% names(table)) {
    txt <- paste0(
      txt,
      ", t(",
      insight::format_value(table$df, protect_integers = TRUE),
      ") = ",
      insight::format_value(table$t)
    )
  } else if ("S" %in% names(table)) {
    txt <- paste0(txt, ", S = ", insight::format_value(table$S))
  } else if ("z" %in% names(table)) {
    txt <- paste0(txt, ", z = ", insight::format_value(table$z))
  } else if ("W" %in% names(table)) {
    txt <- paste0("W = ", insight::format_value(table$W))
  }

  # p-value
  txt <- paste0(
    txt,
    ", ",
    insight::format_p(table$p, stars = FALSE, digits = "apa")
  )

  # Effect size
  if (
    model_info$is_ttest ||
      (model_info$is_ranktest && !model_info$is_correlation) ||
      model_info$is_chi2test
  ) {
    text_full <- paste0(txt, "; ", attributes(effsize)$statistics)
    txt <- paste0(txt, ", ", attributes(effsize)$main)
  } else {
    text_full <- txt
  }

  as.report_statistics(
    text_full,
    summary = txt,
    estimate = table[[estimate]],
    table = table,
    effsize = effsize
  )
}


# report_parameters ------------------------------------------------------------

#' @rdname report.htest
#' @export
report_parameters.htest <- function(x, table = NULL, ...) {
  dot_args <- list(...)
  model_info <- dot_args$model_info
  if (is.null(model_info)) {
    model_info <- suppressWarnings(insight::model_info(x, verbose = FALSE))
  }

  # remove arg, so dots can be passed to other methods
  dot_args[["model_info"]] <- NULL

  call_args <- c(list(x, table = table, model_info = model_info), dot_args)
  stats <- do.call(report_statistics, call_args)
  table <- attributes(stats)$table
  effsize <- attributes(stats)$effsize

  ## TODO see https://github.com/easystats/report/issues/256
  # insight::model_info() returns "$is_correlation" for shapiro-test,
  # but shapiro-test has no "estimate", so this fails. We probably need
  # to handle shapiro separately

  # Correlations
  if (model_info$is_correlation) {
    out <- .report_parameters_correlation(table, stats, ...)
    # t-tests
  } else if (model_info$is_ttest) {
    out <- .report_parameters_ttest(table, stats, effsize, ...)
    # Friedman
  } else if (
    model_info$is_ranktest &&
      grepl("Friedman", attributes(x$statistic)$names, fixed = TRUE)
  ) {
    out <- .report_parameters_friedman(table, stats, effsize, ...)
  } else if (
    !is.null(x$statistic) &&
      grepl(
        "Kruskal",
        attributes(x$statistic)$names,
        fixed = TRUE
      )
  ) {
    # Kruskal
    out <- .report_parameters_kruskal(table, stats, effsize, ...)
    # chi2
  } else if (model_info$is_chi2test) {
    if (chi2_type(x) == "fisher") {
      out <- .report_parameters_fisher(table, stats, effsize, ...)
    } else {
      out <- .report_parameters_chi2(table, stats, effsize, ...)
    }
  } else {
    # TODO: default, same as t-test?
    out <- .report_parameters_htest_default(table, stats, effsize, ...)
  }

  as.report_parameters(
    out$text_full,
    summary = out$text_short,
    table = table,
    effectsize = effsize,
    ...
  )
}

# report_model ------------------------------------------------------------

#' @rdname report.htest
#' @export
report_model.htest <- function(x, table = NULL, ...) {
  dot_args <- list(...)
  model_info <- dot_args$model_info
  if (is.null(model_info)) {
    model_info <- suppressWarnings(insight::model_info(x, verbose = FALSE))
  }

  # remove arg, so dots can be passed to other methods
  dot_args[["model_info"]] <- NULL

  if (is.null(table)) {
    call_args <- c(list(x, model_info = model_info), dot_args)
    table <- do.call(report_table, call_args)
  }

  if (model_info$is_correlation) {
    txt <- paste0(x$method, " between ", x$data.name)
  }

  if (model_info$is_ttest) {
    txt <- .report_model_ttest(x, table)
  }

  if (model_info$is_ranktest && !model_info$is_correlation) {
    # For friedman test ---------------

    if (grepl("Friedman", attributes(x$statistic)$names, fixed = TRUE)) {
      txt <- .report_model_friedman(x, table)
    } else {
      # For wilcox test ---------------

      txt <- .report_model_wilcox(x, table)
    }
  }

  if (model_info$is_chi2test) {
    if (chi2_type(x) == "fisher") {
      txt <- .report_model_fisher(x, table)
    } else {
      txt <- .report_model_chi2(x, table)
    }
  }

  as.report_model(txt, summary = txt)
}


# report_info ------------------------------------------------------------

#' @rdname report.htest
#' @export
report_info.htest <- function(x, effectsize = NULL, ...) {
  dot_args <- list(...)
  model_info <- dot_args$model_info
  if (is.null(model_info)) {
    model_info <- suppressWarnings(insight::model_info(x, verbose = FALSE))
  }

  # remove arg, so dots can be passed to other methods
  dot_args[["model_info"]] <- NULL

  if (is.null(effectsize)) {
    call_args <- c(list(x, model_info = model_info), dot_args)
    effectsize <- do.call(report_effectsize, call_args)
  }

  as.report_info(attributes(effectsize)$rules)
}


# report_text ------------------------------------------------------------

#' @rdname report.htest
#' @export
report_text.htest <- function(x, table = NULL, ...) {
  dot_args <- list(...)
  model_info <- dot_args$model_info
  if (is.null(model_info)) {
    model_info <- suppressWarnings(insight::model_info(x, verbose = FALSE))
  }

  # remove arg, so dots can be passed to other methods
  dot_args[["model_info"]] <- NULL

  call_args <- c(list(x, table = table, model_info = model_info), dot_args)
  params <- do.call(report_parameters, call_args)

  table <- attributes(params)$table
  call_args <- c(list(x, table = table, model_info = model_info), dot_args)
  model <- do.call(report_model, call_args)

  call_args <- c(
    list(
      x,
      effectsize = attributes(params)$effectsize,
      model_info = model_info
    ),
    dot_args
  )
  info <- do.call(report_info, call_args)

  if (model_info$is_correlation) {
    txt <- paste0(
      "The ",
      model,
      " is ",
      params
    )

    text_full <- paste0(
      info,
      "\n\n",
      txt
    )
  } else {
    text_full <- paste0(
      info,
      "\n\nThe ",
      model,
      " suggests that the effect is ",
      params
    )

    txt <- paste0(
      "The ",
      model,
      " suggests that the effect is ",
      summary(params)
    )
  }

  as.report_text(text_full, summary = txt)
}

# report_table -----------------

.report_table_friedman <- function(table_full, effsize) {
  tab_attr <- attributes(effsize)$table
  if (is.null(tab_attr)) {
    tab_attr <- data.frame()
  }
  table_full <- cbind(table_full, tab_attr)
  list(table = NULL, table_full = table_full)
}


# report_effectsize ---------------------

.report_effectsize_friedman <- function(x, dot_args) {
  param_args <- c(list(x, es_type = "kendalls_w"), dot_args)

  # Build effect-size table, tolerate failures → NULL
  friedman_table <- tryCatch(
    do.call(parameters::parameters, param_args),
    error = function(e) NULL
  )

  # Extract CI level if present
  ci <- if (is.null(friedman_table)) NULL else attributes(friedman_table)$ci

  # Interpretation (guarded)
  interpret_args <- c(
    list(if (is.null(friedman_table)) NA_real_ else friedman_table$Kendalls_W),
    dot_args
  )
  interpretation <- tryCatch(
    do.call(effectsize::interpret_kendalls_w, interpret_args),
    error = function(e) NA_character_
  )
  rules <- .text_effectsize(attr(attr(interpretation, "rules"), "rule_name"))

  # Main estimate text
  main <- paste0(
    "Kendall's W = ",
    insight::format_value(
      if (is.null(friedman_table)) NA_real_ else friedman_table$Kendalls_W
    )
  )

  # Detect CI columns (your table uses W_CI_low/high; be flexible)
  has_cols <- function(x, cols) all(cols %in% names(x))
  has_ci <- !is.null(friedman_table) &&
    !is.null(ci) &&
    (has_cols(friedman_table, c("W_CI_low", "W_CI_high")) ||
      has_cols(friedman_table, c("CI_low", "CI_high")))

  if (has_ci) {
    if (has_cols(friedman_table, c("W_CI_low", "W_CI_high"))) {
      lo <- friedman_table$W_CI_low
      hi <- friedman_table$W_CI_high
    } else {
      lo <- friedman_table$CI_low
      hi <- friedman_table$CI_high
    }

    statistics <- paste0(
      main,
      ", ",
      insight::format_ci(CI_low = lo, CI_high = hi, ci = ci)
    )
  } else {
    statistics <- main
  }

  # Keep only stable columns if table exists
  if (!is.null(friedman_table)) {
    keep <- intersect(
      c("Kendalls_W", "W_CI_low", "W_CI_high", "CI_low", "CI_high"),
      names(friedman_table)
    )
    friedman_table <- friedman_table[keep]
  }

  list(
    table = friedman_table,
    statistics = statistics,
    interpretation = interpretation,
    rules = rules,
    ci = ci,
    main = main
  )
}

# report_model ----------------------------

.report_model_friedman <- function(x, table) {
  # two-sample
  if ("Parameter1" %in% names(table)) {
    vars_full <- paste0(table$Parameter1[[1]], ", and ", table$Parameter2[[1]])

    friedman_text <- paste0(
      trimws(x$method),
      " testing the difference in ranks between ",
      vars_full
    )
  } else {
    # one-sample
    vars_full <- paste0(table$Parameter[[1]])

    friedman_text <- paste0(
      trimws(x$method),
      " testing the difference in rank for ",
      vars_full,
      " and true location of 0"
    )
  }

  friedman_text
}

.report_parameters_friedman <- function(table, stats, effsize, ...) {
  text_full <- paste0(
    "statistically ",
    effectsize::interpret_p(table$p, rules = "default"),
    ", and in ",
    attributes(effsize)$interpretation,
    " (",
    stats,
    ")"
  )

  text_short <- paste0(
    "statistically ",
    effectsize::interpret_p(table$p, rules = "default"),
    ", and in ",
    attributes(effsize)$interpretation,
    " (",
    summary(stats),
    ")"
  )

  list(text_short = text_short, text_full = text_full)
}

#' Reporting Bayesian Models
#'
#' Create reports for Bayesian models. The description of the parameters follows
#' the Sequential Effect eXistence and sIgnificance Testing framework (see
#' [SEXIT documentation][bayestestR::sexit]).
#'
#' @inheritParams report.lm
#' @inherit report return seealso
#'
#' @examplesIf requireNamespace("rstanarm", quietly = TRUE)
#' \donttest{
#' # Bayesian models
#' library(rstanarm)
#' model <- suppressWarnings(stan_glm(mpg ~ qsec + wt, data = mtcars, refresh = 0, iter = 500))
#' r <- report(model)
#' r
#' summary(r)
#' as.data.frame(r)
#' }
#' @return An object of class [report()].
#' @include report.lm.R report.lme4.R
#' @export
report.stanreg <- function(x, ...) {
  stanreg_table <- report_table(x, include_effectsize = FALSE, ...)
  report_text_result <- report_text(x, table = stanreg_table, ...)

  as.report(report_text_result, table = stanreg_table, ...)
}

#' @export
report_effectsize.stanreg <- report_effectsize.lm

#' @export
report_table.stanreg <- report_table.lm

#' @export
report_performance.stanreg <- report_performance.lm

#' @export
report_statistics.stanreg <- report_statistics.lm

#' @export
report_random.stanreg <- report_random.merMod

#' @export
report_model.stanreg <- report_model.lm

#' @export
report_text.stanreg <- report_text.lm


# ==================== Specific to Bayes ===================================

# report_priors -----------------------------------------------------------

#' @export
report_priors.stanreg <- function(x, ...) {
  params <- bayestestR::describe_prior(x)
  params <- params[params$Parameter != "(Intercept)", ]

  # Return empty if no priors info
  if (!"Prior_Distribution" %in% names(params) || nrow(params) == 0) {
    return("")
  }

  values <- ifelse(
    params$Prior_Distribution == "normal",
    paste0(
      "mean = ",
      insight::format_value(params$Prior_Location),
      ", SD = ",
      insight::format_value(params$Prior_Scale)
    ),
    paste0(
      "location = ",
      insight::format_value(params$Prior_Location),
      ", scale = ",
      insight::format_value(params$Prior_Scale)
    )
  )

  values <- paste(values, collapse = "; ")
  values <- paste0(params$Prior_Distribution, " (", values, ")")

  if (length(unique(values)) == 1 && nrow(params) > 1) {
    priors_text <- paste0("all set as ", values[1])
  } else {
    priors_text <- paste0("set as ", values)
  }

  priors_text <- paste0(
    "Priors over parameters were ",
    priors_text,
    " distributions"
  )
  as.report_priors(priors_text)
}


# report_parameters -------------------------------------------------------

#' @export
report_parameters.stanreg <- function(
  x,
  include_intercept = TRUE,
  include_diagnostic = TRUE,
  ...
) {
  # Get data
  sexit_data <- bayestestR::sexit(x, ...)

  att <- attributes(sexit_data)
  info <- paste0(att$sexit_info, " ", att$sexit_thresholds)

  params <- as.data.frame(sexit_data)

  # Parameters' names
  param_text <- .parameters_starting_text(x, params)

  # Replace parameters names
  for (i in seq_along(param_text)) {
    att$sexit_textlong[i] <- gsub(
      names(param_text)[i],
      param_text[i],
      att$sexit_textlong[i],
      fixed = TRUE
    )
    att$sexit_textshort[i] <- gsub(
      names(param_text)[i],
      param_text[i],
      att$sexit_textshort[i],
      fixed = TRUE
    )
  }

  # Include intercept
  if (isFALSE(include_intercept)) {
    idx <- params$Parameter != "(Intercept)"
  } else {
    idx <- rep(TRUE, nrow(params))
  }

  param_text <- att$sexit_textshort[idx]
  param_text_full <- att$sexit_textlong[idx]

  # remove NAs...
  param_text <- param_text[!is.na(param_text)]
  param_text_full <- param_text_full[!is.na(param_text_full)]

  # Diagnostic or Convergence
  if (include_diagnostic) {
    diagnostic <- bayestestR::diagnostic_posterior(x, ...)

    param.dgn <- .parameters_diagnostic_bayesian(
      diagnostic,
      only_when_insufficient = TRUE
    )[idx]
    param_text <- datawizard::text_paste(param_text, param.dgn, sep = ". ")

    param.dgn <- .parameters_diagnostic_bayesian(
      diagnostic,
      only_when_insufficient = FALSE
    )[idx]
    param_text_full <- datawizard::text_paste(
      param_text_full,
      param.dgn,
      sep = ". "
    )

    info <- paste(
      info,
      "Convergence and stability of the Bayesian sampling has been assessed using R-hat,",
      "which should be below 1.01 (Vehtari et al., 2019),",
      "and Effective Sample Size (ESS), which should be greater than 1000 (Burkner, 2017)."
    )
  }

  as.report_parameters(
    param_text_full,
    summary = param_text,
    parameters = sexit_data,
    info = info,
    ...
  )
}


# report_intercept --------------------------------------------------------

#' @export
report_intercept.stanreg <- function(x, ...) {
  posteriors <- insight::get_parameters(x)

  if (!("(Intercept)" %in% names(posteriors))) {
    return(as.report_intercept("", summary = "", ...))
  }

  intercept <- posteriors[["(Intercept)"]]

  intercept_data <- bayestestR::sexit(intercept, ...)

  endtext <- paste0(
    " is at ",
    insight::format_value(intercept_data$Median),
    " (",
    insight::format_ci(
      intercept_data$CI_low,
      intercept_data$CI_high,
      ci = intercept_data$CI
    ),
    ")."
  )

  intercept_text <- paste0("The model's intercept", endtext)
  intercept_text_full <- paste0(
    "The model's intercept",
    .find_intercept(x),
    endtext
  )

  as.report_intercept(intercept_text_full, summary = intercept_text, ...)
}


# report_info -------------------------------------------------------------

#' @export
report_info.stanreg <- function(x, parameters = NULL, ...) {
  if (is.null(parameters)) {
    parameters <- report_parameters(x, ...)
  }

  info_text <- attributes(parameters)$info

  as.report_info(info_text)
}

#' @include report.lm.R
#' @export
report.glm <- report.lm

#' @export
report_effectsize.glm <- report_effectsize.lm

#' @export
report_table.glm <- report_table.lm

#' @export
report_statistics.glm <- report_statistics.lm

#' @export
report_parameters.glm <- report_parameters.lm

#' @export
report_intercept.glm <- report_intercept.lm

#' @export
report_model.glm <- report_model.lm

#' @export
report_performance.glm <- report_performance.lm

#' @export
report_info.glm <- report_info.lm

#' @export
report_text.glm <- report_text.lm

# report_parameters -----------------

.report_parameters_correlation <- function(table, stats, ...) {
  text_full <- paste0(
    effectsize::interpret_direction(attributes(stats)$estimate),
    ", statistically ",
    effectsize::interpret_p(table$p, rules = "default"),
    ", and ",
    effectsize::interpret_r(attributes(stats)$estimate, ...),
    " (",
    stats,
    ")"
  )

  text_short <- text_full
  list(text_short = text_short, text_full = text_full)
}

# report_table -----------------

.report_table_correlation <- function(table_full) {
  cor_table <- datawizard::data_remove(table_full, c("t", "df_error"))
  list(table = cor_table, table_full = table_full)
}


# report_effectsize ---------------------

.report_effectsize_correlation <- function(x, dot_args) {
  param_args <- c(list(x), dot_args)
  cor_table <- do.call(parameters::parameters, param_args)
  ci <- attributes(cor_table)$ci
  estimate <- names(cor_table)[3]

  # Pearson
  interpret_args <- c(list(cor_table[[estimate]]), dot_args)
  interpretation <- do.call(effectsize::interpret_r, interpret_args)
  rules <- .text_effectsize(attr(attr(interpretation, "rules"), "rule_name"))
  main <- paste0(estimate, " = ", insight::format_value(cor_table[[estimate]]))

  if ("CI_low" %in% names(cor_table)) {
    statistics <- paste0(
      main,
      ", ",
      insight::format_ci(cor_table$CI_low, cor_table$CI_high, ci)
    )

    result_table <- cor_table[c(estimate, "CI_low", "CI_high")]

    # For Spearman and co.
  } else {
    statistics <- main
    result_table <- cor_table[estimate]
  }

  list(
    table = result_table,
    statistics = statistics,
    interpretation = interpretation,
    rules = rules,
    ci = ci,
    main = main
  )
}

#' Report random effects and factors
#'
#' Reports random effects of mixed models (see list of supported objects in
#' [report()]).
#'
#' @inheritParams report
#' @inheritParams report_table
#' @inheritParams report_text
#' @inheritParams as.report
#'
#' @return An object of class [report_random()].
#'
#' @examplesIf requireNamespace("lme4", quietly = TRUE)
#' \donttest{
#' # Mixed models
#' library(lme4)
#' model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
#' r <- report_random(model)
#' r
#' summary(r)
#' }
#'
#' @examplesIf requireNamespace("rstanarm", quietly = TRUE)
#' \donttest{
#' # Bayesian models
#' library(rstanarm)
#' model <- suppressWarnings(stan_lmer(
#'   mpg ~ disp + (1 | cyl),
#'   data = mtcars, refresh = 0, iter = 1000
#' ))
#' r <- report_random(model)
#' r
#' summary(r)
#' }
#'
#' @examplesIf requireNamespace("brms", quietly = TRUE) && packageVersion("rstan") >= "2.26.0"
#' \donttest{
#' library(brms)
#' model <- suppressWarnings(brm(mpg ~ disp + (1 | cyl), data = mtcars, refresh = 0, iter = 1000))
#' r <- report_random(model)
#' r
#' summary(r)
#' }
#' @export
report_random <- function(x, ...) {
  UseMethod("report_random")
}


# METHODS -----------------------------------------------------------------

#' @rdname as.report
#' @export
as.report_random <- function(x, summary = NULL, ...) {
  class(x) <- unique(c("report_random", class(x)))
  attributes(x) <- c(attributes(x), list(...))

  if (!is.null(summary)) {
    attr(x, "summary") <- summary
  }
  x
}


#' @export
summary.report_random <- function(object, ...) {
  if (is.null(attributes(object)$summary)) {
    object
  } else {
    attributes(object)$summary
  }
}

#' @export
print.report_random <- function(x, ...) {
  cat(paste(x, collapse = "\n"))
}

skip_if_not_installed("nlme")
skip_on_cran() # nlme mixed-effects models can be computationally intensive

test_that("report.lme", {
  # Create a test nlme model
  suppressWarnings({
    model <- nlme::lme(
      Sepal.Length ~ Petal.Length,
      data = iris,
      random = ~ 1 | Species
    )
  })

  # Test that model was created successfully
  expect_s3_class(model, "lme")

  # Test specific lme methods that should work without standardization issues
  rr <- report_random(model)
  expect_s3_class(rr, c("report_random", "character"))
  expect_true(grepl("random effect", rr, fixed = TRUE))
  expect_true(grepl("Species", rr, fixed = TRUE))

  # Test that report.lme method is properly available
  expect_true("report.lme" %in% ls(asNamespace("report")))

  # Test that it works as intended (aliased to report.lm)
  expect_identical(
    get("report.lme", asNamespace("report")),
    get("report.lm", asNamespace("report"))
  )
})

# returns the row-indices for grouped data frames
#' @keywords internal
.group_indices <- function(x) {
  # dplyr < 0.8.0 returns attribute "indices"
  grps <- attr(x, "groups", exact = TRUE)

  # dplyr < 0.8.0?
  if (is.null(grps)) {
    attr(x, "indices", exact = TRUE)
  } else {
    grps[[".rows"]]
  }
}


# returns the variables that were used for grouping data frames (dplyr::group_var())
#' @keywords internal
.group_vars <- function(x) {
  # dplyr < 0.8.0 returns attribute "indices"
  grps <- attr(x, "groups", exact = TRUE)

  # dplyr < 0.8.0?
  if (is.null(grps)) {
    ## TODO fix for dplyr < 0.8
    attr(x, "vars", exact = TRUE)
  } else {
    setdiff(colnames(grps), ".rows")
  }
}


# returns the 'drop' setting for groups on the data frame
#' @keywords internal
.groups_drop <- function(x) {
  !isFALSE(attr(attr(x, "groups"), ".drop"))
}


# returns if the data frame has groups
#' @keywords internal
.has_groups <- function(x) {
  if (length(.group_vars(x)) == 0L) FALSE else TRUE
}


# recompute group indices grouped_df, used after subsetting
#' @keywords internal
.calculate_groups <- function(x, groups, drop = .groups_drop(x)) {
  # if the dplyr namespace is attached, its `[.grouped_df` method produces
  # erroneous warnings and coerces output to tbl_df
  subset_data <- `[.data.frame`
  x <- .ungroup(x)
  unknown <- setdiff(groups, colnames(x))

  if (length(unknown) > 0L) {
    stop(
      insight::format_message(
        sprintf("`groups` missing from `x`: %s.", toString(groups))
      ),
      call. = FALSE
    )
  }

  unique_groups <- unique(x[, groups, drop = FALSE])
  is_factor <- do.call(c, lapply(unique_groups, is.factor))
  n_comb <- nrow(unique_groups)
  rows <- rep(list(NA), n_comb)

  for (i in seq_len(n_comb)) {
    rows[[i]] <- which(
      interaction(x[, groups, drop = TRUE]) %in%
        interaction(unique_groups[i, groups])
    )
  }

  if (!isTRUE(drop) && any(is_factor)) {
    na_lvls <- do.call(
      expand.grid,
      lapply(
        unique_groups,
        function(x) {
          if (is.factor(x)) {
            levels(x)[!(levels(x) %in% x)]
          } else {
            NA
          }
        }
      )
    )
    unique_groups <- rbind(unique_groups, na_lvls)
    for (i in seq_len(nrow(na_lvls))) {
      rows[[length(rows) + 1]] <- integer(0)
    }
  }

  unique_groups[[".rows"]] <- rows
  unique_groups <- unique_groups[
    do.call(
      order,
      lapply(groups, function(x) unique_groups[, x])
    ),
    ,
    drop = FALSE
  ]

  rownames(unique_groups) <- NULL

  structure(unique_groups, .drop = drop)
}


# ungroup data frame
#' @keywords internal
.ungroup <- function(x) {
  attr(x, "groups") <- NULL
  class(x) <- setdiff(class(x), "grouped_df")
  x
}


# re-sets the groups for a grouped_df, used after subsetting
#' @keywords internal
.groups_set <- function(x, groups, drop = .groups_drop(x)) {
  attr(x, "groups") <- if (is.null(groups) || length(groups) == 0L) {
    NULL
  } else {
    .calculate_groups(x, groups, drop)
  }
  x
}

test_that(".error_message() generates proper error messages", {
  # Test with a simple object
  obj <- data.frame(x = 1:5)
  result <- report:::.error_message(obj)

  expect_type(result, "character")
  expect_match(result, "objects of class")
  expect_match(result, "data.frame")
  expect_match(result, "not supported")
  expect_match(result, "report\\(\\)")
  expect_match(result, "https://easystats.github.io/report")
})

test_that(".error_message() works with custom function names", {
  obj <- list(a = 1, b = 2)
  result <- report:::.error_message(obj, fun = "custom_function()")

  expect_match(result, "custom_function\\(\\)")
  expect_match(result, "list")
})

test_that(".error_message() handles multiple classes", {
  obj <- structure(1:5, class = c("custom1", "custom2", "numeric"))
  result <- report:::.error_message(obj)

  expect_match(result, "custom1")
  expect_match(result, "custom2")
  expect_match(result, "numeric")
})

test_that(".combine_tables_effectsize() combines tables correctly", {
  # Create mock parameters table
  parameters <- data.frame(
    Parameter = c("(Intercept)", "x"),
    Coefficient = c(1.5, 2.0),
    SE = c(0.2, 0.3),
    stringsAsFactors = FALSE
  )
  class(parameters) <- c("parameters_model", "data.frame")
  attr(parameters, "pretty_names") <- c(`(Intercept)` = "(Intercept)", x = "x")

  # Create mock effect size table
  effsize_table <- data.frame(
    Parameter = c("(Intercept)", "x"),
    Std_Coefficient = c(0.8, 1.2),
    stringsAsFactors = FALSE
  )

  # Create mock effectsize object with table attribute
  effsize <- list()
  attr(effsize, "table") <- effsize_table

  # Test combination
  result <- report:::.combine_tables_effectsize(parameters, effsize)

  expect_s3_class(result, "parameters_model")
  expect_true("Std_Coefficient" %in% names(result))
  expect_identical(nrow(result), 2L)
  expect_identical(result$Parameter, c("(Intercept)", "x"))
})

test_that(".combine_tables_performance() combines tables correctly", {
  # Create mock parameters table
  parameters <- data.frame(
    Parameter = c("(Intercept)", "x"),
    Coefficient = c(1.5, 2.0),
    stringsAsFactors = FALSE
  )
  class(parameters) <- c("parameters_model", "data.frame")

  # Create mock performance table
  performance <- data.frame(
    R2 = 0.75,
    R2_adjusted = 0.72,
    AIC = 150.2,
    stringsAsFactors = FALSE
  )

  # Test combination
  result <- report:::.combine_tables_performance(parameters, performance)

  expect_s3_class(result, "parameters_model")
  expect_true("Fit" %in% names(result))
  expect_gt(nrow(result), nrow(parameters)) # Should have added performance rows
  expect_true(anyNA(result$Coefficient)) # NA row should exist
})

test_that(".remove_performance() removes performance rows correctly", {
  # Create table with performance information
  table_with_perf <- data.frame(
    Parameter = c("(Intercept)", "x", NA, "R2", "AIC"),
    Coefficient = c(1.5, 2.0, NA, NA, NA),
    Fit = c(NA, NA, NA, 0.75, 150.2),
    stringsAsFactors = FALSE
  )

  result <- report:::.remove_performance(table_with_perf)

  expect_identical(nrow(result), 2L) # Should only keep parameter rows
  expect_identical(result$Parameter, c("(Intercept)", "x"))
  expect_false(anyNA(result$Parameter))
})

test_that(".remove_performance() handles tables without Fit column", {
  # Create table without performance information
  table_no_perf <- data.frame(
    Parameter = c("(Intercept)", "x"),
    Coefficient = c(1.5, 2.0),
    stringsAsFactors = FALSE
  )

  result <- report:::.remove_performance(table_no_perf)

  expect_identical(result, table_no_perf) # Should return unchanged
})

test_that(".check_spelling() works correctly", {
  data <- data.frame(name = 1:3, value = 4:6, score = 7:9)

  # Test with correct column names - should not error
  expect_invisible(report:::.check_spelling(data, c("name", "value")))

  # Test with incorrect column names - might not error in current implementation
  # This function may just return suggestions rather than throw errors
  result <- tryCatch(
    {
      report:::.check_spelling(data, c("nam", "values"))
      "no_error"
    },
    error = function(e) e$message
  )
  # Function may or may not error, but it should handle the case
  expect_type(result, "character")

  # Test with NULL input - should not error
  expect_invisible(report:::.check_spelling(data, NULL))

  # Test with empty string - should not error
  expect_invisible(report:::.check_spelling(data, ""))
})

test_that(".fuzzy_grep() finds approximate matches", {
  test_names <- c(
    "Sepal.Length",
    "Sepal.Width",
    "Petal.Length",
    "Petal.Width",
    "Species"
  )

  # Test with close match
  result1 <- report:::.fuzzy_grep(test_names, "Spela")
  expect_gt(length(result1), 0) # Should find matches

  # Test with exact match
  result2 <- report:::.fuzzy_grep(test_names, "Species")
  expect_true(5 %in% result2) # Should find exact match

  # Test with very different string
  result3 <- report:::.fuzzy_grep(test_names, "xyz")
  expect_length(result3, 0) # Should find no matches

  # Test with custom precision
  result4 <- report:::.fuzzy_grep(test_names, "Sepal", precision = 1)
  expect_gt(length(result4), 0)
})

test_that(".misspelled_string() creates helpful error messages", {
  source_names <- c(
    "Sepal.Length",
    "Sepal.Width",
    "Petal.Length",
    "Petal.Width",
    "Species"
  )

  # Test with likely misspelling
  result1 <- report:::.misspelled_string(source_names, "Spela")
  expect_match(result1, "Did you mean")
  expect_match(result1, "Sepal")

  # Test with multiple possibilities
  result2 <- report:::.misspelled_string(source_names, "Length")
  expect_match(result2, "one of")

  # Test with no close matches - function may return various types
  result3 <- report:::.misspelled_string(source_names, "xyz")
  # Just test that function executes without error and returns something reasonable
  expect_true(is.character(result3) || is.null(result3))

  # Test with default message
  result4 <- report:::.misspelled_string(source_names, "xyz", "Default message")
  expect_identical(result4, "Default message")
})

test_that("grouped dataframe utilities work correctly", {
  # Skip if dplyr not available
  skip_if_not_installed("dplyr")

  # Create a grouped dataframe
  df <- data.frame(
    group = rep(c("A", "B"), each = 3),
    value = 1:6,
    stringsAsFactors = FALSE
  )
  grouped_df <- dplyr::group_by(df, group)

  # Test group detection
  expect_true(report:::.has_groups(grouped_df))
  expect_false(report:::.has_groups(df))

  # Test group variables
  group_vars <- report:::.group_vars(grouped_df)
  expect_identical(group_vars, "group")

  # Test ungrouping
  ungrouped <- report:::.ungroup(grouped_df)
  expect_false(report:::.has_groups(ungrouped))
  expect_false(inherits(ungrouped, "grouped_df"))
})

# report_parameters -----------------

.report_parameters_ttest <- function(table, stats, effsize, ...) {
  text_full <- paste0(
    effectsize::interpret_direction(attributes(stats)$estimate),
    ", statistically ",
    effectsize::interpret_p(table$p, rules = "default"),
    ", and ",
    attributes(effsize)$interpretation,
    " (",
    stats,
    ")"
  )

  text_short <- paste0(
    effectsize::interpret_direction(attributes(stats)$estimate),
    ", statistically ",
    effectsize::interpret_p(table$p, rules = "default"),
    ", and ",
    attributes(effsize)$interpretation,
    " (",
    summary(stats),
    ")"
  )

  list(text_short = text_short, text_full = text_full)
}

.report_parameters_htest_default <- .report_parameters_ttest


# report_table -----------------

.report_table_ttest <- function(table_full, effsize) {
  table_full <- cbind(table_full, attributes(effsize)$table)
  table_small <- datawizard::data_remove(
    table_full,
    c(
      "Parameter",
      "Group",
      "Mean_Group1",
      "Mean_Group2",
      "Method",
      "d_CI_low",
      "d_CI_high"
    )
  )
  list(table = table_small, table_full = table_full)
}


# report_effectsize ---------------------

.report_effectsize_ttest <- function(x, dot_args, type, rules = "cohen1988") {
  es_args <- c(list(x), dot_args)
  es_table <- do.call(effectsize::effectsize, es_args)
  ci <- attributes(es_table)$ci
  estimate <- names(es_table)[1]
  dot_args$rules <- if (is.null(dot_args$rules)) rules else dot_args$rules

  es_args <- c(list(es_table), dot_args)
  interpretation <- do.call(effectsize::interpret, es_args)$Interpretation
  rules <- .text_effectsize(attr(attr(interpretation, "rules"), "rule_name"))

  if (estimate %in% c("d", "Cohens_d")) {
    main <- paste0("Cohen's d = ", insight::format_value(es_table[[estimate]]))
  } else if (estimate %in% c("g", "Hedges_g")) {
    main <- paste0("Hedges's g = ", insight::format_value(es_table[[estimate]]))
  } else {
    main <- paste0(estimate, " = ", insight::format_value(es_table[[estimate]]))
  }

  statistics <- paste0(
    main,
    ", ",
    insight::format_ci(es_table$CI_low, es_table$CI_high, ci)
  )

  result_table <- datawizard::data_rename(
    as.data.frame(es_table),
    select = c("CI_low", "CI_high"),
    replacement = paste0(estimate, c("_CI_low", "_CI_high"))
  )

  result_table <- result_table[c(
    estimate,
    paste0(estimate, c("_CI_low", "_CI_high"))
  )]

  list(
    table = result_table,
    statistics = statistics,
    interpretation = interpretation,
    rules = rules,
    ci = ci,
    main = main
  )
}


# report model ---------------------------

.report_model_ttest <- function(x, table) {
  # If against mu
  if (names(x$null.value) == "mean") {
    # TODO: @DominiqueMakowski why do we need "table" here??

    table$Difference <- x$estimate - x$null.value
    means <- paste0(" (mean = ", insight::format_value(x$estimate), ")")
    vars_full <- paste0(x$data.name, means, " and mu = ", x$null.value)
    vars <- paste0(x$data.name, " and mu = ", x$null.value)

    # If between two groups
  } else {
    table$Difference <- x$estimate[1] - x$estimate[2]
    means <- paste0(
      names(x$estimate),
      " = ",
      insight::format_value(x$estimate),
      collapse = ", "
    )
    vars_full <- paste0(x$data.name, " (", means, ")")
    vars <- paste0(x$data.name)
  }

  final_text <- paste0(
    trimws(x$method),
    " testing the difference ",
    ifelse(grepl(" by ", x$data.name, fixed = TRUE), "of ", "between "),
    vars_full
  )

  final_text
}

#' Report R environment (packages, system, etc.)
#'
#' @inheritParams report
#' @inheritParams as.report_parameters
#' @param session A [sessionInfo][utils::sessionInfo] object.
#' @param include_R Include R in the citations.
#'
#' @return
#'
#'  For `report_packages`, a data frame of class with information on package
#'  name, version and citation.
#'
#' @examples
#'
#' library(report)
#'
#' session <- sessionInfo()
#'
#' r <- report(session)
#' r
#' summary(r)
#' as.data.frame(r)
#' summary(as.data.frame(r))
#'
#' # Convenience functions
#' report_packages(include_R = FALSE)
#' cite_packages(prefix = "> ")
#' report_system()
#'
#' @return An object of class [report()].
#' @export
report.sessionInfo <- function(x, ...) {
  session_table <- report_table(x, ...)
  session_text <- report_text(x, table = session_table)

  as.report(session_text, table = session_table, ...)
}


# Aliases -----------------------------------------------------------------

#' @rdname report.sessionInfo
#' @export
report_packages <- function(session = NULL, include_R = TRUE, ...) {
  if (is.null(session)) {
    session <- utils::sessionInfo()
  }
  report_parameters(session, include_R = include_R, ...)
}


#' @rdname report.sessionInfo
#' @export
cite_packages <- function(session = NULL, include_R = TRUE, ...) {
  if (is.null(session)) {
    session <- utils::sessionInfo()
  }

  # Do not recompute table if passed
  if (is.null(list(...)$table)) {
    x <- report_table(session, include_R = include_R)
  } else {
    x <- list(...)$table
  }

  x <- sort(x$Reference, na.last = TRUE) # Extract the references

  as.report_parameters(x, ...)
}


# report_system --------------------------------------------------------------

#' @rdname report.sessionInfo
#' @export
report_system <- function(session = NULL) {
  if (is.null(session)) {
    session <- utils::sessionInfo()
  }

  r_version <- paste0(session$R.version$major, ".", session$R.version$minor)
  year <- paste0(
    session$R.version$year,
    "-",
    session$R.version$month,
    "-",
    session$R.version$day
  )
  r_citation <- format_citation(
    clean_citation(utils::citation()),
    authorsdate = TRUE,
    intext = TRUE
  )

  result_text <- paste0(
    "Analyses were conducted using the R Statistical language (version ",
    r_version,
    "; ",
    r_citation,
    ") on ",
    session$running
  )

  short <- paste0(
    "The analysis was done using the R Statistical language (v",
    r_version,
    "; ",
    r_citation,
    ") on ",
    datawizard::text_remove(session$running, " \\(build.*")
  )

  as.report_text(result_text, summary = short)
}


# report_table ------------------------------------------------------------

#' @export
report_table.sessionInfo <- function(x, include_R = TRUE, ...) {
  pkgs <- x$otherPkgs

  if (isTRUE(include_R)) {
    citations <- clean_citation(utils::citation("base"))
    versions <- paste0(x$R.version$major, ".", x$R.version$minor)
    pkg_names <- "R"
  } else {
    citations <- NULL
    versions <- NULL
    pkg_names <- NULL
  }

  for (pkg_name in names(pkgs)) {
    citations <- c(citations, clean_citation(utils::citation(pkg_name)))
    versions <- c(versions, as.character(utils::packageVersion(pkg_name)))
    pkg_names <- c(pkg_names, pkg_name)
  }

  session_data <- data.frame(
    Package = pkg_names,
    Version = versions,
    Reference = citations,
    stringsAsFactors = FALSE
  )

  x <- session_data[order(session_data$Package), ]
  row.names(x) <- NULL
  as.report_table(x, summary = x[c("Package", "Version")])
}


# report_parameters -------------------------------------------------------

#' @export
report_parameters.sessionInfo <- function(
  x,
  table = NULL,
  include_R = TRUE,
  ...
) {
  # Get table
  if (is.null(table)) {
    x <- report_table(x)
  } else {
    x <- table
  }

  if (isFALSE(include_R)) {
    x <- x[x$Package != "R", ]
  }

  # Generate text
  x$text <- paste0(
    x$Package,
    " (version ",
    x$Version,
    "; ",
    format_citation(
      x$Reference,
      authorsdate = TRUE,
      short = TRUE,
      intext = TRUE
    ),
    ")"
  )

  x$summary <- paste0(x$Package, " (v", x$Version, ")")

  x <- x[order(x$Reference), ]

  params <- x$text
  names(params) <- x$Package
  short <- x$summary
  names(short) <- x$Package

  as.report_parameters(params, summary = short, ...)
}


# report_text -------------------------------------------------------------

#' @export
report_text.sessionInfo <- function(x, table = NULL, ...) {
  sys <- report_system(x)
  params <- report_parameters(x, table = table, include_R = FALSE)

  result_text <- paste0(
    sys,
    ", using the packages ",
    datawizard::text_concatenate(params),
    ".\n\nReferences\n----------\n",
    as.character(cite_packages(x, table = table, ...))
  )

  short <- paste0(
    summary(sys),
    ", using the packages ",
    datawizard::text_concatenate(summary(params)),
    "."
  )

  as.report_text(result_text, summary = short)
}

test_that("report.sessionInfo - High level", {
  x <- sessionInfo()
  r <- report(x)

  expect_type(r, "character")
  expect_type(summary(r), "character")
  expect_equal(as.report_text(r), r, ignore_attr = TRUE)
  expect_identical(as.report_text(r, summary = TRUE), summary(r))

  expect_equal(ncol(as.data.frame(r)), 3, ignore_attr = TRUE)
  expect_equal(ncol(as.report_table(r)), 3, ignore_attr = TRUE)
  expect_equal(ncol(summary(as.data.frame(r))), 2, ignore_attr = TRUE)
  expect_equal(ncol(as.report_table(r, summary = TRUE)), 2, ignore_attr = TRUE)
})


test_that("report.sessionInfo - Core", {
  x <- sessionInfo()

  r <- report_table(x)
  expect_equal(ncol(r), 3, ignore_attr = TRUE)
  expect_equal(ncol(summary(r)), 2, ignore_attr = TRUE)

  r <- report_parameters(x)
  expect_type(r, "character")
  expect_length(as.character(r), 1)
  expect_length(as.character(r), 1)
  expect_length(as.character(summary(r)), 1)

  r <- report_text(x)
  expect_type(r, "character")
  expect_type(summary(r), "character")
})


test_that("report.sessionInfo - Aliases", {
  x <- sessionInfo()
  expect_type(cite_packages(x), "character")
  expect_type(cite_packages(x, prefix = ""), "character")

  expect_type(report_system(x), "character")
  expect_type(summary(report_system(x)), "character")

  expect_type(report_packages(x), "character")
  expect_type(summary(report_packages(x)), "character")
})

#' Automatic reporting of R objects
#'
#' Create reports of different objects. See the documentation for your object's
#' class:
#'
#'  - [System and packages][report.sessionInfo] (`sessionInfo`)
#'  - [Dataframes and vectors][report.data.frame]
#'  - [Correlations and t-tests][report.htest] (`htest`)
#'  - [ANOVAs][report.aov] (`aov, anova, aovlist, ...`)
#'  - [Regression models][report.lm] (`glm, lm, ...`)
#'  - [Mixed models][report.lm] (`glmer, lmer, glmmTMB, ...`)
#'  - [Bayesian models][report.stanreg] (`stanreg, brms...`)
#'  - [Bayes factors][report.bayesfactor_models] (from `bayestestR`)
#'  - [Structural Equation Models (SEM)][report.lavaan] (from `lavaan`)
#'  - [Model comparison][report.compare_performance] (from [`performance()`][performance::compare_performance])
#'
#' Most of the time, the object created by the `report()` function can be
#' further transformed, for instance summarized (using `summary()`), or
#' converted to a table (using `as.data.frame()`).
#'
#' @param x The R object that you want to report (see list of of supported
#'   objects above).
#' @param ... Arguments passed to or from other methods.
#'
#' @details
#'
#' \subsection{Organization}{
#' `report_table` and `report_text` are the two distal representations
#' of a report, and are the two provided in `report()`. However,
#' intermediate steps are accessible (depending on the object) via specific
#' functions (e.g., `report_parameters`).
#' }
#'
#' \subsection{Output}{
#'
#' The `report()` function generates a report-object that contain in itself
#' different representations (e.g., text, tables, plots). These different
#' representations can be accessed via several functions, such as:
#'
#' - **`as.report_text(r)`**: Detailed text.
#'
#' - **`as.report_text(r, summary=TRUE)`**: Minimal text giving
#' the minimal information.
#'
#' - **`as.report_table(r)`**: Comprehensive table including most
#' available indices.
#'
#' - **`as.report_table(r, summary=TRUE)`**: Minimal table.
#'
#' Note that for some report objects, some of these representations might be
#' identical.
#' }
#'
#' @return A list-object of class `report`, which contains further
#'   list-objects with a short and long description of the model summary, as
#'   well as a short and long table of parameters and fit indices.
#'
#' @seealso Specific components of reports (especially for stats models):
#'   - [report_table()]
#'   - [report_parameters()]
#'   - [report_statistics()]
#'   - [report_effectsize()]
#'   - [report_model()]
#'   - [report_priors()]
#'   - [report_random()]
#'   - [report_performance()]
#'   - [report_info()]
#'   - [report_text()]
#'
#' Other types of reports:
#'   - [report_system()]
#'   - [report_packages()]
#'   - [report_participants()]
#'   - [report_sample()]
#'   - [report_date()]
#'
#' Methods:
#'   - [as.report()]
#'
#' Template file for supporting new models:
#'
#'   - [report.default()]
#'
#' @examples
#'
#' library(report)
#'
#' model <- t.test(mtcars$mpg ~ mtcars$am)
#' r <- report(model)
#'
#' # Text
#' r
#' summary(r)
#'
#' # Tables
#' as.data.frame(r)
#' summary(as.data.frame(r))
#'
#' @export
report <- function(x, ...) {
  UseMethod("report")
}


# Generic Methods --------------------------------------------------

#' @export
print.report <- print.report_text


#' @include report_text.R
#' @export
as.data.frame.report <- function(x, ...) {
  as.report_table(x, ...)
}

#' Create or test objects of class [report].
#'
#' Allows to create or test whether an object is of the `report` class.
#'
#' @param x An arbitrary R object.
#' @param text Text obtained via `report_text()`
#' @param table Table obtained via `report_table()`
#' @param plot Plot obtained via `report_plot()`. Not yet implemented.
#' @param summary Add a summary as attribute (to be extracted via `summary()`).
#' @param prefix The prefix to be displayed in front of each parameter.
#' @param ... Args to be saved as attributes.
#'
#' @return A report object or a `TRUE/FALSE` value.
#'
#' @export
as.report <- function(text, table = NULL, plot = NULL, ...) {
  class(text) <- unique(c("report", class(text)))
  attributes(text) <- c(attributes(text), list(...))

  if (!is.null(table)) {
    attr(text, "table") <- table
  }

  text
}


#' @rdname as.report
#' @export
is.report <- function(x) inherits(x, "report")

skip_if_not_installed("ivreg")

test_that("report-survreg", {
  data("CigaretteDemand", package = "ivreg")

  # model
  set.seed(123)
  ivr <-
    ivreg::ivreg(
      log(packs) ~ log(rprice) + log(rincome) | salestax + log(rincome),
      data = ivreg::CigaretteDemand
    )

  expect_snapshot(variant = "windows", report(ivr))
})

test_that("report.htest-chi2 report", {
  m <- as.table(rbind(c(762, 327, 468), c(484, 239, 477)))
  dimnames(m) <- list(
    gender = c("F", "M"),
    party = c("Democrat", "Independent", "Republican")
  )
  x <- chisq.test(m)

  expect_snapshot(
    variant = "windows",
    report(x)
  )

  # Rules
  expect_snapshot(
    variant = "windows",
    report(x, rules = "funder2019")
  )

  expect_snapshot(
    variant = "windows",
    report(x, rules = "gignac2016")
  )

  expect_snapshot(
    variant = "windows",
    report(x, rules = "cohen1988")
  )

  expect_snapshot(
    variant = "windows",
    report(x, rules = "evans1996")
  )

  expect_snapshot(
    variant = "windows",
    report(x, rules = "lovakov2021")
  )

  # Types
  expect_snapshot(
    variant = "windows",
    report(x, type = "cramers_v")
  )

  expect_snapshot(
    variant = "windows",
    report(x, type = "pearsons_c")
  )

  expect_snapshot(
    variant = "windows",
    report(x, type = "tschuprows_t", adjust = FALSE)
  )

  expect_snapshot(
    variant = "windows",
    report(x, type = "tschuprows_t")
  )

  expect_snapshot(
    variant = "windows",
    report(x, type = "cohens_w")
  )

  # Change dataset for "Error: Phi is not appropriate for non-2x2 tables."
  dat <- structure(
    c(71, 50, 30, 100),
    dim = c(2L, 2L),
    dimnames = list(
      Diagnosis = c("Sick", "Recovered"),
      Group = c("Treatment", "Control")
    ),
    class = "table"
  )
  x <- chisq.test(dat)

  expect_snapshot(
    variant = "windows",
    report(x, type = "phi")
  )

  expect_snapshot(
    variant = "windows",
    report(x, type = "cohens_h", rules = "sawilowsky2009")
  )

  expect_snapshot(
    variant = "windows",
    report(x, type = "riskratio")
  ) # riskratio has no interpretation in effectsize
  # Watch carefully in case effectsize adds support

  # Mattan example comparison
  x <- suppressWarnings(chisq.test(mtcars$cyl, mtcars$am))

  expect_snapshot(
    variant = "windows",
    report(x)
  )
})

test_that("report.htest-chi2 for given probabilities", {
  # Mattan example comparison
  x <- suppressWarnings(chisq.test(table(mtcars$cyl), p = c(0.1, 0.3, 0.6)))

  expect_snapshot(
    variant = "windows",
    report(x)
  )
})

skip_if_not_installed("BayesFactor")
skip_on_cran() # BayesFactor tests can be computationally intensive

test_that("report.BFBayesFactor - t-test", {
  # Test BFBayesFactor objects from BayesFactor package
  set.seed(123)
  rez <- BayesFactor::ttestBF(iris$Sepal.Width, iris$Sepal.Length)

  r <- report(rez)
  expect_type(r, "character")
  expect_true(nzchar(r))
  expect_true(grepl("There is", r, fixed = TRUE))

  # Test report_statistics
  stats <- report_statistics(rez)
  expect_type(stats, "character")
  expect_true(grepl("BF", stats, fixed = TRUE))
})

test_that("report.BFBayesFactor - correlation", {
  skip_if_not_installed("BayesFactor")

  set.seed(123)
  rez <- BayesFactor::correlationBF(iris$Sepal.Width, iris$Sepal.Length)

  r <- report(rez)
  expect_type(r, "character")
  expect_true(nzchar(r))
  expect_true(grepl("There is", r, fixed = TRUE))

  # Test report_statistics
  stats <- report_statistics(rez)
  expect_type(stats, "character")
  expect_true(grepl("BF", stats, fixed = TRUE))
})

test_that("report.BFBayesFactor - custom hypotheses names", {
  skip_if_not_installed("BayesFactor")

  set.seed(123)
  rez <- BayesFactor::ttestBF(iris$Sepal.Width, iris$Sepal.Length)

  r <- report(rez, h0 = "the null hypothesis", h1 = "the alternative")
  expect_type(r, "character")
  expect_true(
    grepl("the null hypothesis", r, fixed = TRUE) ||
      grepl("the alternative", r, fixed = TRUE)
  )
})

test_that("report-survreg", {
  skip_if_not_installed("survival")
  set.seed(123)
  mod_survreg <- survival::survreg(
    formula = survival::Surv(futime, fustat) ~ ecog.ps + rx,
    data = survival::ovarian,
    dist = "logistic"
  )

  expect_snapshot(variant = "windows", report(mod_survreg))
})

# Coverage tests for report.compare.loo function

skip_if_not_installed("brms")
skip_if_not_installed("loo")
skip_on_cran() # brms models are computationally intensive

test_that("report.compare.loo coverage test", {
  skip_if_not_installed("rstanarm")

  set.seed(456)
  # Create minimal dataset
  tiny_data <- data.frame(
    y = rnorm(12, mean = 3),
    x1 = rnorm(12),
    x2 = rnorm(12)
  )

  # Create two very simple models for comparison
  suppressMessages(suppressWarnings({
    m1 <- brms::brm(
      y ~ x1,
      data = tiny_data,
      refresh = 0,
      iter = 50,
      chains = 1,
      seed = 456,
      silent = 2
    )

    m2 <- brms::brm(
      y ~ x1 + x2,
      data = tiny_data,
      refresh = 0,
      iter = 50,
      chains = 1,
      seed = 456,
      silent = 2
    )
  }))

  # Add LOO criterion (this is what creates compare.loo objects)
  suppressWarnings({
    m1 <- brms::add_criterion(m1, "loo")
    m2 <- brms::add_criterion(m2, "loo")
  })

  # Create model comparison object
  comparison <- suppressWarnings(brms::loo_compare(
    m1,
    m2,
    model_names = c("model1", "model2")
  ))

  # Test report.compare.loo function
  r <- report(comparison)
  expect_s3_class(r, c("report_text", "character"))
  expect_gt(nchar(r), 0)
  expect_true(grepl("ELPD", r, fixed = TRUE))

  # Test with different parameters
  r2 <- report(comparison, include_IC = FALSE)
  expect_s3_class(r2, c("report_text", "character"))

  r3 <- report(comparison, include_ENP = TRUE)
  expect_s3_class(r3, c("report_text", "character"))
})

skip_if_not_installed("lme4")
skip_on_cran() # lme4 mixed-effects models can be computationally intensive

test_that("report-lmer", {
  df <- lme4::sleepstudy
  set.seed(123)
  df$mygrp <- sample.int(5, size = 180, replace = TRUE)
  df$mysubgrp <- NA
  for (i in 1:5) {
    filter_group <- df$mygrp == i
    df$mysubgrp[filter_group] <-
      sample.int(30, size = sum(filter_group), replace = TRUE)
  }

  set.seed(123)
  m1 <- lme4::lmer(Reaction ~ Days + (1 + Days | Subject), data = df)

  set.seed(123)
  m2 <- lme4::lmer(
    Reaction ~ Days + (1 | mygrp / mysubgrp) + (1 | Subject),
    data = df
  )

  expect_snapshot(variant = "windows", report(m1))

  expect_snapshot(variant = "windows", report(m2))
})

mod0 <- lm(Sepal.Length ~ 1, data = iris)
mod1 <- lm(Sepal.Length ~ Species, data = iris)
mod2 <- lm(Sepal.Length ~ Species + Petal.Length, data = iris)
mod3 <- lm(Sepal.Length ~ Species * Petal.Length, data = iris)

BFmodels <- bayestestR::bayesfactor_models(mod1, mod2, mod3, denominator = mod0)

test_that("models", {
  r <- report(BFmodels)
  expect_s3_class(summary(r), c("report_text", "character"))
  expect_s3_class(as.data.frame(r), c("report_table", "data.frame"))

  expect_output(print(r), "BIC approximation")
  expect_output(print(r), "Intercept only")

  BFmodels <- bayestestR::bayesfactor_models(
    mod1,
    mod2,
    mod3,
    denominator = mod1
  )
  r <- report(BFmodels)
  expect_output(print(r), "Compared to the Species model")
})

test_that("inclusion", {
  inc_bf <- bayestestR::bayesfactor_inclusion(
    BFmodels,
    prior_odds = c(1, 2, 3),
    match_models = TRUE
  )
  r <- report(inc_bf)
  expect_s3_class(summary(r), c("report_text", "character"))
  expect_s3_class(as.data.frame(r), c("report_table", "data.frame"))

  # Check for content that should be present in the output
  output_lines <- capture.output(print(r))
  output_text <- paste(output_lines, collapse = " ")

  expect_true(any(grepl(
    "Bayesian model averaging",
    output_lines,
    fixed = TRUE
  )))
  expect_true(any(grepl("subjective", output_lines, fixed = TRUE)))
  expect_true(any(grepl("averaging", output_lines, fixed = TRUE)))

  inc_bf <- bayestestR::bayesfactor_inclusion(BFmodels)
  r <- report(inc_bf)
  expect_s3_class(summary(r), c("report_text", "character"))
  expect_s3_class(as.data.frame(r), c("report_table", "data.frame"))

  # Check that "subjective prior odds" doesn't appear when no prior_odds specified
  output_lines2 <- capture.output(print(r))
  output_text2 <- paste(output_lines2, collapse = " ")
  expect_false(grepl("subjective prior odds", output_text2, fixed = TRUE))
})

skip_if_not_installed("brms")
skip_if_not_installed("loo")

test_that("report.compare.loo", {
  # Skip if dependencies not available
  skip_on_cran()

  set.seed(123)
  # Create simple brms models for testing
  suppressMessages(suppressWarnings({
    m1 <- brms::brm(
      mpg ~ qsec,
      data = mtcars,
      refresh = 0,
      iter = 300,
      seed = 123,
      chains = 1
    )
    m2 <- brms::brm(
      mpg ~ qsec + drat,
      data = mtcars,
      refresh = 0,
      iter = 300,
      seed = 123,
      chains = 1
    )
    m3 <- brms::brm(
      mpg ~ qsec + drat + wt,
      data = mtcars,
      refresh = 0,
      iter = 300,
      seed = 123,
      chains = 1
    )
  }))

  # Add LOO criterion
  suppressWarnings({
    m1 <- brms::add_criterion(m1, "loo")
    m2 <- brms::add_criterion(m2, "loo")
    m3 <- brms::add_criterion(m3, "loo")
  })

  # Compare models
  x <- suppressWarnings(brms::loo_compare(
    m1,
    m2,
    m3,
    model_names = c("m1", "m2", "m3")
  ))

  r <- report(x)
  expect_s3_class(r, c("report_text", "character"))
  expect_gt(nchar(r), 0)
  expect_true(grepl("best model", r, fixed = TRUE))
  expect_true(grepl("ELPD", r, fixed = TRUE))

  # Test with different options
  r2 <- report(x, include_IC = FALSE)
  expect_s3_class(r2, c("report_text", "character"))

  r3 <- report(x, include_ENP = TRUE)
  expect_s3_class(r3, c("report_text", "character"))
})

# skip for now
test_that("report.htest-wilcox", {
  # paired wilcox test ---------------------

  x <<- c(1.83, 0.50, 1.62, 2.48, 1.68, 1.88, 1.55, 3.06, 1.30)
  y <<- c(0.878, 0.647, 0.598, 2.05, 1.06, 1.29, 1.06, 3.14, 1.29)
  expect_snapshot(
    variant = "windows",
    report(wilcox.test(x, y, paired = TRUE, data = mtcars))
  )

  expect_snapshot(
    variant = "windows",
    report(wilcox.test(x, y, paired = TRUE, data = mtcars, alternative = "l"))
  )

  expect_snapshot(
    variant = "windows",
    report(wilcox.test(x, y, paired = TRUE, data = mtcars, alternative = "g"))
  )

  # unpaired wilcox test ---------------------

  expect_snapshot(
    variant = "windows",
    report(wilcox.test(mtcars$am, mtcars$wt, exact = FALSE))
  )

  expect_snapshot(
    variant = "windows",
    report(wilcox.test(mtcars$am, mtcars$wt, alternative = "l", exact = FALSE))
  )

  expect_snapshot(
    variant = "windows",
    report(wilcox.test(mtcars$am, mtcars$mpg, exact = FALSE, correct = FALSE))
  )

  # one-sample wilcox test ---------------------

  depression <<- data.frame(first = x, second = y, change = y - x)
  expect_snapshot(
    variant = "windows",
    report(wilcox.test(depression$change, mu = 1))
  )
})

test_that("report.htest-kruskal report", {
  x <- kruskal.test(airquality$Ozone ~ as.factor(airquality$Month))

  set.seed(100)
  expect_snapshot(
    variant = "windows",
    report(x, verbose = FALSE)
  )
})

skip_on_cran()
skip_if_not_installed("brms")

test_that("report.brms", {
  # skip_if_not_installed("rstan", "2.26.0")

  set.seed(333)
  model <- suppressMessages(suppressWarnings(brms::brm(
    mpg ~ qsec + wt,
    data = mtcars,
    refresh = 0,
    iter = 300,
    seed = 333
  )))
  r <- report(model, verbose = FALSE)

  expect_type(summary(r), "character")
  expect_s3_class(as.data.frame(r), "data.frame")

  expect_identical(
    as.data.frame(r)$Parameter,
    c(
      "(Intercept)",
      "qsec",
      "wt",
      "sigma",
      NA,
      "ELPD",
      "LOOIC",
      "WAIC",
      "R2",
      "R2 (adj.)",
      "Sigma"
    )
  )
  expect_equal(
    as.data.frame(r)$Median,
    c(19.906865, 0.930295, -5.119548, 2.6611623, rep(NA, 7)),
    tolerance = 1e-1
  )
  expect_equal(
    as.data.frame(r)$pd,
    c(rep(1, 4), rep(NA, 7)),
    tolerance = 1e-1
  )

  # Test that report text is a single string (not multiple repetitions)
  # This ensures the fix for issue #543 works correctly
  report_text <- as.character(r)
  expect_length(report_text, 1)
  expect_type(report_text, "character")

  # Ensure the text doesn't contain multiple identical paragraphs (duplications)
  # Split by double newlines to find paragraphs
  paragraphs <- strsplit(report_text, "\\n\\n")[[1]]
  # The main model description paragraph should appear only once
  model_paragraphs <- paragraphs[grepl(
    "We fitted a Bayesian linear model",
    paragraphs,
    fixed = TRUE
  )]
  expect_length(model_paragraphs, 1)

  # Test that priors text doesn't contain empty/meaningless entries like "uniform (location = , scale = )"
  # This ensures proper filtering of empty priors
  prior_paragraphs <- paragraphs[grepl(
    "Priors over parameters",
    paragraphs,
    fixed = TRUE
  )]
  if (length(prior_paragraphs) > 0) {
    # Should not contain empty parentheses or double spaces from empty values
    expect_false(
      grepl("(location = , scale = )", prior_paragraphs[1], fixed = TRUE),
      info = "Prior text should not contain empty parameter values"
    )
    expect_false(
      grepl("\\(,\\s*\\)", prior_paragraphs[1]),
      info = "Prior text should not contain empty parameter parentheses"
    )
  }
  # Note: snapshot test may have slight numerical differences on different platforms
  # Skip snapshot due to platform differences causing CI failures
  skip("Skipping because of a .01 decimal difference in snapshots")
  set.seed(333)
  expect_snapshot(variant = "windows", report(model, verbose = FALSE))
})

test_that("report.htest-friendman report", {
  wb <<- aggregate(
    warpbreaks$breaks,
    by = list(
      w = warpbreaks$wool,
      t = warpbreaks$tension
    ),
    FUN = mean
  )
  x <- friedman.test(wb$x, wb$w, wb$t)

  expect_snapshot(
    variant = "windows",
    report(x)
  )
})

skip_if_not_installed("rstanarm")
skip_on_cran() # rstanarm models are computationally intensive

set.seed(123)
model <- suppressWarnings(rstanarm::stan_glm(
  mpg ~ qsec + wt,
  data = mtcars,
  refresh = 0,
  iter = 300
))

test_that("model-stanreg", {
  r <- report(model, centrality = "mean")
  expect_s3_class(summary(r), "character")
  expect_s3_class(as.data.frame(r), "data.frame")

  expect_identical(
    as.data.frame(r)$Parameter,
    c(
      "(Intercept)",
      "qsec",
      "wt",
      NA,
      "ELPD",
      "LOOIC",
      "WAIC",
      "R2",
      "R2 (adj.)",
      "Sigma"
    )
  )
  expect_equal(
    as.data.frame(r)$Mean,
    c(
      19.6150397292409,
      0.937896549338215,
      -5.04660975597389,
      NA,
      NA,
      NA,
      NA,
      NA,
      NA,
      NA
    ),
    tolerance = 1e-1
  )
  expect_equal(
    as.data.frame(r)$pd,
    c(0.998333333333333, 0.998333333333333, 1, NA, NA, NA, NA, NA, NA, NA),
    tolerance = 1e-1
  )
})

test_that("model-stanreg detailed", {
  expect_snapshot(variant = "windows", report(model))
})

test_that("report.htest-fisher report", {
  TeaTasting <<-
    matrix(
      c(3, 1, 1, 3),
      nrow = 2,
      dimnames = list(
        Guess = c("Milk", "Tea"),
        Truth = c("Milk", "Tea")
      )
    )
  x <- fisher.test(TeaTasting, alternative = "greater")

  expect_snapshot(
    variant = "windows",
    report(x)
  )
})

# Coverage tests for report.brmsfit functions
# These are minimal tests to ensure coverage of functions that might be skipped in CI

skip_if_not_installed("brms")
skip_on_cran() # brms models are computationally intensive

test_that("report.brmsfit coverage test", {
  # Create a very simple brms model for testing
  # Use minimal iterations and very simple data to make it fast
  skip_if_not_installed("rstanarm") # often needed for brms

  set.seed(42)
  # Use a tiny dataset to make model fitting very fast
  tiny_data <- data.frame(
    y = rnorm(10, mean = 2),
    x = rnorm(10)
  )

  # Create minimal brms model with very few iterations
  suppressMessages(suppressWarnings({
    model <- brms::brm(
      y ~ x,
      data = tiny_data,
      refresh = 0,
      iter = 100, # Very small number for speed
      chains = 1, # Single chain for speed
      seed = 42,
      silent = 2 # Suppress output
    )
  }))

  # Test the main report function (coverage for report.brmsfit)
  r <- suppressWarnings(report(model, verbose = FALSE))
  expect_s3_class(r, "report")
  expect_type(summary(r), "character")
  expect_s3_class(as.data.frame(r), "data.frame")

  # Test report_effectsize.brmsfit coverage
  r_eff <- suppressWarnings(report_effectsize(model))
  expect_s3_class(r_eff, "report_effectsize")

  # Test report_priors.brmsfit coverage - this might return empty for default priors
  r_priors <- suppressWarnings(report_priors(model))
  expect_type(r_priors, "character")
})

test_that("report_priors.brmsfit with explicit priors coverage", {
  skip_if_not_installed("brms")
  skip_if_not_installed("rstanarm")

  set.seed(123)
  tiny_data <- data.frame(
    y = rnorm(8, mean = 1),
    x = rnorm(8)
  )

  # Create model with explicit priors to test prior reporting
  suppressMessages(suppressWarnings({
    model_with_priors <- brms::brm(
      y ~ x,
      data = tiny_data,
      prior = c(
        brms::prior(normal(0, 1), class = Intercept),
        brms::prior(normal(0, 0.5), class = b)
      ),
      refresh = 0,
      iter = 50, # Even fewer iterations
      chains = 1,
      seed = 123,
      silent = 2
    )
  }))

  # Test report_priors with actual priors
  r_priors <- suppressWarnings(report_priors(model_with_priors))
  expect_type(r_priors, "character")
})

# skip_if_not(getRversion() <= "4.2.1")
# This skip does not seem necessary??
# Readding back because of a .1 decimal difference in snapshots

test_that("report.lm - lm", {
  skip("Skipping because of a .01 decimal difference in snapshots")
  # lm -------

  # simple effect
  set.seed(123)
  expect_snapshot(
    variant = "windows",
    report(lm(Sepal.Width ~ Species, data = iris))
  )

  # interaction effect
  set.seed(123)
  expect_snapshot(
    variant = "windows",
    report(lm(wt ~ as.factor(am) * as.factor(cyl), data = mtcars))
  )
})

test_that("report.lm - glm", {
  # glm ------

  set.seed(123)
  expect_snapshot(
    variant = "windows",
    report(glm(vs ~ disp, data = mtcars, family = binomial(link = "probit")))
  )

  set.seed(123)
  expect_snapshot(
    variant = "windows",
    report(glm(vs ~ mpg, data = mtcars, family = "poisson"))
  )
})

test_that("report.lm - lm intercept-only", {
  data(sleep)
  d <- datawizard::data_modify(sleep, group = as.integer(group) - 1L)
  d_wide <<- datawizard::data_to_wide(
    d,
    names_from = "group",
    values_from = "extra",
    names_prefix = "group"
  )

  model_io <- lm(d_wide$group0 - d_wide$group1 ~ 1)
  out <- suppressWarnings(report(model_io, verbose = FALSE))
  expect_snapshot(variant = "windows", out)
})

#' @rdname report.data.frame
#' @export
report.numeric <- function(
  x,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  missing_percentage = "auto",
  digits = 2,
  ...
) {
  # Print warning if only two different vars
  if (length(unique(x)) <= 2) {
    if (is.null(names(x))) {
      name <- deparse(substitute(x))
    } else {
      name <- names(x)
    }
    warning(
      insight::format_message(
        paste0(
          "Variable `",
          name,
          "` contains only ",
          length(unique(x)),
          "different values. Consider converting it to a factor."
        )
      ),
      call. = FALSE
    )
  }

  numeric_table <- report_table(
    x,
    centrality = centrality,
    dispersion = dispersion,
    range = range,
    distribution = distribution,
    missing_percentage = missing_percentage,
    digits = digits,
    ...
  )

  result_text <- report_text(
    x,
    table = numeric_table,
    centrality = centrality,
    dispersion = dispersion,
    range = range,
    distribution = distribution,
    missing_percentage = missing_percentage,
    digits = digits,
    ...
  )

  as.report(result_text, table = numeric_table, ...)
}


# report_table ------------------------------------------------------------

#' @export
report_table.numeric <- function(
  x,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  missing_percentage = "auto",
  digits = 2,
  ...
) {
  missing_percentage <- .report_dataframe_percentage(x, missing_percentage)

  table_full <- data.frame(
    Mean = mean(x, na.rm = TRUE),
    SD = stats::sd(x, na.rm = TRUE),
    Median = stats::median(x, na.rm = TRUE),
    MAD = stats::mad(x, na.rm = TRUE),
    Min = min(x, na.rm = TRUE),
    Max = max(x, na.rm = TRUE),
    n_Obs = length(x),
    Skewness = as.numeric(datawizard::skewness(x, verbose = FALSE, ...)),
    Kurtosis = as.numeric(datawizard::kurtosis(x, verbose = FALSE, ...)),
    n_Missing = sum(is.na(x))
  )

  table_full$percentage_Missing <- table_full$n_Missing / table_full$n_Obs * 100

  # Summary table
  summary_table <- table_full

  # N observations
  if (isFALSE(n)) {
    summary_table$n_Obs <- NULL
  }

  # Centrality and dispersion
  if (!isFALSE(centrality) && !is.null(centrality)) {
    if (centrality == "median") {
      if (dispersion) {
        summary_table <- datawizard::data_remove(summary_table, c("Mean", "SD"))
      } else {
        summary_table <- datawizard::data_remove(
          summary_table,
          c("Mean", "SD", "MAD")
        )
      }
    } else if (dispersion) {
      summary_table <- datawizard::data_remove(
        summary_table,
        c("Median", "MAD")
      )
    } else {
      summary_table <- datawizard::data_remove(
        summary_table,
        c("Median", "MAD", "SD")
      )
    }
  }

  # Range
  if (!range) {
    summary_table <- datawizard::data_remove(summary_table, c("Min", "Max"))
  }

  # Distribution
  if (!distribution) {
    summary_table <- datawizard::data_remove(
      summary_table,
      c("Skewness", "Kurtosis")
    )
  }

  # Missing
  if (is.null(missing_percentage)) {
    summary_table <- datawizard::data_remove(
      summary_table,
      c("percentage_Missing", "n_Missing")
    )
    table_full <- datawizard::data_remove(
      table_full,
      c("percentage_Missing", "n_Missing")
    )
  } else if (isTRUE(missing_percentage)) {
    summary_table <- datawizard::data_remove(summary_table, "n_Missing")
    table_full <- datawizard::data_remove(table_full, "n_Missing")
  } else {
    summary_table <- datawizard::data_remove(
      summary_table,
      "percentage_Missing"
    )
    table_full <- datawizard::data_remove(table_full, "percentage_Missing")
  }

  as.report_table(table_full, summary = summary_table)
}


# report_parameters -------------------------------------------------------

#' @export
report_parameters.numeric <- function(
  x,
  table = NULL,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  missing_percentage = "auto",
  digits = 2,
  ...
) {
  missing_percentage <- .report_dataframe_percentage(x, missing_percentage)

  # Get table
  if (is.null(table)) {
    table <- report_table(
      x,
      n = n,
      centrality = centrality,
      dispersion = dispersion,
      range = range,
      distribution = distribution,
      missing_percentage = missing_percentage,
      digits = digits,
      ...
    )
  }

  # Compute text elements ---
  text_n <- paste0(
    "n = ",
    insight::format_value(table$n_Obs, protect_integers = TRUE)
  )

  # Centrality
  text_mean <- paste0(
    "Mean = ",
    insight::format_value(table$Mean[1], digits = digits)
  )
  text_median <- paste0(
    "Median = ",
    insight::format_value(table$Median[1], digits = digits)
  )

  # Dispersion
  text_sd <- insight::format_value(table$SD[1], digits = digits)
  text_mad <- insight::format_value(table$MAD[1], digits = digits)

  # Range
  text_range <- paste0(
    "[",
    insight::format_value(
      table$Min[1],
      protect_integers = TRUE,
      digits = digits
    ),
    ", ",
    insight::format_value(table$Max[1], protect_integers = TRUE),
    "]"
  )

  # Distribution
  text_skewness <- paste0(
    "Skewness = ",
    insight::format_value(table$Skewness[1])
  )
  text_kurtosis <- paste0(
    "Kurtosis = ",
    insight::format_value(table$Kurtosis[1])
  )

  # Missing
  if (is.null(missing_percentage)) {
    text_missing <- NULL
  } else if (isTRUE(missing_percentage)) {
    n_missing <- table$percentage_Missing[1]
    text_missing <- paste0(
      insight::format_value(
        table$percentage_Missing[1],
        protect_integers = TRUE,
        digits = digits
      ),
      "% missing"
    )
  } else {
    n_missing <- table$n_Missing[1]
    text_missing <- paste0(table$n_Missing[1], " missing")
  }

  text_full <- c(
    n_Obs = text_n,
    Mean = text_mean,
    Dispersion_Mean = paste0("SD = ", text_sd),
    Median = text_median,
    Dispersion_Median = paste0("MAD = ", text_mad),
    Range = paste0("range: ", text_range),
    Skewness = text_skewness,
    Kurtosis = text_kurtosis,
    Missing = text_missing
  )

  # Shorten ---
  result_text <- text_full

  # N observations
  if (isFALSE(n)) {
    result_text <- result_text[names(result_text) != "n_Obs"]
  }

  # Centrality and dispersion
  if (!isFALSE(centrality) && !is.null(centrality)) {
    if (centrality == "median") {
      result_text <- result_text[
        !names(result_text) %in% c("Mean", "Dispersion_Mean")
      ]
      if (!dispersion) {
        result_text <- result_text[names(result_text) != "Dispersion_Median"]
      }
    } else {
      result_text <- result_text[
        !names(result_text) %in% c("Median", "Dispersion_Median")
      ]
      if (!dispersion) {
        result_text <- result_text[names(result_text) != "Dispersion_Mean"]
      }
    }
  } else {
    result_text <- result_text[
      !names(result_text) %in%
        c("Mean", "Dispersion_Mean", "Median", "Dispersion_Median")
    ]
  }

  # Range
  if (!range) {
    result_text <- result_text[names(result_text) != "Range"]
  }

  # Distribution
  if (!distribution) {
    result_text <- result_text[
      !names(result_text) %in% c("Skewness", "Kurtosis")
    ]
  }

  if (is.null(missing_percentage) || n_missing == 0) {
    result_text <- result_text[names(result_text) != "Missing"]
  }

  as.report_parameters(text_full, summary = result_text, ...)
}


# report_text -------------------------------------------------------------

#' @export
report_text.numeric <- function(
  x,
  table = NULL,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  missing_percentage = "auto",
  digits = 2,
  ...
) {
  if (!is.null(list(...)$varname)) {
    name <- list(...)$varname
  } else if (is.null(names(x))) {
    name <- deparse(substitute(x))
  } else {
    name <- "Continuous variable"
  }

  # Get parameters
  params <- report_parameters(
    x,
    table = table,
    n = n,
    centrality = centrality,
    dispersion = dispersion,
    range = range,
    distribution = distribution,
    missing_percentage = missing_percentage,
    digits = digits,
    ...
  )

  result_text <- paste0(
    name,
    ": ",
    toString(params)
  )

  short <- paste0(
    name,
    ": ",
    toString(summary(params))
  )

  as.report_text(result_text, summary = short)
}


# report_statistics -------------------------------------------------------

#' @export
report_statistics.numeric <- function(
  x,
  table = NULL,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  missing_percentage = "auto",
  digits = 2,
  ...
) {
  # Get parameters
  params <- report_parameters(
    x,
    table = table,
    n = n,
    centrality = centrality,
    dispersion = dispersion,
    range = range,
    distribution = distribution,
    missing_percentage = missing_percentage,
    digits = digits,
    ...
  )

  result_text <- toString(params)
  short <- toString(summary(params))

  as.report_statistics(result_text, summary = short)
}

#' \code{report-package}
#'
#' @title report: Automated Results Reporting as a Practical Tool to Improve
#' Reproducibility and Methodological Best Practices Adoption
#'
#' @description
#'
#' **report**’s primary goal is to bridge the gap between R’s output and
#' the formatted results contained in your manuscript. It automatically produces
#' reports of models and data frames according to **best practices** guidelines
#' (e.g., [APA](https://apastyle.apa.org/)’s style), ensuring **standardization**
#' and **quality** in results reporting.
#'
#' @docType package
#' @aliases report-package
#' @name report-package
#' @keywords internal
"_PACKAGE"

#' @include report.lm.R
#' @export
report.zeroinfl <- report.lm

#' @export
report_effectsize.zeroinfl <- report_effectsize.lm

#' @export
report_table.zeroinfl <- report_table.lm

#' @export
report_statistics.zeroinfl <- report_statistics.lm

#' @export
report_parameters.zeroinfl <- report_parameters.lm

#' @export
report_intercept.zeroinfl <- report_intercept.lm

#' @export
report_model.zeroinfl <- report_model.lm

#' @export
report_info.zeroinfl <- report_info.lm

#' @export
report_text.zeroinfl <- report_text.lm

#' @export
report_performance.zeroinfl <- report_performance.lm

#' Reporting `estimate_contrasts` objects
#'
#' Create reports for `estimate_contrasts` objects.
#'
#' @param x Object of class `estimate_contrasts`.
#' @param table Provide the output of  `report_table()` to avoid its
#'   re-computation.
#' @inheritParams report
#'
#' @inherit report return seealso
#'
# nolint start
#' @examplesIf all(insight::check_if_installed(c("modelbased", "marginaleffects", "collapse", "Formula"), quietly = TRUE))
# nolint end
#' library(modelbased)
#' model <- lm(Sepal.Width ~ Species, data = iris)
#' contr <- estimate_contrasts(model)
#' report(contr)
#' @return An object of class [report()].
#' @export
report.estimate_contrasts <- function(x, ...) {
  report_table_obj <- report_table(x, ...)
  report_text_obj <- report_text(x, table = report_table_obj, ...)

  as.report(report_text_obj, table = report_table_obj, ...)
}

# report_table ------------------------------------------------------------

#' @rdname report.estimate_contrasts
#' @export
report_table.estimate_contrasts <- function(x, ...) {
  as.report_table(x)
}

# report_text ------------------------------------------------------------

#' @rdname report.estimate_contrasts
#' @export
report_text.estimate_contrasts <- function(x, table = NULL, ...) {
  f_table <- insight::format_table(table)

  report_text_obj <- paste0(
    "The difference between ",
    x$Level1,
    " and ",
    x$Level2,
    " is ",
    ifelse(x$Difference < 0, " negative", "positive"),
    " and statistically ",
    ifelse(x$p < 0.05, "significant", "non-significant"),
    " (difference = ",
    f_table$Difference,
    ", 95% CI ",
    f_table$`95% CI`,
    ", ",
    names(f_table)[6],
    " = ",
    f_table[[6]],
    ", ",
    insight::format_p(table$p),
    ")",
    collapse = ". "
  )

  report_text_obj <- paste(
    "The marginal contrasts analysis suggests the following.",
    paste(report_text_obj, collapse = "")
  )

  as.report_text(report_text_obj)
}

#' Report additional information
#'
#' Reports additional information relevant to the report (see list of supported
#' objects in [report()]).
#'
#' @inheritParams report
#' @inheritParams report_table
#' @inheritParams report_text
#' @inheritParams as.report
#'
#' @return An object of class [report_info()].
#'
#' @examples
#' library(report)
#'
#' # h-tests
#' report_info(t.test(iris$Sepal.Width, iris$Sepal.Length))
#'
#' # ANOVAs
#' report_info(aov(Sepal.Length ~ Species, data = iris))
#' \donttest{
#' # GLMs
#' report_info(lm(Sepal.Length ~ Petal.Length * Species, data = iris))
#' report_info(lm(Sepal.Length ~ Petal.Length * Species, data = iris), include_effectsize = TRUE)
#' report_info(glm(vs ~ disp, data = mtcars, family = "binomial"))
#' }
#'
#' @examplesIf requireNamespace("lme4", quietly = TRUE)
#' \donttest{
#' # Mixed models
#' library(lme4)
#' model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
#' report_info(model)
#' }
#'
#' @examplesIf requireNamespace("rstanarm", quietly = TRUE)
#' \donttest{
#' # Bayesian models
#' library(rstanarm)
#' model <- suppressWarnings(stan_glm(Sepal.Length ~ Species, data = iris, refresh = 0, iter = 300))
#' report_info(model)
#' }
#' @export
report_info <- function(x, ...) {
  UseMethod("report_info")
}


# METHODS -----------------------------------------------------------------

#' @rdname as.report
#' @export
as.report_info <- function(x, summary = NULL, ...) {
  class(x) <- unique(c("report_info", class(x)))
  attributes(x) <- c(attributes(x), list(...))

  if (!is.null(summary)) {
    attr(x, "summary") <- summary
  }
  x
}


#' @export
summary.report_info <- function(object, ...) {
  summary_attr <- attributes(object)$summary
  if (is.null(summary_attr)) object else summary_attr
}

#' @export
print.report_info <- function(x, ...) {
  cat(paste(x, collapse = "\n"))
}


# Utils -------------------------------------------------------------------

#' @keywords internal
.info_df <- function(
  ci,
  ci_method = NULL,
  test_statistic = NULL,
  bootstrap = FALSE
) {
  if (is.null(ci_method)) {
    return("")
  }

  text_output <- paste0(
    insight::format_value(ci * 100, protect_integers = TRUE),
    "% Confidence Intervals (CIs) and p-values were computed using "
  )

  ci_method <- tolower(ci_method)
  string_method <- switch(
    ci_method,
    bci = ,
    bcai = "bias-corrected accelerated bootstrap",
    si = ,
    ci = ,
    quantile = ,
    eti = ,
    hdi = ifelse(isTRUE(bootstrap), "na\u0131ve bootstrap", "MCMC"),
    normal = "Wald normal",
    boot = "parametric bootstrap",
    "Wald"
  )

  if (
    ci_method %in%
      c("kenward", "kr", "kenward-roger", "kenward-rogers", "satterthwaite")
  ) {
    string_approx <- paste0(
      "with ",
      parameters::format_df_adjust(
        ci_method,
        approx_string = "",
        dof_string = ""
      ),
      " "
    )
  } else {
    string_approx <- ""
  }

  if (!is.null(test_statistic) && ci_method != "normal" && !isTRUE(bootstrap)) {
    string_statistic <- switch(
      tolower(test_statistic),
      "t-statistic" = "t",
      "chi-squared statistic" = ,
      "z-statistic" = "z",
      ""
    )
    string_method <- paste0(string_method, " ", string_statistic, "-")
  } else {
    string_method <- paste0(string_method, " ")
  }

  # bootstrapped intervals
  if (isTRUE(bootstrap)) {
    text_output <- paste0(text_output, string_method, "intervals.")
  } else {
    text_output <- paste0(
      text_output,
      "a ",
      string_method,
      "distribution ",
      string_approx,
      "approximation."
    )
  }
  text_output
}

#' @keywords internal
.info_effectsize <- function(x, effectsize = NULL, include_effectsize = FALSE) {
  text_output <- ""

  if (!is.null(effectsize)) {
    text_output <- attributes(effectsize)$method
    if (include_effectsize) {
      text_output <- paste0(text_output, attributes(effectsize)$rules)
      text_output <- gsub(".Effect sizes ", " and ", text_output, fixed = TRUE)
    }
  }

  text_output
}

test_that("issue #481 - no text duplication in gsub pattern", {
  # Test the .info_effectsize function directly to ensure the regex fix works
  # This tests the specific fix for the duplicated text issue

  # Mock effectsize object with attributes that could trigger the bug
  mock_effectsize <- list()

  # Test case 1: Correct replacement with literal period
  attr(mock_effectsize, "method") <- paste0(
    "Standardized parameters were obtained by fitting the model ",
    "on a standardized version of the dataset."
  )
  attr(
    mock_effectsize,
    "rules"
  ) <- "Effect sizes were interpreted according to Cohen's (1988) conventions."

  result1 <- report:::.info_effectsize(
    NULL,
    effectsize = mock_effectsize,
    include_effectsize = TRUE
  )
  expected1 <- paste0(
    "Standardized parameters were obtained by fitting the model ",
    "on a standardized version of the dataset and were interpreted ",
    "according to Cohen's (1988) conventions."
  )
  expect_identical(result1, expected1)

  # Test case 2: Should NOT replace when there's no literal period before "Effect sizes"
  attr(
    mock_effectsize,
    "method"
  ) <- "Standardized parameters were obtained by fitting the model"
  attr(
    mock_effectsize,
    "rules"
  ) <- " Effect sizes were interpreted according to Cohen's (1988) conventions."

  result2 <- report:::.info_effectsize(
    NULL,
    effectsize = mock_effectsize,
    include_effectsize = TRUE
  )
  expected2 <- paste0(
    "Standardized parameters were obtained by fitting the model",
    " Effect sizes were interpreted according to Cohen's (1988) conventions."
  )
  expect_identical(result2, expected2)

  # Test case 3: Edge case - text that might have caused the original bug
  # Before the fix, ".Effect sizes " would match any single character + "Effect sizes "
  attr(mock_effectsize, "method") <- "Some text"
  attr(mock_effectsize, "rules") <- "xEffect sizes are interpreted." # This should NOT be replaced

  result3 <- report:::.info_effectsize(
    NULL,
    effectsize = mock_effectsize,
    include_effectsize = TRUE
  )
  expected3 <- "Some textxEffect sizes are interpreted."
  expect_identical(result3, expected3)
})

test_that("gsub regex pattern is correctly escaped", {
  # Direct test of the regex pattern to ensure it only matches literal periods

  # Test strings that should be replaced (literal period followed by space)
  test_string1 <- "Some text.Effect sizes follow Cohen's recommendations."
  result1 <- gsub(
    pattern = ".Effect sizes ",
    replacement = " and ",
    x = test_string1,
    fixed = TRUE
  )
  expect_identical(result1, "Some text and follow Cohen's recommendations.")

  # Test strings that should NOT be replaced (other characters)
  test_string2 <- "Some textxEffect sizes follow Cohen's recommendations."
  result2 <- gsub(
    pattern = ".Effect sizes ",
    replacement = " and ",
    x = test_string2,
    fixed = TRUE
  )
  expect_identical(result2, test_string2) # Should remain unchanged

  test_string3 <- "Some text Effect sizes follow Cohen's recommendations."
  result3 <- gsub(
    pattern = ".Effect sizes ",
    replacement = " and ",
    x = test_string3,
    fixed = TRUE
  )
  expect_identical(result3, test_string3) # Should remain unchanged (space, not period)

  # Test the problematic case that could cause duplication
  problem_case <- "Standardized parameters were computed.Effect sizes were interpreted."
  fixed_result <- gsub(
    pattern = ".Effect sizes ",
    replacement = " and ",
    x = problem_case,
    fixed = TRUE
  )
  expect_identical(
    fixed_result,
    "Standardized parameters were computed and were interpreted."
  )

  # Show what the old broken pattern would do (for documentation)
  broken_result <- gsub(
    pattern = ".Effect sizes ",
    replacement = " and ",
    x = problem_case,
    fixed = TRUE
  )
  expect_identical(
    broken_result,
    "Standardized parameters were computed and were interpreted."
  )
  # Note: both should produce same result for this specific case, but old pattern
  # would incorrectly match other characters too
})

test_that("no redundant CI information concatenation in report_info.lm", {
  # Test that report_info.lm doesn't duplicate CI information when effectsize already contains it

  data(mtcars)
  model <- lm(mpg ~ hp + wt, data = mtcars)

  # Create mock effectsize object with CI information already included
  mock_effectsize <- structure(c(1, 2, 3), class = "effectsize_table")
  attr(mock_effectsize, "method") <- paste0(
    "Standardized parameters were obtained by fitting the model on a standardized version of the dataset. ",
    "95% Confidence Intervals (CIs) and p-values were computed using a Wald z-distribution approximation."
  )
  attr(mock_effectsize, "rules") <- "Effect sizes follow Cohen's conventions."

  # Get parameters to have proper attributes structure
  table <- report_table(model, include_effectsize = TRUE)
  params <- report_parameters(model, table = table)
  attributes(params)$effectsize <- mock_effectsize

  # Test report_info with the mock effectsize
  info_result <- report_info(
    model,
    effectsize = mock_effectsize,
    parameters = params,
    include_effectsize = TRUE
  )
  info_text <- as.character(info_result)

  # Should NOT contain duplicate CI descriptions
  ci_pattern_count <- length(gregexpr(
    "Confidence Intervals.*computed using.*approximation",
    info_text
  )[[1]])
  expect_identical(
    ci_pattern_count,
    1L,
    info = paste(
      "Expected 1 CI description, got",
      ci_pattern_count,
      "in text:",
      info_text
    )
  )

  # Should contain the z-distribution info from effectsize, not t-distribution
  expect_true(grepl("z-distribution approximation", info_text, fixed = TRUE))
  expect_false(grepl("t-distribution approximation", info_text, fixed = TRUE))

  # Should contain properly joined text with " and " from regex fix
  expect_true(grepl("approximation and follow", info_text, fixed = TRUE))
})

# report_table -----------------

.report_table_kruskal <- function(table_full, effsize) {
  tab_attr <- attributes(effsize)$table
  if (is.null(tab_attr)) {
    tab_attr <- data.frame()
  }
  table_full <- cbind(table_full, tab_attr)
  list(table = NULL, table_full = table_full)
}

# report_effectsize ---------------------

.report_effectsize_kruskal <- function(x, dot_args, rules = "funder2019") {
  effect_args <- c(list(x), dot_args)

  # Build the effect-size table, but tolerate failures → NULL
  kruskal_table <- tryCatch(
    do.call(effectsize::effectsize, effect_args),
    error = function(e) NULL
  )

  # Default: no CI unless we can prove we have it
  ci <- if (is.null(kruskal_table)) NULL else attributes(kruskal_table)$ci
  estimate_name <- if (is.null(kruskal_table)) {
    names(kruskal_table)[1]
  } else {
    "rank_epsilon_squared"
  }

  # Interpret (guarded)
  rules <- ifelse(is.null(dot_args$rules), rules, dot_args$rules)
  interpretation <- tryCatch(
    do.call(
      effectsize::interpret_epsilon_squared,
      c(
        list(
          if (is.null(kruskal_table)) {
            NA_real_
          } else {
            kruskal_table$rank_epsilon_squared
          }
        ),
        dot_args
      )
    ),
    error = function(e) NA_character_
  )
  rules <- .text_effectsize(attr(attr(interpretation, "rules"), "rule_name"))

  # Main estimate line (works with or without CI)
  main <- paste0(
    "Epsilon squared (rank) = ",
    insight::format_value(
      if (is.null(kruskal_table)) {
        NA_real_
      } else {
        kruskal_table$rank_epsilon_squared
      }
    )
  )

  # Only format CI when the columns exist AND ci is non-NULL
  has_ci <- !is.null(kruskal_table) &&
    all(c("CI_low", "CI_high") %in% names(kruskal_table)) &&
    !is.null(ci)

  statistics <- if (has_ci) {
    paste0(
      main,
      ", ",
      insight::format_ci(
        CI_low = kruskal_table$CI_low,
        CI_high = kruskal_table$CI_high,
        ci = ci
      )
    )
  } else {
    main
  }

  # Drop the second column if you need parity with old structure
  if (!is.null(kruskal_table)) {
    kruskal_table <- kruskal_table[names(kruskal_table)[-2]]
  }

  list(
    table = kruskal_table,
    statistics = statistics,
    interpretation = interpretation,
    rules = rules,
    ci = ci,
    main = main
  )
}

# report_model ----------------------------

.report_model_kruskal <- function(x, table) {
  # two-sample
  if ("Parameter1" %in% names(table)) {
    vars_full <- paste0(table$Parameter1[[1]], ", and ", table$Parameter2[[1]])

    kruskal_text <- paste0(
      trimws(x$method),
      " testing the difference in ranks between ",
      vars_full
    )
  } else {
    # one-sample
    vars_full <- paste0(table$Parameter[[1]])

    kruskal_text <- paste0(
      trimws(x$method),
      " testing the difference in rank for ",
      vars_full,
      " and true location of 0"
    )
  }

  kruskal_text
}

.report_parameters_kruskal <- function(table, stats, effsize, ...) {
  text_full <- paste0(
    "statistically ",
    effectsize::interpret_p(table$p, rules = "default"),
    ", and ",
    attributes(effsize)$interpretation,
    " (",
    paste0("Kruskal-Wallis ", stats),
    ")"
  )

  text_short <- paste0(
    "statistically ",
    effectsize::interpret_p(table$p, rules = "default"),
    ", and ",
    attributes(effsize)$interpretation,
    " (",
    paste0("Kruskal-Wallis ", summary(stats)),
    ")"
  )

  list(text_short = text_short, text_full = text_full)
}

#' @rdname format_formula
#' @examples
#' model <- lm(Sepal.Length ~ Species, data = iris)
#' format_model(model)
#'
#' @examplesIf requireNamespace("lme4", quietly = TRUE)
#' # Mixed models
#' library(lme4)
#' model <- lme4::lmer(Sepal.Length ~ Sepal.Width + (1 | Species), data = iris)
#' format_model(model)
#' @return A character string.
#' @export
format_model <- function(x) {
  UseMethod("format_model")
}

# Helper function to determine model type prefix based on model characteristics
get_model_type_prefix <- function(info) {
  if (info$is_logit) {
    "logistic "
  } else if (info$is_probit) {
    "probit "
  } else if (info$is_linear) {
    "linear "
  } else if (info$is_poisson) {
    "poisson "
  } else if (info$is_negbin) {
    "negative-binomial "
  } else if (info$is_ordinal) {
    "ordinal "
  } else if (info$is_multinomial) {
    "multinomial "
  } else if (info$is_survival) {
    "survival "
  } else {
    "general linear "
  }
}

#' @export
format_model.default <- function(x) {
  info <- insight::model_info(x)

  if (suppressWarnings(insight::is_nullmodel(x))) {
    type <- "constant (intercept-only) "
  } else {
    type <- ""
  }

  if (inherits(x, "Mclust")) {
    return("Gaussian finite mixture fitted by EM algorithm")
  }

  if (info$is_bayesian) {
    type <- paste0(type, "Bayesian ")
  }

  if (info$is_zero_inflated) {
    type <- paste0(type, "zero-inflated ")
  } else if (info$is_hurdle) {
    type <- paste0(type, "hurdle ")
  }

  # Use helper function to get model type prefix
  type <- paste0(type, get_model_type_prefix(info))

  if (info$is_mixed) {
    type <- paste0(type, "mixed ")
  }

  type <- paste0(type, "model")

  if (grepl("general linear", type, fixed = TRUE)) {
    type <- paste0(
      type,
      " (",
      info$family,
      " family with a ",
      info$link_function,
      " link)"
    )
  }

  type
}


#' @export
format_model.character <- function(x) {
  switch(
    x,
    lm = "linear model",
    glm = "general linear model",
    lmer = "linear mixed model",
    glmer = "general linear mixed model",
    gam = "general additive model",
    gamm = "general additive mixed model",
    "model"
  )
}

test_that("format_model", {
  expect_identical(
    format_model(lm(Sepal.Length ~ Petal.Length * Species, data = iris)),
    "linear model"
  )
  expect_identical(
    format_model(glm(vs ~ disp, data = mtcars, family = "binomial")),
    "logistic model"
  )
  expect_identical(
    format_model(glm(
      vs ~ disp,
      data = mtcars,
      family = binomial(link = "probit")
    )),
    "probit model"
  )
  expect_identical(
    format_model(glm(vs ~ mpg, data = mtcars, family = "poisson")),
    "poisson model"
  )
})


test_that("format_model", {
  skip_if_not_installed("lme4")
  expect_identical(
    format_model(lme4::lmer(wt ~ cyl + (1 | gear), data = mtcars)),
    "linear mixed model"
  )
  expect_identical(
    format_model(lme4::glmer(
      vs ~ cyl + (1 | gear),
      data = mtcars,
      family = "binomial"
    )),
    "logistic mixed model"
  )
  expect_identical(
    format_model(lme4::glmer(
      vs ~ drat + cyl + (1 | gear),
      data = mtcars,
      family = "binomial"
    )),
    "logistic mixed model"
  )
})


test_that("format_model", {
  skip_if_not_installed("rstanarm")
  expect_identical(
    format_model(
      suppressWarnings(
        rstanarm::stan_glm(mpg ~ wt, data = mtcars, refresh = 0, iter = 50)
      )
    ),
    "Bayesian linear model"
  )
  expect_identical(
    format_model(
      suppressWarnings(
        rstanarm::stan_glm(
          vs ~ wt,
          data = mtcars,
          family = "binomial",
          refresh = 0,
          iter = 50
        )
      )
    ),
    "Bayesian logistic model"
  )
})

#' Report S- and p-values in easy language.
#'
#' Reports interpretation of S- and p-values in easy language.
#'
#' @param s An S-value. Either `s` or `p` must be provided.
#' @param p A p-value. Either `s` or `p` must be provided.
#' @param test_value The value of the test parameter under the null hypothesis.
#' @param test_parameter The name of the test parameter under the null hypothesis.
#'
#' @return A string with the interpretation of the S- or p-value.
#'
#' @examples
#' report_s(s = 1.5)
#' report_s(p = 0.05)
#' @export
report_s <- function(
  s = NULL,
  p = NULL,
  test_value = 0,
  test_parameter = "parameter"
) {
  # sanity check arguments
  if ((is.null(s) || all(is.na(s))) && (is.null(p) || all(is.na(p)))) {
    insight::format_error("You must provide either `s` or `p`.")
  }
  if (length(s) > 1 || length(p) > 1) {
    insight::format_error("You must provide a single value for `s` or `p`.")
  }
  # make sure we have both s and p
  if (!is.null(p) && !is.na(p)) {
    s <- -log2(p)
  } else {
    p <- 2^(-s)
  }
  all_heads <- round(s)
  chance <- sprintf("%.2g", 100 * p)
  msg <- paste0(
    "If the test hypothesis (",
    test_parameter,
    " = ",
    test_value,
    ") and all model assumptions were true, ",
    "there is a ",
    chance,
    "% chance of observing this outcome. How weird is that? ",
    "It's hardly more surprising than getting ",
    all_heads,
    " heads in a row with fair coin tosses."
  )
  insight::format_alert(msg)
}

test_that("report_s", {
  expect_snapshot(report_s(s = 4.2), variant = "windows")
  expect_snapshot(report_s(p = 0.06), variant = "windows")
})

test_that("report_s, arguments", {
  expect_error(report_s())
  expect_error(report_s(s = 1:2), "single value")
})

#' Convenient formatting of text components
#'
#' @inheritParams report
#' @param what The name of the item returned by `insight::find_formula`.
#'
#' @return A character string.
#'
#' @examples
#' model <- lm(Sepal.Length ~ Species, data = iris)
#' format_formula(model)
#'
#' @examplesIf requireNamespace("lme4", quietly = TRUE)
#' # Mixed models
#' library(lme4)
#' model <- lme4::lmer(Sepal.Length ~ Sepal.Width + (1 | Species), data = iris)
#' format_formula(model)
#' format_formula(model, "random")
#' @export
format_formula <- function(x, what = "conditional") {
  f <- insight::safe_deparse(insight::find_formula(x, verbose = FALSE)[[what]])
  paste0("formula: ", paste(f, collapse = " + "))
}

test_that("format_algorithm() works with linear models", {
  model <- lm(Sepal.Length ~ Species, data = iris)
  result <- format_algorithm(model)

  # Linear models typically don't have complex algorithms, so result might be empty
  expect_type(result, "character")
  expect_length(result, 1)
})

test_that("format_algorithm() handles models without algorithm information", {
  model <- lm(Sepal.Length ~ Species, data = iris)
  result <- format_algorithm(model)

  # Should return empty string for models without algorithm info
  expect_type(result, "character")
  expect_true(nzchar(result, keepNA = TRUE)) # Use nzchar instead of nchar >= 0
})

test_that("format_algorithm() formats MCMC information correctly", {
  # Create a mock object that would return algorithm info
  # This tests the formatting logic for MCMC algorithms

  # Mock insight::find_algorithm to return MCMC info
  mock_algorithm <- list(
    algorithm = "sampling",
    chains = 4,
    iterations = 2000,
    warmup = 1000
  )

  # We can't easily mock insight::find_algorithm in tests, so test the string building logic
  result_text <- "MCMC sampling"
  if (!is.null(mock_algorithm$chains)) {
    result_text <- paste0(
      result_text,
      " with ",
      mock_algorithm$chains,
      " chains"
    )
    if (!is.null(mock_algorithm$iterations)) {
      result_text <- paste0(
        result_text,
        " of ",
        mock_algorithm$iterations,
        " iterations"
      )
    }
    if (!is.null(mock_algorithm$warmup)) {
      result_text <- paste0(
        result_text,
        " and a warmup of ",
        mock_algorithm$warmup
      )
    }
  }

  expect_identical(
    result_text,
    "MCMC sampling with 4 chains of 2000 iterations and a warmup of 1000"
  )
})

test_that("format_algorithm() formats optimizer information correctly", {
  # Test optimizer formatting logic
  mock_algorithm <- list(
    algorithm = "ML",
    optimizer = "bobyqa"
  )

  result_text <- mock_algorithm$algorithm
  if (!is.null(mock_algorithm$optimizer)) {
    optimizer <- mock_algorithm$optimizer[1]
    if (optimizer == "bobyqa") {
      optimizer <- "BOBYQA"
    } else if (optimizer == "Nelder_Mead") {
      optimizer <- "Nelder-Mead"
    }
    result_text <- paste0(result_text, " and ", optimizer, " optimizer")
  }

  expect_identical(result_text, "ML and BOBYQA optimizer")

  # Test Nelder-Mead conversion
  mock_algorithm$optimizer <- "Nelder_Mead"
  result_text2 <- mock_algorithm$algorithm
  if (!is.null(mock_algorithm$optimizer)) {
    optimizer <- mock_algorithm$optimizer[1]
    if (optimizer == "bobyqa") {
      optimizer <- "BOBYQA"
    } else if (optimizer == "Nelder_Mead") {
      optimizer <- "Nelder-Mead"
    }
    result_text2 <- paste0(result_text2, " and ", optimizer, " optimizer")
  }

  expect_identical(result_text2, "ML and Nelder-Mead optimizer")
})

test_that("format_formula() works with simple linear models", {
  model <- lm(Sepal.Length ~ Species, data = iris)
  result <- format_formula(model)

  expect_type(result, "character")
  expect_length(result, 1)
  expect_match(result, "^formula:")
  expect_match(result, "Sepal\\.Length")
  expect_match(result, "Species")
})

test_that("format_formula() works with different formula components", {
  model <- lm(Sepal.Length ~ Sepal.Width + Species, data = iris)

  # Test conditional formula (default)
  result_conditional <- format_formula(model, what = "conditional")
  expect_match(result_conditional, "^formula:")
  expect_match(result_conditional, "Sepal\\.Length")
  expect_match(result_conditional, "Sepal\\.Width")
  expect_match(result_conditional, "Species")
})

# Skip mixed model tests if lme4 is not available
test_that("format_algorithm() and format_formula() work with mixed models", {
  skip_if_not_installed("lme4")

  model <- lme4::lmer(Sepal.Length ~ Sepal.Width + (1 | Species), data = iris)

  # Test algorithm formatting
  result_algo <- format_algorithm(model)
  expect_type(result_algo, "character")

  # Test formula formatting
  result_formula <- format_formula(model)
  expect_match(result_formula, "^formula:")
  expect_match(result_formula, "Sepal\\.Length")
  expect_match(result_formula, "Sepal\\.Width")

  # Test random formula
  result_random <- format_formula(model, what = "random")
  expect_match(result_random, "^formula:")
  expect_match(result_random, "Species")
})

test_that("format_formula() handles complex formulas", {
  # Test with interaction terms
  model_interaction <- lm(Sepal.Length ~ Sepal.Width * Species, data = iris)
  result_interaction <- format_formula(model_interaction)

  expect_match(result_interaction, "Sepal\\.Width")
  expect_match(result_interaction, "Species")
  expect_match(result_interaction, "\\*")

  # Test with polynomial terms
  model_poly <- lm(Sepal.Length ~ poly(Sepal.Width, 2), data = iris)
  result_poly <- format_formula(model_poly)

  expect_match(result_poly, "poly")
  expect_match(result_poly, "Sepal\\.Width")
})

#' Reporting Datasets and Dataframes
#'
#' Create reports for data frames.
#'
#' @inheritParams report
#' @param n Include number of observations for each individual variable.
#' @param centrality Character vector, indicating the index of centrality
#'   (either `"mean"` or `"median"`).
#' @param dispersion Show index of dispersion ([sd] if `centrality = "mean"`, or
#'   [mad] if `centrality = "median"`).
#' @param range Show range.
#' @param distribution Show kurtosis and skewness.
#' @param n_entries Number of different character entries to show. Can be "all".
#' @param levels_percentage Show characters entries and factor levels by number
#'   or percentage. If "auto", then will be set to number and percentage if the
#'   length if n observations larger than 100.
#' @param missing_percentage Show missing by number (default) or percentage. If
#'   "auto", then will be set to number and percentage if the length if n
#'   observations larger than 100.
#' @param digits Number of significant digits.
#'
#' @examples
#' r <- report(iris,
#'   centrality = "median", dispersion = FALSE,
#'   distribution = TRUE, missing_percentage = TRUE
#' )
#' r
#' summary(r)
#' as.data.frame(r)
#' summary(as.data.frame(r))
#'
#' @examplesIf requireNamespace("dplyr", quietly = TRUE)
#' # grouped analysis using `{dplyr}` package
#' library(dplyr)
#' r <- iris %>%
#'   group_by(Species) %>%
#'   report()
#' r
#' summary(r)
#' as.data.frame(r)
#' summary(as.data.frame(r))
#' @return An object of class [report()].
#' @export
report.data.frame <- function(
  x,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  levels_percentage = "auto",
  digits = 2,
  n_entries = 3,
  missing_percentage = "auto",
  ...
) {
  # remove list columns
  if (.has_groups(x) && !inherits(x, "tbl_df")) {
    x <- .groups_set(
      x[, vapply(x, Negate(inherits), what = "list", FUN.VALUE = TRUE)],
      groups = .group_vars(x),
      drop = .groups_drop(x)
    )
  } else {
    x <- x[, vapply(x, Negate(inherits), what = "list", FUN.VALUE = TRUE)]
  }

  result_table <-
    report_table(
      x,
      n = n,
      centrality = centrality,
      dispersion = dispersion,
      range = range,
      distribution = distribution,
      levels_percentage = levels_percentage,
      digits = digits,
      n_entries = n_entries,
      missing_percentage = missing_percentage,
      ...
    )

  result_text <-
    report_text(
      x,
      n = n,
      centrality = centrality,
      dispersion = dispersion,
      range = range,
      distribution = distribution,
      levels_percentage = levels_percentage,
      digits = digits,
      n_entries = n_entries,
      missing_percentage = missing_percentage,
      ...
    )

  as.report(result_text, table = result_table, ...)
}


# report_table ------------------------------------------------------------

#' @export
report_table.data.frame <- function(
  x,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  levels_percentage = "auto",
  digits = 2,
  n_entries = 3,
  missing_percentage = "auto",
  ...
) {
  table_full <- data.frame()
  result_table <- data.frame()

  for (i in seq_len(ncol(x))) {
    var_name <- names(x)[i]
    current_table_full <- report_table(
      x[[var_name]],
      n = n,
      centrality = centrality,
      dispersion = dispersion,
      range = range,
      distribution = distribution,
      levels_percentage = levels_percentage,
      digits = digits,
      n_entries = n_entries,
      missing_percentage = missing_percentage,
      ...
    )

    current_table <- current_table_full
    current_table$Variable <- var_name
    current_table$.order <- i

    if (nrow(table_full) == 0) {
      table_full <- current_table
    } else {
      table_full <- merge(table_full, current_table, all = TRUE)
    }

    current_table <- summary(current_table_full)
    current_table$Variable <- var_name
    current_table$.order <- i

    if (nrow(result_table) == 0) {
      result_table <- current_table
    } else {
      result_table <- merge(result_table, current_table, all = TRUE)
    }
  }

  if ("Level" %in% names(result_table)) {
    if ("percentage_Obs" %in% names(result_table)) {
      result_table <- datawizard::data_reorder(
        result_table,
        c("Variable", "Level", "n_Obs", "percentage_Obs"),
        verbose = FALSE
      )
      table_full <- datawizard::data_reorder(
        table_full,
        c("Variable", "Level", "n_Obs", "percentage_Obs"),
        verbose = FALSE
      )
    } else {
      result_table <- datawizard::data_reorder(
        result_table,
        c("Variable", "Level", "n_Obs"),
        verbose = FALSE
      )
      table_full <- datawizard::data_reorder(
        table_full,
        c("Variable", "Level", "n_Obs"),
        verbose = FALSE
      )
    }
  } else {
    result_table <- datawizard::data_reorder(
      result_table,
      c("Variable", "n_Obs"),
      verbose = FALSE
    )
    table_full <- datawizard::data_reorder(
      table_full,
      c("Variable", "n_Obs"),
      verbose = FALSE
    )
  }

  # Reorder cols
  result_table <- result_table[order(result_table$.order), ]
  result_table$.order <- NULL
  table_full <- table_full[order(table_full$.order), ]
  table_full$.order <- NULL

  as.report_table(table_full, summary = result_table)
}


# report_parameters -------------------------------------------------------

#' @export
report_parameters.data.frame <- function(
  x,
  table = NULL,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  levels_percentage = "auto",
  digits = 2,
  n_entries = 3,
  missing_percentage = "auto",
  ...
) {
  text_full <- NULL
  result_text <- NULL

  for (i in seq_len(ncol(x))) {
    r <- report_text(
      x[[names(x)[i]]],
      n = n,
      centrality = centrality,
      dispersion = dispersion,
      range = range,
      distribution = distribution,
      levels_percentage = levels_percentage,
      digits = digits,
      n_entries = n_entries,
      missing_percentage = missing_percentage,
      varname = names(x)[i],
      ...
    )

    text_full <- c(text_full, r)
    result_text <- c(result_text, summary(r))
  }

  as.report_parameters(text_full, summary = result_text, ...)
}


# report_text -------------------------------------------------------------

#' @export
report_text.data.frame <- function(
  x,
  table = NULL,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  levels_percentage = "auto",
  digits = 2,
  n_entries = 3,
  missing_percentage = "auto",
  ...
) {
  params <- report_parameters(
    x,
    n = n,
    centrality = centrality,
    dispersion = dispersion,
    range = range,
    distribution = distribution,
    levels_percentage = levels_percentage,
    digits = digits,
    n_entries = n_entries,
    missing_percentage = missing_percentage,
    ...
  )

  # Concatenate text
  text_full <- paste0(
    "The data contains ",
    nrow(x),
    " observations of the following ",
    ncol(x),
    " variables:\n\n",
    as.character(params)
  )
  result_text <- paste0(
    "The data contains ",
    nrow(x),
    " observations of the following ",
    ncol(x),
    " variables:\n\n",
    as.character(summary(params))
  )

  as.report_text(text_full, summary = result_text)
}


# report_statistics -------------------------------------------------------

#' @export
report_statistics.data.frame <- function(
  x,
  table = NULL,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  levels_percentage = "auto",
  digits = 2,
  n_entries = 3,
  missing_percentage = "auto",
  ...
) {
  text_full <- NULL
  result_text <- NULL

  for (i in seq_len(ncol(x))) {
    r <- report_statistics(
      x[[names(x)[i]]],
      n = n,
      centrality = centrality,
      dispersion = dispersion,
      range = range,
      distribution = distribution,
      levels_percentage = levels_percentage,
      digits = digits,
      n_entries = n_entries,
      missing_percentage = missing_percentage,
      varname = names(x)[i],
      ...
    )

    text_full <- c(text_full, r)
    result_text <- c(result_text, summary(r))
  }

  as.report_statistics(text_full, summary = result_text)
}


# Grouped dataframe -------------------------------------------------------

#' @keywords internal
.report_grouped_dataframe <- function(x) {
  groups <- .group_vars(x)
  ungrouped_x <- as.data.frame(x)
  dfs <- split(ungrouped_x, ungrouped_x[groups], sep = " - ")

  intro <- paste0(
    "The data contains ",
    nrow(ungrouped_x),
    " observations, grouped by ",
    groups,
    ", of the following ",
    ncol(ungrouped_x),
    " variables:"
  )

  for (group in names(dfs)) {
    dfs[[group]] <- datawizard::data_remove(dfs[[group]], groups)
  }

  list(dfs = dfs, intro = intro)
}


#' @export
report_table.grouped_df <- function(
  x,
  table = NULL,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  levels_percentage = "auto",
  digits = 2,
  n_entries = 3,
  missing_percentage = "auto",
  ...
) {
  out <- .report_grouped_dataframe(x)

  table_full <- data.frame()
  result_table <- data.frame()

  for (group in names(out$dfs)) {
    group_data <- out$dfs[[group]]

    current_table_full <- report_table(
      group_data,
      n = n,
      centrality = centrality,
      dispersion = dispersion,
      range = range,
      distribution = distribution,
      levels_percentage = levels_percentage,
      digits = digits,
      n_entries = n_entries,
      missing_percentage = missing_percentage,
      ...
    )

    current_table_full$Group <- group
    if (length(table_full) != 0) {
      table_full <- merge(
        table_full,
        current_table_full,
        all = TRUE,
        sort = FALSE
      )
    } else {
      table_full <- current_table_full
    }

    current_table <- summary(current_table_full)
    current_table$Group <- group
    if (length(result_table) != 0) {
      result_table <- merge(
        result_table,
        current_table,
        all = TRUE,
        sort = FALSE
      )
    } else {
      result_table <- current_table
    }
  }

  result_table <- datawizard::data_reorder(result_table, "Group")
  table_full <- datawizard::data_reorder(table_full, "Group")

  as.report_table(table_full, summary = result_table)
}


#' @export
report_parameters.grouped_df <- function(
  x,
  table = NULL,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  levels_percentage = "auto",
  digits = 2,
  n_entries = 3,
  missing_percentage = "auto",
  ...
) {
  out <- .report_grouped_dataframe(x)

  params_full <- NULL
  params <- NULL

  for (group in names(out$dfs)) {
    group_data <- out$dfs[[group]]

    r <- report_parameters(
      group_data,
      n = n,
      centrality = centrality,
      dispersion = dispersion,
      range = range,
      distribution = distribution,
      levels_percentage = levels_percentage,
      digits = digits,
      n_entries = n_entries,
      missing_percentage = missing_percentage,
      ...
    )

    params_full <- c(params_full, paste0(group, " - ", r))
    params <- c(params, paste0(group, " - ", summary(r)))
  }

  as.report_parameters(params_full, summary = params, ...)
}


#' @export
report_text.grouped_df <- function(
  x,
  table = NULL,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  levels_percentage = "auto",
  digits = 2,
  n_entries = 3,
  missing_percentage = "auto",
  ...
) {
  out <- .report_grouped_dataframe(x)

  text_full <- out$intro
  result_text <- out$intro

  for (group in names(out$dfs)) {
    group_data <- out$dfs[[group]]

    r <- report_parameters(
      group_data,
      n = n,
      centrality = centrality,
      dispersion = dispersion,
      range = range,
      distribution = distribution,
      levels_percentage = levels_percentage,
      digits = digits,
      n_entries = n_entries,
      missing_percentage = missing_percentage,
      ...
    )

    text_group <- paste0("\n- ", group, " (n = ", nrow(group_data), "):\n")

    text_full <- paste0(text_full, "\n", paste0(text_group, as.character(r)))
    result_text <- paste0(
      result_text,
      "\n",
      paste0(text_group, as.character(summary(r)))
    )
  }

  as.report_text(text_full, summary = result_text)
}

#' @export
report.grouped_df <- report.data.frame


#' @export
report_statistics.grouped_df <- function(
  x,
  table = NULL,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  levels_percentage = "auto",
  digits = 2,
  n_entries = 3,
  missing_percentage = "auto",
  ...
) {
  out <- .report_grouped_dataframe(x)

  text_full <- NULL
  result_text <- NULL

  for (group in names(out$dfs)) {
    group_data <- out$dfs[[group]]

    r <- report_statistics(
      group_data,
      n = n,
      centrality = centrality,
      dispersion = dispersion,
      range = range,
      distribution = distribution,
      levels_percentage = levels_percentage,
      digits = digits,
      n_entries = n_entries,
      missing_percentage = missing_percentage,
      ...
    )

    text_full <- c(text_full, paste0(group, ", ", as.character(r)))
    result_text <- c(result_text, paste0(group, ", ", as.character(summary(r))))
  }

  as.report_statistics(text_full, summary = result_text)
}


# Utils -------------------------------------------------------------------

#' @keywords internal
.report_dataframe_percentage <- function(x, percentage = "auto") {
  if (is.null(percentage) || percentage != "auto") {
    return(percentage)
  }

  if (any(inherits(x, c("data.frame", "grouped_df")))) {
    n <- nrow(x)
  } else {
    n <- length(x)
  }

  if (n >= 100) {
    percentage <- TRUE
  } else {
    percentage <- FALSE
  }
  percentage
}

#' @include report.lm.R
#' @export
report.survreg <- report.lm

#' @export
report_effectsize.survreg <- report_effectsize.lm

#' @export
report_table.survreg <- report_table.lm

#' @export
report_statistics.survreg <- report_statistics.lm

#' @export
report_parameters.survreg <- report_parameters.lm

#' @export
report_intercept.survreg <- report_intercept.lm

#' @export
report_model.survreg <- report_model.lm

#' @export
report_performance.survreg <- report_performance.lm

#' @export
report_info.survreg <- report_info.lm

#' @export
report_text.survreg <- report_text.lm

#' Citation formatting
#'
#' Convenience functions to manipulate and format citations. Only works with APA
#' formatted citations, for now.
#'
#' @param citation A character string of a citation.
#' @param authorsdate Only show authors and date (remove title, journal, etc.).
#' @param short If more than one authors, replace by `et al.`
#' @param intext Remove brackets around the date (so that it can be placed
#'   inside larger parentheses).
#'
#' @return A character string.
#'
#' @examples
#' library(report)
#'
#' citation <- "Makowski, D., Ben-Shachar, M. S., Patil, I., & Ludecke, D. (2020).
#' Methods and Algorithms for Correlation Analysis in R. Journal of Open Source
#' Software, 5(51), 2306."
#'
#' format_citation(citation, authorsdate = TRUE)
#' format_citation(citation, authorsdate = TRUE, short = TRUE)
#' format_citation(citation, authorsdate = TRUE, short = TRUE, intext = TRUE)
#'
#' cite_citation(citation)
#' clean_citation(citation())
#' @export
format_citation <- function(
  citation,
  authorsdate = FALSE,
  short = FALSE,
  intext = FALSE
) {
  if (isTRUE(authorsdate)) {
    citation <- trimws(gsub(")..*", ")", citation)) # Remove everything after first parenthesis (hopefully, the date)
    citation <- gsub("[A-Z]\\., ", "", citation) # Remove last first names
    citation <- gsub("[A-Z]\\. ", "", citation) # Remove remaining first names
    citation <- gsub(", (", " (", citation, fixed = TRUE)
  }

  if (isTRUE(short)) {
    n_authors <- lengths(regmatches(
      citation,
      gregexpr(",", citation, fixed = TRUE)
    ))
    for (i in seq_along(n_authors)) {
      if (n_authors[i] > 1 || grepl("&", citation[i], fixed = TRUE)) {
        citation[i] <- trimws(gsub(",.*\\(", " et al. (", citation[i])) # Replace remaining authors by et al.
      }
    }
  }

  if (isTRUE(intext)) {
    citation <- trimws(gsub(").*", "", citation))
    citation <- trimws(gsub(" (", ", ", citation, fixed = TRUE))
  }

  citation
}


#' @rdname format_citation
#' @export
cite_citation <- function(citation) {
  citation <- format_citation(
    citation,
    authorsdate = TRUE,
    short = TRUE,
    intext = TRUE
  )
  paste0("(", citation, ")")
}


#' @rdname format_citation
#' @export
clean_citation <- function(citation) {
  if (isTRUE(inherits(citation, "citation"))) {
    citation <- format(citation, style = "text")
  }
  citation <- unlist(strsplit(citation, "\n", fixed = TRUE))
  citation <- paste(citation, collapse = "SPLIT")
  citation <- unlist(strsplit(citation, "SPLITSPLIT", fixed = TRUE))
  i <- 1
  while (grepl("To cite ", citation[i], fixed = TRUE)) {
    i <- i + 1
  }
  citation <- gsub(
    "  ",
    " ",
    trimws(gsub("SPLIT", " ", citation[i], fixed = TRUE), which = "both"),
    fixed = TRUE
  )
  as.character(citation)
}

test_that("format_citation() works with basic formatting", {
  citation <- paste0(
    "Makowski, D., Ben-Shachar, M. S., Patil, I., & Ludecke, D. (2020). Methods and ",
    "Algorithms for Correlation Analysis in R. Journal of Open Source Software, 5(51), 2306."
  )

  # Test basic citation (no changes)
  result_basic <- format_citation(citation)
  expect_identical(result_basic, citation)

  # Test authorsdate option
  result_authorsdate <- format_citation(citation, authorsdate = TRUE)
  expect_match(result_authorsdate, "^Makowski")
  expect_match(result_authorsdate, "\\(2020\\)$")
  expect_false(grepl("Journal", result_authorsdate, fixed = TRUE))

  # Test short option (with authorsdate)
  result_short <- format_citation(citation, authorsdate = TRUE, short = TRUE)
  expect_match(result_short, "Makowski.*et al\\.")
  expect_false(grepl("Ben-Shachar", result_short, fixed = TRUE))

  # Test intext option (remove parentheses around date)
  result_intext <- format_citation(citation, authorsdate = TRUE, intext = TRUE)
  expect_match(result_intext, "Makowski.*2020")
  expect_false(grepl("(2020)", result_intext, fixed = TRUE))

  # Test all options combined
  result_all <- format_citation(
    citation,
    authorsdate = TRUE,
    short = TRUE,
    intext = TRUE
  )
  expect_match(result_all, "^Makowski.*et al.*2020$")
  expect_false(grepl("[\\(\\)]", result_all))
})

test_that("format_citation() handles multiple citations", {
  citations <- c(
    "Makowski, D., Ben-Shachar, M. S., Patil, I., & Ludecke, D. (2020). Title 1.",
    "Smith, J. (2019). Title 2."
  )

  result <- format_citation(citations, authorsdate = TRUE, short = TRUE)
  expect_length(result, 2)
  expect_match(result[1], "Makowski.*et al")
  expect_match(result[2], "Smith.*\\(2019\\)")
})

test_that("cite_citation() creates proper parenthetical citation", {
  citation <- paste0(
    "Makowski, D., Ben-Shachar, M. S., Patil, I., & Ludecke, D. (2020). ",
    "Methods and Algorithms for Correlation Analysis in R."
  )

  result <- cite_citation(citation)
  expect_match(result, "^\\(Makowski.*et al.*2020\\)$")
})

test_that("clean_citation() works with citation objects", {
  # Test with character string (basic case)
  citation_text <- "Makowski, D. et al. (2020). Some Title."
  result_text <- clean_citation(citation_text)
  expect_type(result_text, "character")
  expect_length(result_text, 1)

  # Test with citation object format (simulated)
  citation_with_prefix <- paste0(
    "To cite package 'report' in publications use:\n\n  Makowski, D. et al. (2020). ",
    "Some Title.\n\nA BibTeX entry for LaTeX users is"
  )
  result_clean <- clean_citation(citation_with_prefix)
  expect_false(grepl("To cite", result_clean, fixed = TRUE))
  expect_match(result_clean, "Makowski")
})

test_that("format_citation() handles edge cases", {
  # Single author
  single_author <- "Smith, J. (2020). Title."
  result_single <- format_citation(
    single_author,
    authorsdate = TRUE,
    short = TRUE
  )
  expect_false(grepl("et al", result_single, fixed = TRUE))
  expect_match(result_single, "Smith.*\\(2020\\)")

  # Citation with middle initials
  with_initials <- "Makowski, D. M., Ben-Shachar, M. S. (2020). Title."
  result_initials <- format_citation(with_initials, authorsdate = TRUE)
  expect_false(grepl("D. M.", result_initials, fixed = TRUE))
  expect_match(result_initials, "Makowski.*Ben-Shachar")
})

#' @include report.lm.R
#' @export
report.ivreg <- report.lm

#' @export
report_effectsize.ivreg <- report_effectsize.lm

#' @export
report_table.ivreg <- report_table.lm

#' @export
report_statistics.ivreg <- report_statistics.lm

#' @export
report_parameters.ivreg <- report_parameters.lm

#' @export
report_intercept.ivreg <- report_intercept.lm

#' @export
report_model.ivreg <- report_model.lm

#' @export
report_performance.ivreg <- report_performance.lm

#' @export
report_info.ivreg <- report_info.lm

#' @export
report_text.ivreg <- report_text.lm

#' Template to add report support for new objects
#'
#' Template file to add report support for new objects. Check-out the vignette on
#' [Supporting
#' New Models](https://easystats.github.io/report/articles/new_models.html).
#'
#' @param x Object of class `NEW OBJECT`.
#' @inheritParams report
#'
#' @inherit report return seealso
#'
#' @examples
#' library(report)
#'
#' # Add a reproducible example instead of the following
#' model <- lm(Sepal.Length ~ Petal.Length * Species, data = iris)
#' r <- report(model)
#' r
#' summary(r)
#' as.data.frame(r)
#' summary(as.data.frame(r))
#' @return An object of class [report()].
#' @export
report.default <- function(x, ...) {
  stop(.error_message(x, "report()"), call. = FALSE)
}


# report_effectsize -------------------------------------------------------

#' @rdname report.default
#' @export
report_effectsize.default <- function(x, ...) {
  stop(.error_message(x, "report_effectsize()"), call. = FALSE)
}


# report_table ------------------------------------------------------------

#' @rdname report.default
#' @export
report_table.default <- function(x, ...) {
  stop(.error_message(x, "report_table()"), call. = FALSE)
}


# report_statistics ------------------------------------------------------------

#' @rdname report.default
#' @export
report_statistics.default <- function(x, ...) {
  stop(.error_message(x, "report_statistics()"), call. = FALSE)
}


# report_parameters ------------------------------------------------------------

#' @rdname report.default
#' @export
report_parameters.default <- function(x, ...) {
  stop(.error_message(x, "report_parameters()"), call. = FALSE)
}


# report_intercept ------------------------------------------------------------

#' @rdname report.default
#' @export
report_intercept.default <- function(x, ...) {
  stop(.error_message(x, "report_intercept()"), call. = FALSE)
}


# report_model ------------------------------------------------------------

#' @rdname report.default
#' @export
report_model.default <- function(x, ...) {
  stop(.error_message(x, "report_model()"), call. = FALSE)
}


# report_random ------------------------------------------------------------

#' @rdname report.default
#' @export
report_random.default <- function(x, ...) {
  stop(.error_message(x, "report_random()"), call. = FALSE)
}


# report_priors ------------------------------------------------------------

#' @rdname report.default
#' @export
report_priors.default <- function(x, ...) {
  stop(.error_message(x, "report_priors()"), call. = FALSE)
}


# report_performance ------------------------------------------------------------

#' @rdname report.default
#' @export
report_performance.default <- function(x, ...) {
  stop(.error_message(x, "report_performance()"), call. = FALSE)
}


# report_info ------------------------------------------------------------

#' @rdname report.default
#' @export
report_info.default <- function(x, ...) {
  stop(.error_message(x, "report_info()"), call. = FALSE)
}


# report_text ------------------------------------------------------------

#' @rdname report.default
#' @export
report_text.default <- function(x, ...) {
  stop(.error_message(x, "report_text()"), call. = FALSE)
}

#' @rdname format_formula
#' @examples
#' model <- lm(Sepal.Length ~ Species, data = iris)
#' format_algorithm(model)
#'
#' @examplesIf requireNamespace("lme4", quietly = TRUE)
#' # Mixed models
#' library(lme4)
#' model <- lme4::lmer(Sepal.Length ~ Sepal.Width + (1 | Species), data = iris)
#' format_algorithm(model)
#' @return A character string.
#' @export
format_algorithm <- function(x) {
  algorithm <- suppressWarnings(insight::find_algorithm(x))

  result <- ""

  if (is.null(algorithm$algorithm)) {
    return(result)
  }

  # Name
  result <- algorithm$algorithm
  if (result == "sampling") {
    result <- "MCMC sampling"
  }

  # Chains
  if (!is.null(algorithm$chains)) {
    result <- paste0(
      result,
      " with ",
      algorithm$chains,
      " chains"
    )
    if (!is.null(algorithm$iterations)) {
      result <- paste0(
        result,
        " of ",
        algorithm$iterations,
        " iterations"
      )
    }
    if (!is.null(algorithm$warmup)) {
      result <- paste0(
        result,
        " and a warmup of ",
        algorithm$warmup
      )
    }
  }

  # Optimizer
  if (!is.null(algorithm$optimizer)) {
    optimizer <- algorithm$optimizer[1]

    if (optimizer == "bobyqa") {
      optimizer <- "BOBYQA"
    } else if (optimizer == "Nelder_Mead") {
      optimizer <- "Nelder-Mead"
    }
    result <- paste0(
      result,
      " and ",
      optimizer,
      " optimizer"
    )
  }

  result
}

#' Report the parameters of a model
#'
#' Creates a list containing a description of the parameters of R objects (see
#' list of supported objects in [report()]).
#'
#' @inheritParams report
#' @inheritParams report_table
#' @inheritParams report_text
#' @inheritParams as.report
#'
#' @return A `vector`.
#'
#' @examples
#' \donttest{
#' library(report)
#'
#' # Miscellaneous
#' r <- report_parameters(sessionInfo())
#' r
#' summary(r)
#'
#' # Data
#' report_parameters(iris$Sepal.Length)
#' report_parameters(as.character(round(iris$Sepal.Length, 1)))
#' report_parameters(iris$Species)
#' report_parameters(iris)
#'
#' # h-tests
#' report_parameters(t.test(iris$Sepal.Width, iris$Sepal.Length))
#'
#' # ANOVA
#' report_parameters(aov(Sepal.Length ~ Species, data = iris))
#'
#' # GLMs
#' report_parameters(lm(Sepal.Length ~ Petal.Length * Species, data = iris))
#' report_parameters(lm(Petal.Width ~ Species, data = iris), include_intercept = FALSE)
#' report_parameters(glm(vs ~ disp, data = mtcars, family = "binomial"))
#' }
#'
#' @examplesIf requireNamespace("lme4", quietly = TRUE)
#' \donttest{
#' # Mixed models
#' library(lme4)
#' model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
#' report_parameters(model)
#' }
#'
#' @examplesIf requireNamespace("rstanarm", quietly = TRUE)
#' \donttest{
#' # Bayesian models
#' library(rstanarm)
#' model <- suppressWarnings(stan_glm(Sepal.Length ~ Species, data = iris, refresh = 0, iter = 600))
#' report_parameters(model)
#' }
#' @export
report_parameters <- function(x, ...) {
  UseMethod("report_parameters")
}


# METHODS -----------------------------------------------------------------

#' @rdname as.report
#' @export
as.report_parameters <- function(x, summary = NULL, prefix = "  - ", ...) {
  class(x) <- unique(c("report_parameters", class(x)))
  attributes(x) <- c(attributes(x), list(...))
  attr(x, "prefix") <- prefix

  if (!is.null(summary)) {
    class(summary) <- unique(c("report_parameters", class(summary)))
    attr(summary, "prefix") <- prefix
    attr(x, "summary") <- summary
  }
  x
}

#' @export
as.character.report_parameters <- function(x, prefix = NULL, ...) {
  # Find prefix
  if (is.null(prefix)) {
    prefix <- attributes(x)$prefix
  }
  if (is.null(prefix)) {
    prefix <- ""
  }

  # Concatenate
  text_output <- paste0(prefix, x)
  text_output <- paste(text_output, collapse = "\n")
  text_output
}

#' @export
summary.report_parameters <- function(object, ...) {
  if (is.null(attributes(object)$summary)) {
    object
  } else {
    attributes(object)$summary
  }
}


#' @export
print.report_parameters <- function(x, ...) {
  cat(as.character(x, ...))
}


# Utils -------------------------------------------------------------------

#' @keywords internal
.format_parameters_aov <- function(names) {
  for (i in seq_along(names)) {
    if (grepl(":", names[i], fixed = TRUE)) {
      interaction_terms <- unlist(strsplit(names[i], ":", fixed = TRUE))
      names[i] <- paste0(
        "The interaction between ",
        datawizard::text_concatenate(interaction_terms)
      )
    } else {
      names[i] <- paste0("The main effect of ", names[i])
    }
  }
  names
}

#' @keywords internal
.format_parameters_regression <- function(names) {
  for (i in seq_along(names)) {
    # Interaction
    if (grepl(" * ", names[i], fixed = TRUE)) {
      parts <- unlist(strsplit(names[i], " * ", fixed = TRUE))
      basis <- paste(utils::head(parts, -1), collapse = " * ")
      names[i] <- paste0(
        "The interaction effect of ",
        utils::tail(parts, 1),
        " on ",
        basis
      )

      # Intercept
    } else if (names[i] == "(Intercept)") {
      names[i] <- paste0("The intercept")

      # No interaction
    } else {
      names[i] <- paste0("The effect of ", names[i])
    }
  }
  names
}

#' @keywords internal
.parameters_starting_text <- function(x, params) {
  if ("pretty_names" %in% attributes(params)) {
    pretty_name <- attributes(params)$pretty_names[params$Parameter]
  } else if (
    inherits(x, "glmmTMB") &&
      "Component" %in% colnames(params) &&
      length(unique(params$Component[!is.na(params$Component)])) > 1
  ) {
    # For glmmTMB models with multiple components, need to handle component-specific naming
    # Create component-aware parameter names for glmmTMB models
    formatted_params <- parameters::format_parameters(x)
    # Map parameter names to include component information
    pretty_name <- character(nrow(params))
    for (i in seq_len(nrow(params))) {
      param_name <- params$Parameter[i]
      component <- params$Component[i]

      if (
        !is.na(param_name) &&
          !is.na(component) &&
          param_name %in% names(formatted_params)
      ) {
        if (component == "dispersion") {
          # For dispersion component, modify the name to indicate it affects variability
          if (param_name == "(Intercept)") {
            pretty_name[i] <- "(Intercept) (dispersion)"
          } else {
            pretty_name[i] <- paste0(
              formatted_params[param_name],
              " (on dispersion)"
            )
          }
        } else {
          # For conditional component, use standard name
          pretty_name[i] <- formatted_params[param_name]
        }
      } else {
        pretty_name[i] <- param_name
      }
    }
    names(pretty_name) <- params$Parameter
  } else {
    # Try to get formatted parameter names from the model
    # If this fails (e.g., with mock objects), fall back to using parameter names from table
    pretty_name <- tryCatch(
      {
        parameters::format_parameters(x)
      },
      warning = function(w) {
        # If we can't get parameters from the model, use the parameter names from the table
        param_names <- params$Parameter
        names(param_names) <- param_names
        param_names
      },
      error = function(e) {
        # If we can't get parameters from the model, use the parameter names from the table
        param_names <- params$Parameter
        names(param_names) <- param_names
        param_names
      }
    )
  }

  text_output <- vapply(
    pretty_name,
    .format_parameters_regression,
    USE.NAMES = TRUE,
    "character"
  )

  text_output
}


#' @keywords internal
.parameters_diagnostic_bayesian <- function(
  diagnostic,
  only_when_insufficient = FALSE,
  ...
) {
  # init
  convergence <- stability <- NULL
  # Convergence
  if ("Rhat" %in% names(diagnostic)) {
    convergence <- effectsize::interpret_rhat(diagnostic$Rhat, ...)
    text_output <- ifelse(
      convergence == "converged",
      paste0(
        "The estimation successfully converged (Rhat = ",
        insight::format_value(diagnostic$Rhat, digits = 3),
        ")"
      ),
      paste0(
        "However, the estimation might not have successfuly converged (Rhat = ",
        insight::format_value(diagnostic$Rhat, digits = 3),
        ")"
      )
    )

    if ("ESS" %in% names(diagnostic)) {
      stability <- effectsize::interpret_ess(diagnostic$ESS, ...)
      text_output <- lapply(seq_along(stability), function(x) {
        y <- switch(
          EXPR = paste(
            stability[x] == "sufficient",
            convergence[x] == "converged"
          ),
          "TRUE TRUE" = paste0(
            text_output,
            " and the indices are reliable (ESS = ",
            insight::format_value(diagnostic$ESS, digits = 0),
            ")"
          ),
          "TRUE FALSE" = paste0(
            text_output,
            " even though the indices appear as reliable (ESS = ",
            insight::format_value(diagnostic$ESS, digits = 0),
            ")"
          ),
          "FALSE TRUE" = paste0(
            text_output,
            " but the indices are unreliable (ESS = ",
            insight::format_value(diagnostic$ESS, digits = 0),
            ")"
          ),
          "FALSE FALSE" = paste0(
            text_output,
            " and the indices are unreliable (ESS = ",
            insight::format_value(diagnostic$ESS, digits = 0),
            ")"
          )
        )
        y[[x]]
      })
      text_output <- unlist(text_output)
    }
  } else {
    text_output <- ""
  }

  if (only_when_insufficient) {
    ifelse(
      !identical(convergence, "converged") |
        !identical(stability, "sufficient"),
      text_output,
      ""
    )
  } else {
    text_output
  }
}

test_that("glmmTMB duplication fix - parameters are deduplicated", {
  # Test that the fix for glmmTMB parameter duplication works correctly
  # This test reproduces the issue described in #551 and verifies the fix

  # Mock glmmTMB model
  mock_model <- structure(list(), class = c("glmmTMB", "lm"))

  # Create table with duplicated rows (simulating the problematic scenario)
  # This represents what might come from report_table for glmmTMB with duplicated
  # parameters
  duplicated_table <- data.frame(
    Parameter = c(
      "(Intercept)",
      "treatmentcontrol",
      "treatmentcontrol",
      "treatmentcontrol",
      "treatmentcontrol",
      "treatmentcontrol"
    ),
    Component = c("cond", "cond", "cond", "cond", "cond", "cond"),
    Coefficient = c(0.13, -0.27, -0.27, -0.27, -0.27, -0.27),
    SE = c(0.25, 0.36, 0.36, 0.36, 0.36, 0.36),
    CI_low = c(-0.37, -0.99, -0.99, -0.99, -0.99, -0.99),
    CI_high = c(0.64, 0.45, 0.45, 0.45, 0.45, 0.45),
    t = c(0.52, -0.75, -0.75, -0.75, -0.75, -0.75),
    df_error = c(18, 18, 18, 18, 18, 18),
    p = c(0.606, 0.466, 0.466, 0.466, 0.466, 0.466),
    stringsAsFactors = FALSE
  )

  # Test report_parameters with duplicated table (this should be deduplicated)
  params_result <- report_parameters(
    mock_model,
    table = duplicated_table,
    include_intercept = FALSE
  )
  params_text <- as.character(params_result)

  # Count the number of parameter lines in the output
  lines <- trimws(strsplit(params_text, "\n", fixed = TRUE)[[1]])
  non_empty_lines <- lines[nzchar(lines)]

  # Should have exactly 1 parameter line (not 5) after deduplication
  expect_equal(
    length(non_empty_lines),
    1L,
    info = paste(
      "Expected 1 parameter line after deduplication, got",
      length(non_empty_lines),
      "lines:",
      paste(non_empty_lines, collapse = " | ")
    )
  )

  # Verify the content is correct (contains the treatment parameter information)
  expect_true(
    grepl("beta = -0.27", params_text, fixed = TRUE),
    info = paste(
      "Expected to find treatment parameter (beta = -0.27) in:",
      params_text
    )
  )

  # Verify it's marked as non-significant and negative as expected
  expect_true(
    grepl(
      "statistically non-significant and negative",
      params_text,
      fixed = TRUE
    ),
    info = paste(
      "Expected to find significance interpretation in:",
      params_text
    )
  )
})

test_that("glmmTMB deduplication preserves unique parameters", {
  # Test that deduplication doesn't remove legitimately different parameters

  mock_model <- structure(list(), class = c("glmmTMB", "lm"))

  # Create table with different parameters (these should NOT be deduplicated)
  mixed_table <- data.frame(
    Parameter = c("(Intercept)", "treatmentcontrol", "covariate1"),
    Component = c("cond", "cond", "cond"),
    Coefficient = c(0.13, -0.27, 0.45),
    SE = c(0.25, 0.36, 0.20),
    CI_low = c(-0.37, -0.99, 0.05),
    CI_high = c(0.64, 0.45, 0.85),
    t = c(0.52, -0.75, 2.25),
    df_error = c(18, 18, 18),
    p = c(0.606, 0.466, 0.033),
    stringsAsFactors = FALSE
  )

  # Test report_parameters with mixed table
  params_result <- report_parameters(
    mock_model,
    table = mixed_table,
    include_intercept = FALSE
  )
  params_text <- as.character(params_result)

  # Count the number of parameter lines in the output
  lines <- trimws(strsplit(params_text, "\n", fixed = TRUE)[[1]])
  non_empty_lines <- lines[nzchar(lines)]

  # Should have exactly 2 parameter lines (excluding intercept)
  expect_equal(
    length(non_empty_lines),
    2L,
    info = paste(
      "Expected 2 parameter lines for different parameters, got",
      length(non_empty_lines),
      "lines:",
      paste(non_empty_lines, collapse = " | ")
    )
  )

  # Verify both parameters are present
  expect_true(
    grepl("beta = -0.27", params_text, fixed = TRUE),
    info = paste("Expected to find treatment parameter in:", params_text)
  )

  expect_true(
    grepl("beta = 0.45", params_text, fixed = TRUE),
    info = paste("Expected to find covariate parameter in:", params_text)
  )
})

test_that("glmmTMB deduplication works without Component column", {
  # Test that the fix works for glmmTMB tables that don't have a Component column

  mock_model <- structure(list(), class = c("glmmTMB", "lm"))

  # Create table with duplicated rows but no Component column
  duplicated_table_no_component <- data.frame(
    Parameter = c(
      "(Intercept)",
      "treatmentcontrol",
      "treatmentcontrol",
      "treatmentcontrol"
    ),
    Coefficient = c(0.13, -0.27, -0.27, -0.27),
    SE = c(0.25, 0.36, 0.36, 0.36),
    CI_low = c(-0.37, -0.99, -0.99, -0.99),
    CI_high = c(0.64, 0.45, 0.45, 0.45),
    t = c(0.52, -0.75, -0.75, -0.75),
    df_error = c(18, 18, 18, 18),
    p = c(0.606, 0.466, 0.466, 0.466),
    stringsAsFactors = FALSE
  )

  # Test report_parameters with duplicated table (should be deduplicated)
  params_result <- report_parameters(
    mock_model,
    table = duplicated_table_no_component,
    include_intercept = FALSE
  )
  params_text <- as.character(params_result)

  # Count the number of parameter lines in the output
  lines <- trimws(strsplit(params_text, "\n", fixed = TRUE)[[1]])
  non_empty_lines <- lines[nzchar(lines)]

  # Should have exactly 1 parameter line after deduplication
  expect_equal(
    length(non_empty_lines),
    1L,
    info = paste(
      "Expected 1 parameter line after deduplication (no Component), got",
      length(non_empty_lines),
      "lines:",
      paste(non_empty_lines, collapse = " | ")
    )
  )

  # Verify the content is correct
  expect_true(
    grepl("beta = -0.27", params_text, fixed = TRUE),
    info = paste(
      "Expected to find treatment parameter (beta = -0.27) in:",
      params_text
    )
  )
})

test_that("glmmTMB deduplication only applies to glmmTMB models", {
  # Test that deduplication logic only applies to glmmTMB models, not other models

  # Mock regular lm model (not glmmTMB)
  mock_lm <- structure(list(), class = "lm")

  # Create table with duplicated rows
  duplicated_table <- data.frame(
    Parameter = c("(Intercept)", "treatmentcontrol", "treatmentcontrol"),
    Coefficient = c(0.13, -0.27, -0.27),
    SE = c(0.25, 0.36, 0.36),
    CI_low = c(-0.37, -0.99, -0.99),
    CI_high = c(0.64, 0.45, 0.45),
    t = c(0.52, -0.75, -0.75),
    df_error = c(18, 18, 18),
    p = c(0.606, 0.466, 0.466),
    stringsAsFactors = FALSE
  )

  # Test report_parameters with regular lm model (should NOT deduplicate)
  params_result <- report_parameters(
    mock_lm,
    table = duplicated_table,
    include_intercept = FALSE
  )
  params_text <- as.character(params_result)

  # Count the number of parameter lines in the output
  lines <- trimws(strsplit(params_text, "\n", fixed = TRUE)[[1]])
  non_empty_lines <- lines[nzchar(lines)]

  # Should have 2 parameter lines (no deduplication for non-glmmTMB models)
  expect_equal(
    length(non_empty_lines),
    2L,
    info = paste(
      "Expected 2 parameter lines for regular lm (no deduplication), got",
      length(non_empty_lines),
      "lines:",
      paste(non_empty_lines, collapse = " | ")
    )
  )
})

test_that("report_statistics() works with linear models", {
  model <- lm(Sepal.Length ~ Species, data = iris)

  result <- report_statistics(model)

  expect_s3_class(result, "report_statistics")
  expect_type(result, "character")
  # Just check that it's not empty, not specific length
  expect_gt(length(result), 0)
})

test_that("report_statistics() works with t-test", {
  test_result <- t.test(iris$Sepal.Width, iris$Sepal.Length)

  result <- report_statistics(test_result)

  expect_s3_class(result, "report_statistics")
  expect_type(result, "character")
})

test_that("report_statistics() works with ANOVA", {
  aov_result <- aov(Sepal.Length ~ Species, data = iris)

  result <- report_statistics(aov_result)

  expect_s3_class(result, "report_statistics")
  expect_type(result, "character")
})

test_that("report_statistics() works with basic data types", {
  # Character vector
  char_data <- c("red", "blue", "red", "green")
  result1 <- report_statistics(char_data)
  expect_s3_class(result1, "report_statistics")
  expect_match(as.character(result1), "red")

  # Factor
  factor_data <- factor(c("low", "medium", "high", "low"))
  result2 <- report_statistics(factor_data)
  expect_s3_class(result2, "report_statistics")
  expect_match(as.character(result2), "low")
})

test_that("as.report_statistics() works correctly", {
  text_stats <- "Mean = 5.0, SD = 1.2"
  summary_stats <- "Mean = 5.0"

  result <- as.report_statistics(text_stats, summary = summary_stats)

  expect_s3_class(result, "report_statistics")
  expect_identical(as.character(result), text_stats)
  expect_identical(as.character(summary(result)), summary_stats)
})

test_that("report_statistics methods work", {
  text_stats <- "Test statistics"
  result <- as.report_statistics(text_stats)

  # Test summary (should return itself if no summary attribute)
  summ <- summary(result)
  expect_identical(summ, result)

  # Test print
  expect_output(print(result), "Test statistics")
})

test_that("report_parameters() works with linear models", {
  model <- lm(Sepal.Length ~ Species, data = iris)

  result <- report_parameters(model)

  expect_s3_class(result, "report_parameters")
  expect_type(result, "character")
  expect_true(nzchar(as.character(result)))
})

test_that("report_parameters() works with t-test", {
  test_result <- t.test(iris$Sepal.Width, iris$Sepal.Length)

  result <- report_parameters(test_result)

  expect_s3_class(result, "report_parameters")
  expect_type(result, "character")
})

test_that("report_parameters() works with basic data types", {
  # Character vector
  char_data <- c("red", "blue", "red", "green", "blue")
  result1 <- report_parameters(char_data)
  expect_s3_class(result1, "report_parameters")
  expect_match(as.character(result1), "red")

  # Factor
  factor_data <- factor(c("A", "B", "A", "C"))
  result2 <- report_parameters(factor_data)
  expect_s3_class(result2, "report_parameters")
  expect_match(as.character(result2), "A")
})

test_that("as.report_parameters() works correctly", {
  params_text <- c("Parameter A = 1.5", "Parameter B = 2.0")
  summary_text <- c("A = 1.5", "B = 2.0")

  result <- as.report_parameters(params_text, summary = summary_text)

  expect_s3_class(result, "report_parameters")
  expect_true(all(grepl("Parameter", as.character(result), fixed = TRUE)))
  expect_true(all(grepl("A|B", as.character(summary(result)))))
})

test_that("report_parameters methods work", {
  params_text <- "Test parameters"
  result <- as.report_parameters(params_text)

  # Test summary
  summ <- summary(result)
  expect_identical(summ, result) # Should return itself if no summary

  # Test print
  expect_output(print(result), "Test parameters")
})

#' Reporting Bayesian Model Comparison
#'
#' Automatically report the results of Bayesian model comparison using the `loo` package.
#'
#' @param x An object of class [brms::loo_compare].
#' @param include_IC Whether to include the information criteria (IC).
#' @param include_ENP Whether to include the effective number of parameters (ENP).
#' @param ... Additional arguments (not used for now).
#'
# nolint start
#' @examplesIf requireNamespace("brms", quietly = TRUE) && requireNamespace("RcppEigen", quietly = TRUE) && requireNamespace("BH", quietly = TRUE)
# nolint end
#' \donttest{
#' library(brms)
#'
#' m1 <- brms::brm(mpg ~ qsec, data = mtcars)
#' m2 <- brms::brm(mpg ~ qsec + drat, data = mtcars)
#' m3 <- brms::brm(mpg ~ qsec + drat + wt, data = mtcars)
#'
#' x <- suppressWarnings(brms::loo_compare(
#'   brms::add_criterion(m1, "loo"),
#'   brms::add_criterion(m2, "loo"),
#'   brms::add_criterion(m3, "loo"),
#'   model_names = c("m1", "m2", "m3")
#' ))
#' report(x)
#' report(x, include_IC = FALSE)
#' report(x, include_ENP = TRUE)
#' }
#'
#' @details
#' The rule of thumb is that the models are "very similar" if |elpd_diff| (the
#' absolute value of elpd_diff) is less than 4 (Sivula, Magnusson and Vehtari, 2020).
#' If superior to 4, then one can use the SE to obtain a standardized difference
#' (Z-diff) and interpret it as such, assuming that the difference is normally
#' distributed. The corresponding p-value is then calculated as `2 * pnorm(-abs(Z-diff))`.
#' However, note that if the raw ELPD difference is small (less than 4), it doesn't
#' make much sense to rely on its standardized value: it is not very useful to
#' conclude that a model is much better than another if both models make very
#' similar predictions.
#'
#' @return Objects of class [report_text()].
#' @export
report.compare.loo <- function(x, include_IC = TRUE, include_ENP = FALSE, ...) {
  # nolint start
  # https://stats.stackexchange.com/questions/608881/how-to-interpret-elpd-diff-of-bayesian-loo-estimate-in-bayesian-logistic-regress
  # nolint end
  # https://users.aalto.fi/%7Eave/CV-FAQ.html#12_What_is_the_interpretation_of_ELPD__elpd_loo__elpd_diff
  # https://users.aalto.fi/%7Eave/CV-FAQ.html#se_diff

  # The difference in expected log predictive density (elpd) between each model
  # and the best model as well as the standard error of this difference (assuming
  # the difference is approximately normal).

  # The values in the first row are 0s because the models are ordered from best to worst according to their elpd.
  x <- as.data.frame(x)
  modnames <- rownames(x)

  elpd_diff <- x[["elpd_diff"]]
  se_elpd_diff <- x[["se_diff"]]
  ic_diff <- -2 * elpd_diff

  z_elpd_diff <- elpd_diff / se_elpd_diff
  p_elpd_diff <- 2 * stats::pnorm(-abs(z_elpd_diff))
  z_ic_diff <- -z_elpd_diff

  if ("looic" %in% colnames(x)) {
    elpd <- x[["elpd_loo"]]
    enp <- x[["p_loo"]]
    index_label <- "ELPD-LOO"
    ic <- x[["looic"]]
    index_ic <- "LOOIC"
  } else {
    elpd <- x[["elpd_waic"]]
    enp <- x[["p_waic"]]
    index_label <- "ELPD-WAIC"
    ic <- x[["waic"]]
    index_ic <- "WAIC"
  }

  # TODO: The above indices-computation and name-matching should be implemented
  # in a parameters.compare.loo() function which would be run here.

  # Starting text -----
  text1 <- sprintf(
    paste0(
      "The difference in predictive accuracy, as indexed by Expected Log ",
      "Predictive Density (%s), suggests that '%s' is the best model ("
    ),
    index_label,
    modnames[1]
  )
  if (all(c(include_IC, include_ENP))) {
    if (include_IC) {
      text1 <- sprintf(paste0(text1, "%s = %.2f"), index_ic, ic[1])
    }
    if (include_ENP) {
      if (include_IC) {
        text1 <- sprintf(paste0(text1, ", ENP = %.2f)"), enp[1])
      } else {
        text1 <- sprintf(paste0(text1, "ENP = %.2f)"), enp[1])
      }
    } else {
      text1 <- paste0(text1, ")")
    }
  } else {
    text1 <- sprintf(paste0(text1, "ELPD = %.2f)"), elpd[1])
  }

  # Other models ---
  text_models <- sprintf(
    "'%s' (diff-ELPD = %.2f +- %.2f, %s",
    modnames[-1],
    elpd_diff[-1],
    se_elpd_diff[-1],
    insight::format_p(p_elpd_diff[-1])
  )

  if (all(c(include_IC, include_ENP))) {
    if (include_IC) {
      text_models <- sprintf(
        paste0(text_models, ", %s = %.2f"),
        index_ic,
        ic[-1]
      )
    }
    if (include_ENP) {
      if (include_IC) {
        text_models <- sprintf(paste0(text_models, ", ENP = %.2f)"), enp[-1])
      } else {
        text_models <- sprintf(paste0(text_models, "ENP = %.2f)"), enp[-1])
      }
    } else {
      text_models <- sprintf(paste0(text_models, ")"))
    }
  } else {
    text_models <- paste0(text_models, ")")
  }

  text1 <- paste0(
    text1,
    ", followed by ",
    datawizard::text_concatenate(text_models)
  )
  class(text1) <- c("report_text", class(text1))
  text1
}

#' @keywords internal
.error_message <- function(x, fun = "report()") {
  paste0(
    "Oops, objects of class [",
    toString(class(x)),
    "] are not supported (yet) by ",
    fun,
    " :(\n\nWant to help? Check out https://easystats.github.io/report/articles/new_models.html"
  )
}

#' Reporting models comparison
#'
#' Create reports for model comparison as obtained by the
#' [performance::compare_performance()]
#' function in the `performance` package.
#'
#' @param x Object of class `NEW OBJECT`.
#' @inheritParams report
#' @inheritParams report.lm
#'
#' @inherit report return seealso
#'
#' @examples
#' \donttest{
#' library(report)
#' library(performance)
#'
#' m1 <- lm(Sepal.Length ~ Petal.Length * Species, data = iris)
#' m2 <- lm(Sepal.Length ~ Petal.Length + Species, data = iris)
#' m3 <- lm(Sepal.Length ~ Petal.Length, data = iris)
#'
#' x <- performance::test_performance(m1, m2, m3)
#' r <- report(x)
#' r
#' summary(r)
#' as.data.frame(r)
#' summary(as.data.frame(r))
#'
#' # Specific reports
#' report_table(x)
#' report_statistics(x)
#' report_parameters(x)
#' }
#' @return An object of class [report()].
#' @export
report.test_performance <- function(x, ...) {
  performance_table <- report_table(x, table = performance_table, ...)
  performance_text <- report_text(x, ...)
  as.report(text = performance_text, table = performance_table, ...)
}

# report_table ------------------------------------------------------------

#' @rdname report.test_performance
#' @export
report_table.test_performance <- function(x, ...) {
  as.report_table(x, summary = x, as_is = TRUE)
}


# report_statistics ------------------------------------------------------------

#' @rdname report.test_performance
#' @export
report_statistics.test_performance <- function(x, table = NULL, ...) {
  if (is.null(table)) {
    perf_table <- report_table(x, ...)
  }

  stats_text <- text_short <- ""
  if ("BF" %in% names(perf_table)) {
    bf_formatted <- insight::format_bf(
      stats::na.omit(perf_table$BF),
      exact = TRUE
    )
    val <- stats_text <- datawizard::text_paste(stats_text, bf_formatted)
  }

  if ("Omega2" %in% names(perf_table)) {
    val <- stats::na.omit(perf_table$Omega2)
    text2 <- paste0(
      "Omega2 = ",
      insight::format_value(stats::na.omit(perf_table$Omega2)),
      ", ",
      insight::format_p(stats::na.omit(perf_table$p_Omega2))
    )
    stats_text <- datawizard::text_paste(stats_text, text2, sep = "; ")
  }

  if ("LR" %in% names(perf_table)) {
    val <- stats::na.omit(perf_table$LR)
    text2 <- paste0(
      "LR = ",
      insight::format_value(stats::na.omit(perf_table$LR)),
      ", ",
      insight::format_p(stats::na.omit(perf_table$p_LR))
    )
    stats_text <- datawizard::text_paste(stats_text, text2, sep = "; ")
  }

  as.report_statistics(stats_text, summary = text_short, table = perf_table)
}

# report_parameters ------------------------------------------------------------

#' @rdname report.test_performance
#' @export
report_parameters.test_performance <- report_parameters.compare_performance


# report_text ------------------------------------------------------------

#' @rdname report.test_performance
#' @export
report_text.test_performance <- function(x, table = NULL, ...) {
  stats <- report_statistics(x, table = table)
  performance_table <- attributes(stats)$table

  # Get indices
  models <- performance_table$Model
  comparison_text <- datawizard::text_concatenate(paste0(
    models,
    " (",
    stats,
    ")"
  ))
  text_short <- datawizard::text_concatenate(paste0(
    models,
    " (",
    summary(stats),
    ")"
  ))

  # Add intro sentence
  text_start <- paste0(
    "We compared ",
    insight::format_number(nrow(performance_table)),
    " ",
    ifelse(
      length(unique(performance_table$Type)) == 1,
      format_model(unique(performance_table$Type)),
      "model"
    ),
    "s"
  )
  comparison_text <- paste0(text_start, "; ", comparison_text, ".")
  text_short <- paste0(text_start, "; ", text_short, ".")

  as.report_text(comparison_text, summary = text_short)
}

#' Reporting `BFBayesFactor` objects from the `BayesFactor` package
#'
#' Interpretation of the Bayes factor output from the `BayesFactor` package.
#'
#' @param x An object of class `BFBayesFactor`.
#' @param h0,h1 Names of the null and alternative hypotheses.
#' @param table A `parameters` table (this argument is meant for internal use).
#' @param ... Other arguments to be passed to [effectsize::interpret_bf] and [insight::format_bf].
#'
#' @examplesIf requireNamespace("BayesFactor", quietly = TRUE)
#' \donttest{
#' library(BayesFactor)
#'
#' rez <- BayesFactor::ttestBF(iris$Sepal.Width, iris$Sepal.Length)
#' report_statistics(rez, exact = TRUE) # Print exact BF
#' report(rez, h0 = "the null hypothesis", h1 = "the alternative")
#'
#' rez <- BayesFactor::correlationBF(iris$Sepal.Width, iris$Sepal.Length)
#' report(rez)
#' }
#'
#' @export
report.BFBayesFactor <- function(x, h0 = "H0", h1 = "H1", ...) {
  if (inherits("BFlinearModel", class(x@numerator[[1]]))) {
    return(report(bayestestR::bayesfactor_models(x), ...))
  }

  if (length(x@numerator) > 1) {
    insight::format_alert(
      "Multiple `BFBayesFactor` models detected - reporting for the first numerator model.",
      "See help(\"get_parameters\", package = \"insight\")."
    )
    x <- x[1]
  }

  param <- parameters::parameters(x[1], ...)
  bf <- param$BF
  other_dir <- ifelse(bf < 1, "h0", "h1")

  if (other_dir == "h1") {
    other_text <- paste0(
      "There is ",
      effectsize::interpret_bf(bf, ...),
      " ",
      h1,
      " over ",
      h0,
      " (",
      report_statistics(x, ...),
      ")."
    )
  } else {
    other_text <- paste0(
      "There is ",
      effectsize::interpret_bf(1 / bf, ...),
      " ",
      h0,
      " over ",
      h1,
      " (",
      report_statistics(x, ...),
      ")."
    )
  }
  other_text
}


#' @rdname report.BFBayesFactor
#' @export
report_statistics.BFBayesFactor <- function(x, table = NULL, ...) {
  if (is.null(table)) {
    if (length(x@numerator) > 1) {
      insight::format_alert(
        "Multiple `BFBayesFactor` models detected - reporting for the first numerator model.",
        "See help(\"get_parameters\", package = \"insight\")."
      )
      x <- x[1]
    }
    table <- parameters::parameters(x, ...)
  }

  bf <- table$BF
  other_text <- ifelse(
    bf < 1,
    insight::format_bf(1 / bf, name = "BF01", ...),
    insight::format_bf(bf, name = "BF10", ...)
  )
  other_text
}

# call this function to check arguments. "select" is the argument where user
# specified column names. "arg_name" is the name of that argument, can be NULL
.check_spelling <- function(data, select) {
  wrong_arg <- paste0("specified in `", deparse(substitute(select)), "` ")
  if (
    !is.null(select) &&
      isTRUE(nzchar(select)) &&
      !all(select %in% colnames(data))
  ) {
    not_found <- select[!select %in% colnames(data)]
    insight::format_error(
      paste0(
        sprintf(
          "The following column(s) %sdon't exist in the dataset: ",
          wrong_arg
        ),
        datawizard::text_concatenate(not_found),
        "."
      ),
      .misspelled_string(colnames(data), not_found, "Possibly misspelled?")
    )
  }
}


#' Fuzzy grep, matches pattern that are close, but not identical
#' @examples
#' colnames(iris)
#' p <- sprintf("(%s){~%i}", "Spela", 2)
#' grep(pattern = p, x = colnames(iris), ignore.case = FALSE)
#' @keywords internal
#' @noRd
.fuzzy_grep <- function(x, pattern, precision = NULL) {
  if (is.null(precision)) {
    precision <- round(nchar(pattern) / 3)
  }
  if (precision > nchar(pattern)) {
    return(NULL)
  }
  p <- sprintf("(%s){~%i}", pattern, precision)
  grep(pattern = p, x = x, ignore.case = FALSE)
}


#' create a message string to tell user about matches that could possibly
#' be the string they were looking for
#'
#' @keywords internal
#' @noRd
.misspelled_string <- function(source, searchterm, default_message = NULL) {
  if (is.null(searchterm) || length(searchterm) < 1) {
    return(default_message)
  }
  # used for many matches
  more_found <- ""
  # init default
  msg <- ""
  # guess the misspelled string
  possible_strings <- unlist(
    lapply(searchterm, function(s) {
      source[.fuzzy_grep(source, s)] # nolint
    }),
    use.names = FALSE
  )
  if (length(possible_strings)) {
    msg <- "Did you mean "
    if (length(possible_strings) > 1) {
      # make sure we don't print dozens of alternatives for larger data frames
      if (length(possible_strings) > 5) {
        more_found <- sprintf(
          " We even found %i more possible matches, not shown here.",
          length(possible_strings) - 5
        )
        possible_strings <- possible_strings[1:5]
      }
      msg <- paste0(
        msg,
        "one of ",
        datawizard::text_concatenate(
          possible_strings,
          enclose = "\"",
          last = " or "
        )
      )
    } else {
      msg <- paste0(msg, "\"", possible_strings, "\"")
    }
    msg <- paste0(msg, "?", more_found)
  } else {
    msg <- default_message
  }
  # no double white space
  insight::trim_ws(msg)
}

#' Sample Description
#'
#' Create sample description table (also referred to as "Table 1").
#'
#' @param data A data frame for which descriptive statistics should be created.
#' @param by Character vector, indicating the column(s) for possible grouping
#'   of the descriptive table. Note that weighting (see `weights`) does not work
#'   with more than one grouping column.
#' @param centrality Character, indicates the statistics that should be
#'   calculated for numeric variables. May be `"mean"` (for mean and
#'   standard deviation) or `"median"` (for median and median absolute
#'   deviation) as summary.
#' @param ci Level of confidence interval for relative frequencies (proportions).
#'   If not `NULL`, confidence intervals are shown for proportions of factor
#'   levels.
#' @param ci_method Character, indicating the method how to calculate confidence
#'   intervals for proportions. Currently implemented methods are `"wald"` and
#'   `"wilson"`. Note that `"wald"` can produce intervals outside the plausible
#'   range of \[0, 1\], and thus it is recommended to prefer the `"wilson"` method.
#'   The formulae for the confidence intervals are:
#' - `"wald"`:
#'
#'   \deqn{p \pm z \sqrt{\frac{p (1 - p)}{n}}}
#'
#' - `"wilson"`:
#'
#'   \deqn{\frac{2np + z^2 \pm z \sqrt{z^2 + 4npq}}{2(n + z^2)}}
#'
#'   where `p` is the proportion (of a factor level), `q` is `1-p`, `z` is the
#'   critical z-score based on the interval level and `n` is the length of the
#'   vector (cf. *Newcombe 1998*, *Wilson 1927*).
#' @param ci_correct Logical, it `TRUE`, applies continuity correction. See
#'   *Newcombe 1998* for different correction-methods based on the chosen
#'   `ci_method`.
#' @param select Character vector, with column names that should be included in
#'   the descriptive table.
#' @param exclude Character vector, with column names that should be excluded
#'   from the descriptive table.
#' @param weights Character vector, indicating the name of a potential
#'   weight-variable. Reported descriptive statistics will be weighted by
#'   `weight`.
#' @param total Add a `Total` column.
#' @param digits Number of decimals.
#' @param n Logical, actual sample size used in the calculation of the
#' reported descriptive statistics (i.e., without the missing values).
#' @param group_by Deprecated. Use `by` instead.
#' @inheritParams report.data.frame
#'
#' @return A data frame of class `report_sample` with variable names and
#'   their related summary statistics.
#'
#' @references
#' - Newcombe, R. G. (1998). Two-sided confidence intervals for the single
#'   proportion: comparison of seven methods. Statistics in Medicine. 17 (8):
#'   857–872
#'
#' - Wilson, E. B. (1927). Probable inference, the law of succession, and statistical
#'   inference. Journal of the American Statistical Association. 22 (158): 209–212
#'
#' @examples
#' library(report)
#'
#' report_sample(iris[, 1:4])
#' report_sample(iris, select = c("Sepal.Length", "Petal.Length", "Species"))
#' report_sample(iris, by = "Species")
#' report_sample(airquality, by = "Month", n = TRUE, total = FALSE)
#'
#' # confidence intervals for proportions
#' set.seed(123)
#' d <- data.frame(x = factor(sample(letters[1:3], 100, TRUE, c(0.01, 0.39, 0.6))))
#' report_sample(d, ci = 0.95, ci_method = "wald") # ups, negative CI
#' report_sample(d, ci = 0.95, ci_method = "wilson") # negative CI fixed
#' report_sample(d, ci = 0.95, ci_correct = TRUE) # continuity correction
#' @export
report_sample <- function(
  data,
  by = NULL,
  centrality = "mean",
  ci = NULL,
  ci_method = "wilson",
  ci_correct = FALSE,
  select = NULL,
  exclude = NULL,
  weights = NULL,
  total = TRUE,
  digits = 2,
  n = FALSE,
  group_by = NULL,
  ...
) {
  ## TODO: deprecate later
  if (!is.null(group_by)) {
    insight::format_warning(
      "Argument `group_by` is deprecated and will be removed in a future release. Please use `by` instead."
    ) # nolint
    by <- group_by
  }

  # check for correct input type
  if (!is.data.frame(data)) {
    data <- tryCatch(
      as.data.frame(data, stringsAsFactors = FALSE),
      error = function(e) {
        insight::format_error(
          "`data` must be a data frame, or an object that can be coerced to a data frame."
        )
      }
    )
  }

  variables <- colnames(data)

  # select all?
  if (is.null(select)) {
    select <- colnames(data)
  }

  # for numeric indices, select related column names
  if (is.numeric(select)) {
    select <- colnames(data)[select]
  }

  # sanity check for existing columns
  .check_spelling(data, select)
  .check_spelling(data, exclude)
  .check_spelling(data, by)
  .check_spelling(data, weights)

  # variables to keep
  if (!is.null(weights)) {
    select <- unique(c(select, weights))
  }

  # variables to keep
  variables <- intersect(select, variables)

  # variables to exclude
  if (!is.null(exclude)) {
    variables <- setdiff(variables, exclude)
  }

  # for grouped data frames, use groups as by argument
  if (inherits(data, "grouped_df") && is.null(by)) {
    by <- setdiff(colnames(attributes(data)$groups), ".rows")
  }

  # grouped by?
  do_grouping <- !is.null(by) && all(by %in% colnames(data))

  # sanity check - weights and grouping
  if (!is.null(by) && length(by) > 1 && !is.null(weights)) {
    insight::format_error(
      "Cannot apply `weights` when grouping is done by more than one variable."
    )
  }

  # make clean data frame
  class(data) <- "data.frame"

  # character to factor
  data[] <- lapply(data, function(i) {
    if (is.character(i)) {
      i <- as.factor(i)
    }
    i
  })

  # coerce by columns to factor
  groups <- as.data.frame(lapply(data[by], factor))

  out <- if (isTRUE(do_grouping)) {
    result <- lapply(split(data[variables], groups), function(x) {
      x[by] <- NULL
      .generate_descriptive_table(
        x,
        centrality,
        weights,
        digits,
        n,
        ci,
        ci_method,
        ci_correct
      )
    })
    # for more than one group, fix column names. we don't want "a.b (n=10)",
    # but rather ""a, b (n=10)""
    if (length(by) > 1) {
      old_names <- datawizard::data_unite(
        unique(groups),
        new_column = ".old_names",
        separator = "."
      )[".old_names"]

      new_names <- datawizard::data_unite(
        unique(groups),
        new_column = ".new_names",
        separator = ", "
      )[".new_names"]

      name_map <- stats::setNames(new_names[[1]], old_names[[1]])
      names(result) <- name_map[names(result)]
    }
    # remember values of first columns
    variable <- result[[1]]["Variable"]
    # number of observation, based on weights
    if (is.null(weights)) {
      n_obs <- as.vector(table(data[by]))
    } else {
      n_obs <- round(as.vector(stats::xtabs(data[[weights]] ~ data[[by]])))
    }
    # column names for groups
    cn <- sprintf("%s (n=%g)", names(result), n_obs)
    # just extract summary columns
    summaries <- do.call(cbind, lapply(result, function(i) i["Summary"]))
    colnames(summaries) <- cn
    # generate data for total column, but make sure to remove missings
    total_data <- data[
      stats::complete.cases(data[by]),
      unique(c(variables, by))
    ]
    # bind all together, including total column
    final <- cbind(
      variable,
      summaries,
      Total = .generate_descriptive_table(
        total_data[setdiff(variables, by)],
        centrality,
        weights,
        digits,
        n,
        ci,
        ci_method,
        ci_correct
      )[["Summary"]]
    )
    # Remove Total column if need be
    if (isFALSE(total)) {
      final$Total <- NULL
    }
    # define total N, based on weights
    if (is.null(weights)) {
      total_n <- sum(as.vector(table(data[by])))
    } else {
      total_n <- round(
        sum(as.vector(table(data[by]))) * mean(data[[weights]], na.rm = TRUE)
      )
    }
    # add N to column name
    colnames(final)[ncol(final)] <- sprintf(
      "%s (n=%g)",
      colnames(final)[ncol(final)],
      total_n
    )
    final
  } else {
    .generate_descriptive_table(
      data[variables],
      centrality,
      weights,
      digits,
      n,
      ci,
      ci_method,
      ci_correct
    )
  }

  attr(out, "weighted") <- !is.null(weights)
  class(out) <- c("report_sample", class(out))
  out
}


# helper ------------------------

.generate_descriptive_table <- function(
  x,
  centrality,
  weights,
  digits,
  n = FALSE,
  ci = NULL,
  ci_method = "wilson",
  ci_correct = FALSE
) {
  if (is.null(weights)) {
    w <- NULL
    columns <- colnames(x)
  } else {
    w <- x[[weights]]
    columns <- setdiff(colnames(x), weights)
  }

  do.call(
    rbind,
    lapply(columns, function(cn) {
      .report_sample_row(
        x[[cn]],
        column = cn,
        centrality = centrality,
        weights = w,
        digits = digits,
        n = n,
        ci = ci,
        ci_method = ci_method,
        ci_correct = ci_correct
      )
    })
  )
}


# create a "table row", i.e. a summary from a variable ------------------------

.report_sample_row <- function(x, ...) {
  UseMethod(".report_sample_row")
}

.report_sample_row.numeric <- function(
  x,
  column,
  centrality = "mean",
  weights = NULL,
  digits = 1,
  n = FALSE,
  ...
) {
  n_stat <- ifelse(n, paste0(", ", sum(!is.na(x))), "")

  .summary <- if (centrality == "mean") {
    sprintf(
      "%.*f (%.*f)%s",
      digits,
      .weighted_mean(x, weights),
      digits,
      .weighted_sd(x, weights),
      n_stat
    )
  } else {
    sprintf(
      "%.*f (%.*f)%s",
      digits,
      .weighted_median(x, weights),
      digits,
      .weighted_mad(x, weights),
      n_stat
    )
  }

  n_label <- ifelse(n, ", n", "")
  if (centrality == "mean") {
    column <- sprintf("Mean %s (SD)%s", column, n_label)
  } else {
    column <- sprintf("Median %s (MAD)%s", column, n_label)
  }

  data.frame(
    Variable = column,
    Summary = .summary,
    stringsAsFactors = FALSE
  )
}


.report_sample_row.factor <- function(
  x,
  column,
  weights = NULL,
  digits = 1,
  ci = NULL,
  ci_method = "wilson",
  ci_correct = FALSE,
  n = FALSE,
  ...
) {
  # we need a factor here, to determine number of levels. For binary variables,
  # only one category is shown. However, for grouped data, sometimes not all
  # levels are present in each group. A character would then have only two values
  # and recognized as binary, even if it actually would be a factor with more
  # than two levels...
  if (is.character(x)) {
    x <- as.factor(x)
  }

  if (is.null(weights)) {
    table_proportions <- prop.table(table(x))
  } else {
    x[is.na(weights)] <- NA
    weights[is.na(x)] <- NA
    weights <- stats::na.omit(weights)
    x <- stats::na.omit(x)
    table_proportions <- prop.table(stats::xtabs(weights ~ x))
  }

  # for binary factors, just need one level
  if (nlevels(x) == 2) {
    table_proportions <- table_proportions[2]
  }

  # CI for proportions?
  if (is.null(ci)) {
    .summary <- sprintf("%.1f", 100 * table_proportions)
  } else {
    ci_low_high <- .ci_proportion(
      x,
      table_proportions,
      weights,
      ci,
      ci_method,
      ci_correct
    )
    .summary <- sprintf(
      "%.1f [%.1f, %.1f]",
      100 * table_proportions,
      100 * ci_low_high$ci_low,
      100 * ci_low_high$ci_high
    )
  }

  if (isTRUE(n)) {
    .summary <- paste0(
      .summary,
      ", ",
      round(sum(!is.na(x)) * as.vector(table_proportions))
    )
  }

  n_label <- ifelse(n, ", n", "")
  data.frame(
    Variable = sprintf(
      "%s [%s], %%%s",
      column,
      names(table_proportions),
      n_label
    ),
    Summary = as.vector(.summary),
    stringsAsFactors = FALSE
  )
}


.report_sample_row.character <- .report_sample_row.factor


# Standard error for confidence interval of proportions ----

.ci_proportion <- function(
  x,
  table_proportions,
  weights,
  ci,
  ci_method,
  ci_correct
) {
  ci_method <- match.arg(tolower(ci_method), c("wald", "wilson"))

  # variables
  p <- as.vector(table_proportions)
  quant <- 1 - p
  n <- length(stats::na.omit(x))
  z <- stats::qnorm((1 + ci) / 2)

  ## FIXME: Once know how to do, calculate accurate SE for weighted data

  # There is a formula how to deal with SE for weighted data, see
  # https://en.wikipedia.org/wiki/Binomial_proportion_confidence_interval
  # but it seems "p" is unknown. For now, we give a warning instead of
  # estimating p for weighted data
  if (!is.null(weights)) {
    insight::format_warning(
      "Confidence intervals are not accurate for weighted data."
    )
  }

  if (ci_method == "wilson") {
    # Wilson CIs -------------------
    if (isTRUE(ci_correct)) {
      ci_low <- (2 *
        n *
        p +
        z^2 -
        1 -
        z * sqrt(z^2 - 2 - 1 / n + 4 * p * (n * quant + 1))) /
        (2 * (n + z^2))
      ci_high <- (2 *
        n *
        p +
        z^2 +
        1 +
        z * sqrt(z^2 + 2 - 1 / n + 4 * p * (n * quant - 1))) /
        (2 * (n + z^2))
      # close to 0 or 1, then CI is 0 resp. 1
      fix_ci <- p < 0.00001 | ci_low < 0.00001
      if (any(fix_ci)) {
        ci_low[fix_ci] <- 0
      }
      fix_ci <- p > 0.99999 | ci_high > 0.99999
      if (any(fix_ci)) {
        ci_high[fix_ci] <- 1
      }
      out <- list(ci_low = ci_low, ci_high = ci_high)
    } else {
      prop <- (2 * n * p) + z^2
      moe <- z * sqrt(z^2 + 4 * n * p * quant)
      correction <- 2 * (n + z^2)
      out <- list(
        ci_low = (prop - moe) / correction,
        ci_high = (prop + moe) / correction
      )
    }
  } else {
    # Wald CIs -------------------
    moe <- z * suppressWarnings(sqrt(p * quant / n))
    if (isTRUE(ci_correct)) {
      moe <- moe + 1 / (2 * n)
    }
    out <- list(ci_low = p - moe, ci_high = p + moe)
  }
  out
}


# print-method --------------------------------------------

#' @export
print.report_sample <- function(x, layout = "horizontal", ...) {
  layout <- match.arg(layout, choices = c("horizontal", "vertical"))
  if (isTRUE(attributes(x)$weighted)) {
    header <- "# Descriptive Statistics (weighted)\n\n"
  } else {
    header <- "# Descriptive Statistics\n\n"
  }

  if (layout == "vertical") {
    x <- datawizard::data_transpose(x, colnames = TRUE, rownames = "Groups")
  }
  insight::print_colour(header, "blue")
  cat(insight::export_table(x))
}

#' @export
print_html.report_sample <- function(x, layout = "horizontal", ...) {
  layout <- match.arg(layout, choices = c("horizontal", "vertical"))
  if (isTRUE(attributes(x)$weighted)) {
    caption <- "Descriptive Statistics (weighted)"
  } else {
    caption <- "Descriptive Statistics"
  }
  if (layout == "vertical") {
    x <- datawizard::data_transpose(x, colnames = TRUE, rownames = "Groups")
  }
  insight::export_table(x, format = "html", caption = caption, ...)
}

#' @export
print_md.report_sample <- function(x, layout = "horizontal", ...) {
  layout <- match.arg(layout, choices = c("horizontal", "vertical"))
  if (isTRUE(attributes(x)$weighted)) {
    caption <- "Descriptive Statistics (weighted)"
  } else {
    caption <- "Descriptive Statistics"
  }
  if (layout == "vertical") {
    x <- datawizard::data_transpose(x, colnames = TRUE, rownames = "Groups")
  }
  insight::export_table(x, format = "markdown", caption = caption, ...)
}


# helper for weighted stuff --------------------------

.weighted_variance <- function(x, weights = NULL) {
  if (is.null(weights)) {
    return(stats::var(x, na.rm = TRUE))
  }
  x[is.na(weights)] <- NA
  weights[is.na(x)] <- NA
  weights <- stats::na.omit(weights)
  x <- stats::na.omit(x)
  xbar <- sum(weights * x) / sum(weights)
  sum(weights * ((x - xbar)^2)) / (sum(weights) - 1)
}


.weighted_sd <- function(x, weights = NULL) {
  if (is.null(weights)) {
    return(stats::sd(x, na.rm = TRUE))
  }
  sqrt(.weighted_variance(x, weights))
}


.weighted_median <- function(x, weights = NULL, p = 0.5) {
  if (is.null(weights)) {
    return(stats::median(x, na.rm = TRUE))
  }
  x[is.na(weights)] <- NA
  weights[is.na(x)] <- NA
  weights <- stats::na.omit(weights)
  x <- stats::na.omit(x)
  x_order <- order(x)
  x <- x[x_order]
  weights <- weights[x_order]
  rw <- cumsum(weights) / sum(weights)
  md_values <- min(which(rw >= p))
  if (rw[md_values] == p) {
    mean(x[md_values:(md_values + 1)])
  } else {
    x[md_values]
  }
}


.weighted_mean <- function(x, weights = NULL) {
  if (is.null(weights)) {
    return(mean(x, na.rm = TRUE))
  }
  x[is.na(weights)] <- NA
  weights[is.na(x)] <- NA
  weights <- stats::na.omit(weights)
  x <- stats::na.omit(x)
  stats::weighted.mean(x, w = weights, na.rm = TRUE)
}


.weighted_mad <- function(x, weights = NULL, constant = 1.4826) {
  center <- .weighted_median(x, weights = weights)
  x <- abs(x - center)
  constant * .weighted_median(x, weights = weights)
}

test_that("report_sample weights, coorect weighted N", {
  d <- data.frame(
    x = c("a", "a", "a", "a", "b", "b", "b", "b", "c", "c", "c", "c"),
    g = c(1, 1, 2, 2, 1, 1, 2, 2, 1, 1, 2, 2),
    w = c(0.5, 0.5, 1, 1, 1.5, 1.5, 2, 2, 1, 1, 1.5, 1.5),
    stringsAsFactors = FALSE
  )

  out1 <- report_sample(d, select = "x", by = "g")
  out2 <- report_sample(d, select = "x", by = "g", weights = "w")
  expect_identical(
    capture.output(print(out1)),
    c(
      "# Descriptive Statistics",
      "",
      "Variable | 1 (n=6) | 2 (n=6) | Total (n=12)",
      "-------------------------------------------",
      "x [a], % |    33.3 |    33.3 |         33.3",
      "x [b], % |    33.3 |    33.3 |         33.3",
      "x [c], % |    33.3 |    33.3 |         33.3"
    )
  )
  expect_identical(
    capture.output(print(out2)),
    c(
      "# Descriptive Statistics (weighted)",
      "",
      "Variable | 1 (n=6) | 2 (n=9) | Total (n=15)",
      "-------------------------------------------",
      "x [a], % |    16.7 |    22.2 |         20.0",
      "x [b], % |    50.0 |    44.4 |         46.7",
      "x [c], % |    33.3 |    33.3 |         33.3"
    )
  )

  d <- data.frame(
    x = c("a", "a", "a", "a", "b", "b", "b", "b", "c", "c", "c", "c"),
    g1 = c(1, 1, 2, 2, 1, 1, 2, 2, 1, 1, 2, 2),
    g2 = c(3, 2, 1, 3, 2, 1, 3, 2, 1, 3, 2, 1),
    w = c(0.5, 0.5, 1, 1, 1.5, 1.5, 2, 2, 1, 1, 1.5, 1.5),
    stringsAsFactors = FALSE
  )
  expect_error(
    report_sample(d, select = "x", by = c("g1", "g2"), weights = "w"),
    regex = "Cannot apply"
  )
})

test_that("report_sample check input", {
  skip_if(packageVersion("parameters") < "0.20.3")
  data(iris)
  expect_error(report_sample(lm(Sepal.Length ~ Species, data = iris)))
  expect_silent(report_sample(iris$Species))
  expect_error(
    report_sample(iris, by = "Spedies"),
    regex = "The following column"
  )
  expect_error(
    report_sample(iris, select = "Spedies"),
    regex = "The following column"
  )
})

test_that("report_sample default", {
  expect_snapshot(
    variant = "windows",
    report_sample(airquality)
  )
  expect_snapshot(
    variant = "windows",
    report_sample(mtcars)
  )
  expect_snapshot(
    variant = "windows",
    report_sample(iris)
  )
})

test_that("report_sample n = TRUE", {
  expect_snapshot(
    variant = "windows",
    report_sample(airquality, n = TRUE)
  )
  expect_equal(
    nchar(report_sample(airquality, n = TRUE)),
    c(Variable = 131, Summary = 128),
    ignore_attr = TRUE
  )
  expect_snapshot(
    variant = "windows",
    report_sample(mtcars, n = TRUE)
  )
  expect_snapshot(
    variant = "windows",
    report_sample(iris, n = TRUE)
  )
})

test_that("report_sample CI", {
  expect_snapshot(
    variant = "windows",
    report_sample(
      iris,
      select = c("Sepal.Length", "Species"),
      ci = 0.95,
      ci_method = "wald"
    )
  )

  expect_snapshot(
    variant = "windows",
    report_sample(
      iris,
      select = c("Sepal.Length", "Species"),
      ci = 0.95,
      ci_method = "wilson"
    )
  )

  set.seed(123)
  d <- data.frame(
    x = as.factor(rbinom(1000, 1, prob = 0.03)),
    w = abs(rnorm(1000, 1, 0.1))
  )
  expect_snapshot(
    variant = "windows",
    report_sample(d, ci = 0.95, select = "x", ci_method = "wald")
  )
  expect_snapshot(
    variant = "windows",
    report_sample(d, ci = 0.95, select = "x", ci_method = "wilson")
  )
  expect_snapshot(
    variant = "windows",
    report_sample(
      d,
      ci = 0.95,
      ci_correct = TRUE,
      select = "x",
      ci_method = "wald"
    )
  )
  expect_snapshot(
    variant = "windows",
    report_sample(
      d,
      ci = 0.95,
      ci_correct = TRUE,
      select = "x",
      ci_method = "wilson"
    )
  )
  expect_warning(
    report_sample(d, ci = 0.95, weights = "w", ci_method = "wald"),
    regex = "accurate"
  )
})

test_that("report_sample by", {
  expect_snapshot(
    variant = "windows",
    report_sample(airquality, by = "Month")
  )
  expect_snapshot(
    variant = "windows",
    report_sample(mtcars, by = "cyl")
  )
  expect_snapshot(
    variant = "windows",
    report_sample(iris, by = "Species")
  )
})

test_that("report_sample centrality", {
  expect_snapshot(
    variant = "windows",
    report_sample(airquality, centrality = "mean")
  )
  expect_snapshot(
    variant = "windows",
    report_sample(mtcars, centrality = "mean")
  )
  expect_snapshot(
    variant = "windows",
    report_sample(iris, centrality = "mean")
  )
  expect_snapshot(
    variant = "windows",
    report_sample(airquality, centrality = "median")
  )
  expect_snapshot(
    variant = "windows",
    report_sample(mtcars, centrality = "median")
  )
  expect_snapshot(
    variant = "windows",
    report_sample(iris, centrality = "median")
  )
})

test_that("report_sample select", {
  expect_snapshot(
    variant = "windows",
    report_sample(airquality, select = "Temp")
  )
  expect_snapshot(
    variant = "windows",
    report_sample(mtcars, select = c("mpg", "disp"))
  )
  expect_snapshot(
    variant = "windows",
    report_sample(iris, select = "Petal.Width")
  )
})

test_that("report_sample exclude", {
  expect_snapshot(
    variant = "windows",
    report_sample(airquality, exclude = "Temp")
  )
  expect_snapshot(
    variant = "windows",
    report_sample(mtcars, exclude = c("mpg", "disp"))
  )
  expect_snapshot(
    variant = "windows",
    report_sample(iris, exclude = "Petal.Width")
  )
})

test_that("report_sample total", {
  expect_snapshot(
    variant = "windows",
    report_sample(airquality, total = TRUE)
  )
  expect_snapshot(
    variant = "windows",
    report_sample(airquality, total = FALSE)
  )
  expect_snapshot(
    variant = "windows",
    report_sample(airquality, by = "Month", total = TRUE)
  )
  expect_snapshot(
    variant = "windows",
    report_sample(airquality, by = "Month", total = FALSE)
  )
  expect_snapshot(
    variant = "windows",
    report_sample(airquality, by = "Month", total = FALSE, n = TRUE)
  )
  expect_snapshot(
    variant = "windows",
    report_sample(airquality, by = "Month", total = TRUE, n = TRUE)
  )
})

test_that("report_sample digits", {
  expect_snapshot(
    variant = "windows",
    report_sample(airquality, digits = 5)
  )
  expect_snapshot(
    variant = "windows",
    report_sample(mtcars, digits = 5)
  )
  expect_snapshot(
    variant = "windows",
    report_sample(iris, digits = 5)
  )
})

test_that("report_sample weights", {
  expect_snapshot(
    report_sample(airquality, weights = "Temp"),
    variant = "windows"
  )
  expect_snapshot(report_sample(mtcars, weights = "carb"), variant = "windows")
  expect_snapshot(
    report_sample(iris, weights = "Petal.Width"),
    variant = "windows"
  )
})

test_that("report_sample grouped data frames", {
  skip_if_not_installed("datawizard")
  data(mtcars)
  mtcars_grouped <- datawizard::data_group(mtcars, "gear")
  out1 <- report_sample(mtcars_grouped, select = c("hp", "mpg"))
  out2 <- report_sample(mtcars, by = "gear", select = c("hp", "mpg"))
  expect_identical(out1, out2)
})

test_that("report_sample, with more than one grouping variable", {
  data(iris)
  set.seed(123)
  iris$grp <- sample(letters[1:3], nrow(iris), TRUE)
  out <- report_sample(
    iris,
    by = c("Species", "grp"),
    select = c("Sepal.Length", "Sepal.Width")
  )
  # verified against
  expected <- aggregate(iris["Sepal.Length"], iris[c("Species", "grp")], mean)
  expect_snapshot(variant = "windows", out)
})

test_that("report_sample, numeric select", {
  data(iris)
  out1 <- report_sample(
    iris,
    select = c("Sepal.Length", "Sepal.Width")
  )
  out2 <- report_sample(
    iris,
    select = 1:2
  )
  expect_identical(out1, out2)
})

test_that("report_sample, print vertical", {
  skip_if_not_installed("datawizard")
  data(iris)
  set.seed(123)
  iris$grp <- sample(letters[1:3], nrow(iris), TRUE)
  iris_grp <- datawizard::data_group(iris, c("Species", "grp"))
  out <- report_sample(iris_grp, select = 1:3)
  expect_snapshot(variant = "windows", print(out, layout = "vertical"))
})

#' Report the model type
#'
#' Reports the type of different R objects (see list of supported objects in
#' [report()]).
#'
#' @inheritParams report
#' @inheritParams report_table
#' @inheritParams report_text
#' @inheritParams as.report
#'
#' @return A `character` string.
#'
#' @examples
#' \donttest{
#' library(report)
#'
#' # h-tests
#' report_model(t.test(iris$Sepal.Width, iris$Sepal.Length))
#'
#' # ANOVA
#' report_model(aov(Sepal.Length ~ Species, data = iris))
#'
#' # GLMs
#' report_model(lm(Sepal.Length ~ Petal.Length * Species, data = iris))
#' report_model(glm(vs ~ disp, data = mtcars, family = "binomial"))
#' }
#'
#' @examplesIf requireNamespace("lme4", quietly = TRUE)
#' \donttest{
#' # Mixed models
#' library(lme4)
#' model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
#' report_model(model)
#' }
#'
#' @examplesIf requireNamespace("rstanarm", quietly = TRUE)
#' \donttest{
#' # Bayesian models
#' library(rstanarm)
#' model <- suppressWarnings(stan_glm(Sepal.Length ~ Species, data = iris, refresh = 0, iter = 600))
#' report_model(model)
#' }
#' @export
report_model <- function(x, table = NULL, ...) {
  UseMethod("report_model")
}


# METHODS -----------------------------------------------------------------

#' @rdname as.report
#' @export
as.report_model <- function(x, summary = NULL, ...) {
  class(x) <- unique(c("report_model", class(x)))
  attributes(x) <- c(attributes(x), list(...))

  if (!is.null(summary)) {
    attr(x, "summary") <- summary
  }
  x
}


#' @export
summary.report_model <- function(object, ...) {
  if (is.null(attributes(object)$summary)) {
    object
  } else {
    attributes(object)$summary
  }
}

#' @export
print.report_model <- function(x, ...) {
  cat(paste(x, collapse = "\n"))
}

#' Reporting models comparison
#'
#' Create reports for model comparison as obtained by the
#' [performance::compare_performance()]
#' function in the `performance` package.
#'
#' @param x Object of class `NEW OBJECT`.
#' @inheritParams report
#' @inheritParams report.lm
#'
#' @inherit report return seealso
#'
#' @examples
#' \donttest{
#' library(report)
#' library(performance)
#'
#' m1 <- lm(Sepal.Length ~ Petal.Length * Species, data = iris)
#' m2 <- lm(Sepal.Length ~ Petal.Length + Species, data = iris)
#' m3 <- lm(Sepal.Length ~ Petal.Length, data = iris)
#'
#' x <- performance::compare_performance(m1, m2, m3)
#' r <- report(x)
#' r
#' summary(r)
#' as.data.frame(r)
#' summary(as.data.frame(r))
#'
#' # Specific reports
#' report_table(x)
#' report_statistics(x)
#' report_parameters(x)
#' }
#' @return An object of class [report()].
#' @export

report.compare_performance <- function(x, ...) {
  result_table <- report_table(x, ...)
  result_text <- report_text(x, ...)
  as.report(text = result_text, table = result_table, ...)
}

# report_table ------------------------------------------------------------

#' @rdname report.compare_performance
#' @export
report_table.compare_performance <- function(x, ...) {
  result_table <- x
  table_short <- x[!names(x) %in% c("Type", "Sigma")]
  as.report_table(result_table, summary = table_short)
}


# report_statistics ------------------------------------------------------------

#' @rdname report.compare_performance
#' @export

report_statistics.compare_performance <- function(x, table = NULL, ...) {
  if (is.null(table)) {
    table <- report_table(x, ...)
  }

  result_text <- text_short <- ""

  if ("R2" %in% names(table)) {
    result_text <- datawizard::text_paste(
      result_text,
      paste0("R2 = ", insight::format_value(table$R2))
    )
    if ("R2_adjusted" %in% names(table)) {
      result_text <- datawizard::text_paste(
        result_text,
        paste0("adj. R2 = ", insight::format_value(table$R2_adjusted))
      )
      text_short <- datawizard::text_paste(
        text_short,
        paste0("adj. R2 = ", insight::format_value(table$R2_adjusted))
      )
    } else {
      text_short <- datawizard::text_paste(
        result_text,
        paste0("R2 = ", insight::format_value(table$R2))
      )
    }
  }

  if ("AIC" %in% names(table)) {
    result_text <- datawizard::text_paste(
      result_text,
      paste0("AIC = ", insight::format_value(table$AIC))
    )
  }

  if ("BIC" %in% names(table)) {
    result_text <- datawizard::text_paste(
      result_text,
      paste0("BIC = ", insight::format_value(table$BIC))
    )
    text_short <- datawizard::text_paste(
      text_short,
      paste0("BIC = ", insight::format_value(table$BIC))
    )
  }

  if ("WAIC" %in% names(table)) {
    result_text <- datawizard::text_paste(
      result_text,
      paste0("WAIC = ", insight::format_value(table$WAIC))
    )
    text_short <- datawizard::text_paste(
      text_short,
      paste0("WAIC = ", insight::format_value(table$WAIC))
    )
  }

  if ("RMSE" %in% names(table)) {
    result_text <- datawizard::text_paste(
      result_text,
      paste0("RMSE = ", insight::format_value(table$RMSE))
    )
  }

  if ("Sigma" %in% names(table)) {
    result_text <- datawizard::text_paste(
      result_text,
      paste0("Sigma = ", insight::format_value(table$Sigma))
    )
  }

  as.report_statistics(result_text, summary = text_short, table = table)
}

# report_parameters ------------------------------------------------------------

#' @rdname report.compare_performance
#' @export
report_parameters.compare_performance <- function(x, table = NULL, ...) {
  stats <- report_statistics(x, table = table, ...)
  result_table <- attributes(stats)$table

  result_text <- paste0(result_table$Model, " (", stats, ")")
  text_short <- paste0(result_table$Model, " (", summary(stats), ")")

  as.report_parameters(result_text, summary = text_short, table = result_table)
}


# report_text ------------------------------------------------------------

#' @rdname report.compare_performance
#' @export

report_text.compare_performance <- function(x, table = NULL, ...) {
  stats <- report_statistics(x, table = table)
  result_table <- attributes(stats)$table

  # Get indices
  models <- result_table$Model
  result_text <- datawizard::text_concatenate(paste0(models, " (", stats, ")"))
  text_short <- datawizard::text_concatenate(paste0(
    models,
    " (",
    summary(stats),
    ")"
  ))

  # Add intro sentence
  text_start <- paste0(
    "We compared ",
    insight::format_number(nrow(result_table)),
    " ",
    ifelse(
      length(unique(result_table$Type)) == 1,
      format_model(unique(result_table$Type)),
      "model"
    ),
    "s"
  )
  result_text <- paste0(text_start, "; ", result_text, ".")
  text_short <- paste0(text_start, "; ", text_short, ".")

  as.report_text(result_text, summary = text_short)
}

#' Report the effect size(s) of a model or a test
#'
#' Computes, interpret and formats the effect sizes of a variety of models and
#' statistical tests (see list of supported objects in [report()]).
#'
#' @inheritParams report
#' @inheritParams report_table
#' @inheritParams report_text
#' @inheritParams as.report
#'
#' @return An object of class [report_effectsize()].
#'
#' @examples
#' library(report)
#'
#' # h-tests
#' report_effectsize(t.test(iris$Sepal.Width, iris$Sepal.Length))
#'
#' # ANOVAs
#' report_effectsize(aov(Sepal.Length ~ Species, data = iris))
#'
#' # GLMs
#' report_effectsize(lm(Sepal.Length ~ Petal.Length * Species, data = iris))
#' report_effectsize(glm(vs ~ disp, data = mtcars, family = "binomial"))
#'
#' @examplesIf requireNamespace("lme4", quietly = TRUE)
#' \donttest{
#' # Mixed models
#' library(lme4)
#' model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
#' report_effectsize(model)
#' }
#'
#' @examplesIf requireNamespace("rstanarm", quietly = TRUE)
#' \donttest{
#' # Bayesian models
#' library(rstanarm)
#' model <- suppressWarnings(stan_glm(Sepal.Length ~ Species, data = iris, refresh = 0, iter = 600))
#' report_effectsize(model, effectsize_method = "basic")
#' }
#' @export
report_effectsize <- function(x, ...) {
  UseMethod("report_effectsize")
}


# METHODS -----------------------------------------------------------------

#' @rdname as.report
#' @export
as.report_effectsize <- function(x, summary = NULL, prefix = "  - ", ...) {
  class(x) <- unique(c("report_effectsize", class(x)))
  attributes(x) <- c(attributes(x), list(...))
  attr(x, "prefix") <- prefix

  if (!is.null(summary)) {
    class(summary) <- unique(c("report_effectsize", class(summary)))
    attr(summary, "prefix") <- prefix
    attr(x, "summary") <- summary
  }

  x
}


#' @export
summary.report_effectsize <- function(object, ...) {
  if (is.null(attributes(object)$summary)) {
    object
  } else {
    attributes(object)$summary
  }
}

#' @export
print.report_effectsize <- function(x, ...) {
  if (!is.null(attributes(x)$rules)) {
    cat(attributes(x)$rules, "\n\n")
  }

  cat(paste(x, collapse = "\n"))
}


# Utilities ---------------------------------------------------------------

#' @keywords internal
.text_effectsize <- function(interpretation) {
  # Effect size
  if (is.null(interpretation)) {
    effect_text <- ""
  } else if (is.character(interpretation)) {
    effsize_name <- switch(
      interpretation,
      cohen1988 = "Cohen's (1988)",
      sawilowsky2009 = "Savilowsky's (2009)",
      gignac2016 = "Gignac's (2016)",
      funder2019 = "Funder's (2019)",
      lovakov2021 = "Lovakov's (2021)",
      evans1996 = "Evans's (1996)",
      chen2010 = "Chen's (2010)",
      field2013 = "Field's (2013)",
      landis1977 = "Landis' (1977)"
    )
    effect_text <- paste0(
      "Effect sizes were labelled following ",
      effsize_name,
      " recommendations."
    )
  } else {
    effect_text <- paste0(
      "Effect sizes were labelled following a custom set of rules."
    )
  }
  effect_text
}


#' @keywords internal
.text_standardize <- function(x, ...) {
  method <- attributes(x)$std_method
  robust <- attributes(x)$robust
  two_sd <- attributes(x)$two_sd

  if (method == "refit") {
    if (robust) {
      standard_text <- "(using the median and the MAD, a robust equivalent of the SD) "
    } else {
      standard_text <- ""
    }
    standard_text <- paste0(
      "Standardized parameters were obtained by fitting the model on a standardized version ",
      standard_text,
      "of the dataset."
    )
  } else if (method == "2sd") {
    if (robust) {
      standard_text <- "MAD (a median-based equivalent of the SD) "
    } else {
      standard_text <- "SD "
    }
    standard_text <- paste0(
      "Standardized parameters were obtained by standardizing the data by 2 times the ",
      standard_text,
      " (see Gelman, 2008)."
    )
  } else if (method %in% c("smart", "basic", "posthoc")) {
    if (robust) {
      standard_text <- "median and the MAD (a median-based equivalent of the SD) of the response variable."
    } else {
      standard_text <- "mean and the SD of the response variable."
    }
    standard_text <- paste0("Parameters were scaled by the ", standard_text)
  } else {
    standard_text <- paste0(
      "Parameters were standardized using the ",
      method,
      " method."
    )
  }

  standard_text
}

#' Report intercept
#'
#' Reports intercept of regression models (see list of supported objects in
#' [report()]).
#'
#' @inheritParams report
#' @inheritParams report_table
#' @inheritParams report_text
#' @inheritParams as.report
#'
#' @return An object of class [report_intercept()].
#'
#' @examples
#' \donttest{
#' library(report)
#'
#' # GLMs
#' report_intercept(lm(Sepal.Length ~ Species, data = iris))
#' report_intercept(glm(vs ~ disp, data = mtcars, family = "binomial"))
#' }
#'
#' @examplesIf requireNamespace("lme4", quietly = TRUE)
#' \donttest{
#' # Mixed models
#' library(lme4)
#' model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
#' report_intercept(model)
#' }
#'
#' @examplesIf requireNamespace("rstanarm", quietly = TRUE)
#' \donttest{
#' # Bayesian models
#' library(rstanarm)
#' model <- suppressWarnings(stan_glm(Sepal.Length ~ Species, data = iris, refresh = 0, iter = 600))
#' report_intercept(model)
#' }
#' @export

report_intercept <- function(x, ...) {
  UseMethod("report_intercept")
}


# METHODS -----------------------------------------------------------------

#' @rdname as.report
#' @export
as.report_intercept <- function(x, summary = NULL, ...) {
  class(x) <- unique(c("report_intercept", class(x)))
  attributes(x) <- c(attributes(x), list(...))

  if (!is.null(summary)) {
    attr(x, "summary") <- summary
  }
  x
}


#' @export
summary.report_intercept <- function(object, ...) {
  if (is.null(attributes(object)$summary)) {
    object
  } else {
    attributes(object)$summary
  }
}

#' @export
print.report_intercept <- function(x, ...) {
  cat(paste(x, collapse = "\n"))
}


# Utils -------------------------------------------------------------------

#' @keywords internal
.find_intercept <- function(model) {
  # Intercept-only
  if (suppressWarnings(insight::is_nullmodel(model))) {
    return("")
  }

  model_terms <- insight::find_variables(model)$conditional
  model_data <- insight::get_data(model)
  intercept_data <- model_data[model_terms[model_terms %in% names(model_data)]]
  intercept_text <- NULL
  for (col in names(intercept_data)) {
    if (is.numeric(intercept_data[[col]])) {
      intercept_text <- c(intercept_text, paste0(col, " = 0"))
    } else if (is.character(intercept_data[[col]])) {
      intercept_text <- c(
        intercept_text,
        paste0(col, " = ", levels(as.factor(intercept_data[[col]]))[1])
      )
    } else if (is.factor(intercept_data[[col]])) {
      ref_level <- .find_reference_level(intercept_data[[col]])
      intercept_text <- c(
        intercept_text,
        paste0(col, " = ", levels(intercept_data[[col]])[ref_level])
      )
    } else {
      intercept_text <- c(intercept_text, paste0(col, " = [?]"))
    }
  }
  paste0(
    ", corresponding to ",
    datawizard::text_concatenate(intercept_text),
    ","
  )
}


.find_reference_level <- function(f) {
  tryCatch(
    {
      con <- stats::contrasts(f)
      unname(which(apply(con, 1, function(i) sum(i) == 0)))
    },
    error = function(e) {
      1
    }
  )
}

set.seed(123)
response <- rnorm(30)

data1 <- data.frame(
  y = response,
  f = factor(rep(c(1, 2, 3), 10)),
  stringsAsFactors = FALSE
)

data2 <- data.frame(
  y = response,
  f = factor(rep(c(1, 2, 3), 10)),
  stringsAsFactors = FALSE
)

data3 <- data.frame(
  y = response,
  f = factor(rep(c(1, 2, 3), 10)),
  stringsAsFactors = FALSE
)

contrasts(data1$f) <- cbind(c(1, 0, 0), c(0, 1, 0))
data2$f <- relevel(data2$f, 3)

# insight::get_data needs access to data in global environment
data1 <<- data1
data2 <<- data2
data3 <<- data3

# contrasts, ref. level = category 3
m1 <- lm(y ~ f, data = data1)

# relevel, ref. level = category 3
m2 <- lm(y ~ f, data = data2)

# default ref. level = category 1
m3 <- lm(y ~ f, data = data3)

test_that("reflevel", {
  expect_identical(
    as.character(report_intercept(m1)),
    "The model's intercept, corresponding to f = 3, is at 0.07 (95% CI [-0.57, 0.71], t(27) = 0.23, p = 0.819)."
  )
  expect_identical(
    as.character(report_intercept(m2)),
    "The model's intercept, corresponding to f = 3, is at 0.07 (95% CI [-0.57, 0.71], t(27) = 0.23, p = 0.819)."
  )
  expect_identical(
    as.character(report_intercept(m3)),
    "The model's intercept, corresponding to f = 1, is at 0.17 (95% CI [-0.47, 0.81], t(27) = 0.55, p = 0.584)."
  )
})

#' Cite the easystats ecosystem
#'
#' A convenient function for those who wish to cite the easystats packages.
#'
#' @param packages A character vector of packages to cite. Can be `"all"` for
#'   all *easystats* packages or a vector with specific package names.
#' @param format The format to generate citations. Can be `"text"` for plain
#'   text, `"markdown"` for markdown citations and CSL bibliography
#'   (recommended for writing in RMarkdown), or `"biblatex"` for BibLaTeX
#'   citations and bibliography.
#' @param intext_prefix A character vector of length 1 containing text to
#'   include before in-text citations. If `TRUE`, defaults to `"Analyses were
#'   conducted using the easystats collection of packages "`. If `FALSE` or
#'   `NA`, no prefix is included.
#' @param intext_suffix A character vector of length 1 containing text to
#'   include after in-text citations. Defaults to `"."`. If `FALSE` or `NA`,
#'   no suffix is included.
#' @param x,object A `"cite_easystats"` object to print.
#' @param what What elements of the citations to print, can be `"all"`,
#'   `"intext"`, or `"refs"`.
#' @param ... Not used. Included for compatibility with the generic function.
#'
#' @return A list of class `"cite_easystats"` with elements:
#'   - `intext`: In-text citations in the requested `format`
#'   - `refs`: References or bibliography in the requested `format`
#'
#' @examples
#' \donttest{
#' # Cite just the 'easystats' umbrella package:
#' cite_easystats()
#' summary(cite_easystats(), what = "all")
#'
#' # Cite every easystats package:
#' cite_easystats(packages = "all")
#' summary(cite_easystats(packages = "all"), what = "all")
#'
#' # Cite specific packages:
#' cite_easystats(packages = c("modelbased", "see"))
#' summary(cite_easystats(packages = c("modelbased", "see")), what = "all")
#'
#' # To cite easystats packages in an RMarkdown document, use:
#'
#' ## In-text citations:
#' print(cite_easystats(format = "markdown"), what = "intext")
#'
#' ## Bibliography (print with the  `output = 'asis'` option on the code chunk)
#' print(cite_easystats(format = "markdown"), what = "refs")
#' }
#'
#' @export
cite_easystats <- function(
  packages = "easystats",
  format = c("text", "markdown", "biblatex"),
  intext_prefix = TRUE,
  intext_suffix = "."
) {
  format <- match.arg(format, choices = c("text", "markdown", "biblatex"))
  installed_packages <- utils::installed.packages()[, "Version"]
  if (length(packages) == 1 && packages == "all") {
    packages <- c(
      "easystats",
      "insight",
      "datawizard",
      "bayestestR",
      "performance",
      "parameters",
      "effectsize",
      "correlation",
      "modelbased",
      "see",
      "report"
    )
    packages <- packages[packages %in% names(installed_packages)]
  } else if (length(packages) == 1 && packages == "easystats") {
    if (!packages %in% names(installed_packages)) {
      installed_packages <- c(easystats = "")
    }
  } else {
    missing_packages <- setdiff(packages, names(installed_packages))
    if (length(missing_packages)) {
      message(insight::format_message(
        "Requested package(s) not installed:",
        toString(missing_packages),
        "Citations to these packages omitted."
      ))
      packages <- setdiff(packages, missing_packages)
    }
  }

  # in-text
  if (format == "text") {
    letters_ludecke_packages <- .disamguation_letters(
      c(
        "easystats",
        "insight",
        "performance",
        "parameters",
        "see"
      ) %in%
        packages
    )
    if (sum(nzchar(letters_ludecke_packages)) == 1) {
      letters_ludecke_packages <- rep("", length(letters_ludecke_packages))
    }
    letters_ludecke_articles <- .disamguation_letters(
      c("performance", "see") %in% packages
    )
    if (sum(nzchar(letters_ludecke_articles)) == 1) {
      letters_ludecke_articles <- rep("", length(letters_ludecke_articles))
    }
    letters_makowski_packages <- .disamguation_letters(
      c(
        "datawizard",
        "bayestestR",
        "correlation",
        "modelbased",
        "report"
      ) %in%
        packages
    )
    if (sum(nzchar(letters_makowski_packages)) == 1) {
      letters_makowski_packages <- rep("", length(letters_makowski_packages))
    }
    easystats <- "_easystats_"
    cit_packages <- sprintf(
      "(%s)",
      toString(c(
        easystats = sprintf(
          "L\u00fcdecke et al., 2019/2023%s",
          letters_ludecke_packages[1]
        ),
        insight = sprintf(
          "L\u00fcdecke et al., 2019, 2019/2022%s",
          letters_ludecke_packages[2]
        ),
        datawizard = sprintf(
          "Makowski et al., 2021/2022%s",
          letters_makowski_packages[1]
        ),
        bayestestR = sprintf(
          "Makowski et al., 2019, 2019/2022%s",
          letters_makowski_packages[2]
        ),
        performance = sprintf(
          "L\u00fcdecke et al., 2021%s, 2019/2022%s",
          letters_ludecke_articles[1],
          letters_ludecke_packages[3]
        ),
        parameters = sprintf(
          "L\u00fcdecke et al., 2020, 2019/2022%s",
          letters_ludecke_packages[4]
        ),
        effectsize = "Ben-Shachar et al., 2020, 2019/2022",
        correlation = sprintf(
          "Makowski et al., 2020, 2020/2022%s",
          letters_makowski_packages[3]
        ),
        modelbased = sprintf(
          "Makowski et al., 2020/2022%s",
          letters_makowski_packages[4]
        ),
        see = sprintf(
          "L\u00fcdecke et al., 2021%s, 2019/2022%s",
          letters_ludecke_articles[2],
          letters_ludecke_packages[5]
        ),
        report = sprintf(
          "Makowski et al., 2021/2023%s",
          letters_makowski_packages[5]
        )
      )[packages])
    )
  } else if (format == "markdown") {
    easystats <- "_easystats_"
    cit_packages <- sprintf(
      "[%s]",
      toString(c(
        easystats = "@easystatsPackage",
        insight = "@insightArticle, @insightPackage",
        datawizard = "@datawizardPackage",
        bayestestR = "@bayestestRArticle, @bayestestRPackage",
        performance = "@performanceArticle, @performancePackage",
        parameters = "@parametersArticle, @parametersPackage",
        effectsize = "@effectsizeArticle, @effectsizePackage",
        correlation = "@correlationArticle, @correlationPackage",
        modelbased = "@modelbasedPackage",
        see = "@seeArticle, @seePackage",
        report = "@reportPackage"
      )[packages])
    )
  } else {
    easystats <- "\\emph{easystats}"
    cit_packages <- sprintf(
      "\\cite{%s}",
      toString(c(
        easystats = "easystatsPackage",
        insight = "insightArticle, insightPackage",
        datawizard = "datawizardPackage",
        bayestestR = "bayestestRArticle, bayestestRPackage",
        performance = "performanceArticle, performancePackage",
        parameters = "parametersArticle, parametersPackage",
        effectsize = "effectsizeArticle, effectsizePackage",
        correlation = "correlationArticle, correlationPackage",
        modelbased = "modelbasedPackage",
        see = "seeArticle, seePackage",
        report = "reportPackage"
      )[packages])
    )
  }

  if (isTRUE(intext_prefix)) {
    intext_prefix <- sprintf(
      "Analyses were conducted using the %s collection of packages ",
      easystats
    )
  } else if (isFALSE(intext_prefix)) {
    intext_prefix <- ""
  }
  if (isTRUE(intext_suffix)) {
    intext_suffix <- "."
  }
  if (isFALSE(intext_suffix) || is.na(intext_suffix)) {
    intext_suffix <- ""
  }
  intext <- sprintf("%s%s%s", intext_prefix, cit_packages, intext_suffix)

  # references
  if (format == "text") {
    ref_packages <- paste0(
      "- ",
      sort(unlist(list(
        easystats = sprintf(
          paste(
            "L\u00fcdecke, D., Makowski, D., Ben-Shachar, M. S., Patil, I.,",
            "Wiernik, B. M., Bacher, Etienne, & Th\u00e9riault, R. (2023).",
            "easystats: Streamline model interpretation, visualization,",
            "and reporting%s [R package].",
            "https://easystats.github.io/easystats/",
            "(Original work published 2019)"
          ),
          ifelse(
            nzchar(installed_packages["easystats"]),
            paste0(" (", installed_packages["easystats"], ")"),
            ""
          )
        ),
        insight = c(
          article = paste(
            "L\u00fcdecke, D., Waggoner, P., & Makowski, D. (2019).",
            "insight: A unified interface to access information from",
            "model objects in R.",
            "Journal of Open Source Software, 4(38), 1412.",
            "https://doi.org/10.21105/joss.01412"
          ),
          package = sprintf(
            paste(
              "L\u00fcdecke, D., Makowski, D., Patil, I., Waggoner,",
              "P., Ben-Shachar, M. S., Wiernik, B. M., & Arel-Bundock, V. (2022).",
              "insight: Easy access to model information for various model objects (%s) [R package].",
              "https://CRAN.R-project.org/package=insight (Original work published 2019)"
            ),
            installed_packages["insight"]
          )
        ),
        datawizard = sprintf(
          paste(
            "Makowski, D., L\u00fcdecke, D., Patil, I., Ben-Shachar, M. S., & Wiernik, B. M. (2022).",
            "datawizard: Easy data wrangling (%s) [R package].",
            "https://CRAN.R-project.org/package=datawizard (Original work published 2021)"
          ),
          installed_packages["datawizard"]
        ),
        bayestestR = c(
          article = paste(
            "Makowski, D., Ben-Shachar, M., & L\u00fcdecke, D. (2019).",
            "bayestestR: Describing effects and their uncertainty, existence",
            "and significance within the Bayesian framework.",
            "Journal of Open Source Software, 4(40), 1541. https://doi.org/10.21105/joss.01541"
          ),
          package = sprintf(
            paste(
              "Makowski, D., L\u00fcdecke, D., Ben-Shachar, M. S., Patil, I.,",
              "Wilson, M. D., & Wiernik, B. M. (2021).",
              "bayestestR: Understand and describe Bayesian models and posterior distributions (%s) [R package].",
              "https://CRAN.R-project.org/package=bayestestR (Original work published 2019)"
            ),
            installed_packages["bayestestR"]
          )
        ),
        performance = c(
          article = paste(
            "L\u00fcdecke, D., Ben-Shachar, M., Patil, I., Waggoner, P., & Makowski, D. (2021).",
            "performance: An R package for assessment, comparison and testing of statistical models.",
            "Journal of Open Source Software, 6(60), 3139. https://doi.org/10.21105/joss.03139"
          ),
          package = sprintf(
            paste(
              "L\u00fcdecke, D., Makowski, D., Ben-Shachar, M. S., Patil,",
              "I., Waggoner, P., & Wiernik, B. M. (2021).",
              "performance: Assessment of regression models performance (%s) [R package].",
              "https://CRAN.R-project.org/package=performance (Original work published 2019)"
            ),
            installed_packages["performance"]
          )
        ),
        parameters = c(
          article = paste(
            "L\u00fcdecke, D., Ben-Shachar, M., Patil, I., & Makowski, D. (2020).",
            "Extracting, computing and exploring the parameters of statistical models using R.",
            "Journal of Open Source Software, 5(53), 2445. https://doi.org/10.21105/joss.02445"
          ),
          package = sprintf(
            paste(
              "L\u00fcdecke, D., Makowski, D., Ben-Shachar, M. S., Patil, I.,",
              "H\u00F8jsgaard, S., & Wiernik, B. M. (2022).",
              "parameters: Processing of model parameters (%s) [R package].",
              "https://CRAN.R-project.org/package=parameters (Original work published 2019)"
            ),
            installed_packages["parameters"]
          )
        ),
        effectsize = c(
          article = paste(
            "Ben-Shachar, M. S., L\u00fcdecke, D., & Makowski, D. (2020).",
            "effectsize: Estimation of effect size indices and standardized parameters.",
            "Journal of Open Source Software, 5(56), 2815. https://doi.org/10.21105/joss.02815"
          ),
          package = sprintf(
            paste(
              "Ben-Shachar, M. S., Makowski, D., L\u00fcdecke, D., Patil, I., & Wiernik, B. M. (2022).",
              "effectsize: Indices of effect size and standardized parameters (%s) [R package].",
              "https://CRAN.R-project.org/package=effectsize (Original work published 2019)"
            ),
            installed_packages["effectsize"]
          )
        ),
        correlation = c(
          article = paste(
            "Makowski, D., Ben-Shachar, M., Patil, I., & L\u00fcdecke, D. (2020).",
            "Methods and algorithms for correlation analysis in R.",
            "Journal of Open Source Software, 5(51), 2306. https://doi.org/10.21105/joss.02306"
          ),
          package = sprintf(
            paste(
              "Makowski, D., Wiernik, B. M., Patil, I., L\u00fcdecke, D., & Ben-Shachar, M. S. (2022).",
              "correlation: Methods for correlation analysis (%s) [R package].",
              "https://CRAN.R-project.org/package=correlation (Original work published 2020)"
            ),
            installed_packages["correlation"]
          )
        ),
        modelbased = sprintf(
          paste(
            "Makowski, D., L\u00fcdecke, D., Ben-Shachar, M. S., & Patil, I. (2022).",
            "modelbased: Estimation of model-based predictions, contrasts and means (%s) [R package].",
            "https://CRAN.R-project.org/package=modelbased (Original work published 2020)"
          ),
          installed_packages["modelbased"]
        ),
        see = c(
          article = paste(
            "L\u00fcdecke, D., Patil, I., Ben-Shachar, M. S., Wiernik,",
            "B. M., Waggoner, P., & Makowski, D. (2021).",
            "see: An R package for visualizing statistical models.",
            "Journal of Open Source Software, 6(64), 3393. https://doi.org/10.21105/joss.03393"
          ),
          package = sprintf(
            paste(
              "L\u00fcdecke, D., Makowski, D., Patil, I., Ben-Shachar, M. S., Wiernik,",
              "B. M., & Waggoner, P. (2022).",
              "see: Visualisation toolbox for 'easystats' (%s) [R package].",
              "https://CRAN.R-project.org/package=see (Original work published 2019)"
            ),
            installed_packages["see"]
          )
        ),
        report = sprintf(
          paste(
            "Makowski, D., L\u00fcdecke, D., Patil, I., Th\U00E9riault, R., Ben-Shachar, M. S.,",
            "& Wiernik, B. M. (2023).",
            "report: Automated reporting of results and statistical models (%s) [R package].",
            "https://easystats.github.io/easystats/ (Original work published 2021)"
          ),
          installed_packages["report"]
        )
      )[packages])),
      "\n"
    )
  } else if (format == "markdown") {
    ref_packages <- readLines(system.file(
      "easystats_bib.yaml",
      package = "report"
    ))
    ref_packages[ref_packages == "  version: %s"] <- sprintf(
      ref_packages[ref_packages == "  version: %s"],
      c(
        installed_packages["bayestestR"],
        installed_packages["correlation"],
        installed_packages["datawizard"],
        installed_packages["easystats"],
        installed_packages["effectsize"],
        installed_packages["insight"],
        installed_packages["modelbased"],
        installed_packages["parameters"],
        installed_packages["performance"],
        installed_packages["report"],
        installed_packages["see"]
      )
    )
    ref_packages <- list(
      ref_packages[1:2],
      ref_packages[-c(1:2, length(ref_packages))],
      ref_packages[length(ref_packages)]
    )
    ref_packages[[2]] <- split(
      ref_packages[[2]],
      cumsum(!nzchar(ref_packages[[2]]))
    )
    ref_packages_index <- grep(
      paste0("id: ((", paste(packages, collapse = ")|("), "))"),
      ref_packages[[2]]
    )
    ref_packages[[2]] <- unlist(
      ref_packages[[2]][ref_packages_index],
      use.names = FALSE
    )
    ref_packages <- paste(
      unlist(ref_packages, use.names = FALSE),
      collapse = "\n"
    )
  } else {
    ref_packages <- readLines(system.file(
      "easystats_bib.bib",
      package = "report"
    ))
    ref_packages[ref_packages == "  version = {%s}"] <- sprintf(
      ref_packages[ref_packages == "  version = {%s}"],
      c(
        installed_packages["bayestestR"],
        installed_packages["correlation"],
        installed_packages["datawizard"],
        installed_packages["easystats"],
        installed_packages["effectsize"],
        installed_packages["insight"],
        installed_packages["modelbased"],
        installed_packages["parameters"],
        installed_packages["performance"],
        installed_packages["report"],
        installed_packages["see"]
      )
    )
    ref_packages <- split(ref_packages, cumsum(!nzchar(ref_packages)))
    ref_packages_index <- grep(
      paste0(
        "((article)|(software))\\{((",
        paste(packages, collapse = ")|("),
        "))"
      ),
      ref_packages
    )
    ref_packages <- unlist(ref_packages[ref_packages_index], use.names = FALSE)
    ref_packages <- paste(
      unlist(ref_packages, use.names = FALSE),
      collapse = "\n"
    )
  }

  out <- list(
    intext = intext,
    refs = ref_packages
  )

  class(out) <- c("cite_easystats", class(out))
  out
}


#' @export
#' @rdname cite_easystats
summary.cite_easystats <- function(object, what = "all", ...) {
  what <- match.arg(what, choices = c("all", "cite", "intext", "bib", "refs"))
  what <- switch(
    what,
    all = "all",
    cite = ,
    intext = "intext",
    bib = ,
    refs = "refs"
  )
  switch(
    what,
    all = {
      insight::print_colour("\nCitations\n----------\n\n", "blue")
      cat(
        strwrap(object$intext, exdent = 0, width = 0.95 * getOption("width")),
        sep = "\n"
      )
      cat("\n")
      insight::print_colour("\nReferences\n----------\n\n", "blue")
      cat(
        unlist(lapply(
          object$refs,
          strwrap,
          exdent = 4,
          width = 0.95 * getOption("width")
        )),
        sep = "\n"
      )
      cat("\n")
    },
    intext = {
      cat(
        strwrap(object$intext, exdent = 0, width = 0.95 * getOption("width")),
        sep = "\n"
      )
    },
    refs = {
      cat(
        unlist(lapply(
          object$refs,
          strwrap,
          exdent = 4,
          width = 0.95 * getOption("width")
        )),
        sep = "\n"
      )
    }
  )
}


#' @export
#' @rdname cite_easystats
print.cite_easystats <- function(x, what = "all", ...) {
  what <- match.arg(what, choices = c("all", "cite", "intext", "bib", "refs"))
  what <- switch(
    what,
    all = "all",
    cite = ,
    intext = "intext",
    bib = ,
    refs = "refs"
  )
  if (what == "all") {
    insight::print_colour(
      sprintf(
        "Thanks for crediting us! %s You can cite the easystats ecosystem as follows:",
        ifelse(.support_unicode, "\U1F600", ":)")
      ),
      "blue"
    )
    cat("\n")
  }
  summary(x, what = what, ...)
}


.disamguation_letters <- function(x) {
  if (!is.logical(x)) {
    stop("`x` must be a logical vector.", call. = FALSE)
  }
  count <- sum(x)
  x[!x] <- ""
  x[nzchar(x)] <- letters[seq_len(count)]
  x
}

test_that("cite_easystats() works with default parameters", {
  result <- cite_easystats()
  expect_s3_class(result, "cite_easystats")
  expect_type(result, "list")
  expect_named(result, c("intext", "refs"))
  expect_type(result$intext, "character")
  expect_type(result$refs, "character")
  expect_match(result$intext, "easystats")
  expect_match(result$refs, "Lüdecke")
})

test_that("cite_easystats() works with different formats", {
  # Text format (default)
  result_text <- cite_easystats(format = "text")
  expect_match(result_text$intext, "collection of packages")
  expect_match(result_text$refs, "- Lüdecke")

  # Markdown format
  result_md <- cite_easystats(format = "markdown")
  expect_match(result_md$intext, "@easystatsPackage")
  expect_match(result_md$refs, "id: easystatsPackage")

  # BibLaTeX format
  result_bib <- cite_easystats(format = "biblatex")
  expect_match(result_bib$intext, "\\\\cite\\{")
  expect_match(result_bib$refs, "@software\\{")
})

test_that("cite_easystats() works with different package specifications", {
  # Single package
  result_single <- cite_easystats(packages = "insight")
  expect_true(any(grepl("insight", result_single$refs, fixed = TRUE)))

  # Multiple specific packages
  result_multi <- cite_easystats(packages = c("insight", "parameters"))
  expect_true(any(grepl("insight", result_multi$refs, fixed = TRUE)))
  expect_true(any(grepl("parameters", result_multi$refs, fixed = TRUE)))
})

test_that("cite_easystats() handles prefix and suffix correctly", {
  # With default prefix/suffix
  result_default <- cite_easystats()
  expect_match(result_default$intext, "^Analyses were conducted")
  expect_match(result_default$intext, "\\.$")

  # Without prefix
  result_no_prefix <- cite_easystats(intext_prefix = FALSE)
  expect_false(startsWith(result_no_prefix$intext, "Analyses were conducted"))

  # Without suffix
  result_no_suffix <- cite_easystats(intext_suffix = FALSE)
  expect_false(endsWith(result_no_suffix$intext, "."))

  # Custom prefix and suffix
  result_custom <- cite_easystats(
    intext_prefix = "Custom prefix ",
    intext_suffix = " custom suffix"
  )
  expect_match(result_custom$intext, "^Custom prefix")
  expect_match(result_custom$intext, "custom suffix$")
})

test_that("cite_easystats() handles missing packages gracefully", {
  # Test with non-existent package (should show warning and omit)
  expect_message(
    cite_easystats(packages = c("insight", "nonexistent_package")),
    "not installed"
  )

  # Now get the result for testing
  suppressMessages({
    result <- cite_easystats(packages = c("insight", "nonexistent_package"))
  })
  expect_true(any(grepl("insight", result$refs, fixed = TRUE)))
  expect_false(any(grepl("nonexistent", result$refs, fixed = TRUE)))
})

test_that("print.cite_easystats() works correctly", {
  result <- cite_easystats()

  # Test different 'what' arguments
  expect_output(print(result, what = "all"), "Thanks for crediting")
  expect_output(print(result, what = "intext"), "easystats")
  expect_output(print(result, what = "refs"), "Lüdecke")

  # Test with 'cite' and 'bib' aliases
  expect_output(print(result, what = "cite"), "easystats")
  expect_output(print(result, what = "bib"), "Lüdecke")
})

test_that("summary.cite_easystats() works correctly", {
  result <- cite_easystats()

  # Test different 'what' arguments
  expect_output(summary(result, what = "all"), "Citations")
  expect_output(summary(result, what = "intext"), "easystats")
  expect_output(summary(result, what = "refs"), "Lüdecke")

  # Test with 'cite' and 'bib' aliases
  expect_output(summary(result, what = "cite"), "easystats")
  expect_output(summary(result, what = "bib"), "Lüdecke")
})

test_that(".disamguation_letters() helper function works", {
  # Test with logical vector
  result1 <- report:::.disamguation_letters(c(TRUE, FALSE, TRUE))
  expect_identical(result1, c("a", "", "b"))

  result2 <- report:::.disamguation_letters(TRUE)
  expect_identical(result2, "a") # Single TRUE should return "a", not empty

  result3 <- report:::.disamguation_letters(c(FALSE, FALSE))
  expect_identical(result3, c("", ""))

  # Test error handling
  expect_error(
    report:::.disamguation_letters(c(1, 2, 3)),
    "must be a logical vector"
  )
})

#' @keywords internal
.combine_tables_effectsize <- function(parameters, effsize) {
  effsize_table <- attributes(effsize)$table
  combined_table <- merge(parameters, effsize_table, sort = FALSE, all = TRUE)
  # combined_table <- combined_table[order(
  #   match(combined_table$Parameter, parameters$Parameter)), ]
  row.names(combined_table) <- NULL

  # Prepare output
  class(combined_table) <- class(parameters)
  attributes(combined_table) <- utils::modifyList(
    attributes(parameters),
    attributes(combined_table)
  )

  combined_table
}


#' @keywords internal
.combine_tables_performance <- function(parameters, performance) {
  combined_table <- parameters

  # Pretty names
  if (!is.null(attributes(parameters)$pretty_names)) {
    combined_table$Parameter <- attributes(parameters)$pretty_names[
      parameters$Parameter
    ]
  }

  # Skip row
  combined_table[nrow(combined_table) + 1, ] <- NA

  # Prettify performance names
  perf_names <- colnames(performance)
  perf_names[perf_names == "R2_adjusted"] <- "R2 (adj.)"
  perf_names[perf_names == "R2_Tjur"] <- "Tjur's R2"
  perf_names[perf_names == "BIC_adjusted"] <- "BIC (adj.)"
  perf_names[perf_names == "R2_conditional"] <- "R2 (conditional)"
  perf_names[perf_names == "R2_marginal"] <- "R2 (marginal)"

  # add performance
  perf_vertical <- data.frame(
    Parameter = perf_names,
    Fit = as.numeric(performance[1, ]),
    stringsAsFactors = FALSE
  )

  # remove missing values
  perf_vertical <- perf_vertical[!is.na(perf_vertical$Fit), ]

  # Name parameter column
  name_parameter <- names(parameters)[
    names(parameters) %in% c("Parameter", "Link", "To")
  ][1]
  names(perf_vertical)[1] <- name_parameter

  # Merge
  combined_table <- merge(
    combined_table,
    perf_vertical,
    by = name_parameter,
    all = TRUE,
    sort = FALSE
  )

  # Prepare output
  class(combined_table) <- class(parameters)
  attributes(combined_table) <- utils::modifyList(
    attributes(parameters),
    attributes(combined_table)
  )

  # Add pretty names
  pretty_names <- combined_table$Parameter
  # pretty_names <- pretty_names[!is.na(pretty_names)]
  names(pretty_names) <- pretty_names
  attr(combined_table, "pretty_names") <- pretty_names

  combined_table
}


#' @keywords internal
.remove_performance <- function(table) {
  if ("Fit" %in% names(table)) {
    params_table <- table[is.na(table$Fit), ]
    params_table <- params_table[!is.na(params_table$Parameter), ]
  } else {
    params_table <- table
  }
  params_table
}

test_that("BRMS effectsize method uses faster default", {
  # Test that the brmsfit method exists and has correct default parameters
  expect_true(exists(
    "report_effectsize.brmsfit",
    envir = getNamespace("report")
  ))

  # Get the function
  brms_func <- get("report_effectsize.brmsfit", envir = getNamespace("report"))

  # Check that it's a function
  expect_type(brms_func, "closure")

  # Check the arguments - should have effectsize_method = "basic" as default
  args <- formals(brms_func)
  expect_true("effectsize_method" %in% names(args))
  expect_identical(eval(args$effectsize_method), "basic")

  # Ensure it's different from the lm method which defaults to "refit"
  lm_func <- get("report_effectsize.lm", envir = getNamespace("report"))
  lm_args <- formals(lm_func)
  expect_identical(eval(lm_args$effectsize_method), "refit")

  # This confirms that BRMS models will use the faster "basic" method
  # while LM models continue using "refit" for consistency
})

---
title: "Publication-ready Tables"
output: 
  rmarkdown::html_vignette:
    toc: true
    fig_width: 10.08
    fig_height: 6
tags: [r, report]
vignette: >
  %\VignetteIndexEntry{Publication-ready Tables}
  \usepackage[utf8]{inputenc}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, echo = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  comment = "#"
)

options(
  knitr.kable.NA = "",
  width = 60
)

if (!requireNamespace("dplyr", quietly = TRUE)) {
  knitr::opts_chunk$set(eval = FALSE)
}
```


First, load the `{report}` package:

```{r}
library(report)
```

## Getting started with correlations

Let's start by demonstrating some features with simple tests. The function `report_table()` can be used to create a table for many R objects.

```{r}
results <- cor.test(mtcars$mpg, mtcars$wt)

report_table(results)
```

We can also obtain a shorter version by running `summary()` on the output (that we are going to store in a variable called `t` - like table)

```{r}
t <- summary(report_table(results))
t
```

In the example above, running just `t` ran `print(t)` under the hood, which prints the table inside the console. However, one can nicely display that table in markdown documents using `display()`.

```{r}
display(t)
```

We can further customize this table, by adding significance stars, 


```{r, results='asis'}
display(t,
  stars = TRUE,
  title = "Table 1",
  footer = "Correlation in the mtcars (n = 32) dataset.\n*** p < .001"
)
```

## *t*-tests, ...

It works similarly for *t*-tests.

```{r}
results <- t.test(mtcars$mpg ~ mtcars$am)

t <- summary(report_table(results))
t
```

Note that, by default, `report_table()` prettifies the printing: that means that the column names and its content is, underneath, not necessarily what is printed, which can be a bit confusing. For instance, while the confidence interval `CI` appears as one column, it this actually made of three columns! One can access this *raw* table as a dataframe:

```{r}
as.data.frame(t)
```

In fact, the function used to *prettify* the output is called `insight::format_table()` and is accessible to you too, so that you can prettify the output while keeping it as a data frame.

```{r}
insight::format_table(as.data.frame(t), stars = TRUE)
```

Also, you can join the results of multiple tables:

```{r}
results1 <- t.test(mtcars$mpg ~ mtcars$am)
results2 <- t.test(mtcars$wt ~ mtcars$am)
results3 <- t.test(mtcars$qsec ~ mtcars$am)

results <- c(
  report_table(results1),
  report_table(results2),
  report_table(results3)
)
display(results)
```


## ANOVAs



## Linear Models

```{r}
model <- lm(Petal.Length ~ Species * Petal.Width, data = iris)

report_table(model)
```

---
title: "Report and Cite Packages"
output: 
  rmarkdown::html_vignette:
    toc: true
    fig_width: 10.08
    fig_height: 6
tags: [r, report]
vignette: >
  %\VignetteIndexEntry{Report and Cite Packages}
  \usepackage[utf8]{inputenc}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, echo = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  comment = "#"
)

options(
  knitr.kable.NA = "",
  width = 60
)

if (!requireNamespace("dplyr", quietly = TRUE)) {
  knitr::opts_chunk$set(eval = FALSE)
}
```

**Citing the packages, modules and software** you used for your analysis is
important to acknowledge the time and effort spent by people who create theses
tools (sometimes in their free-time, or at the expense of their own research),
but also for **reproducibility**. Indeed, statistical routines are often
implemented in different ways by different packages, which can lead to possible
discrepancies in the results. Explicitly mentioning that *"I did this using this function from that package version 1.2.3"* is a way of **protecting yourself** by being transparent about what you have found doing what you have
done.

But, understandably, you must have a lot of questions- 

> **That's great, but how to *actually* cite them?**

> **I used about 100 packages, should I cite them *all*?**

> **How should I report the system (the OS, the R version, etc.)?**

We attempt to answer these questions below :)

## What should I cite?

Ideally, you should indeed cite all the packages that you used. However, it's
often not possible to cite them all in the manuscript body. Therefore, we would
recommend the following guidelines:

### 1. Cite the main/important packages in the manuscript

This should be done for the packages that were central to your specific study
(*i.e.,* that got you the results that you reported) rather than data
manipulation tools (even though these are as much, if not *more*, important).
For example:

> Statistical analysis were carried out using R 4.1.0 (R Core Team, 2021), the
*rstanarm* (*v2.13.1*; Gabry \& Goodrich, 2016) and the *report* (*v0.2.0*;
Makowski, Patil, \& Lüdecke, 2019) packages. The full reproducible code is available in **Supplementary Materials**.

### 2. Present everything in Supplementary Materials

Then, in *Supplementary Materials*, you can show all the packages and functions
you used. To do it quickly, explicitly and in a reproducible fashion, we
recommend writing the *Supplementary Materials* with [**R Markdown**](https://rmarkdown.rstudio.com/), which can generate *docs* and *pdf*
files that you can submit along with your manuscript. Moreover, if you're using
R, you can include (usually at the end) every used package's citation using the
`cite_packages()` function from the
[**report**](https://github.com/easystats/report) package. For example:

```{r, results='asis'}
library(report)

cite_packages()
```

## Where 

Finding the right citation information is sometimes complicated. In `R`, this process is made quite easy, you simply run `citation("packagename")`. For instance, `citation("bayestestR")`:

    To cite bayestestR in publications use:

      Makowski, D., Ben-Shachar, M., \& Lüdecke, D. (2019). bayestestR:
      Describing Effects and their Uncertainty, Existence and Significance
      within the Bayesian Framework. Journal of Open Source Software,
      4(40), 1541. doi:10.21105/joss.01541
  
    A BibTeX entry for LaTeX users is
  
      @Article{,
        title = {bayestestR: Describing Effects and their Uncertainty, Existence and Significance within the Bayesian Framework.},
        author = {Dominique Makowski and Mattan S. Ben-Shachar and Daniel Lüdecke},
        journal = {Journal of Open Source Software},
        doi = {10.21105/joss.01541},
        year = {2019},
        number = {40},
        volume = {4},
        pages = {1541},
        url = {https://joss.theoj.org/papers/10.21105/joss.01541},
      }

For other languages, such as `Python` or `Julia`, it might be a little trickier,
but a quick search on Google (or github) should provide you with all the
necessary information (version, authors and date). 

**Keep in mind that it's better to have a slightly incomplete citation than no citation at all.**

## cite_easystats()

If you want to cite the **easystats** ecosystem, you can use the [`cite_easystats()`](https://easystats.github.io/report/reference/cite_easystats.html) function:

```{r results='asis'}
cite_easystats()
```

---
title: "Automated Reporting: Getting Started"
output: 
  rmarkdown::html_vignette:
    toc: true
    fig_width: 10.08
    fig_height: 6
tags: [r, report]
vignette: >
  %\VignetteIndexEntry{Automated Reporting: Getting Started}
  \usepackage[utf8]{inputenc}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, echo=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  message = FALSE,
  out.width = "100%",
  comment = "#"
)

options(
  knitr.kable.NA = "",
  width = 60
)

pkgs <- c("dplyr", "lme4")
successfully_loaded <- vapply(pkgs, requireNamespace, FUN.VALUE = logical(1L), quietly = TRUE)
can_evaluate <- all(successfully_loaded)

if (can_evaluate) {
  knitr::opts_chunk$set(eval = TRUE)
  vapply(pkgs, require, FUN.VALUE = logical(1L), quietly = TRUE, character.only = TRUE)
} else {
  knitr::opts_chunk$set(eval = FALSE)
}

library(report)
```

# Installation

First, install R and R studio. Then, copy and paste the following lines in the
console:

```{r eval=FALSE}
install.packages("remotes")
remotes::install_github("easystats/report") # You only need to do that once
```

```{r eval=FALSE}
library("report") # Load the package every time you start R
```

Great! The `report` package is now installed and loaded in your session.

# Supported Objects

The `report` package works in a two step fashion:
- First, you create a `report` object with the `report()` function.
- Second, this report object can be displayed either textually (the default
output) or as a table, using `as.data.frame()`. Moreover, you can also access a
more compact version of the report using `summary()` on the report object.

## Dataframes

If an entire dataframe is supplied, `report` will provide descriptive statistics
for all columns:

```{r}
report(iris)
```

## Grouped Dataframes

The dataframe can also be a *grouped* dataframe (from `{dplyr}` package), in which case
`report` would return a separate report for each level of the grouping variable.
Additionally, instead of textual summary, `report` also allows one to return a
tabular summary using the `report_table()` function:

```{r}
iris |>
  group_by(Species) |>
  report_table()
```

## Correlations, t-test, and Wilcox test

`report` can also be used to provide automated summaries for statistical model
objects from correlation, *t*-tests, Wilcoxon tests, etc.

```{r}
report(t.test(formula = mtcars$wt ~ mtcars$am))
```

```{r, eval=FALSE}
report(cor.test(mtcars$mpg, mtcars$wt))
```

## Regression models

### Linear regression (`lm`)

We will start out simple: a simple linear regression

```{r}
model <- lm(wt ~ am + mpg, data = mtcars)

report(model)
```

### anova (`aov`)

And its close cousin ANOVA is also covered by `report`:

```{r}
model <- aov(wt ~ am + mpg, data = mtcars)

report(model)
```

### General Linear Models (GLMs) (`glm`)

```{r}
model <- glm(vs ~ mpg + cyl, data = mtcars, family = "binomial")

report(model)
```

### Linear Mixed-Effects Models (`merMod`)

```{r}
library(lme4)

model <- lmer(Reaction ~ Days + (Days | Subject), data = sleepstudy)

report(model)
```

---
title: "Supporting New Models"
output: 
  rmarkdown::html_vignette:
    toc: true
    fig_width: 10.08
    fig_height: 6
tags: [r, report, support new models]
vignette: >
  %\VignetteIndexEntry{Supporting New Models}
  \usepackage[utf8]{inputenc}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, echo=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE
)
```

## 1. Open an Issue on GitHub

If there is a model that you would like to report but is not currently supported
in `report`, you can suggest it to us. You can do so by opening an issue on
[https://github.com/easystats/report/issues](https://github.com/easystats/report/issues),
and saying what would you like to see, and how you can eventually help out (e.g., you can help us test the new functionality).

## 2. Use template file

Copy and paste the [**template file**](https://github.com/easystats/report/blob/master/R/report.default.R), and replace the function with the one you would like to see supported.

# report.lm - glm

    Code
      report(glm(vs ~ disp, data = mtcars, family = binomial(link = "probit")))
    Output
      We fitted a probit model (estimated using ML) to predict vs with disp (formula:
      vs ~ disp). The model's explanatory power is substantial (Nagelkerke's R2 =
      0.66). The model's intercept, corresponding to disp = 0, is at 2.51 (95% CI
      [1.13, 4.28], p = 0.001). Within this model:
      
        - The effect of disp is statistically significant and negative (beta = -0.01,
      95% CI [-0.02, -6.45e-03], p < .001; Std. beta = -1.62, 95% CI [-2.79, -0.80])
      
      Standardized parameters were obtained by fitting the model on a standardized
      version of the dataset. 95% Confidence Intervals (CIs) and p-values were
      computed using a Wald z-distribution approximation.

---

    Code
      report(glm(vs ~ mpg, data = mtcars, family = "poisson"))
    Output
      We fitted a poisson model (estimated using ML) to predict vs with mpg (formula:
      vs ~ mpg). The model's explanatory power is substantial (Nagelkerke's R2 =
      0.39). The model's intercept, corresponding to mpg = 0, is at -3.27 (95% CI
      [-5.46, -1.36], p = 0.002). Within this model:
      
        - The effect of mpg is statistically significant and positive (beta = 0.11, 95%
      CI [0.03, 0.19], p = 0.007; Std. beta = 0.66, 95% CI [0.18, 1.15])
      
      Standardized parameters were obtained by fitting the model on a standardized
      version of the dataset. 95% Confidence Intervals (CIs) and p-values were
      computed using a Wald z-distribution approximation.

# report.lm - lm intercept-only

    Code
      out
    Output
       We fitted a constant (intercept-only) linear model (estimated using OLS) to
      predict d_wide$group0 - d_wide$group1 (formula: d_wide$group0 - d_wide$group1 ~
      1). The model's intercept is at -1.58 (95% CI [-2.46, -0.70], t(9) = -4.06, p =
      0.003).
      
      Standardized parameters were obtained by fitting the model on a standardized
      version of the dataset. 95% Confidence Intervals (CIs) and p-values were
      computed using a Wald t-distribution approximation.

# report.htest-fisher report

    Code
      report(x)
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Fisher's Exact Test for Count Data testing the association between the
      variables of the TeaTasting dataset suggests that the effect is statistically
      not significant, and large (odds ratio = 6.41, 95% CI [0.31, Inf], p = 0.243;
      Adjusted Cramer's v = 0.35, 95% CI [0.00, 1.00])

# report_performance Bayesian)

    Code
      report_performance(x5)
    Output
      The model's explanatory power is substantial (R2 = 0.62, 95% CI [0.53, 0.69],
      adj. R2 = 0.61)

---

    Code
      summary(report_performance(x5))
    Output
      [1] "The model's explanatory power is substantial (R2 = 0.62, adj. R2 = 0.61)"

---

    Code
      report_performance(x6)
    Output
      The model's explanatory power is substantial (R2 = 0.54, 95% CI [0.27, 0.77])

---

    Code
      summary(report_performance(x6))
    Output
      [1] "The model's explanatory power is substantial (R2 = 0.54)"

# report_performance Bayesian 2)

    Code
      summary(report_performance(x7))
    Output
      [1] "The model's explanatory power is substantial (R2 = 0.83, adj. R2 = 0.83) and the part related to the fixed effects alone (marginal R2) is of 0.74"

# report.htest-wilcox

    Code
      report(wilcox.test(x, y, paired = TRUE, data = mtcars))
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Wilcoxon signed rank exact test testing the difference in ranks between x
      and y suggests that the effect is positive, statistically significant, and very
      large (W = 40.00, p = 0.039; r (rank biserial) = 0.78, 95% CI [0.30, 0.94])

---

    Code
      report(wilcox.test(x, y, paired = TRUE, data = mtcars, alternative = "l"))
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Wilcoxon signed rank exact test testing the difference in ranks between x
      and y suggests that the effect is positive, statistically not significant, and
      very large (W = 40.00, p = 0.986; r (rank biserial) = 0.78, 95% CI [-1.00,
      0.93])

---

    Code
      report(wilcox.test(x, y, paired = TRUE, data = mtcars, alternative = "g"))
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Wilcoxon signed rank exact test testing the difference in ranks between x
      and y suggests that the effect is positive, statistically significant, and very
      large (W = 40.00, p = 0.020; r (rank biserial) = 0.78, 95% CI [0.40, 1.00])

---

    Code
      report(wilcox.test(mtcars$am, mtcars$wt, exact = FALSE))
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Wilcoxon rank sum test with continuity correction testing the difference in
      ranks between mtcars$am and mtcars$wt suggests that the effect is negative,
      statistically significant, and very large (W = 0.00, p < .001; r (rank
      biserial) = -1.00, 95% CI [-1.00, -1.00])

---

    Code
      report(wilcox.test(mtcars$am, mtcars$wt, alternative = "l", exact = FALSE))
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Wilcoxon rank sum test with continuity correction testing the difference in
      ranks between mtcars$am and mtcars$wt suggests that the effect is negative,
      statistically significant, and very large (W = 0.00, p < .001; r (rank
      biserial) = -1.00, 95% CI [-1.00, -1.00])

---

    Code
      report(wilcox.test(mtcars$am, mtcars$mpg, exact = FALSE, correct = FALSE))
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Wilcoxon rank sum test testing the difference in ranks between mtcars$am
      and mtcars$mpg suggests that the effect is negative, statistically significant,
      and very large (W = 0.00, p < .001; r (rank biserial) = -1.00, 95% CI [-1.00,
      -1.00])

---

    Code
      report(wilcox.test(depression$change, mu = 1))
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Wilcoxon signed rank exact test testing the difference in rank for
      depression$change and true location of 0 suggests that the effect is negative,
      statistically significant, and very large (W = 0.00, p = 0.004; r (rank
      biserial) = -1.00, 95% CI [-1.00, -1.00])

# report-survreg

    Code
      report(mod_survreg)
    Output
      Can't calculate log-loss.
    Message
      Can't calculate proper scoring rules for models without integer response
        values.
    Output
      `performance_pcp()` only works for models with binary response values.
      Can't calculate log-loss.
    Message
      Can't calculate proper scoring rules for models without integer response
        values.
    Output
      `performance_pcp()` only works for models with binary response values.
      We fitted a logistic model to predict survival::Surv(futime, fustat) with
      ecog.ps and rx (formula: survival::Surv(futime, fustat) ~ ecog.ps + rx). The
      model's explanatory power is weak (Nagelkerke's R2 = 0.07). The model's
      intercept, corresponding to ecog.ps = 0 and rx = 0, is at 667.43 (95% CI
      [-415.59, 1750.45], p = 0.227). Within this model:
      
        - The effect of ecog ps is statistically non-significant and negative (beta =
      -210.59, 95% CI [-726.18, 305.01], p = 0.423; Std. beta = -107.06, 95% CI
      [-369.19, 155.07])
        - The effect of rx is statistically non-significant and positive (beta =
      320.10, 95% CI [-194.11, 834.32], p = 0.222; Std. beta = 163.22, 95% CI
      [-98.98, 425.42])
        - The effect of Log(scale) is statistically significant and positive (beta =
      5.82, 95% CI [5.35, 6.29], p < .001; Std. beta = 5.82, 95% CI [5.35, 6.29])
      
      Standardized parameters were obtained by fitting the model on a standardized
      version of the dataset. 95% Confidence Intervals (CIs) and p-values were
      computed using a Wald z-distribution approximation.

# report.htest-friendman report

    Code
      report(x)
    Output
      Effect sizes were labelled following Landis' (1977) recommendations.
      
      The Friedman rank sum test testing the difference in ranks between wb$x, wb$w,
      and wb$t suggests that the effect is statistically not significant, and in
      slight agreement (chi2 = 0.33, p = 0.564; Kendall's W = 0.11, 95% CI [0.11,
      1.00])

# report.htest-correlation

    Code
      report(cor.test(mtcars$wt, mtcars$mpg))
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Pearson's product-moment correlation between mtcars$wt and mtcars$mpg is
      negative, statistically significant, and very large (r = -0.87, 95% CI [-0.93,
      -0.74], t(30) = -9.56, p < .001)

---

    Code
      report(suppressWarnings(cor.test(mtcars$wt, mtcars$mpg, method = "spearman")))
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Spearman's rank correlation rho between mtcars$wt and mtcars$mpg is
      negative, statistically significant, and very large (rho = -0.89, S = 10292.32,
      p < .001)

---

    Code
      report(suppressWarnings(cor.test(mtcars$wt, mtcars$mpg, method = "kendall")))
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Kendall's rank correlation tau between mtcars$wt and mtcars$mpg is
      negative, statistically significant, and very large (tau = -0.73, z = -5.80, p
      < .001)

# model-stanreg detailed

    Code
      report(model)
    Output
      We fitted a Bayesian linear model (estimated using MCMC sampling with 4 chains
      of 300 iterations and a warmup of 150) to predict mpg with qsec and wt
      (formula: mpg ~ qsec + wt). Priors over parameters were all set as normal (mean
      = 0.00, SD = 8.43; mean = 0.00, SD = 15.40) distributions. The model's
      explanatory power is substantial (R2 = 0.81, 95% CI [0.70, 0.90], adj. R2 =
      0.80). The model's intercept, corresponding to qsec = 0 and wt = 0, is at 19.71
      (95% CI [9.04, 30.18]). Within this model:
      
        - The effect of qsec (Median = 0.92, 95% CI [0.40, 1.47]) has a 99.83%
      probability of being positive (> 0), 99.00% of being significant (> 0.30), and
      0.33% of being large (> 1.81). The estimation successfully converged (Rhat =
      0.999) but the indices are unreliable (ESS = 451)
        - The effect of wt (Median = -5.06, 95% CI [-6.02, -4.18]) has a 100.00%
      probability of being negative (< 0), 100.00% of being significant (< -0.30),
      and 100.00% of being large (< -1.81). However, the estimation might not have
      successfuly converged (Rhat = 1.013) and the indices are unreliable (ESS = 478)
      
      Following the Sequential Effect eXistence and sIgnificance Testing (SEXIT)
      framework, we report the median of the posterior distribution and its 95% CI
      (Highest Density Interval), along the probability of direction (pd), the
      probability of significance and the probability of being large. The thresholds
      beyond which the effect is considered as significant (i.e., non-negligible) and
      large are |0.30| and |1.81| (corresponding respectively to 0.05 and 0.30 of the
      outcome's SD). Convergence and stability of the Bayesian sampling has been
      assessed using R-hat, which should be below 1.01 (Vehtari et al., 2019), and
      Effective Sample Size (ESS), which should be greater than 1000 (Burkner, 2017).

# report-lmer

    Code
      report(m1)
    Output
      We fitted a linear mixed model (estimated using REML and nloptwrap optimizer)
      to predict Reaction with Days (formula: Reaction ~ Days). The model included
      Days as random effects (formula: ~1 + Days | Subject). The model's total
      explanatory power is substantial (conditional R2 = 0.80) and the part related
      to the fixed effects alone (marginal R2) is of 0.28. The model's intercept,
      corresponding to Days = 0, is at 251.41 (95% CI [237.94, 264.87], t(174) =
      36.84, p < .001). Within this model:
      
        - The effect of Days is statistically significant and positive (beta = 10.47,
      95% CI [7.42, 13.52], t(174) = 6.77, p < .001; Std. beta = 0.54, 95% CI [0.38,
      0.69])
      
      Standardized parameters were obtained by fitting the model on a standardized
      version of the dataset. 95% Confidence Intervals (CIs) and p-values were
      computed using a Wald t-distribution approximation.

---

    Code
      report(m2)
    Message
      boundary (singular) fit: see help('isSingular')
    Output
      Random effect variances not available. Returned R2 does not account for random effects.
    Message
      boundary (singular) fit: see help('isSingular')
    Output
      Random effect variances not available. Returned R2 does not account for random effects.
      We fitted a linear mixed model (estimated using REML and nloptwrap optimizer)
      to predict Reaction with Days (formula: Reaction ~ Days). The model included
      mysubgrp as random effects (formula: list(~1 | mysubgrp:mygrp, ~1 | mygrp, ~1 |
      Subject)). The model's explanatory power related to the fixed effects alone
      (marginal R2) is 0.49. The model's intercept, corresponding to Days = 0, is at
      252.09 (95% CI [232.50, 271.69], t(174) = 25.39, p < .001). Within this model:
      
        - The effect of Days is statistically significant and positive (beta = 10.35,
      95% CI [8.78, 11.93], t(174) = 12.95, p < .001; Std. beta = 0.53, 95% CI [0.45,
      0.61])
      
      Standardized parameters were obtained by fitting the model on a standardized
      version of the dataset. 95% Confidence Intervals (CIs) and p-values were
      computed using a Wald t-distribution approximation.

# report-survreg

    Code
      report(ivr)
    Message
      Formula contains log- or sqrt-terms.
        See help("standardize") for how such terms are standardized.
      Formula contains log- or sqrt-terms.
        See help("standardize") for how such terms are standardized.
    Output
      We fitted a linear model to predict packs with rprice, rincome and salestax
      (formula: log(packs) ~ log(rprice) + log(rincome)). The model's explanatory
      power is substantial (R2 = 0.42, adj. R2 = 0.39). The model's intercept,
      corresponding to rprice = 0 and rincome = 0, is at 9.43 (95% CI [6.69, 12.17],
      t(45) = 6.94, p < .001). Within this model:
      
        - The effect of rprice [log] is statistically significant and negative (beta =
      -1.14, 95% CI [-1.87, -0.42], t(45) = -3.18, p = 0.003; Std. beta = -0.53, 95%
      CI [-0.86, -0.20])
        - The effect of rincome [log] is statistically non-significant and positive
      (beta = 0.21, 95% CI [-0.33, 0.76], t(45) = 0.80, p = 0.429; Std. beta = 0.12,
      95% CI [-0.14, 0.39])
      
      Standardized parameters were obtained by fitting the model on a standardized
      version of the dataset. 95% Confidence Intervals (CIs) and p-values were
      computed using a Wald t-distribution approximation.

# report.htest-t-test

    Code
      report(t.test(iris$Sepal.Width, mu = 1))
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations.
      
      The One Sample t-test testing the difference between iris$Sepal.Width (mean =
      3.06) and mu = 1 suggests that the effect is positive, statistically
      significant, and large (difference = 2.06, 95% CI [2.99, 3.13], t(149) = 57.81,
      p < .001; Cohen's d = 4.72, 95% CI [4.15, 5.27])

---

    Code
      report(t.test(iris$Sepal.Width, mu = -1, alternative = "l"))
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations.
      
      The One Sample t-test testing the difference between iris$Sepal.Width (mean =
      3.06) and mu = -1 suggests that the effect is positive, statistically not
      significant, and large (difference = 4.06, 95% CI [-Inf, 3.12], t(149) =
      114.01, p > .999; Cohen's d = 9.31, 95% CI [-Inf, 10.19])

---

    Code
      report(t.test(iris$Sepal.Width, mu = 5, alternative = "g"))
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations.
      
      The One Sample t-test testing the difference between iris$Sepal.Width (mean =
      3.06) and mu = 5 suggests that the effect is negative, statistically not
      significant, and large (difference = -1.94, 95% CI [3.00, Inf], t(149) =
      -54.59, p > .999; Cohen's d = -4.46, 95% CI [-4.89, Inf])

---

    Code
      report(t.test(mtcars$wt ~ mtcars$am))
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations.
      
      The Welch Two Sample t-test testing the difference of mtcars$wt by mtcars$am
      (mean in group 0 = 3.77, mean in group 1 = 2.41) suggests that the effect is
      positive, statistically significant, and large (difference = 1.36, 95% CI
      [0.85, 1.86], t(29.23) = 5.49, p < .001; Cohen's d = 1.93, 95% CI [1.08, 2.77])

---

    Code
      report(t.test(mtcars$wt ~ mtcars$am, alternative = "l"))
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations.
      
      The Welch Two Sample t-test testing the difference of mtcars$wt by mtcars$am
      (mean in group 0 = 3.77, mean in group 1 = 2.41) suggests that the effect is
      positive, statistically not significant, and large (difference = 1.36, 95% CI
      [-Inf, 1.78], t(29.23) = 5.49, p > .999; Cohen's d = 1.93, 95% CI [-Inf, 2.63])

---

    Code
      report(t.test(mtcars$wt ~ mtcars$am, alternative = "g"))
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations.
      
      The Welch Two Sample t-test testing the difference of mtcars$wt by mtcars$am
      (mean in group 0 = 3.77, mean in group 1 = 2.41) suggests that the effect is
      positive, statistically significant, and large (difference = 1.36, 95% CI
      [0.94, Inf], t(29.23) = 5.49, p < .001; Cohen's d = 1.93, 95% CI [1.21, Inf])

---

    Code
      report(t.test(x, y, paired = TRUE))
    Message
      For paired samples, 'repeated_measures_d()' provides more options.
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations.
      
      The Paired t-test testing the difference between x and y (mean difference =
      0.43) suggests that the effect is positive, statistically significant, and
      large (difference = 0.43, 95% CI [0.10, 0.76], t(8) = 3.04, p = 0.016; Cohen's
      d = 1.01, 95% CI [0.18, 1.81])

---

    Code
      report(t.test(x, y, paired = TRUE, alternative = "l"))
    Message
      For paired samples, 'repeated_measures_d()' provides more options.
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations.
      
      The Paired t-test testing the difference between x and y (mean difference =
      0.43) suggests that the effect is positive, statistically not significant, and
      large (difference = 0.43, 95% CI [-Inf, 0.70], t(8) = 3.04, p = 0.992; Cohen's
      d = 1.01, 95% CI [-Inf, 1.67])

---

    Code
      report(t.test(x, y, paired = TRUE, alternative = "g"))
    Message
      For paired samples, 'repeated_measures_d()' provides more options.
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations.
      
      The Paired t-test testing the difference between x and y (mean difference =
      0.43) suggests that the effect is positive, statistically significant, and
      large (difference = 0.43, 95% CI [0.17, Inf], t(8) = 3.04, p = 0.008; Cohen's d
      = 1.01, 95% CI [0.30, Inf])

---

    Code
      report(t.test(sleep2$extra.1, sleep2$extra.2, paired = TRUE))
    Message
      For paired samples, 'repeated_measures_d()' provides more options.
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations.
      
      The Paired t-test testing the difference between sleep2$extra.1 and
      sleep2$extra.2 (mean difference = -1.58) suggests that the effect is negative,
      statistically significant, and large (difference = -1.58, 95% CI [-2.46,
      -0.70], t(9) = -4.06, p = 0.003; Cohen's d = -1.28, 95% CI [-2.12, -0.41])

---

    Code
      report_effectsize(x)
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations. 
      
      large (Cohen's d = -1.70, 95% CI [-2.55, -0.82])

---

    Code
      report_effectsize(x, type = "d")
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations. 
      
      large (Cohen's d = -1.70, 95% CI [-2.55, -0.82])

---

    Code
      report_effectsize(x, type = "g")
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations. 
      
      large (Hedges's g = -1.64, 95% CI [-2.46, -0.79])

---

    Code
      report_effectsize(x, rules = "cohen1988")
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations. 
      
      large (Cohen's d = -1.70, 95% CI [-2.55, -0.82])

---

    Code
      report_effectsize(x, rules = "cohen1988", type = "d")
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations. 
      
      large (Cohen's d = -1.70, 95% CI [-2.55, -0.82])

---

    Code
      report_effectsize(x, rules = "cohen1988", type = "g")
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations. 
      
      large (Hedges's g = -1.64, 95% CI [-2.46, -0.79])

---

    Code
      report_effectsize(x, rules = "sawilowsky2009")
    Output
      Effect sizes were labelled following Savilowsky's (2009) recommendations. 
      
      very large (Cohen's d = -1.70, 95% CI [-2.55, -0.82])

---

    Code
      report_effectsize(x, rules = "sawilowsky2009", type = "d")
    Output
      Effect sizes were labelled following Savilowsky's (2009) recommendations. 
      
      very large (Cohen's d = -1.70, 95% CI [-2.55, -0.82])

---

    Code
      report_effectsize(x, rules = "sawilowsky2009", type = "g")
    Output
      Effect sizes were labelled following Savilowsky's (2009) recommendations. 
      
      very large (Hedges's g = -1.64, 95% CI [-2.46, -0.79])

---

    Code
      report_effectsize(x, rules = "gignac2016")
    Output
      Effect sizes were labelled following Gignac's (2016) recommendations. 
      
      large (Cohen's d = -1.70, 95% CI [-2.55, -0.82])

---

    Code
      report_effectsize(x, rules = "gignac2016", type = "d")
    Output
      Effect sizes were labelled following Gignac's (2016) recommendations. 
      
      large (Cohen's d = -1.70, 95% CI [-2.55, -0.82])

---

    Code
      report_effectsize(x, rules = "gignac2016", type = "g")
    Output
      Effect sizes were labelled following Gignac's (2016) recommendations. 
      
      large (Hedges's g = -1.64, 95% CI [-2.46, -0.79])

# report.htest-chi2 report

    Code
      report(x)
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Pearson's Chi-squared test of independence between gender and party
      suggests that the effect is statistically significant, and small (chi2 = 30.07,
      p < .001; Adjusted Cramer's v = 0.10, 95% CI [0.07, 1.00])

---

    Code
      report(x, rules = "funder2019")
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Pearson's Chi-squared test of independence between gender and party
      suggests that the effect is statistically significant, and small (chi2 = 30.07,
      p < .001; Adjusted Cramer's v = 0.10, 95% CI [0.07, 1.00])

---

    Code
      report(x, rules = "gignac2016")
    Output
      Effect sizes were labelled following Gignac's (2016) recommendations.
      
      The Pearson's Chi-squared test of independence between gender and party
      suggests that the effect is statistically significant, and small (chi2 = 30.07,
      p < .001; Adjusted Cramer's v = 0.10, 95% CI [0.07, 1.00])

---

    Code
      report(x, rules = "cohen1988")
    Output
      Effect sizes were labelled following Cohen's (1988) recommendations.
      
      The Pearson's Chi-squared test of independence between gender and party
      suggests that the effect is statistically significant, and small (chi2 = 30.07,
      p < .001; Adjusted Cramer's v = 0.10, 95% CI [0.07, 1.00])

---

    Code
      report(x, rules = "evans1996")
    Output
      Effect sizes were labelled following Evans's (1996) recommendations.
      
      The Pearson's Chi-squared test of independence between gender and party
      suggests that the effect is statistically significant, and very weak (chi2 =
      30.07, p < .001; Adjusted Cramer's v = 0.10, 95% CI [0.07, 1.00])

---

    Code
      report(x, rules = "lovakov2021")
    Output
      Effect sizes were labelled following Lovakov's (2021) recommendations.
      
      The Pearson's Chi-squared test of independence between gender and party
      suggests that the effect is statistically significant, and very small (chi2 =
      30.07, p < .001; Adjusted Cramer's v = 0.10, 95% CI [0.07, 1.00])

---

    Code
      report(x, type = "cramers_v")
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Pearson's Chi-squared test of independence between gender and party
      suggests that the effect is statistically significant, and small (chi2 = 30.07,
      p < .001; Adjusted Cramer's v = 0.10, 95% CI [0.07, 1.00])

---

    Code
      report(x, type = "pearsons_c")
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Pearson's Chi-squared test of independence between gender and party
      suggests that the effect is statistically significant, and small (chi2 = 30.07,
      p < .001; Pearson's c = 0.10, 95% CI [0.07, 1.00])

---

    Code
      report(x, type = "tschuprows_t", adjust = FALSE)
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Pearson's Chi-squared test of independence between gender and party
      suggests that the effect is statistically significant, and very small (chi2 =
      30.07, p < .001; Tschuprow's t = 0.09, 95% CI [0.06, 1.00])

---

    Code
      report(x, type = "tschuprows_t")
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Pearson's Chi-squared test of independence between gender and party
      suggests that the effect is statistically significant, and very small (chi2 =
      30.07, p < .001; Adjusted Tschuprow's t = 0.08, 95% CI [0.06, 1.00])

---

    Code
      report(x, type = "cohens_w")
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Pearson's Chi-squared test of independence between gender and party
      suggests that the effect is statistically significant, and small (chi2 = 30.07,
      p < .001; Cohens_w = 0.10, 95% CI [0.07, 1.00])

---

    Code
      report(x, type = "phi")
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Pearson's Chi-squared test with Yates' continuity correction of
      independence between Diagnosis and Group suggests that the effect is
      statistically significant, and large (chi2 = 31.57, p < .001; Adjusted Phi =
      0.36, 95% CI [0.25, 1.00])

---

    Code
      report(x, type = "cohens_h", rules = "sawilowsky2009")
    Output
      Effect sizes were labelled following Savilowsky's (2009) recommendations.
      
      The Pearson's Chi-squared test with Yates' continuity correction of
      independence between Diagnosis and Group suggests that the effect is
      statistically significant, and medium (chi2 = 31.57, p < .001; Cohen's h =
      0.74, 95% CI [0.50, 0.99])

---

    Code
      report(x, type = "riskratio")
    Output
      
      
      The Pearson's Chi-squared test with Yates' continuity correction of
      independence between Diagnosis and Group suggests that the effect is
      statistically significant (chi2 = 31.57, p < .001; Risk_ratio = 2.54, 95% CI
      [1.80, 3.60])

---

    Code
      report(x)
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Pearson's Chi-squared test of independence between mtcars$cyl and mtcars$am
      suggests that the effect is statistically significant, and very large (chi2 =
      8.74, p = 0.013; Adjusted Cramer's v = 0.46, 95% CI [0.00, 1.00])

# report.htest-chi2 for given probabilities

    Code
      report(x)
    Output
      Effect sizes were labelled following Funder's (2019) recommendations.
      
      The Chi-squared test for given probabilities / goodness of fit of
      table(mtcars$cyl) to a distribution of [4: n=3.2, 6: n=9.6, 8: n=19.2] suggests
      that the effect is statistically significant, and medium (chi2 = 21.12, p <
      .001; Fei = 0.27, 95% CI [0.17, 1.00])

# report_s

    Code
      report_s(s = 4.2)
    Message
      If the test hypothesis (parameter = 0) and all model assumptions were
        true, there is a 5.4% chance of observing this outcome. How weird is
        that? It's hardly more surprising than getting 4 heads in a row with
        fair coin tosses.

---

    Code
      report_s(p = 0.06)
    Message
      If the test hypothesis (parameter = 0) and all model assumptions were
        true, there is a 6% chance of observing this outcome. How weird is that?
        It's hardly more surprising than getting 4 heads in a row with fair coin
        tosses.

# model-lavaan detailed report

    Code
      report(model)
    Output
      Support for lavaan not fully implemented yet :(

# model-lavaan detailed table

    Code
      report_table(model)
    Output
      Parameter     | Coefficient |       95% CI |     z |      p |  Component |     Fit
      ----------------------------------------------------------------------------------
      ind60 =~ x1   |        1.00 | [1.00, 1.00] |       | < .001 |    Loading |        
      ind60 =~ x2   |        2.18 | [1.91, 2.45] | 15.59 | < .001 |    Loading |        
      ind60 =~ x3   |        1.82 | [1.52, 2.12] | 11.96 | < .001 |    Loading |        
      dem60 =~ y1   |        1.00 | [1.00, 1.00] |       | < .001 |    Loading |        
      dem60 =~ y2   |        1.04 | [0.66, 1.43] |  5.33 | < .001 |    Loading |        
      dem60 =~ y3   |        0.98 | [0.65, 1.30] |  5.89 | < .001 |    Loading |        
      dem60 ~ ind60 |        1.37 | [0.53, 2.21] |  3.20 | 0.001  | Regression |        
                    |             |              |       |        |            |        
      Chi2          |             |              |       |        |            |    7.98
      Chi2_df       |             |              |       |        |            |    8.00
      p_Chi2        |             |              |       |        |            |    0.44
      p_Baseline    |             |              |       |        |            |    0.00
      GFI           |             |              |       |        |            |    0.97
      AGFI          |             |              |       |        |            |    0.91
      NFI           |             |              |       |        |            |    0.97
      NNFI          |             |              |       |        |            |    1.00
      CFI           |             |              |       |        |            |    1.00
      RMSEA         |             |              |       |        |            |    0.00
      RMSEA_CI_low  |             |              |       |        |            |    0.00
      RMSEA_CI_high |             |              |       |        |            |    0.14
      p_RMSEA       |             |              |       |        |            |    0.57
      RMR           |             |              |       |        |            |    0.10
      SRMR          |             |              |       |        |            |    0.03
      RFI           |             |              |       |        |            |    0.95
      PNFI          |             |              |       |        |            |    0.52
      IFI           |             |              |       |        |            |    1.00
      RNI           |             |              |       |        |            |    1.00
      Loglikelihood |             |              |       |        |            | -778.27
      AIC           |             |              |       |        |            | 1582.54
      BIC           |             |              |       |        |            | 1612.67
      BIC (adj.)    |             |              |       |        |            | 1571.69

# model-lavaan detailed performance

    Code
      report_performance(model)
    Output
      The model is not significantly different from a baseline model (Chi2(8) = 7.98,
      p = 0.435). The GFI (.97 > .95) suggest a satisfactory fit., The model is not
      significantly different from a baseline model (Chi2(8) = 7.98, p = 0.435). The
      AGFI (.91 > .90) suggest a satisfactory fit., The model is not significantly
      different from a baseline model (Chi2(8) = 7.98, p = 0.435). The NFI (.97 >
      .90) suggest a satisfactory fit., The model is not significantly different from
      a baseline model (Chi2(8) = 7.98, p = 0.435). The NNFI (.00 > .90) suggest a
      satisfactory fit., The model is not significantly different from a baseline
      model (Chi2(8) = 7.98, p = 0.435). The CFI (.00 > .90) suggest a satisfactory
      fit., The model is not significantly different from a baseline model (Chi2(8) =
      7.98, p = 0.435). The RMSEA (.00 < .05) suggest a satisfactory fit., The model
      is not significantly different from a baseline model (Chi2(8) = 7.98, p =
      0.435). The SRMR (.03 < .08) suggest a satisfactory fit., The model is not
      significantly different from a baseline model (Chi2(8) = 7.98, p = 0.435). The
      RFI (.95 > .90) suggest a satisfactory fit., The model is not significantly
      different from a baseline model (Chi2(8) = 7.98, p = 0.435). The PNFI (.52 >
      .50) suggest a satisfactory fit. and The model is not significantly different
      from a baseline model (Chi2(8) = 7.98, p = 0.435). The IFI (.00 > .90) suggest
      a satisfactory fit.

# report.factor

    Code
      r
    Output
      x: 3 levels, namely A (n = 10, 33.33%), B (n = 10, 33.33%) and C (n = 10,
      33.33%)

# report.data.frame

    Code
      r
    Output
      The data contains 150 observations, grouped by Species, of the following 5
      variables:
      
      - setosa (n = 50):
        - Sepal.Length: n = 50, Mean = 5.01, SD = 0.35, Median = 5.00, MAD = 0.30,
      range: [4.30, 5.80], Skewness = 0.12, Kurtosis = -0.25, 0 missing
        - Sepal.Width: n = 50, Mean = 3.43, SD = 0.38, Median = 3.40, MAD = 0.37,
      range: [2.30, 4.40], Skewness = 0.04, Kurtosis = 0.95, 0 missing
        - Petal.Length: n = 50, Mean = 1.46, SD = 0.17, Median = 1.50, MAD = 0.15,
      range: [1, 1.90], Skewness = 0.11, Kurtosis = 1.02, 0 missing
        - Petal.Width: n = 50, Mean = 0.25, SD = 0.11, Median = 0.20, MAD = 0.00,
      range: [0.10, 0.60], Skewness = 1.25, Kurtosis = 1.72, 0 missing
      
      - versicolor (n = 50):
        - Sepal.Length: n = 50, Mean = 5.94, SD = 0.52, Median = 5.90, MAD = 0.52,
      range: [4.90, 7], Skewness = 0.11, Kurtosis = -0.53, 0 missing
        - Sepal.Width: n = 50, Mean = 2.77, SD = 0.31, Median = 2.80, MAD = 0.30,
      range: [2, 3.40], Skewness = -0.36, Kurtosis = -0.37, 0 missing
        - Petal.Length: n = 50, Mean = 4.26, SD = 0.47, Median = 4.35, MAD = 0.52,
      range: [3, 5.10], Skewness = -0.61, Kurtosis = 0.05, 0 missing
        - Petal.Width: n = 50, Mean = 1.33, SD = 0.20, Median = 1.30, MAD = 0.22,
      range: [1, 1.80], Skewness = -0.03, Kurtosis = -0.41, 0 missing
      
      - virginica (n = 50):
        - Sepal.Length: n = 50, Mean = 6.59, SD = 0.64, Median = 6.50, MAD = 0.59,
      range: [4.90, 7.90], Skewness = 0.12, Kurtosis = 0.03, 0 missing
        - Sepal.Width: n = 50, Mean = 2.97, SD = 0.32, Median = 3.00, MAD = 0.30,
      range: [2.20, 3.80], Skewness = 0.37, Kurtosis = 0.71, 0 missing
        - Petal.Length: n = 50, Mean = 5.55, SD = 0.55, Median = 5.55, MAD = 0.67,
      range: [4.50, 6.90], Skewness = 0.55, Kurtosis = -0.15, 0 missing
        - Petal.Width: n = 50, Mean = 2.03, SD = 0.27, Median = 2.00, MAD = 0.30,
      range: [1.40, 2.50], Skewness = -0.13, Kurtosis = -0.60, 0 missing

# report.data.frame - with NAs

    Code
      report_grouped_df
    Output
      The data contains 32 observations, grouped by cyl, of the following 11
      variables:
      
      - 4 (n = 11):
        - mpg: n = 11, Mean = 26.66, SD = 4.51, Median = 26.00, MAD = 6.52, range:
      [21.40, 33.90], Skewness = 0.35, Kurtosis = -1.43, 0 missing
        - disp: n = 11, Mean = 105.14, SD = 26.87, Median = 108.00, MAD = 43.00, range:
      [71.10, 146.70], Skewness = 0.16, Kurtosis = -1.41, 0 missing
        - hp: n = 11, Mean = 82.64, SD = 20.93, Median = 91.00, MAD = 32.62, range:
      [52, 113], Skewness = 8.41e-03, Kurtosis = -1.56, 0 missing
        - drat: n = 11, Mean = 4.07, SD = 0.37, Median = 4.08, MAD = 0.34, range:
      [3.69, 4.93], Skewness = 1.34, Kurtosis = 2.13, 0 missing
        - wt: n = 11, Mean = 2.29, SD = 0.57, Median = 2.20, MAD = 0.54, range: [1.51,
      3.19], Skewness = 0.40, Kurtosis = -0.85, 0 missing
        - qsec: n = 11, Mean = 19.14, SD = 1.68, Median = 18.90, MAD = 1.48, range:
      [16.70, 22.90], Skewness = 0.74, Kurtosis = 1.84, 0 missing
        - vs: n = 11, Mean = 0.91, SD = 0.30, Median = 1.00, MAD = 0.00, range: [0, 1],
      Skewness = -3.32, Kurtosis = 11.00, 0 missing
        - am: n = 11, Mean = 0.73, SD = 0.47, Median = 1.00, MAD = 0.00, range: [0, 1],
      Skewness = -1.19, Kurtosis = -0.76, 0 missing
        - gear: n = 11, Mean = 4.09, SD = 0.54, Median = 4.00, MAD = 0.00, range: [3,
      5], Skewness = 0.15, Kurtosis = 1.86, 0 missing
        - carb: n = 11, Mean = 1.55, SD = 0.52, Median = 2.00, MAD = 0.00, range: [1,
      2], Skewness = -0.21, Kurtosis = -2.44, 0 missing
      
      - 6 (n = 6):
        - mpg: n = 6, Mean = 19.53, SD = 1.47, Median = 19.45, MAD = 2.15, range:
      [17.80, 21.40], Skewness = 0.14, Kurtosis = -1.77, 0 missing
        - disp: n = 6, Mean = 187.20, SD = 44.11, Median = 167.60, MAD = 22.39, range:
      [145, 258], Skewness = 1.04, Kurtosis = -0.49, 0 missing
        - hp: n = 6, Mean = 124.33, SD = 25.90, Median = 116.50, MAD = 9.64, range:
      [105, 175], Skewness = 2.02, Kurtosis = 4.34, 0 missing
        - drat: n = 6, Mean = 3.53, SD = 0.50, Median = 3.76, MAD = 0.24, range: [2.76,
      3.92], Skewness = -0.94, Kurtosis = -1.02, 0 missing
        - wt: n = 6, Mean = 3.20, SD = 0.31, Median = 3.33, MAD = 0.18, range: [2.77,
      3.46], Skewness = -0.71, Kurtosis = -1.84, 0 missing
        - qsec: n = 6, Mean = 18.23, SD = 1.72, Median = 18.60, MAD = 1.79, range:
      [15.50, 20.22], Skewness = -0.72, Kurtosis = -0.20, 0 missing
        - vs: n = 6, Mean = 0.67, SD = 0.52, Median = 1.00, MAD = 0.00, range: [0, 1],
      Skewness = -0.97, Kurtosis = -1.88, 0 missing
        - am: n = 6, Mean = 0.33, SD = 0.52, Median = 0.00, MAD = 0.00, range: [0, 1],
      Skewness = 0.97, Kurtosis = -1.87, 0 missing
        - gear: n = 6, Mean = 3.83, SD = 0.75, Median = 4.00, MAD = 0.74, range: [3,
      5], Skewness = 0.31, Kurtosis = -0.10, 0 missing
        - carb: n = 6, Mean = 3.33, SD = 1.97, Median = 4.00, MAD = 1.48, range: [1,
      6], Skewness = -0.22, Kurtosis = -1.08, 0 missing
      
      - 8 (n = 14):
        - mpg: n = 14, Mean = 15.10, SD = 2.56, Median = 15.20, MAD = 1.56, range:
      [10.40, 19.20], Skewness = -0.46, Kurtosis = 0.33, 0 missing
        - disp: n = 14, Mean = 353.10, SD = 67.77, Median = 350.50, MAD = 73.39, range:
      [275.80, 472], Skewness = 0.57, Kurtosis = -0.86, 0 missing
        - hp: n = 14, Mean = 209.21, SD = 50.98, Median = 192.50, MAD = 44.48, range:
      [150, 335], Skewness = 1.14, Kurtosis = 1.46, 0 missing
        - drat: n = 14, Mean = 3.23, SD = 0.37, Median = 3.12, MAD = 0.16, range:
      [2.76, 4.22], Skewness = 1.68, Kurtosis = 3.14, 0 missing
        - wt: n = 14, Mean = 4.00, SD = 0.76, Median = 3.75, MAD = 0.41, range: [3.17,
      5.42], Skewness = 1.24, Kurtosis = 0.08, 0 missing
        - qsec: n = 14, Mean = 16.77, SD = 1.20, Median = 17.18, MAD = 0.79, range:
      [14.50, 18], Skewness = -1.01, Kurtosis = -0.28, 0 missing
        - vs: n = 14, Mean = 0.00, SD = 0.00, Median = 0.00, MAD = 0.00, range: [0, 0],
      Skewness = , Kurtosis = , 0 missing
        - am: n = 14, Mean = 0.14, SD = 0.36, Median = 0.00, MAD = 0.00, range: [0, 1],
      Skewness = 2.29, Kurtosis = 3.79, 0 missing
        - gear: n = 14, Mean = 3.29, SD = 0.73, Median = 3.00, MAD = 0.00, range: [3,
      5], Skewness = 2.29, Kurtosis = 3.79, 0 missing
        - carb: n = 14, Mean = 3.50, SD = 1.56, Median = 3.50, MAD = 0.74, range: [2,
      8], Skewness = 1.86, Kurtosis = 5.14, 0 missing

# report.data.frame - with list columns

    Code
      report(dplyr::starwars)
    Output
      The data contains 87 observations of the following 11 variables:
      
        - name: 87 entries, such as Ackbar (n = 1); Adi Gallia (n = 1); Anakin
      Skywalker (n = 1) and 84 others (0 missing)
        - height: n = 87, Mean = 174.60, SD = 34.77, Median = 180.00, MAD = 17.79,
      range: [66, 264], Skewness = -1.09, Kurtosis = 2.13, 6 missing
        - mass: n = 87, Mean = 97.31, SD = 169.46, Median = 79.00, MAD = 16.31, range:
      [15, 1358], Skewness = 7.34, Kurtosis = 55.42, 28 missing
        - hair_color: 11 entries, such as none (n = 38); brown (n = 18); black (n = 13)
      and 8 others (5 missing)
        - skin_color: 31 entries, such as fair (n = 17); light (n = 11); dark (n = 6)
      and 28 others (0 missing)
        - eye_color: 15 entries, such as brown (n = 21); blue (n = 19); yellow (n = 11)
      and 12 others (0 missing)
        - birth_year: n = 87, Mean = 87.57, SD = 154.69, Median = 52.00, MAD = 29.65,
      range: [8, 896], Skewness = 4.45, Kurtosis = 20.59, 44 missing
        - sex: 4 entries, such as male (n = 60); female (n = 16); none (n = 6) and 1
      other (4 missing)
        - gender: 2 entries, such as masculine (n = 66); feminine (n = 17) (4 missing)
        - homeworld: 48 entries, such as Naboo (n = 11); Tatooine (n = 10); Alderaan (n
      = 3) and 45 others (10 missing)
        - species: 37 entries, such as Human (n = 35); Droid (n = 6); Gungan (n = 3)
      and 34 others (4 missing)

# report.aov

    Code
      suppressWarnings(report(anova(lm(Sepal.Width ~ Species, data = iris))))
    Output
      The ANOVA suggests that:
      
        - The main effect of Species is statistically significant and large (F(2, 147)
      = 49.16, p < .001; Eta2 = 0.40, 95% CI [0.30, 1.00])
      
      Effect sizes were labelled following Field's (2013) recommendations.

---

    Code
      suppressWarnings(report(anova(lm(wt ~ as.factor(am) * as.factor(cyl), data = mtcars))))
    Output
      The ANOVA suggests that:
      
        - The main effect of as.factor(am) is statistically significant and large (F(1,
      26) = 45.39, p < .001; Eta2 (partial) = 0.64, 95% CI [0.43, 1.00])
        - The main effect of as.factor(cyl) is statistically significant and large
      (F(2, 26) = 11.53, p < .001; Eta2 (partial) = 0.47, 95% CI [0.21, 1.00])
        - The interaction between as.factor(am) and as.factor(cyl) is statistically not
      significant and very small (F(2, 26) = 0.11, p = 0.899; Eta2 (partial) =
      8.13e-03, 95% CI [0.00, 1.00])
      
      Effect sizes were labelled following Field's (2013) recommendations.

---

    Code
      suppressWarnings(report(aov(wt ~ cyl + Error(gear), data = mtcars)))
    Output
      The repeated-measures ANOVA (formula: wt ~ cyl + Error(gear)) suggests that:
      
        - The main effect of cyl is statistically significant and large (F(1, 29) =
      27.94, p < .001; Eta2 (partial) = 1.00, 95% CI [, 1.00])
        - The main effect of cyl is statistically significant and large (F(1, 29) =
      27.94, p < .001; Eta2 (partial) = 0.49, 95% CI [0.27, 1.00])
      
      Effect sizes were labelled following Field's (2013) recommendations.

# report.htest-kruskal report

    Code
      report(x, verbose = FALSE)
    Output
      Effect sizes were labelled following Field's (2013) recommendations.
      
      The Kruskal-Wallis rank sum test testing the difference in ranks between
      airquality$Ozone and as.factor(airquality$Month) suggests that the effect is
      statistically significant, and large (Kruskal-Wallis chi2 = 29.27, p < .001;
      Epsilon squared (rank) = 0.25, 95% CI [0.15, 1.00])

# report_participants

    Code
      report_participants(data, age = "Age", sex = "Sex", participant = "Participant")
    Output
      [1] "3 participants (Mean age = 28.0, SD = 23.6, range: [8, 54]; Sex: 33.3% females, 33.3% males, 33.3% other; Gender: 33.3% women, 33.3% men, 33.33% non-binary)"

---

    Code
      report_participants(data, participant = "Participant", spell_n = TRUE)
    Output
      [1] "Three participants (Mean age = 28.0, SD = 23.6, range: [8, 54]; Sex: 33.3% females, 33.3% males, 33.3% other; Gender: 33.3% women, 33.3% men, 33.33% non-binary)"

---

    Code
      report_participants(data2)
    Output
      [1] "6 participants (Mean age = 28.0, SD = 21.1, range: [8, 54]; Sex: 50.0% females, 16.7% males, 33.3% other; Gender: 50.0% women, 16.7% men, 33.33% non-binary)"

---

    Code
      report_participants(data3)
    Output
      [1] "6 participants (Mean age = 52.0, SD = 42.4, range: [22, 82], 66.7% missing; Sex: 33.3% females, 0.0% males, 0.0% other, 66.67% missing; Gender: 33.3% women, 0.0% men, 0.00% non-binary, 66.67% missing)"

---

    Code
      report_participants(data4)
    Output
      [1] "7 participants (Mean education = 1.3, SD = 4.9, range: [-5, 8]; Country: 28.57% Canada, 28.57% USA, 14.29% Germany, 14.29% India, 14.29% missing; Race: 42.86% Black, 28.57% White, 14.29% Asian, 14.29% missing)"

---

    Code
      report_participants(data4, education = "Education2")
    Output
      [1] "7 participants (Education: Bachelor, 42.86%; Highschool, 28.57%; PhD, 14.29%; missing, 14.29%; Country: 28.57% Canada, 28.57% USA, 14.29% Germany, 14.29% India, 14.29% missing; Race: 42.86% Black, 28.57% White, 14.29% Asian, 14.29% missing)"

---

    Code
      report_participants(data4, threshold = 15)
    Output
      [1] "7 participants (Mean education = 1.3, SD = 4.9, range: [-5, 8]; Country: 28.57% Canada, 28.57% USA, 42.86% other; Race: 42.86% Black, 28.57% White, 28.57% other)"

# report_participants test NAs no warning

    Code
      report_participants(data)
    Output
      [1] "6 participants (Mean age = 28.3, SD = 16.6, range: [8, 54]; Sex: 16.7% females, 33.3% males, 16.7% other, 33.33% missing; Gender: 33.3% women, 16.7% men, 16.67% non-binary, 33.33% missing; Education: -5, 16.67%; -3, 16.67%; 0, 16.67%; 3, 16.67%; 5, 16.67%; 8, 16.67%; Country: 33.33% Canada, 16.67% Germany, 16.67% India, 16.67% USA, 16.67% missing; Race: 16.67% A, 16.67% B, 16.67% C, 16.67% D, 16.67% E, 16.67% missing)"

---

    Code
      report_participants(data, age = "Age", sex = "Sex")
    Output
      [1] "6 participants (Mean age = 3.5, SD = 1.9, range: [1, 6]; Sex: 16.7% females, 33.3% males, 16.7% other, 33.33% missing; Gender: 33.3% women, 16.7% men, 16.67% non-binary, 33.33% missing; Education: -5, 16.67%; -3, 16.67%; 0, 16.67%; 3, 16.67%; 5, 16.67%; 8, 16.67%; Country: 33.33% Canada, 16.67% Germany, 16.67% India, 16.67% USA, 16.67% missing; Race: 16.67% A, 16.67% B, 16.67% C, 16.67% D, 16.67% E, 16.67% missing)"

# report_participants age as character

    Code
      report_participants(data)
    Output
      [1] "6 participants (Mean age = 29.5, SD = 15.0, range: [11, 52])"

# report_participants different gender spellings

    Code
      report_participants(data)
    Output
      [1] "20 participants (Gender: 50.0% women, 45.0% men, 0.00% non-binary, 5.00% missing)"

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.htest.R
\name{report.htest}
\alias{report.htest}
\alias{report_effectsize.htest}
\alias{report_table.htest}
\alias{report_statistics.htest}
\alias{report_parameters.htest}
\alias{report_model.htest}
\alias{report_info.htest}
\alias{report_text.htest}
\title{Reporting \code{htest} objects (Correlation, t-test...)}
\usage{
\method{report}{htest}(x, ...)

\method{report_effectsize}{htest}(x, table = NULL, ...)

\method{report_table}{htest}(x, ...)

\method{report_statistics}{htest}(x, table = NULL, ...)

\method{report_parameters}{htest}(x, table = NULL, ...)

\method{report_model}{htest}(x, table = NULL, ...)

\method{report_info}{htest}(x, effectsize = NULL, ...)

\method{report_text}{htest}(x, table = NULL, ...)
}
\arguments{
\item{x}{Object of class \code{htest}.}

\item{...}{Arguments passed to or from other methods.}

\item{table}{Provide the output of \code{report_table()} to avoid its re-computation.}

\item{effectsize}{Provide the output of \code{report_effectsize()} to avoid
its re-computation.}
}
\value{
An object of class \code{\link[=report]{report()}}.
}
\description{
Create reports for \code{htest} objects (\code{t.test()}, \code{cor.test()},
etc.).
}
\examples{
# t-tests
report(t.test(iris$Sepal.Width, iris$Sepal.Length))
report(t.test(iris$Sepal.Width, iris$Sepal.Length, var.equal = TRUE))
report(t.test(mtcars$mpg ~ mtcars$vs))
report(t.test(mtcars$mpg, mtcars$vs, paired = TRUE), verbose = FALSE)
report(t.test(iris$Sepal.Width, mu = 1))

# Correlations
report(cor.test(iris$Sepal.Width, iris$Sepal.Length))
}
\seealso{
Specific components of reports (especially for stats models):
\itemize{
\item \code{\link[=report_table]{report_table()}}
\item \code{\link[=report_parameters]{report_parameters()}}
\item \code{\link[=report_statistics]{report_statistics()}}
\item \code{\link[=report_effectsize]{report_effectsize()}}
\item \code{\link[=report_model]{report_model()}}
\item \code{\link[=report_priors]{report_priors()}}
\item \code{\link[=report_random]{report_random()}}
\item \code{\link[=report_performance]{report_performance()}}
\item \code{\link[=report_info]{report_info()}}
\item \code{\link[=report_text]{report_text()}}
}

Other types of reports:
\itemize{
\item \code{\link[=report_system]{report_system()}}
\item \code{\link[=report_packages]{report_packages()}}
\item \code{\link[=report_participants]{report_participants()}}
\item \code{\link[=report_sample]{report_sample()}}
\item \code{\link[=report_date]{report_date()}}
}

Methods:
\itemize{
\item \code{\link[=as.report]{as.report()}}
}

Template file for supporting new models:
\itemize{
\item \code{\link[=report.default]{report.default()}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_intercept.R
\name{report_intercept}
\alias{report_intercept}
\title{Report intercept}
\usage{
report_intercept(x, ...)
}
\arguments{
\item{x}{The R object that you want to report (see list of of supported
objects above).}

\item{...}{Arguments passed to or from other methods.}
}
\value{
An object of class \code{\link[=report_intercept]{report_intercept()}}.
}
\description{
Reports intercept of regression models (see list of supported objects in
\code{\link[=report]{report()}}).
}
\examples{
\donttest{
library(report)

# GLMs
report_intercept(lm(Sepal.Length ~ Species, data = iris))
report_intercept(glm(vs ~ disp, data = mtcars, family = "binomial"))
}

\dontshow{if (requireNamespace("lme4", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Mixed models
library(lme4)
model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
report_intercept(model)
}
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("rstanarm", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Bayesian models
library(rstanarm)
model <- suppressWarnings(stan_glm(Sepal.Length ~ Species, data = iris, refresh = 0, iter = 600))
report_intercept(model)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_effectsize.R
\name{report_effectsize}
\alias{report_effectsize}
\title{Report the effect size(s) of a model or a test}
\usage{
report_effectsize(x, ...)
}
\arguments{
\item{x}{The R object that you want to report (see list of of supported
objects above).}

\item{...}{Arguments passed to or from other methods.}
}
\value{
An object of class \code{\link[=report_effectsize]{report_effectsize()}}.
}
\description{
Computes, interpret and formats the effect sizes of a variety of models and
statistical tests (see list of supported objects in \code{\link[=report]{report()}}).
}
\examples{
library(report)

# h-tests
report_effectsize(t.test(iris$Sepal.Width, iris$Sepal.Length))

# ANOVAs
report_effectsize(aov(Sepal.Length ~ Species, data = iris))

# GLMs
report_effectsize(lm(Sepal.Length ~ Petal.Length * Species, data = iris))
report_effectsize(glm(vs ~ disp, data = mtcars, family = "binomial"))

\dontshow{if (requireNamespace("lme4", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Mixed models
library(lme4)
model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
report_effectsize(model)
}
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("rstanarm", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Bayesian models
library(rstanarm)
model <- suppressWarnings(stan_glm(Sepal.Length ~ Species, data = iris, refresh = 0, iter = 600))
report_effectsize(model, effectsize_method = "basic")
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.stanreg.R
\name{report.stanreg}
\alias{report.stanreg}
\title{Reporting Bayesian Models}
\usage{
\method{report}{stanreg}(x, ...)
}
\arguments{
\item{x}{Object of class \code{lm} or \code{glm}.}

\item{...}{Arguments passed to or from other methods.}
}
\value{
An object of class \code{\link[=report]{report()}}.
}
\description{
Create reports for Bayesian models. The description of the parameters follows
the Sequential Effect eXistence and sIgnificance Testing framework (see
\link[bayestestR:sexit]{SEXIT documentation}).
}
\examples{
\dontshow{if (requireNamespace("rstanarm", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Bayesian models
library(rstanarm)
model <- suppressWarnings(stan_glm(mpg ~ qsec + wt, data = mtcars, refresh = 0, iter = 500))
r <- report(model)
r
summary(r)
as.data.frame(r)
}
\dontshow{\}) # examplesIf}
}
\seealso{
Specific components of reports (especially for stats models):
\itemize{
\item \code{\link[=report_table]{report_table()}}
\item \code{\link[=report_parameters]{report_parameters()}}
\item \code{\link[=report_statistics]{report_statistics()}}
\item \code{\link[=report_effectsize]{report_effectsize()}}
\item \code{\link[=report_model]{report_model()}}
\item \code{\link[=report_priors]{report_priors()}}
\item \code{\link[=report_random]{report_random()}}
\item \code{\link[=report_performance]{report_performance()}}
\item \code{\link[=report_info]{report_info()}}
\item \code{\link[=report_text]{report_text()}}
}

Other types of reports:
\itemize{
\item \code{\link[=report_system]{report_system()}}
\item \code{\link[=report_packages]{report_packages()}}
\item \code{\link[=report_participants]{report_participants()}}
\item \code{\link[=report_sample]{report_sample()}}
\item \code{\link[=report_date]{report_date()}}
}

Methods:
\itemize{
\item \code{\link[=as.report]{as.report()}}
}

Template file for supporting new models:
\itemize{
\item \code{\link[=report.default]{report.default()}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.character.R, R/report.data.frame.R,
%   R/report.factor.R, R/report.numeric.R
\name{report.character}
\alias{report.character}
\alias{report.data.frame}
\alias{report.factor}
\alias{report.numeric}
\title{Reporting Datasets and Dataframes}
\usage{
\method{report}{character}(
  x,
  n_entries = 3,
  levels_percentage = "auto",
  missing_percentage = "auto",
  ...
)

\method{report}{data.frame}(
  x,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  levels_percentage = "auto",
  digits = 2,
  n_entries = 3,
  missing_percentage = "auto",
  ...
)

\method{report}{factor}(x, levels_percentage = "auto", ...)

\method{report}{numeric}(
  x,
  n = FALSE,
  centrality = "mean",
  dispersion = TRUE,
  range = TRUE,
  distribution = FALSE,
  missing_percentage = "auto",
  digits = 2,
  ...
)
}
\arguments{
\item{x}{The R object that you want to report (see list of of supported
objects above).}

\item{n_entries}{Number of different character entries to show. Can be "all".}

\item{levels_percentage}{Show characters entries and factor levels by number
or percentage. If "auto", then will be set to number and percentage if the
length if n observations larger than 100.}

\item{missing_percentage}{Show missing by number (default) or percentage. If
"auto", then will be set to number and percentage if the length if n
observations larger than 100.}

\item{...}{Arguments passed to or from other methods.}

\item{n}{Include number of observations for each individual variable.}

\item{centrality}{Character vector, indicating the index of centrality
(either \code{"mean"} or \code{"median"}).}

\item{dispersion}{Show index of dispersion (\link{sd} if \code{centrality = "mean"}, or
\link{mad} if \code{centrality = "median"}).}

\item{range}{Show range.}

\item{distribution}{Show kurtosis and skewness.}

\item{digits}{Number of significant digits.}
}
\value{
An object of class \code{\link[=report]{report()}}.
}
\description{
Create reports for data frames.
}
\examples{
r <- report(iris,
  centrality = "median", dispersion = FALSE,
  distribution = TRUE, missing_percentage = TRUE
)
r
summary(r)
as.data.frame(r)
summary(as.data.frame(r))

\dontshow{if (requireNamespace("dplyr", quietly = TRUE)) withAutoprint(\{ # examplesIf}
# grouped analysis using `{dplyr}` package
library(dplyr)
r <- iris \%>\%
  group_by(Species) \%>\%
  report()
r
summary(r)
as.data.frame(r)
summary(as.data.frame(r))
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.default.R
\name{report.default}
\alias{report.default}
\alias{report_effectsize.default}
\alias{report_table.default}
\alias{report_statistics.default}
\alias{report_parameters.default}
\alias{report_intercept.default}
\alias{report_model.default}
\alias{report_random.default}
\alias{report_priors.default}
\alias{report_performance.default}
\alias{report_info.default}
\alias{report_text.default}
\title{Template to add report support for new objects}
\usage{
\method{report}{default}(x, ...)

\method{report_effectsize}{default}(x, ...)

\method{report_table}{default}(x, ...)

\method{report_statistics}{default}(x, ...)

\method{report_parameters}{default}(x, ...)

\method{report_intercept}{default}(x, ...)

\method{report_model}{default}(x, ...)

\method{report_random}{default}(x, ...)

\method{report_priors}{default}(x, ...)

\method{report_performance}{default}(x, ...)

\method{report_info}{default}(x, ...)

\method{report_text}{default}(x, ...)
}
\arguments{
\item{x}{Object of class \verb{NEW OBJECT}.}

\item{...}{Arguments passed to or from other methods.}
}
\value{
An object of class \code{\link[=report]{report()}}.
}
\description{
Template file to add report support for new objects. Check-out the vignette on
\href{https://easystats.github.io/report/articles/new_models.html}{Supporting New Models}.
}
\examples{
library(report)

# Add a reproducible example instead of the following
model <- lm(Sepal.Length ~ Petal.Length * Species, data = iris)
r <- report(model)
r
summary(r)
as.data.frame(r)
summary(as.data.frame(r))
}
\seealso{
Specific components of reports (especially for stats models):
\itemize{
\item \code{\link[=report_table]{report_table()}}
\item \code{\link[=report_parameters]{report_parameters()}}
\item \code{\link[=report_statistics]{report_statistics()}}
\item \code{\link[=report_effectsize]{report_effectsize()}}
\item \code{\link[=report_model]{report_model()}}
\item \code{\link[=report_priors]{report_priors()}}
\item \code{\link[=report_random]{report_random()}}
\item \code{\link[=report_performance]{report_performance()}}
\item \code{\link[=report_info]{report_info()}}
\item \code{\link[=report_text]{report_text()}}
}

Other types of reports:
\itemize{
\item \code{\link[=report_system]{report_system()}}
\item \code{\link[=report_packages]{report_packages()}}
\item \code{\link[=report_participants]{report_participants()}}
\item \code{\link[=report_sample]{report_sample()}}
\item \code{\link[=report_date]{report_date()}}
}

Methods:
\itemize{
\item \code{\link[=as.report]{as.report()}}
}

Template file for supporting new models:
\itemize{
\item \code{\link[=report.default]{report.default()}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_table.R
\name{report_table}
\alias{report_table}
\title{Report a descriptive table}
\usage{
report_table(x, ...)
}
\arguments{
\item{x}{The R object that you want to report (see list of of supported
objects above).}

\item{...}{Arguments passed to or from other methods.}
}
\value{
An object of class \code{\link[=report_table]{report_table()}}.
}
\description{
Creates tables to describe different objects (see list of supported objects
in \code{\link[=report]{report()}}).
}
\examples{
\donttest{
# Miscellaneous
r <- report_table(sessionInfo())
r
summary(r)

# Data
report_table(iris$Sepal.Length)
report_table(as.character(round(iris$Sepal.Length, 1)))
report_table(iris$Species)
report_table(iris)

# h-tests
report_table(t.test(mtcars$mpg ~ mtcars$am))

# ANOVAs
report_table(aov(Sepal.Length ~ Species, data = iris))

# GLMs
report_table(lm(Sepal.Length ~ Petal.Length * Species, data = iris))
report_table(glm(vs ~ disp, data = mtcars, family = "binomial"))
}

\dontshow{if (requireNamespace("lme4", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Mixed models
library(lme4)
model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
report_table(model)
}
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("rstanarm", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Bayesian models
library(rstanarm)
model <- suppressWarnings(stan_glm(Sepal.Length ~ Species, data = iris, refresh = 0, iter = 600))
report_table(model, effectsize_method = "basic")
}
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("lavaan", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Structural Equation Models (SEM)
library(lavaan)
structure <- "ind60 =~ x1 + x2 + x3
              dem60 =~ y1 + y2 + y3
              dem60 ~ ind60"
model <- lavaan::sem(structure, data = PoliticalDemocracy)
suppressWarnings(report_table(model))
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.aov.R
\name{report.aov}
\alias{report.aov}
\alias{report_effectsize.aov}
\alias{report_table.aov}
\alias{report_statistics.aov}
\alias{report_parameters.aov}
\alias{report_model.aov}
\alias{report_info.aov}
\alias{report_text.aov}
\title{Reporting ANOVAs}
\usage{
\method{report}{aov}(x, ...)

\method{report_effectsize}{aov}(x, include_intercept = FALSE, ...)

\method{report_table}{aov}(x, include_intercept = FALSE, ...)

\method{report_statistics}{aov}(x, table = NULL, ...)

\method{report_parameters}{aov}(x, ...)

\method{report_model}{aov}(x, table = NULL, ...)

\method{report_info}{aov}(x, effectsize = NULL, ...)

\method{report_text}{aov}(x, table = NULL, ...)
}
\arguments{
\item{x}{Object of class \code{aov}, \code{anova} or \code{aovlist}.}

\item{...}{Arguments passed to or from other methods.}

\item{include_intercept}{Set to \code{TRUE} to include the intercept (relevant for type-3 ANOVA tables).}

\item{table}{Provide the output of \code{report_table()} to avoid its re-computation.}

\item{effectsize}{Provide the output of \code{report_effectsize()} to avoid
its re-computation.}
}
\value{
An object of class \code{\link[=report]{report()}}.
}
\description{
Create reports for ANOVA models.
}
\examples{
data <- iris
data$Cat1 <- rep(c("A", "B"), length.out = nrow(data))

model <- aov(Sepal.Length ~ Species * Cat1, data = data)
r <- report(model)
r
summary(r)
as.data.frame(r)
summary(as.data.frame(r))
}
\seealso{
Specific components of reports (especially for stats models):
\itemize{
\item \code{\link[=report_table]{report_table()}}
\item \code{\link[=report_parameters]{report_parameters()}}
\item \code{\link[=report_statistics]{report_statistics()}}
\item \code{\link[=report_effectsize]{report_effectsize()}}
\item \code{\link[=report_model]{report_model()}}
\item \code{\link[=report_priors]{report_priors()}}
\item \code{\link[=report_random]{report_random()}}
\item \code{\link[=report_performance]{report_performance()}}
\item \code{\link[=report_info]{report_info()}}
\item \code{\link[=report_text]{report_text()}}
}

Other types of reports:
\itemize{
\item \code{\link[=report_system]{report_system()}}
\item \code{\link[=report_packages]{report_packages()}}
\item \code{\link[=report_participants]{report_participants()}}
\item \code{\link[=report_sample]{report_sample()}}
\item \code{\link[=report_date]{report_date()}}
}

Methods:
\itemize{
\item \code{\link[=as.report]{as.report()}}
}

Template file for supporting new models:
\itemize{
\item \code{\link[=report.default]{report.default()}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_s.R
\name{report_s}
\alias{report_s}
\title{Report S- and p-values in easy language.}
\usage{
report_s(s = NULL, p = NULL, test_value = 0, test_parameter = "parameter")
}
\arguments{
\item{s}{An S-value. Either \code{s} or \code{p} must be provided.}

\item{p}{A p-value. Either \code{s} or \code{p} must be provided.}

\item{test_value}{The value of the test parameter under the null hypothesis.}

\item{test_parameter}{The name of the test parameter under the null hypothesis.}
}
\value{
A string with the interpretation of the S- or p-value.
}
\description{
Reports interpretation of S- and p-values in easy language.
}
\examples{
report_s(s = 1.5)
report_s(p = 0.05)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_parameters.R
\name{report_parameters}
\alias{report_parameters}
\title{Report the parameters of a model}
\usage{
report_parameters(x, ...)
}
\arguments{
\item{x}{The R object that you want to report (see list of of supported
objects above).}

\item{...}{Arguments passed to or from other methods.}
}
\value{
A \code{vector}.
}
\description{
Creates a list containing a description of the parameters of R objects (see
list of supported objects in \code{\link[=report]{report()}}).
}
\examples{
\donttest{
library(report)

# Miscellaneous
r <- report_parameters(sessionInfo())
r
summary(r)

# Data
report_parameters(iris$Sepal.Length)
report_parameters(as.character(round(iris$Sepal.Length, 1)))
report_parameters(iris$Species)
report_parameters(iris)

# h-tests
report_parameters(t.test(iris$Sepal.Width, iris$Sepal.Length))

# ANOVA
report_parameters(aov(Sepal.Length ~ Species, data = iris))

# GLMs
report_parameters(lm(Sepal.Length ~ Petal.Length * Species, data = iris))
report_parameters(lm(Petal.Width ~ Species, data = iris), include_intercept = FALSE)
report_parameters(glm(vs ~ disp, data = mtcars, family = "binomial"))
}

\dontshow{if (requireNamespace("lme4", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Mixed models
library(lme4)
model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
report_parameters(model)
}
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("rstanarm", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Bayesian models
library(rstanarm)
model <- suppressWarnings(stan_glm(Sepal.Length ~ Species, data = iris, refresh = 0, iter = 600))
report_parameters(model)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.bayesfactor_models.R
\name{report.bayesfactor_models}
\alias{report.bayesfactor_models}
\alias{report.bayesfactor_inclusion}
\title{Reporting Models' Bayes Factor}
\usage{
\method{report}{bayesfactor_models}(
  x,
  interpretation = "jeffreys1961",
  exact = TRUE,
  protect_ratio = TRUE,
  ...
)

\method{report}{bayesfactor_inclusion}(
  x,
  interpretation = "jeffreys1961",
  exact = TRUE,
  protect_ratio = TRUE,
  ...
)
}
\arguments{
\item{x}{Object of class \code{bayesfactor_inclusion}.}

\item{interpretation}{Effect size interpretation set of rules (see
\link[effectsize:interpret_bf]{interpret_bf}).}

\item{exact}{Should very large or very small values be reported with a
scientific format (e.g., 4.24e5), or as truncated values (as "> 1000" and
"< 1/1000").}

\item{protect_ratio}{Should values smaller than 1 be represented as ratios?}

\item{...}{Arguments passed to or from other methods.}
}
\value{
An object of class \code{\link[=report]{report()}}.
}
\description{
Create reports of Bayes factors for model comparison.
}
\examples{
\dontshow{if (requireNamespace("bayestestR", quietly = TRUE)) withAutoprint(\{ # examplesIf}
library(bayestestR)
# Bayes factor - models
mo0 <- lm(Sepal.Length ~ 1, data = iris)
mo1 <- lm(Sepal.Length ~ Species, data = iris)
mo2 <- lm(Sepal.Length ~ Species + Petal.Length, data = iris)
mo3 <- lm(Sepal.Length ~ Species * Petal.Length, data = iris)
BFmodels <- bayesfactor_models(mo1, mo2, mo3, denominator = mo0)

r <- report(BFmodels)
r

# Bayes factor - inclusion
inc_bf <- bayesfactor_inclusion(BFmodels, prior_odds = c(1, 2, 3), match_models = TRUE)

r <- report(inc_bf)
r
as.data.frame(r)
\dontshow{\}) # examplesIf}
}
\seealso{
Specific components of reports (especially for stats models):
\itemize{
\item \code{\link[=report_table]{report_table()}}
\item \code{\link[=report_parameters]{report_parameters()}}
\item \code{\link[=report_statistics]{report_statistics()}}
\item \code{\link[=report_effectsize]{report_effectsize()}}
\item \code{\link[=report_model]{report_model()}}
\item \code{\link[=report_priors]{report_priors()}}
\item \code{\link[=report_random]{report_random()}}
\item \code{\link[=report_performance]{report_performance()}}
\item \code{\link[=report_info]{report_info()}}
\item \code{\link[=report_text]{report_text()}}
}

Other types of reports:
\itemize{
\item \code{\link[=report_system]{report_system()}}
\item \code{\link[=report_packages]{report_packages()}}
\item \code{\link[=report_participants]{report_participants()}}
\item \code{\link[=report_sample]{report_sample()}}
\item \code{\link[=report_date]{report_date()}}
}

Methods:
\itemize{
\item \code{\link[=as.report]{as.report()}}
}

Template file for supporting new models:
\itemize{
\item \code{\link[=report.default]{report.default()}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.compare.loo.R
\name{report.compare.loo}
\alias{report.compare.loo}
\title{Reporting Bayesian Model Comparison}
\usage{
\method{report}{compare.loo}(x, include_IC = TRUE, include_ENP = FALSE, ...)
}
\arguments{
\item{x}{An object of class \link[brms:loo_compare]{brms::loo_compare}.}

\item{include_IC}{Whether to include the information criteria (IC).}

\item{include_ENP}{Whether to include the effective number of parameters (ENP).}

\item{...}{Additional arguments (not used for now).}
}
\value{
Objects of class \code{\link[=report_text]{report_text()}}.
}
\description{
Automatically report the results of Bayesian model comparison using the \code{loo} package.
}
\details{
The rule of thumb is that the models are "very similar" if |elpd_diff| (the
absolute value of elpd_diff) is less than 4 (Sivula, Magnusson and Vehtari, 2020).
If superior to 4, then one can use the SE to obtain a standardized difference
(Z-diff) and interpret it as such, assuming that the difference is normally
distributed. The corresponding p-value is then calculated as \code{2 * pnorm(-abs(Z-diff))}.
However, note that if the raw ELPD difference is small (less than 4), it doesn't
make much sense to rely on its standardized value: it is not very useful to
conclude that a model is much better than another if both models make very
similar predictions.
}
\examples{
\dontshow{if (requireNamespace("brms", quietly = TRUE) && requireNamespace("RcppEigen", quietly = TRUE) && requireNamespace("BH", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
library(brms)

m1 <- brms::brm(mpg ~ qsec, data = mtcars)
m2 <- brms::brm(mpg ~ qsec + drat, data = mtcars)
m3 <- brms::brm(mpg ~ qsec + drat + wt, data = mtcars)

x <- suppressWarnings(brms::loo_compare(
  brms::add_criterion(m1, "loo"),
  brms::add_criterion(m2, "loo"),
  brms::add_criterion(m3, "loo"),
  model_names = c("m1", "m2", "m3")
))
report(x)
report(x, include_IC = FALSE)
report(x, include_ENP = TRUE)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.lavaan.R
\name{report.lavaan}
\alias{report.lavaan}
\alias{report_performance.lavaan}
\title{Reports of Structural Equation Models (SEM)}
\usage{
\method{report}{lavaan}(x, ...)

\method{report_performance}{lavaan}(x, table = NULL, ...)
}
\arguments{
\item{x}{Object of class \code{lavaan}.}

\item{...}{Arguments passed to or from other methods.}

\item{table}{Provide the output of \code{report_table()} to avoid its re-computation.}
}
\value{
An object of class \code{\link[=report]{report()}}.
}
\description{
Create a report for \code{lavaan} objects.
}
\examples{
\dontshow{if (requireNamespace("lavaan", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Structural Equation Models (SEM)
library(lavaan)
structure <- "ind60 =~ x1 + x2 + x3
              dem60 =~ y1 + y2 + y3
              dem60 ~ ind60"
model <- lavaan::sem(structure, data = PoliticalDemocracy)
r <- report(model)
r
summary(r)
as.data.frame(r)
summary(as.data.frame(r))

# Specific reports
suppressWarnings(report_table(model))
suppressWarnings(report_performance(model))
}
\dontshow{\}) # examplesIf}
}
\seealso{
Specific components of reports (especially for stats models):
\itemize{
\item \code{\link[=report_table]{report_table()}}
\item \code{\link[=report_parameters]{report_parameters()}}
\item \code{\link[=report_statistics]{report_statistics()}}
\item \code{\link[=report_effectsize]{report_effectsize()}}
\item \code{\link[=report_model]{report_model()}}
\item \code{\link[=report_priors]{report_priors()}}
\item \code{\link[=report_random]{report_random()}}
\item \code{\link[=report_performance]{report_performance()}}
\item \code{\link[=report_info]{report_info()}}
\item \code{\link[=report_text]{report_text()}}
}

Other types of reports:
\itemize{
\item \code{\link[=report_system]{report_system()}}
\item \code{\link[=report_packages]{report_packages()}}
\item \code{\link[=report_participants]{report_participants()}}
\item \code{\link[=report_sample]{report_sample()}}
\item \code{\link[=report_date]{report_date()}}
}

Methods:
\itemize{
\item \code{\link[=as.report]{as.report()}}
}

Template file for supporting new models:
\itemize{
\item \code{\link[=report.default]{report.default()}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format_algorithm.R, R/format_formula.R,
%   R/format_model.R
\name{format_algorithm}
\alias{format_algorithm}
\alias{format_formula}
\alias{format_model}
\title{Convenient formatting of text components}
\usage{
format_algorithm(x)

format_formula(x, what = "conditional")

format_model(x)
}
\arguments{
\item{x}{The R object that you want to report (see list of of supported
objects above).}

\item{what}{The name of the item returned by \code{insight::find_formula}.}
}
\value{
A character string.

A character string.

A character string.
}
\description{
Convenient formatting of text components
}
\examples{
model <- lm(Sepal.Length ~ Species, data = iris)
format_algorithm(model)

\dontshow{if (requireNamespace("lme4", quietly = TRUE)) withAutoprint(\{ # examplesIf}
# Mixed models
library(lme4)
model <- lme4::lmer(Sepal.Length ~ Sepal.Width + (1 | Species), data = iris)
format_algorithm(model)
\dontshow{\}) # examplesIf}
model <- lm(Sepal.Length ~ Species, data = iris)
format_formula(model)

\dontshow{if (requireNamespace("lme4", quietly = TRUE)) withAutoprint(\{ # examplesIf}
# Mixed models
library(lme4)
model <- lme4::lmer(Sepal.Length ~ Sepal.Width + (1 | Species), data = iris)
format_formula(model)
format_formula(model, "random")
\dontshow{\}) # examplesIf}
model <- lm(Sepal.Length ~ Species, data = iris)
format_model(model)

\dontshow{if (requireNamespace("lme4", quietly = TRUE)) withAutoprint(\{ # examplesIf}
# Mixed models
library(lme4)
model <- lme4::lmer(Sepal.Length ~ Sepal.Width + (1 | Species), data = iris)
format_model(model)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_text.R
\name{report_text}
\alias{report_text}
\title{Report a textual description of an object}
\usage{
report_text(x, table = NULL, ...)
}
\arguments{
\item{x}{The R object that you want to report (see list of of supported
objects above).}

\item{table}{A table obtained via \code{report_table()}. If not provided,
will run it.}

\item{...}{Arguments passed to or from other methods.}
}
\value{
An object of class \code{\link[=report_text]{report_text()}}.
}
\description{
Creates text containing a description of the parameters of R objects (see
list of supported objects in \code{\link[=report]{report()}}).
}
\examples{
library(report)

# Miscellaneous
r <- report_text(sessionInfo())
r
summary(r)

# Data
report_text(iris$Sepal.Length)
report_text(as.character(round(iris$Sepal.Length, 1)))
report_text(iris$Species)
report_text(iris)

# h-tests
report_text(t.test(iris$Sepal.Width, iris$Sepal.Length))

# ANOVA
r <- report_text(aov(Sepal.Length ~ Species, data = iris))
r
summary(r)

# GLMs
r <- report_text(lm(Sepal.Length ~ Petal.Length * Species, data = iris))
r
summary(r)

\dontshow{if (requireNamespace("lme4", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
library(lme4)
model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
r <- report_text(model)
r
summary(r)
}
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("rstanarm", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Bayesian models
library(rstanarm)
model <- suppressWarnings(stan_glm(mpg ~ cyl + wt, data = mtcars, refresh = 0, iter = 600))
r <- report_text(model)
r
summary(r)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.sessionInfo.R
\name{report.sessionInfo}
\alias{report.sessionInfo}
\alias{report_packages}
\alias{cite_packages}
\alias{report_system}
\title{Report R environment (packages, system, etc.)}
\usage{
\method{report}{sessionInfo}(x, ...)

report_packages(session = NULL, include_R = TRUE, ...)

cite_packages(session = NULL, include_R = TRUE, ...)

report_system(session = NULL)
}
\arguments{
\item{x}{The R object that you want to report (see list of of supported
objects above).}

\item{...}{Arguments passed to or from other methods.}

\item{session}{A \link[utils:sessionInfo]{sessionInfo} object.}

\item{include_R}{Include R in the citations.}
}
\value{
For \code{report_packages}, a data frame of class with information on package
name, version and citation.

An object of class \code{\link[=report]{report()}}.
}
\description{
Report R environment (packages, system, etc.)
}
\examples{

library(report)

session <- sessionInfo()

r <- report(session)
r
summary(r)
as.data.frame(r)
summary(as.data.frame(r))

# Convenience functions
report_packages(include_R = FALSE)
cite_packages(prefix = "> ")
report_system()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.brmsfit.R
\name{report.brmsfit}
\alias{report.brmsfit}
\alias{report_effectsize.brmsfit}
\title{Reporting Bayesian Models from brms}
\usage{
\method{report}{brmsfit}(x, ...)

\method{report_effectsize}{brmsfit}(x, effectsize_method = "basic", ...)
}
\arguments{
\item{x}{Object of class \code{lm} or \code{glm}.}

\item{...}{Arguments passed to or from other methods.}

\item{effectsize_method}{Method for computing effect sizes. For \code{brmsfit} objects,
defaults to \code{"basic"} (faster, no refitting) instead of \code{"refit"} to improve
performance with large Bayesian models. See documentation for
\code{\link[effectsize:effectsize]{effectsize::effectsize()}}.}
}
\value{
An object of class \code{\link[=report]{report()}}.
}
\description{
Create reports for Bayesian models. The description of the parameters
follows the Sequential Effect eXistence and sIgnificance Testing framework
(see \link[bayestestR:sexit]{SEXIT documentation}).
}
\details{
Message from the \code{rstan} package: "To avoid recompilation of unchanged
Stan programs, we recommend calling \code{rstan_options(auto_write = TRUE)}"
}
\examples{
\dontshow{if (require("brms", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Bayesian models
library(brms)
model <- suppressWarnings(brm(mpg ~ qsec + wt,
  data = mtcars,
  refresh = 0, iter = 300
))
r <- report(model, verbose = FALSE)
r
summary(r)
as.data.frame(r)
summary(as.data.frame(r))
}
\dontshow{\}) # examplesIf}
}
\seealso{
Specific components of reports (especially for stats models):
\itemize{
\item \code{\link[=report_table]{report_table()}}
\item \code{\link[=report_parameters]{report_parameters()}}
\item \code{\link[=report_statistics]{report_statistics()}}
\item \code{\link[=report_effectsize]{report_effectsize()}}
\item \code{\link[=report_model]{report_model()}}
\item \code{\link[=report_priors]{report_priors()}}
\item \code{\link[=report_random]{report_random()}}
\item \code{\link[=report_performance]{report_performance()}}
\item \code{\link[=report_info]{report_info()}}
\item \code{\link[=report_text]{report_text()}}
}

Other types of reports:
\itemize{
\item \code{\link[=report_system]{report_system()}}
\item \code{\link[=report_packages]{report_packages()}}
\item \code{\link[=report_participants]{report_participants()}}
\item \code{\link[=report_sample]{report_sample()}}
\item \code{\link[=report_date]{report_date()}}
}

Methods:
\itemize{
\item \code{\link[=as.report]{as.report()}}
}

Template file for supporting new models:
\itemize{
\item \code{\link[=report.default]{report.default()}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_priors.R
\name{report_priors}
\alias{report_priors}
\title{Report priors of Bayesian models}
\usage{
report_priors(x, ...)
}
\arguments{
\item{x}{The R object that you want to report (see list of of supported
objects above).}

\item{...}{Arguments passed to or from other methods.}
}
\value{
An object of class \code{\link[=report_priors]{report_priors()}}.
}
\description{
Reports priors of Bayesian models (see list of supported objects in
\code{\link[=report]{report()}}).
}
\examples{
\dontshow{if (requireNamespace("rstanarm", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Bayesian models
library(rstanarm)
model <- stan_glm(mpg ~ disp, data = mtcars, refresh = 0, iter = 1000)
r <- report_priors(model)
r
summary(r)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.estimate_contrasts.R
\name{report.estimate_contrasts}
\alias{report.estimate_contrasts}
\alias{report_table.estimate_contrasts}
\alias{report_text.estimate_contrasts}
\title{Reporting \code{estimate_contrasts} objects}
\usage{
\method{report}{estimate_contrasts}(x, ...)

\method{report_table}{estimate_contrasts}(x, ...)

\method{report_text}{estimate_contrasts}(x, table = NULL, ...)
}
\arguments{
\item{x}{Object of class \code{estimate_contrasts}.}

\item{...}{Arguments passed to or from other methods.}

\item{table}{Provide the output of  \code{report_table()} to avoid its
re-computation.}
}
\value{
An object of class \code{\link[=report]{report()}}.
}
\description{
Create reports for \code{estimate_contrasts} objects.
}
\examples{
\dontshow{if (all(insight::check_if_installed(c("modelbased", "marginaleffects", "collapse", "Formula"), quietly = TRUE))) withAutoprint(\{ # examplesIf}
library(modelbased)
model <- lm(Sepal.Width ~ Species, data = iris)
contr <- estimate_contrasts(model)
report(contr)
\dontshow{\}) # examplesIf}
}
\seealso{
Specific components of reports (especially for stats models):
\itemize{
\item \code{\link[=report_table]{report_table()}}
\item \code{\link[=report_parameters]{report_parameters()}}
\item \code{\link[=report_statistics]{report_statistics()}}
\item \code{\link[=report_effectsize]{report_effectsize()}}
\item \code{\link[=report_model]{report_model()}}
\item \code{\link[=report_priors]{report_priors()}}
\item \code{\link[=report_random]{report_random()}}
\item \code{\link[=report_performance]{report_performance()}}
\item \code{\link[=report_info]{report_info()}}
\item \code{\link[=report_text]{report_text()}}
}

Other types of reports:
\itemize{
\item \code{\link[=report_system]{report_system()}}
\item \code{\link[=report_packages]{report_packages()}}
\item \code{\link[=report_participants]{report_participants()}}
\item \code{\link[=report_sample]{report_sample()}}
\item \code{\link[=report_date]{report_date()}}
}

Methods:
\itemize{
\item \code{\link[=as.report]{as.report()}}
}

Template file for supporting new models:
\itemize{
\item \code{\link[=report.default]{report.default()}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_misc.R
\name{report_date}
\alias{report_date}
\alias{report_story}
\title{Miscellaneous reports}
\usage{
report_date(...)

report_story(...)
}
\arguments{
\item{...}{Arguments passed to or from other methods.}
}
\value{
Objects of class \code{\link[=report_text]{report_text()}}.
}
\description{
Other convenient or totally useless reports.
}
\examples{
library(report)

report_date()
summary(report_date())
report_story()
}
\seealso{
Specific components of reports (especially for stats models):
\itemize{
\item \code{\link[=report_table]{report_table()}}
\item \code{\link[=report_parameters]{report_parameters()}}
\item \code{\link[=report_statistics]{report_statistics()}}
\item \code{\link[=report_effectsize]{report_effectsize()}}
\item \code{\link[=report_model]{report_model()}}
\item \code{\link[=report_priors]{report_priors()}}
\item \code{\link[=report_random]{report_random()}}
\item \code{\link[=report_performance]{report_performance()}}
\item \code{\link[=report_info]{report_info()}}
\item \code{\link[=report_text]{report_text()}}
}

Other types of reports:
\itemize{
\item \code{\link[=report_system]{report_system()}}
\item \code{\link[=report_packages]{report_packages()}}
\item \code{\link[=report_participants]{report_participants()}}
\item \code{\link[=report_sample]{report_sample()}}
\item \code{\link[=report_date]{report_date()}}
}

Methods:
\itemize{
\item \code{\link[=as.report]{as.report()}}
}

Template file for supporting new models:
\itemize{
\item \code{\link[=report.default]{report.default()}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.lm.R, R/report.lme4.R
\name{report.lm}
\alias{report.lm}
\alias{report_effectsize.lm}
\alias{report_table.lm}
\alias{report_statistics.lm}
\alias{report_parameters.lm}
\alias{report_intercept.lm}
\alias{report_model.lm}
\alias{report_performance.lm}
\alias{report_info.lm}
\alias{report_text.lm}
\alias{report_random.merMod}
\title{Reporting (General) Linear Models}
\usage{
\method{report}{lm}(x, include_effectsize = TRUE, effectsize_method = "refit", ...)

\method{report_effectsize}{lm}(x, effectsize_method = "refit", ...)

\method{report_table}{lm}(x, include_effectsize = TRUE, ...)

\method{report_statistics}{lm}(
  x,
  table = NULL,
  include_effectsize = TRUE,
  include_diagnostic = TRUE,
  ...
)

\method{report_parameters}{lm}(
  x,
  table = NULL,
  include_effectsize = TRUE,
  include_intercept = TRUE,
  ...
)

\method{report_intercept}{lm}(x, table = NULL, ...)

\method{report_model}{lm}(x, table = NULL, ...)

\method{report_performance}{lm}(x, table = NULL, ...)

\method{report_info}{lm}(
  x,
  effectsize = NULL,
  include_effectsize = FALSE,
  parameters = NULL,
  ...
)

\method{report_text}{lm}(x, table = NULL, ...)

\method{report_random}{merMod}(x, ...)
}
\arguments{
\item{x}{Object of class \code{lm} or \code{glm}.}

\item{include_effectsize}{If \code{FALSE}, won't include effect-size related
indices (standardized coefficients, etc.).}

\item{effectsize_method}{See documentation for
\code{\link[effectsize:effectsize]{effectsize::effectsize()}}.}

\item{...}{Arguments passed to or from other methods.}

\item{table}{Provide the output of \code{report_table()} to avoid its re-computation.}

\item{include_diagnostic}{If \code{FALSE}, won't include diagnostic related
indices for Bayesian models (ESS, Rhat).}

\item{include_intercept}{If \code{FALSE}, won't include the intercept.}

\item{effectsize}{Provide the output of \code{report_effectsize()} to avoid
its re-computation.}

\item{parameters}{Provide the output of \code{report_parameters()} to avoid
its re-computation.}
}
\value{
An object of class \code{\link[=report]{report()}}.
}
\description{
Create reports for (general) linear models.
}
\examples{
\donttest{
library(report)

# Linear models
model <- lm(Sepal.Length ~ Petal.Length * Species, data = iris)
r <- report(model)
r
summary(r)
as.data.frame(r)
summary(as.data.frame(r))

# Logistic models
model <- glm(vs ~ disp, data = mtcars, family = "binomial")
r <- report(model)
r
summary(r)
as.data.frame(r)
summary(as.data.frame(r))
}

\dontshow{if (requireNamespace("lme4", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Mixed models
library(lme4)
model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
r <- report(model)
r
summary(r)
as.data.frame(r)
summary(as.data.frame(r))
}
\dontshow{\}) # examplesIf}
}
\seealso{
Specific components of reports (especially for stats models):
\itemize{
\item \code{\link[=report_table]{report_table()}}
\item \code{\link[=report_parameters]{report_parameters()}}
\item \code{\link[=report_statistics]{report_statistics()}}
\item \code{\link[=report_effectsize]{report_effectsize()}}
\item \code{\link[=report_model]{report_model()}}
\item \code{\link[=report_priors]{report_priors()}}
\item \code{\link[=report_random]{report_random()}}
\item \code{\link[=report_performance]{report_performance()}}
\item \code{\link[=report_info]{report_info()}}
\item \code{\link[=report_text]{report_text()}}
}

Other types of reports:
\itemize{
\item \code{\link[=report_system]{report_system()}}
\item \code{\link[=report_packages]{report_packages()}}
\item \code{\link[=report_participants]{report_participants()}}
\item \code{\link[=report_sample]{report_sample()}}
\item \code{\link[=report_date]{report_date()}}
}

Methods:
\itemize{
\item \code{\link[=as.report]{as.report()}}
}

Template file for supporting new models:
\itemize{
\item \code{\link[=report.default]{report.default()}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_performance.R
\name{report_performance}
\alias{report_performance}
\title{Report the model's quality and fit indices}
\usage{
report_performance(x, table = NULL, ...)
}
\arguments{
\item{x}{The R object that you want to report (see list of of supported
objects above).}

\item{table}{A table obtained via \code{report_table()}. If not provided,
will run it.}

\item{...}{Arguments passed to or from other methods.}
}
\value{
An object of class \code{\link[=report_performance]{report_performance()}}.
}
\description{
Investigating the fit of statistical models to data often involves selecting
the best fitting model amongst many competing models. This function helps
report indices of model fit for various models.  Reports the type of
different R objects . For a list of supported objects, see
\code{\link[=report]{report()}}).
}
\examples{
\donttest{
# GLMs
report_performance(lm(Sepal.Length ~ Petal.Length * Species, data = iris))
report_performance(glm(vs ~ disp, data = mtcars, family = "binomial"))
}

\dontshow{if (requireNamespace("lme4", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Mixed models
library(lme4)
model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
report_performance(model)
}
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("rstanarm", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Bayesian models
library(rstanarm)
model <- suppressWarnings(stan_glm(Sepal.Length ~ Species, data = iris, refresh = 0, iter = 600))
report_performance(model)
}
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("lavaan", quietly = TRUE) && packageVersion("effectsize") >= "0.6.0.1") withAutoprint(\{ # examplesIf}
\donttest{
# Structural Equation Models (SEM)
library(lavaan)
structure <- "ind60 =~ x1 + x2 + x3
              dem60 =~ y1 + y2 + y3
              dem60 ~ ind60 "
model <- lavaan::sem(structure, data = PoliticalDemocracy)
suppressWarnings(report_performance(model))
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_info.R
\name{report_info}
\alias{report_info}
\title{Report additional information}
\usage{
report_info(x, ...)
}
\arguments{
\item{x}{The R object that you want to report (see list of of supported
objects above).}

\item{...}{Arguments passed to or from other methods.}
}
\value{
An object of class \code{\link[=report_info]{report_info()}}.
}
\description{
Reports additional information relevant to the report (see list of supported
objects in \code{\link[=report]{report()}}).
}
\examples{
library(report)

# h-tests
report_info(t.test(iris$Sepal.Width, iris$Sepal.Length))

# ANOVAs
report_info(aov(Sepal.Length ~ Species, data = iris))
\donttest{
# GLMs
report_info(lm(Sepal.Length ~ Petal.Length * Species, data = iris))
report_info(lm(Sepal.Length ~ Petal.Length * Species, data = iris), include_effectsize = TRUE)
report_info(glm(vs ~ disp, data = mtcars, family = "binomial"))
}

\dontshow{if (requireNamespace("lme4", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Mixed models
library(lme4)
model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
report_info(model)
}
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("rstanarm", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Bayesian models
library(rstanarm)
model <- suppressWarnings(stan_glm(Sepal.Length ~ Species, data = iris, refresh = 0, iter = 300))
report_info(model)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_participants.R
\name{report_participants}
\alias{report_participants}
\title{Reporting the participant data}
\usage{
report_participants(
  data,
  age = NULL,
  sex = NULL,
  gender = NULL,
  education = NULL,
  country = NULL,
  race = NULL,
  participants = NULL,
  by = NULL,
  spell_n = FALSE,
  digits = 1,
  threshold = 10,
  group = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{age}{The name of the column containing the age of the participant.}

\item{sex}{The name of the column containing the sex of the participant. The
classes should be one of \code{c("Male", "M", "Female", "F")}. Note that
you can specify other characters here as well (e.g., \code{"Intersex"}), but
the function will group all individuals in those groups as \code{"Other"}.}

\item{gender}{The name of the column containing the gender of the
classes should be one of \verb{c("Man", "M", "Male", Woman", "W", "F", "Female", Non-Binary", "N")}. Note that you can specify other characters
here as well (e.g., \code{"Gender Fluid"}), but the function will group all
individuals in those groups as \code{"Non-Binary"}.}

\item{education}{The name of the column containing education information.}

\item{country}{The name of the column containing country information.}

\item{race}{The name of the column containing race/ethnicity information.}

\item{participants}{The name of the participants' identifier column (for
instance in the case of repeated measures).}

\item{by}{A character vector indicating the name(s) of the column(s) used
for stratified description.}

\item{spell_n}{Logical, fully spell the sample size (\code{"Three participants"}
instead of \code{"3 participants"}).}

\item{digits}{Number of significant digits.}

\item{threshold}{Percentage after which to combine, e.g., countries (default is 10\%,
so countries that represent less than 10\% will be combined in the "other" category).}

\item{group}{Deprecated. Use \code{by} instead.}

\item{...}{Arguments passed to or from other methods.}
}
\value{
A character vector with description of the "participants", based on
the information provided in \code{data}.
}
\description{
A helper function to help you format the participants data (age, sex, ...) in
the participants section.
}
\examples{
library(report)
data <- data.frame(
  "Age" = c(22, 23, 54, 21, 8, 42),
  "Sex" = c("Intersex", "F", "M", "M", "NA", NA),
  "Gender" = c("N", "W", "W", "M", "NA", NA)
)
report_participants(data, age = "Age", sex = "Sex")

# Years of education (relative to high school graduation)
data$Education <- c(0, 8, -3, -5, 3, 5)
report_participants(data,
  age = "Age", sex = "Sex", gender = "Gender",
  education = "Education"
)

# Education as factor
data$Education2 <- c(
  "Bachelor", "PhD", "Highschool",
  "Highschool", "Bachelor", "Bachelor"
)
report_participants(data, age = "Age", sex = "Sex", gender = "Gender", education = "Education2")

# Country
data <- data.frame(
  "Age" = c(22, 23, 54, 21, 8, 42, 18, 32, 24, 27, 45),
  "Sex" = c("Intersex", "F", "F", "M", "M", "M", "F", "F", "F", "F", "F"),
  "Gender" = c("N", "W", "W", "M", "M", "M", "W", "W", "W", "W", "W"),
  "Country" = c(
    "USA", NA, "Canada", "Canada", "India", "Germany",
    "USA", "USA", "USA", "USA", "Canada"
  )
)
report_participants(data)

# Country, control presentation treshold
report_participants(data, threshold = 5)

# Race/ethnicity
data <- data.frame(
  "Age" = c(22, 23, 54, 21, 8, 42, 18, 32, 24, 27, 45),
  "Sex" = c("Intersex", "F", "F", "M", "M", "M", "F", "F", "F", "F", "F"),
  "Gender" = c("N", "W", "W", "M", "M", "M", "W", "W", "W", "W", "W"),
  "Race" = c(
    "Black", NA, "White", "Asian", "Black", "Arab", "Black",
    "White", "Asian", "Southeast Asian", "Mixed"
  )
)
report_participants(data)

# Race/ethnicity, control presentation treshold
report_participants(data, threshold = 5)

# Repeated measures data
data <- data.frame(
  "Age" = c(22, 22, 54, 54, 8, 8),
  "Sex" = c("I", "F", "M", "M", "F", "F"),
  "Gender" = c("N", "W", "W", "M", "M", "M"),
  "Participant" = c("S1", "S1", "s2", "s2", "s3", "s3")
)
report_participants(data, age = "Age", sex = "Sex", gender = "Gender", participants = "Participant")

# Grouped data
data <- data.frame(
  "Age" = c(22, 22, 54, 54, 8, 8, 42, 42),
  "Sex" = c("I", "I", "M", "M", "F", "F", "F", "F"),
  "Gender" = c("N", "N", "W", "M", "M", "M", "Non-Binary", "Non-Binary"),
  "Participant" = c("S1", "S1", "s2", "s2", "s3", "s3", "s4", "s4"),
  "Condition" = c("A", "A", "A", "A", "B", "B", "B", "B")
)

report_participants(data,
  age = "Age",
  sex = "Sex",
  gender = "Gender",
  participants = "Participant",
  by = "Condition"
)

# Spell sample size
paste(
  report_participants(data, participants = "Participant", spell_n = TRUE),
  "were recruited in the study by means of torture and coercion."
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_text.R, R/report.R,
%   R/report_effectsize.R, R/report_info.R, R/report_intercept.R,
%   R/report_model.R, R/report_parameters.R, R/report_performance.R,
%   R/report_priors.R, R/report_random.R, R/report_statistics.R,
%   R/report_table.R
\name{as.report_text}
\alias{as.report_text}
\alias{as.report}
\alias{is.report}
\alias{as.report_effectsize}
\alias{as.report_info}
\alias{as.report_intercept}
\alias{as.report_model}
\alias{as.report_parameters}
\alias{as.report_performance}
\alias{as.report_priors}
\alias{as.report_random}
\alias{as.report_statistics}
\alias{as.report_table}
\title{Create or test objects of class \link{report}.}
\usage{
as.report_text(x, ...)

as.report(text, table = NULL, plot = NULL, ...)

is.report(x)

as.report_effectsize(x, summary = NULL, prefix = "  - ", ...)

as.report_info(x, summary = NULL, ...)

as.report_intercept(x, summary = NULL, ...)

as.report_model(x, summary = NULL, ...)

as.report_parameters(x, summary = NULL, prefix = "  - ", ...)

as.report_performance(x, summary = NULL, ...)

as.report_priors(x, summary = NULL, ...)

as.report_random(x, summary = NULL, ...)

as.report_statistics(x, summary = NULL, prefix = "  - ", ...)

as.report_table(x, ...)
}
\arguments{
\item{x}{An arbitrary R object.}

\item{...}{Args to be saved as attributes.}

\item{text}{Text obtained via \code{report_text()}}

\item{table}{Table obtained via \code{report_table()}}

\item{plot}{Plot obtained via \code{report_plot()}. Not yet implemented.}

\item{summary}{Add a summary as attribute (to be extracted via \code{summary()}).}

\item{prefix}{The prefix to be displayed in front of each parameter.}
}
\value{
A report object or a \code{TRUE/FALSE} value.
}
\description{
Allows to create or test whether an object is of the \code{report} class.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_model.R
\name{report_model}
\alias{report_model}
\title{Report the model type}
\usage{
report_model(x, table = NULL, ...)
}
\arguments{
\item{x}{The R object that you want to report (see list of of supported
objects above).}

\item{table}{A table obtained via \code{report_table()}. If not provided,
will run it.}

\item{...}{Arguments passed to or from other methods.}
}
\value{
A \code{character} string.
}
\description{
Reports the type of different R objects (see list of supported objects in
\code{\link[=report]{report()}}).
}
\examples{
\donttest{
library(report)

# h-tests
report_model(t.test(iris$Sepal.Width, iris$Sepal.Length))

# ANOVA
report_model(aov(Sepal.Length ~ Species, data = iris))

# GLMs
report_model(lm(Sepal.Length ~ Petal.Length * Species, data = iris))
report_model(glm(vs ~ disp, data = mtcars, family = "binomial"))
}

\dontshow{if (requireNamespace("lme4", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Mixed models
library(lme4)
model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
report_model(model)
}
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("rstanarm", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Bayesian models
library(rstanarm)
model <- suppressWarnings(stan_glm(Sepal.Length ~ Species, data = iris, refresh = 0, iter = 600))
report_model(model)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.test_performance.R
\name{report.test_performance}
\alias{report.test_performance}
\alias{report_table.test_performance}
\alias{report_statistics.test_performance}
\alias{report_parameters.test_performance}
\alias{report_text.test_performance}
\title{Reporting models comparison}
\usage{
\method{report}{test_performance}(x, ...)

\method{report_table}{test_performance}(x, ...)

\method{report_statistics}{test_performance}(x, table = NULL, ...)

\method{report_parameters}{test_performance}(x, table = NULL, ...)

\method{report_text}{test_performance}(x, table = NULL, ...)
}
\arguments{
\item{x}{Object of class \verb{NEW OBJECT}.}

\item{...}{Arguments passed to or from other methods.}

\item{table}{Provide the output of \code{report_table()} to avoid its re-computation.}
}
\value{
An object of class \code{\link[=report]{report()}}.
}
\description{
Create reports for model comparison as obtained by the
\code{\link[performance:compare_performance]{performance::compare_performance()}}
function in the \code{performance} package.
}
\examples{
\donttest{
library(report)
library(performance)

m1 <- lm(Sepal.Length ~ Petal.Length * Species, data = iris)
m2 <- lm(Sepal.Length ~ Petal.Length + Species, data = iris)
m3 <- lm(Sepal.Length ~ Petal.Length, data = iris)

x <- performance::test_performance(m1, m2, m3)
r <- report(x)
r
summary(r)
as.data.frame(r)
summary(as.data.frame(r))

# Specific reports
report_table(x)
report_statistics(x)
report_parameters(x)
}
}
\seealso{
Specific components of reports (especially for stats models):
\itemize{
\item \code{\link[=report_table]{report_table()}}
\item \code{\link[=report_parameters]{report_parameters()}}
\item \code{\link[=report_statistics]{report_statistics()}}
\item \code{\link[=report_effectsize]{report_effectsize()}}
\item \code{\link[=report_model]{report_model()}}
\item \code{\link[=report_priors]{report_priors()}}
\item \code{\link[=report_random]{report_random()}}
\item \code{\link[=report_performance]{report_performance()}}
\item \code{\link[=report_info]{report_info()}}
\item \code{\link[=report_text]{report_text()}}
}

Other types of reports:
\itemize{
\item \code{\link[=report_system]{report_system()}}
\item \code{\link[=report_packages]{report_packages()}}
\item \code{\link[=report_participants]{report_participants()}}
\item \code{\link[=report_sample]{report_sample()}}
\item \code{\link[=report_date]{report_date()}}
}

Methods:
\itemize{
\item \code{\link[=as.report]{as.report()}}
}

Template file for supporting new models:
\itemize{
\item \code{\link[=report.default]{report.default()}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format_citation.R
\name{format_citation}
\alias{format_citation}
\alias{cite_citation}
\alias{clean_citation}
\title{Citation formatting}
\usage{
format_citation(citation, authorsdate = FALSE, short = FALSE, intext = FALSE)

cite_citation(citation)

clean_citation(citation)
}
\arguments{
\item{citation}{A character string of a citation.}

\item{authorsdate}{Only show authors and date (remove title, journal, etc.).}

\item{short}{If more than one authors, replace by \verb{et al.}}

\item{intext}{Remove brackets around the date (so that it can be placed
inside larger parentheses).}
}
\value{
A character string.
}
\description{
Convenience functions to manipulate and format citations. Only works with APA
formatted citations, for now.
}
\examples{
library(report)

citation <- "Makowski, D., Ben-Shachar, M. S., Patil, I., & Ludecke, D. (2020).
Methods and Algorithms for Correlation Analysis in R. Journal of Open Source
Software, 5(51), 2306."

format_citation(citation, authorsdate = TRUE)
format_citation(citation, authorsdate = TRUE, short = TRUE)
format_citation(citation, authorsdate = TRUE, short = TRUE, intext = TRUE)

cite_citation(citation)
clean_citation(citation())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cite_easystats.R
\name{cite_easystats}
\alias{cite_easystats}
\alias{summary.cite_easystats}
\alias{print.cite_easystats}
\title{Cite the easystats ecosystem}
\usage{
cite_easystats(
  packages = "easystats",
  format = c("text", "markdown", "biblatex"),
  intext_prefix = TRUE,
  intext_suffix = "."
)

\method{summary}{cite_easystats}(object, what = "all", ...)

\method{print}{cite_easystats}(x, what = "all", ...)
}
\arguments{
\item{packages}{A character vector of packages to cite. Can be \code{"all"} for
all \emph{easystats} packages or a vector with specific package names.}

\item{format}{The format to generate citations. Can be \code{"text"} for plain
text, \code{"markdown"} for markdown citations and CSL bibliography
(recommended for writing in RMarkdown), or \code{"biblatex"} for BibLaTeX
citations and bibliography.}

\item{intext_prefix}{A character vector of length 1 containing text to
include before in-text citations. If \code{TRUE}, defaults to \code{"Analyses were conducted using the easystats collection of packages "}. If \code{FALSE} or
\code{NA}, no prefix is included.}

\item{intext_suffix}{A character vector of length 1 containing text to
include after in-text citations. Defaults to \code{"."}. If \code{FALSE} or \code{NA},
no suffix is included.}

\item{what}{What elements of the citations to print, can be \code{"all"},
\code{"intext"}, or \code{"refs"}.}

\item{...}{Not used. Included for compatibility with the generic function.}

\item{x, object}{A \code{"cite_easystats"} object to print.}
}
\value{
A list of class \code{"cite_easystats"} with elements:
\itemize{
\item \code{intext}: In-text citations in the requested \code{format}
\item \code{refs}: References or bibliography in the requested \code{format}
}
}
\description{
A convenient function for those who wish to cite the easystats packages.
}
\examples{
\donttest{
# Cite just the 'easystats' umbrella package:
cite_easystats()
summary(cite_easystats(), what = "all")

# Cite every easystats package:
cite_easystats(packages = "all")
summary(cite_easystats(packages = "all"), what = "all")

# Cite specific packages:
cite_easystats(packages = c("modelbased", "see"))
summary(cite_easystats(packages = c("modelbased", "see")), what = "all")

# To cite easystats packages in an RMarkdown document, use:

## In-text citations:
print(cite_easystats(format = "markdown"), what = "intext")

## Bibliography (print with the  `output = 'asis'` option on the code chunk)
print(cite_easystats(format = "markdown"), what = "refs")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report-package.R
\docType{package}
\name{report-package}
\alias{report-package}
\title{report: Automated Results Reporting as a Practical Tool to Improve
Reproducibility and Methodological Best Practices Adoption}
\description{
\strong{report}’s primary goal is to bridge the gap between R’s output and
the formatted results contained in your manuscript. It automatically produces
reports of models and data frames according to \strong{best practices} guidelines
(e.g., \href{https://apastyle.apa.org/}{APA}’s style), ensuring \strong{standardization}
and \strong{quality} in results reporting.
}
\details{
\code{report-package}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://easystats.github.io/report/}
  \item Report bugs at \url{https://github.com/easystats/report/issues}
}

}
\author{
\strong{Maintainer}: Rémi Thériault \email{remi.theriault@mail.mcgill.ca} (\href{https://orcid.org/0000-0003-4315-6788}{ORCID})

Authors:
\itemize{
  \item Rémi Thériault \email{remi.theriault@mail.mcgill.ca} (\href{https://orcid.org/0000-0003-4315-6788}{ORCID})
  \item Dominique Makowski \email{dom.makowski@gmail.com} (\href{https://orcid.org/0000-0001-5375-9967}{ORCID})
  \item Daniel Lüdecke \email{d.luedecke@uke.de} (\href{https://orcid.org/0000-0002-8895-3206}{ORCID})
  \item Indrajeet Patil \email{patilindrajeet.science@gmail.com} (\href{https://orcid.org/0000-0003-1995-6531}{ORCID})
  \item Mattan S. Ben-Shachar \email{matanshm@post.bgu.ac.il} (\href{https://orcid.org/0000-0002-4287-4801}{ORCID})
  \item Brenton M. Wiernik \email{brenton@wiernik.org} (\href{https://orcid.org/0000-0001-9560-6336}{ORCID})
}

Other contributors:
\itemize{
  \item Rudolf Siegel \email{mutlusun@users.noreply.github.com} (\href{https://orcid.org/0000-0002-6021-804X}{ORCID}) [contributor]
  \item Camden Bock \email{camden.bock@maine.edu} (\href{https://orcid.org/0000-0002-3907-7748}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_random.R
\name{report_random}
\alias{report_random}
\title{Report random effects and factors}
\usage{
report_random(x, ...)
}
\arguments{
\item{x}{The R object that you want to report (see list of of supported
objects above).}

\item{...}{Arguments passed to or from other methods.}
}
\value{
An object of class \code{\link[=report_random]{report_random()}}.
}
\description{
Reports random effects of mixed models (see list of supported objects in
\code{\link[=report]{report()}}).
}
\examples{
\dontshow{if (requireNamespace("lme4", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Mixed models
library(lme4)
model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
r <- report_random(model)
r
summary(r)
}
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("rstanarm", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Bayesian models
library(rstanarm)
model <- suppressWarnings(stan_lmer(
  mpg ~ disp + (1 | cyl),
  data = mtcars, refresh = 0, iter = 1000
))
r <- report_random(model)
r
summary(r)
}
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("brms", quietly = TRUE) && packageVersion("rstan") >= "2.26.0") withAutoprint(\{ # examplesIf}
\donttest{
library(brms)
model <- suppressWarnings(brm(mpg ~ disp + (1 | cyl), data = mtcars, refresh = 0, iter = 1000))
r <- report_random(model)
r
summary(r)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_statistics.R
\name{report_statistics}
\alias{report_statistics}
\title{Report the statistics of a model}
\usage{
report_statistics(x, table = NULL, ...)
}
\arguments{
\item{x}{The R object that you want to report (see list of of supported
objects above).}

\item{table}{A table obtained via \code{report_table()}. If not provided,
will run it.}

\item{...}{Arguments passed to or from other methods.}
}
\value{
An object of class \code{\link[=report_statistics]{report_statistics()}}.
}
\description{
Creates a list containing a description of the parameters' values of R
objects (see list of supported objects in \code{\link[=report]{report()}}). Useful to
insert in parentheses in plots or reports.
}
\examples{
\donttest{
library(report)

# Data
report_statistics(iris$Sepal.Length)
report_statistics(as.character(round(iris$Sepal.Length, 1)))
report_statistics(iris$Species)
report_statistics(iris)

# h-tests
report_statistics(t.test(iris$Sepal.Width, iris$Sepal.Length))

# ANOVA
report_statistics(aov(Sepal.Length ~ Species, data = iris))

# GLMs
report_statistics(lm(Sepal.Length ~ Petal.Length * Species, data = iris))
report_statistics(glm(vs ~ disp, data = mtcars, family = "binomial"))
}

\dontshow{if (requireNamespace("lme4", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Mixed models
library(lme4)
model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
report_statistics(model)
}
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("rstanarm", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
# Bayesian models
library(rstanarm)
model <- suppressWarnings(stan_glm(Sepal.Length ~ Species, data = iris, refresh = 0, iter = 600))
report_statistics(model)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_sample.R
\name{report_sample}
\alias{report_sample}
\title{Sample Description}
\usage{
report_sample(
  data,
  by = NULL,
  centrality = "mean",
  ci = NULL,
  ci_method = "wilson",
  ci_correct = FALSE,
  select = NULL,
  exclude = NULL,
  weights = NULL,
  total = TRUE,
  digits = 2,
  n = FALSE,
  group_by = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame for which descriptive statistics should be created.}

\item{by}{Character vector, indicating the column(s) for possible grouping
of the descriptive table. Note that weighting (see \code{weights}) does not work
with more than one grouping column.}

\item{centrality}{Character, indicates the statistics that should be
calculated for numeric variables. May be \code{"mean"} (for mean and
standard deviation) or \code{"median"} (for median and median absolute
deviation) as summary.}

\item{ci}{Level of confidence interval for relative frequencies (proportions).
If not \code{NULL}, confidence intervals are shown for proportions of factor
levels.}

\item{ci_method}{Character, indicating the method how to calculate confidence
intervals for proportions. Currently implemented methods are \code{"wald"} and
\code{"wilson"}. Note that \code{"wald"} can produce intervals outside the plausible
range of [0, 1], and thus it is recommended to prefer the \code{"wilson"} method.
The formulae for the confidence intervals are:
\itemize{
\item \code{"wald"}:

\deqn{p \pm z \sqrt{\frac{p (1 - p)}{n}}}
\item \code{"wilson"}:

\deqn{\frac{2np + z^2 \pm z \sqrt{z^2 + 4npq}}{2(n + z^2)}}

where \code{p} is the proportion (of a factor level), \code{q} is \code{1-p}, \code{z} is the
critical z-score based on the interval level and \code{n} is the length of the
vector (cf. \emph{Newcombe 1998}, \emph{Wilson 1927}).
}}

\item{ci_correct}{Logical, it \code{TRUE}, applies continuity correction. See
\emph{Newcombe 1998} for different correction-methods based on the chosen
\code{ci_method}.}

\item{select}{Character vector, with column names that should be included in
the descriptive table.}

\item{exclude}{Character vector, with column names that should be excluded
from the descriptive table.}

\item{weights}{Character vector, indicating the name of a potential
weight-variable. Reported descriptive statistics will be weighted by
\code{weight}.}

\item{total}{Add a \code{Total} column.}

\item{digits}{Number of decimals.}

\item{n}{Logical, actual sample size used in the calculation of the
reported descriptive statistics (i.e., without the missing values).}

\item{group_by}{Deprecated. Use \code{by} instead.}

\item{...}{Arguments passed to or from other methods.}
}
\value{
A data frame of class \code{report_sample} with variable names and
their related summary statistics.
}
\description{
Create sample description table (also referred to as "Table 1").
}
\examples{
library(report)

report_sample(iris[, 1:4])
report_sample(iris, select = c("Sepal.Length", "Petal.Length", "Species"))
report_sample(iris, by = "Species")
report_sample(airquality, by = "Month", n = TRUE, total = FALSE)

# confidence intervals for proportions
set.seed(123)
d <- data.frame(x = factor(sample(letters[1:3], 100, TRUE, c(0.01, 0.39, 0.6))))
report_sample(d, ci = 0.95, ci_method = "wald") # ups, negative CI
report_sample(d, ci = 0.95, ci_method = "wilson") # negative CI fixed
report_sample(d, ci = 0.95, ci_correct = TRUE) # continuity correction
}
\references{
\itemize{
\item Newcombe, R. G. (1998). Two-sided confidence intervals for the single
proportion: comparison of seven methods. Statistics in Medicine. 17 (8):
857–872
\item Wilson, E. B. (1927). Probable inference, the law of succession, and statistical
inference. Journal of the American Statistical Association. 22 (158): 209–212
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.R
\name{report}
\alias{report}
\title{Automatic reporting of R objects}
\usage{
report(x, ...)
}
\arguments{
\item{x}{The R object that you want to report (see list of of supported
objects above).}

\item{...}{Arguments passed to or from other methods.}
}
\value{
A list-object of class \code{report}, which contains further
list-objects with a short and long description of the model summary, as
well as a short and long table of parameters and fit indices.
}
\description{
Create reports of different objects. See the documentation for your object's
class:
}
\details{
\itemize{
\item \link[=report.sessionInfo]{System and packages} (\code{sessionInfo})
\item \link[=report.data.frame]{Dataframes and vectors}
\item \link[=report.htest]{Correlations and t-tests} (\code{htest})
\item \link[=report.aov]{ANOVAs} (\verb{aov, anova, aovlist, ...})
\item \link[=report.lm]{Regression models} (\verb{glm, lm, ...})
\item \link[=report.lm]{Mixed models} (\verb{glmer, lmer, glmmTMB, ...})
\item \link[=report.stanreg]{Bayesian models} (\verb{stanreg, brms...})
\item \link[=report.bayesfactor_models]{Bayes factors} (from \code{bayestestR})
\item \link[=report.lavaan]{Structural Equation Models (SEM)} (from \code{lavaan})
\item \link[=report.compare_performance]{Model comparison} (from \code{\link[performance:compare_performance]{performance()}})
}

Most of the time, the object created by the \code{report()} function can be
further transformed, for instance summarized (using \code{summary()}), or
converted to a table (using \code{as.data.frame()}).

\subsection{Organization}{
\code{report_table} and \code{report_text} are the two distal representations
of a report, and are the two provided in \code{report()}. However,
intermediate steps are accessible (depending on the object) via specific
functions (e.g., \code{report_parameters}).
}

\subsection{Output}{

The \code{report()} function generates a report-object that contain in itself
different representations (e.g., text, tables, plots). These different
representations can be accessed via several functions, such as:
\itemize{
\item \strong{\code{as.report_text(r)}}: Detailed text.
\item \strong{\code{as.report_text(r, summary=TRUE)}}: Minimal text giving
the minimal information.
\item \strong{\code{as.report_table(r)}}: Comprehensive table including most
available indices.
\item \strong{\code{as.report_table(r, summary=TRUE)}}: Minimal table.
}

Note that for some report objects, some of these representations might be
identical.
}
}
\examples{

library(report)

model <- t.test(mtcars$mpg ~ mtcars$am)
r <- report(model)

# Text
r
summary(r)

# Tables
as.data.frame(r)
summary(as.data.frame(r))

}
\seealso{
Specific components of reports (especially for stats models):
\itemize{
\item \code{\link[=report_table]{report_table()}}
\item \code{\link[=report_parameters]{report_parameters()}}
\item \code{\link[=report_statistics]{report_statistics()}}
\item \code{\link[=report_effectsize]{report_effectsize()}}
\item \code{\link[=report_model]{report_model()}}
\item \code{\link[=report_priors]{report_priors()}}
\item \code{\link[=report_random]{report_random()}}
\item \code{\link[=report_performance]{report_performance()}}
\item \code{\link[=report_info]{report_info()}}
\item \code{\link[=report_text]{report_text()}}
}

Other types of reports:
\itemize{
\item \code{\link[=report_system]{report_system()}}
\item \code{\link[=report_packages]{report_packages()}}
\item \code{\link[=report_participants]{report_participants()}}
\item \code{\link[=report_sample]{report_sample()}}
\item \code{\link[=report_date]{report_date()}}
}

Methods:
\itemize{
\item \code{\link[=as.report]{as.report()}}
}

Template file for supporting new models:
\itemize{
\item \code{\link[=report.default]{report.default()}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.BFBayesFactor.R
\name{report.BFBayesFactor}
\alias{report.BFBayesFactor}
\alias{report_statistics.BFBayesFactor}
\title{Reporting \code{BFBayesFactor} objects from the \code{BayesFactor} package}
\usage{
\method{report}{BFBayesFactor}(x, h0 = "H0", h1 = "H1", ...)

\method{report_statistics}{BFBayesFactor}(x, table = NULL, ...)
}
\arguments{
\item{x}{An object of class \code{BFBayesFactor}.}

\item{h0, h1}{Names of the null and alternative hypotheses.}

\item{...}{Other arguments to be passed to \link[effectsize:interpret_bf]{effectsize::interpret_bf} and \link[insight:format_bf]{insight::format_bf}.}

\item{table}{A \code{parameters} table (this argument is meant for internal use).}
}
\description{
Interpretation of the Bayes factor output from the \code{BayesFactor} package.
}
\examples{
\dontshow{if (requireNamespace("BayesFactor", quietly = TRUE)) withAutoprint(\{ # examplesIf}
\donttest{
library(BayesFactor)

rez <- BayesFactor::ttestBF(iris$Sepal.Width, iris$Sepal.Length)
report_statistics(rez, exact = TRUE) # Print exact BF
report(rez, h0 = "the null hypothesis", h1 = "the alternative")

rez <- BayesFactor::correlationBF(iris$Sepal.Width, iris$Sepal.Length)
report(rez)
}
\dontshow{\}) # examplesIf}
}
